@@ -3,11 +3,11 @@ A parser for ADQL.
 
 The grammar follows the official BNF grammar quite closely, except where
 pyparsing makes a different approach desirable; the names should mostly
-match except for the obious underscore to camel case map.
+match except for the obvious underscore to camel case map.
 
 The grammar given in the spec has some nasty rules when you're parsing
 without backtracking and by recursive descent (which is what pyparsing
-does).  I need some reformulations.  The more interesting of those 
+does).  I need some reformulations.  The more interesting of those
 include:
 
 TableReference
@@ -93,17 +93,17 @@ By substitution, one has::
 
 hence::
 
-	<table_name> ::= [[ <identifier> <period> ] <identifier> <period> ] 
+	<table_name> ::= [[ <identifier> <period> ] <identifier> <period> ]
 		<identifier>
 
 hence::
 
-	<qualifier> ::= [[ <identifier> <period> ] <identifier> <period> ] 
+	<qualifier> ::= [[ <identifier> <period> ] <identifier> <period> ]
 		<identifier>
 
 (which matches both table_name and correlation_name) and thus::
 
-	<column_reference> ::= [[[ <identifier> <period> ] <identifier> <period> ] 
+	<column_reference> ::= [[[ <identifier> <period> ] <identifier> <period> ]
 		<identifier> <period> ] <identifier>
 
 We need the table_name, qualifier, and column_reference productions.
@@ -119,11 +119,11 @@ string expressions.  I'm not sure why generalLiterals are allowed
 in there anyway.  If this bites at some point, we'll face a major rewrite
 of the grammar (or we need to dump pyparsing).
 
-To make the whole thing work, I added the generalLiteral to the 
+To make the whole thing work, I added the generalLiteral to the
 characterPrimary production.
 """
 
-#c Copyright 2008-2022, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -134,9 +134,10 @@ import functools
 from gavo.utils.parsetricks import (
 	Word, Literal, Optional, alphas, CaselessKeyword,
 	ZeroOrMore, OneOrMore, StringEnd,
-	Forward, 
+	Forward,
+	MatchFirstWithSaneError,   # temporary hack, see parsetricks
 	CaselessLiteral, ParseException, Regex, sglQuotedString, alphanums,
-	White, 
+	White,
 	ParseExpression,
 	Suppress, pyparsingWhitechars)
 from gavo.utils.parsetricks import ParseSyntaxException #noflake: exported name
@@ -158,7 +159,7 @@ ADQL_RESERVED_WORDS = set([ "ABS", "ACOS", "AREA", "ASIN", "ATAN", "ATAN2",
 	"MOD", "PI", "POINT", "POLYGON", "POWER", "RADIANS", "REGION",
 	"RAND", "ROUND", "SIN", "SQUARE", "SQRT", "TOP", "TAN", "TRUNCATE",
 	"OFFSET", "BITWISE_AND", "BITWISE_OR", "BITWISE_XOR", "BITWISE_NOT",
-	"MOC"])
+	"MOC", "STDDEV"])
 
 SQL_RESERVED_WORDS = set([
 	"ABSOLUTE", "ACTION", "ADD", "ALL", "ALLOCATE", "ALTER", "AND", "ANY",
@@ -224,7 +225,7 @@ class RegularIdentifier(Word):
 	Filtering these in the parse action doesn't always work properly for
 	all versions of pyparsing, thus this special class.
 
-	reservedWords are assumed to be in upper case, but matching 
+	reservedWords are assumed to be in upper case, but matching
 	case-insensitively.
 	"""
 	def __init__(self, reservedWords):
@@ -278,7 +279,7 @@ class LongestMatch(ParseExpression):
 			if maxException is not None:
 				raise maxException
 			else:
-				raise ParseException(instring, loc, 
+				raise ParseException(instring, loc,
 					"no defined alternatives to match", self)
 
 		return maxMatchExp._parse(instring, loc, doActions)
@@ -323,20 +324,21 @@ def getADQLGrammarCopy(nodes):
 			).addParseAction(lambda s,p,t: str(int(t[0], 16))
 			)
 		g.unsignedHexadecimal.setName("numeric literal")
-		g.unsignedNumericLiteral = ( 
+		g.unsignedNumericLiteral = (
 			g.unsignedHexadecimal
-			| g.approximateNumericLiteral 
+			| g.approximateNumericLiteral
 			| g.exactNumericLiteral)
 		g.unsignedNumericLiteral.setName("numericLiteral")
 		g.characterStringLiteral = sglQuotedString + ZeroOrMore( sglQuotedString )
 		g.generalLiteral = g.characterStringLiteral.copy()
-		g.unsignedLiteral = g.unsignedNumericLiteral 
+		g.unsignedLiteral = g.unsignedNumericLiteral
 			# !!! DEVIATION: we're not accepting | generalLiteral here.
 		g.sign = Literal("+") | "-"
-		g.signedInteger = (Optional( g.sign ) 
+		g.signedInteger = (Optional( g.sign )
 			+ g.unsignedInteger)
 		g.signedInteger.setName("signed integer")
 		g.nullLiteral = CaselessKeyword("NULL")
+		g.nullLiteral.setName("NULL")
 
 		g.regularIdentifier = RegularIdentifier(ALL_RESERVED_WORDS)
 		g.regularIdentifier.setName("identifier")
@@ -352,7 +354,7 @@ def getADQLGrammarCopy(nodes):
 		g.identifier = g.regularIdentifier | g.delimitedIdentifier
 
 # Operators
-		g.compOp = Regex("=|!=|<=|>=|<|>")
+		g.compOp = Regex("=|!=|<=|>=|<>|<|>")
 		g.compOp.setName("comparison operator")
 		g.multOperator = Literal("*") | Literal("/")
 		g.addOperator =  Literal("+") | Literal("-")
@@ -362,16 +364,16 @@ def getADQLGrammarCopy(nodes):
 		g.columnName = g.identifier.copy()
 		g.columnName.setName("column name")
 		g.correlationName = g.identifier.copy()
-		g.qualifier = (g.identifier 
+		g.qualifier = (g.identifier
 			+ Optional( "." + g.identifier )
 			+ Optional( "." + g.identifier ))
 		g.tableName = g.qualifier("tableName")
 		g.columnReferenceByUCD = (
-			CaselessKeyword("UCDCOL") 
+			CaselessKeyword("UCDCOL")
 			+ '(' + g.characterStringLiteral + ')')
 		g.columnReference = (
 			g.columnReferenceByUCD
-			| g.identifier 
+			| g.identifier
 				+ Optional( "." + g.identifier )
 				+ Optional( "." + g.identifier )
 				+ Optional( "." + g.identifier ))
@@ -381,11 +383,11 @@ def getADQLGrammarCopy(nodes):
 		g.valueExpression = Forward()
 
 # set functions
-		g.setFunctionType = Regex("(?i)AVG|MAX|MIN|SUM|COUNT")
+		g.setFunctionType = Regex("(?i)AVG|MAX|MIN|SUM|COUNT|STDDEV")
 		g.setQuantifier = Regex("(?i)DISTINCT|ALL")
-		g.generalSetFunction = (g.setFunctionType("fName") 
+		g.generalSetFunction = (g.setFunctionType("fName")
 			+ '(' + Optional(g.setQuantifier) + Args(g.valueExpression) + ')')
-		g.countAll = (CaselessLiteral("COUNT")("fName") 
+		g.countAll = (CaselessLiteral("COUNT")("fName")
 			+ '(' + Args(Literal('*')) + ')')
 		g.setFunctionSpecification = (g.countAll | g.generalSetFunction)
 
@@ -396,7 +398,7 @@ def getADQLGrammarCopy(nodes):
 		g.castOperand.setName("value to cast")
 		g.length = g.unsignedInteger | '*'
 		g.sqlTypeLength = Literal("(") - g.length - Literal(")")
-		g.castTarget = (
+		g.sqlAtomType = (
 			((CaselessKeyword("CHAR") | CaselessKeyword("VARCHAR"))
     		+ Optional(g.sqlTypeLength))
     	| (CaselessKeyword("NATIONAL") - CaselessKeyword("CHAR")
@@ -406,12 +408,20 @@ def getADQLGrammarCopy(nodes):
     	| CaselessKeyword("SMALLINT")
     	| CaselessKeyword("REAL")
     	| CaselessKeyword("DOUBLE") + CaselessKeyword("PRECISION")
-    	| CaselessKeyword("TIMESTAMP")).addParseAction(
+    	| CaselessKeyword("TIMESTAMP")
+    	| CaselessKeyword("POINT")
+    	| CaselessKeyword("CIRCLE")
+    	| CaselessKeyword("POLYGON"))
+		g.sqlArraySpec = (Literal("[")
+   		+ Optional(g.unsignedInteger)
+   		+ Literal("]"))
+		g.castTarget = (g.sqlAtomType
+			+ Optional(g.sqlArraySpec)).addParseAction(
     		lambda s,p,t: " ".join(t).upper())
 		g.castSpecification = (
 			CaselessLiteral("CAST")
 				- "(" + g.castOperand.setResultsName("value")
-				- CaselessLiteral("AS") 
+				- CaselessLiteral("AS")
 				+ g.castTarget.setResultsName("newType") - ')')
 
 # value expressions
@@ -420,12 +430,12 @@ def getADQLGrammarCopy(nodes):
 			+ Literal('[')
 			+ g.valueExpression
 			+ Literal(']'))
-		g.valueExpressionPrimary = ( 
+		g.valueExpressionPrimary = (
 			CaselessKeyword("NULL")
-			| g.unsignedLiteral 
+			| g.unsignedLiteral
 			| g.arrayReference
-			| g.columnReference 
-			| g.setFunctionSpecification 
+			| g.columnReference
+			| g.setFunctionSpecification
 			# this should allow whole character value expressions by the
 			# spec.  If we ever actually want that, we'd have to see how
 			# to avoid infinite recursion.
@@ -438,7 +448,7 @@ def getADQLGrammarCopy(nodes):
 		g.characterPrimary = Forward()
 		g.characterPrimary.setName("String expression")
 		g.characterFactor = g.characterPrimary
-		g.characterValueExpression = (g.characterFactor + 
+		g.characterValueExpression = (g.characterFactor +
 			ZeroOrMore("||" + g.characterFactor))
 		g.stringValueExpression = g.characterValueExpression
 
@@ -448,16 +458,16 @@ def getADQLGrammarCopy(nodes):
 		g.numericValueFunction = Forward()
 		g.numericValueFunction.setName("Numeric value function")
 		g.numericExpressionPrimary = (
-			g.unsignedLiteral 
+			g.unsignedLiteral
 			| g.columnReference
-			| g.setFunctionSpecification 
+			| g.setFunctionSpecification
 			| '(' + g.valueExpression + ')')
-		g.numericPrimary = ( 
-			g.numericValueFunction 
+		g.numericPrimary = (
+			g.numericValueFunction
 			| g.valueExpressionPrimary)
 		g.numericPrimary.setName("numeric expression")
 		g.factor = Optional(g.sign) + g.numericPrimary
-		g.term = (g.factor 
+		g.term = (g.factor
 			+ ZeroOrMore(g.multOperator + g.factor))
 		g.numericValueExpression << (
 			g.term + ZeroOrMore(g.addOperator + g.term)
@@ -478,25 +488,25 @@ def getADQLGrammarCopy(nodes):
 			).setResultsName("coordSys")
 		g.coordSys.setName("coordinate system literal (ICRS, GALACTIC,...)")
 		g.coordinates = (g.nvArgs + ',' + g.nvArgs)
-		g.box = (CaselessKeyword("BOX")("fName") 
+		g.box = (CaselessKeyword("BOX")("fName")
 			- '(' + Optional(g.coordSys + ',')
 			+ g.coordinates +  ','
 			+ g.coordinates + ')')
-		g.point = (CaselessKeyword("POINT")("fName") 
+		g.point = (CaselessKeyword("POINT")("fName")
 			- '(' + Optional(g.coordSys - ',')
 			+ g.coordinates + ')')
 		g.coordValue = (
-			g.userDefinedFunction 
-			| g.point 
+			g.userDefinedFunction
+			| g.point
 			| g.columnReference)
 
 		g.circleSplitCooArgs = (g.coordinates + ',' - g.nvArgs)
 		g.circlePointCooArgs = (
 			Args(g.coordValue) + ','
 			- g.nvArgs)
-		g.circle = (CaselessKeyword("CIRCLE")("fName") 
+		g.circle = (CaselessKeyword("CIRCLE")("fName")
 			- '(' + Optional(g.coordSys + ',')
-			+ (g.circleSplitCooArgs | g.circlePointCooArgs) 
+			+ (g.circleSplitCooArgs | g.circlePointCooArgs)
 			+ ')')
 
 		g.polygonSplitCooArgs = (g.coordinates
@@ -505,59 +515,59 @@ def getADQLGrammarCopy(nodes):
 		g.polygonPointCooArgs = (Args(g.coordValue)
 			- ',' - Args(g.coordValue)
 			- OneOrMore(',' - Args(g.coordValue)))
-		g.polygon = (CaselessKeyword("POLYGON")("fName") 
+		g.polygon = (CaselessKeyword("POLYGON")("fName")
 			- '(' + Optional(g.coordSys + ',')
 			+ Args(g.polygonSplitCooArgs | g.polygonPointCooArgs)
 			+ ')')
 
 		g.mocArgs = (
-			Args(g.stringValueExpression) 
+			Args(g.stringValueExpression)
 			^ Args(
-				g.numericValueExpression) - ',' 
+				g.numericValueExpression) - ','
 				- Args(g.geometryValueExpression))
 		g.moc = (CaselessKeyword("MOC")("fName")
 			- '(' + g.mocArgs + ')')
 
-		g.region = (CaselessKeyword("REGION")("fName") 
-			+ '(' 
+		g.region = (CaselessKeyword("REGION")("fName")
+			+ '('
 			+ Args(g.stringValueExpression) + ')')
-		g.geometryExpression = (g.box 
-			| g.point 
-			| g.circle 
-			| g.polygon 
-			| g.region 
+		g.geometryExpression = (g.box
+			| g.point
+			| g.circle
+			| g.polygon
+			| g.region
 			| g.moc)
 		g.geometryValue = g.columnReference.copy()
-		g.centroid = (CaselessKeyword("CENTROID")("fName") 
+		g.centroid = (CaselessKeyword("CENTROID")("fName")
 			+ '(' + Args(g.geometryValueExpression) + ')')
-		g.geometryValueExpression << (g.geometryExpression 
+		g.geometryValueExpression << (g.geometryExpression
 			| g.userDefinedFunction
-			| g.geometryValue 
+			| g.geometryValue
 			| g.centroid)
 		g.geometryValueExpression.setName("geometry value expression")
 
 # geometry functions
-		g.distanceSplitCooArgs = (g.coordinates + ',' 
+		g.distanceSplitCooArgs = (g.coordinates + ','
 			+ g.coordinates)
 		g.distanceSplitCooArgs.setName("Numeric coordinates")
-		g.distancePointCooArgs = (Args(g.coordValue) + ',' 
+		g.distancePointCooArgs = (Args(g.coordValue) + ','
 			+ Args(g.coordValue))
 		g.distancePointCooArgs.setName("Geometry argument")
-		g.distanceFunction = (CaselessKeyword("DISTANCE")("fName") 
+		g.distanceFunction = (CaselessKeyword("DISTANCE")("fName")
 			- '(' +  (g.distanceSplitCooArgs | g.distancePointCooArgs) + ')')
 		g.pointFunction = (Regex("(?i)COORD[12]|COORDSYS")("fName") + '(' +
 			Args(g.coordValue) + ')')
-		g.area = (CaselessKeyword("AREA")("fName") 
+		g.area = (CaselessKeyword("AREA")("fName")
 			+ '(' + Args(g.geometryValueExpression) + ')')
 		g.nonPredicateGeometryFunction = (
-			g.distanceFunction 
-			| g.pointFunction 
+			g.distanceFunction
+			| g.pointFunction
 			| g.area )
 		g.predicateGeoFunctionName = Regex("(?i)CONTAINS|INTERSECTS")
-		g.predicateGeometryFunction = (g.predicateGeoFunctionName("fName") 
-			+ '(' + Args(g.geometryValueExpression) 
+		g.predicateGeometryFunction = (g.predicateGeoFunctionName("fName")
+			+ '(' + Args(g.geometryValueExpression)
 			+ ',' + Args(g.geometryValueExpression) + ')')
-		g.numericGeometryFunction = (g.predicateGeometryFunction 
+		g.numericGeometryFunction = (g.predicateGeometryFunction
 			| g.nonPredicateGeometryFunction)
 
 # numeric, system, user defined functions
@@ -568,9 +578,9 @@ def getADQLGrammarCopy(nodes):
 		g.trig1ArgFunctionName.setName("numeric expression")
 		g.trigFunction = (
 				g.trig1ArgFunctionName("fName") + '('
-					+ g.nvArgs + ')' 
+					+ g.nvArgs + ')'
 			|	CaselessKeyword("ATAN2")("fName") + '('
-					+ g.nvArgs + ',' 
+					+ g.nvArgs + ','
 					+ g.nvArgs + ')')
 		g.math0ArgFunctionName = Regex("(?i)PI")
 		g.optIntFunctionName = Regex("(?i)RAND")
@@ -580,16 +590,16 @@ def getADQLGrammarCopy(nodes):
 		g.math2ArgFunctionName = Regex("(?i)POWER|MOD|BITWISE_AND|BITWISE_OR"
 			"|BITWISE_XOR")
 		g.mathFunction = (
-				g.math0ArgFunctionName("fName") + '(' + ')' 
-			|	g.optIntFunctionName("fName") + '(' 
-				+ Optional( Args(g.unsignedInteger) ) + ')' 
-			|	g.math1ArgFunctionName("fName") + '(' 
-				+ g.nvArgs + ')' 
-			| g.optPrecArgFunctionName("fName") + '(' 
+				g.math0ArgFunctionName("fName") + '(' + ')'
+			|	g.optIntFunctionName("fName") + '('
+				+ Optional( Args(g.unsignedInteger) ) + ')'
+			|	g.math1ArgFunctionName("fName") + '('
+				+ g.nvArgs + ')'
+			| g.optPrecArgFunctionName("fName") + '('
 				+ g.nvArgs +
-				Optional( ',' + Args(g.signedInteger) ) + ')' 
-			|	g.math2ArgFunctionName("fName") + '(' 
-				+ g.nvArgs 
+				Optional( ',' + Args(g.signedInteger) ) + ')'
+			|	g.math2ArgFunctionName("fName") + '('
+				+ g.nvArgs
 				+ ',' + g.nvArgs + ')')
 		g.inUnitFunction = (
 			CaselessKeyword("IN_UNIT")
@@ -598,18 +608,37 @@ def getADQLGrammarCopy(nodes):
 			- ','
 			- g.characterStringLiteral
 			- ')')
+		g.scalarArray1FunctionName = Regex("(?i)arr_(min|max|avg|stddev|sum)")
+		g.scalarArray1Function = (
+			g.scalarArray1FunctionName("fName")
+			- '(' - g.nvArgs - ')')
+		g.scalarArray2FunctionName = Regex("(?i)arr_dot")
+		g.scalarArray2Function = (
+			g.scalarArray2FunctionName("fName")
+			- '(' - g.nvArgs - ',' - g.nvArgs - ')')
+		g.scalarArrayFunction = (
+			g.scalarArray2Function
+			| g.scalarArray1Function)
+		
+		g.arrayMapFunction = (
+			CaselessKeyword("arr_map")("fName") - '('
+			- Args(g.numericValueExpression)
+			- ',' - g.nvArgs - ')')
+
 		g.userDefinedFunctionParam = g.valueExpression
 		g.userDefinedFunctionName = Regex("(?i)"+userFunctionPrefix+"_[A-Za-z_]+")
 		g.userDefinedFunctionName.setName("Name of locally defined function")
-		g.userDefinedFunction << (g.userDefinedFunctionName("fName") 
-			+ '(' 
-			+ Optional(Args(g.userDefinedFunctionParam) 
+		g.userDefinedFunction << (g.userDefinedFunctionName("fName")
+			+ '('
+			+ Optional(Args(g.userDefinedFunctionParam)
 				+ ZeroOrMore("," + Args(g.userDefinedFunctionParam)))
 			+ ')')
-		g.numericValueFunction << (g.trigFunction 
-			| g.mathFunction 
+		g.numericValueFunction << (g.trigFunction
+			| g.mathFunction
 			| g.inUnitFunction
-			| g.userDefinedFunction 
+			| g.scalarArrayFunction
+			| g.arrayMapFunction
+			| g.userDefinedFunction
 			| g.numericGeometryFunction)
 
 		g.foldFunction = (
@@ -619,24 +648,54 @@ def getADQLGrammarCopy(nodes):
 
 		g.characterPrimary << (
 			g.stringValueFunction
-			| g.generalLiteral 
+			| g.generalLiteral
 			| g.userDefinedFunction
 			| g.valueExpressionPrimary)
 
 		g.timestampFunction = (
-			CaselessKeyword('TIMESTAMP')("fName") - '(' 
+			CaselessKeyword('TIMESTAMP')("fName") - '('
 				+ Args(g.stringValueExpression) - ')')
 		g.dateValueExpression = (g.timestampFunction)
 
 		g.coalesceExpression = (
-			CaselessKeyword('COALESCE') - '(' 
-				+ Args(g.valueExpression) 
+			CaselessKeyword('COALESCE') - '('
+				+ Args(g.valueExpression)
 					+ ZeroOrMore(',' + Args(g.valueExpression)) - ')')
-		
+
+		g.searchCondition = Forward()
+		g.searchCondition.setName("Condition")
+		g.caseResult = g.valueExpression
+		g.searchedWhenClause = (
+			CaselessKeyword('WHEN')
+			+ g.searchCondition
+			+ CaselessKeyword('THEN')
+			+ g.caseResult)
+		g.simpleWhenClause = (
+			CaselessKeyword('WHEN')
+			+ g.valueExpression
+			+ CaselessKeyword('THEN')
+			+ g.caseResult)
+		g.caseElseClause = CaselessKeyword('ELSE') + g.caseResult
+		g.searchedCase = (
+			CaselessKeyword('CASE')
+			+ OneOrMore(g.searchedWhenClause)
+			+ Optional(g.caseElseClause)
+			- CaselessKeyword('END'))
+		g.simpleCase = (
+			CaselessKeyword('CASE')
+			+ g.valueExpression
+			+ OneOrMore(g.simpleWhenClause)
+			+ Optional(g.caseElseClause)
+			- CaselessKeyword('END'))
+		g.caseExpression = (
+			g.coalesceExpression
+			| g.searchedCase
+			| g.simpleCase)
+	
 # toplevel value expression
 		g.valueExpression << (
 			LongestMatch([
-				g.coalesceExpression,
+				g.caseExpression,
 				g.numericValueExpression,
 				g.stringValueExpression,
 				g.dateValueExpression,
@@ -645,7 +704,7 @@ def getADQLGrammarCopy(nodes):
 		g.derivedColumn = g.valueExpression("expr") + Optional(g.asClause)
 
 # parts of select clauses
-		g.setQuantifier = (CaselessKeyword( "DISTINCT" ) 
+		g.setQuantifier = (CaselessKeyword( "DISTINCT" )
 			| CaselessKeyword( "ALL" ))("setQuantifier")
 		g.setLimit = CaselessKeyword( "TOP" ) - g.unsignedInteger("setLimit")
 		g.offsetSpec = CaselessKeyword( "OFFSET" )- g.unsignedInteger("offset")
@@ -661,57 +720,55 @@ def getADQLGrammarCopy(nodes):
 # boolean terms
 		g.subquery = Forward()
 		g.subquery.setName("Subquery")
-		g.searchCondition = Forward()
-		g.searchCondition.setName("Condition")
 		g.comparisonPredicate = g.valueExpression + g.compOp + g.valueExpression
-		g.betweenPredicate = (g.valueExpression 
-			+ Optional(g.notKeyword) 
-			+ CaselessKeyword("BETWEEN") 
-			- g.valueExpression 
-			+ CaselessKeyword("AND") 
+		g.betweenPredicate = (g.valueExpression
+			+ Optional(g.notKeyword)
+			+ CaselessKeyword("BETWEEN")
+			- g.valueExpression
+			+ CaselessKeyword("AND")
 			- g.valueExpression)
 		g.inValueList = g.valueExpression + ZeroOrMore(',' + g.valueExpression)
 		g.inPredicateValue = g.subquery | ("(" + g.inValueList + ")")
-		g.inPredicate = (g.valueExpression + Optional(g.notKeyword) 
+		g.inPredicate = (g.valueExpression + Optional(g.notKeyword)
 			+ CaselessKeyword("IN") + g.inPredicateValue)
 		g.existsPredicate = CaselessKeyword("EXISTS") - g.subquery
 		g.likePredicate = (
-			g.characterValueExpression 
-			+ Optional(g.notKeyword) 
+			g.characterValueExpression
+			+ Optional(g.notKeyword)
 			+ (CaselessKeyword("LIKE") | CaselessKeyword("ILIKE"))
 			+ g.characterValueExpression)
-		g.nullPredicate = (g.columnReference 
-			+ CaselessKeyword("IS") 
+		g.nullPredicate = (g.columnReference
+			+ CaselessKeyword("IS")
 			+ Optional(g.notKeyword) - g.nullLiteral)
-		g.predicate = (g.comparisonPredicate 
-			| g.betweenPredicate 
-			| g.inPredicate 
-			| g.likePredicate 
-			| g.nullPredicate 
+		g.predicate = (g.comparisonPredicate
+			| g.betweenPredicate
+			| g.inPredicate
+			| g.likePredicate
+			| g.nullPredicate
 			| g.existsPredicate)
 		g.booleanPrimaryOpener = Literal('(')  # for error messages
 		g.booleanPrimaryOpener.setName("boolean expression")
 		g.booleanPrimary = (
-			g.booleanPrimaryOpener + g.searchCondition + ')' 
+			g.booleanPrimaryOpener + g.searchCondition + ')'
 			| g.predicate)
 		g.booleanPrimary.setName("boolean expression")
 		g.booleanFactor = Optional(g.notKeyword) + g.booleanPrimary
-		g.booleanTerm = (g.booleanFactor + 
+		g.booleanTerm = (g.booleanFactor +
 			ZeroOrMore(CaselessKeyword("AND") - g.booleanFactor))
 
 # WHERE clauses and such
-		g.searchCondition << (g.booleanTerm 
+		g.searchCondition << (g.booleanTerm
 			+ ZeroOrMore(CaselessKeyword("OR") - g.booleanTerm)
 			)
 		g.searchCondition.setName("search condition")
-		g.whereClause = (CaselessKeyword("WHERE") 
+		g.whereClause = (CaselessKeyword("WHERE")
 			- g.searchCondition)("whereClause")
 
 # Referencing tables
 		g.queryExpression = Forward()
 		g.queryExpression.setName("Query expression")
-		g.correlationSpecification = ( 
-			Optional(CaselessKeyword("AS")  | g.whitespace) 
+		g.correlationSpecification = (
+			Optional(CaselessKeyword("AS")  | g.whitespace)
 				+ g.correlationName("alias"))
 		g.subqueryOpener = Literal('(')  # for error reporting
 		g.subqueryOpener.setName("subquery")
@@ -719,10 +776,10 @@ def getADQLGrammarCopy(nodes):
 		g.derivedTable = g.subquery.copy() + g.correlationSpecification
 		g.tablesample = (CaselessKeyword("TABLESAMPLE")
 			- Literal('(')
-			- g.unsignedNumericLiteral 
+			- g.unsignedNumericLiteral
 			- Literal(')'))
 		g.possiblyAliasedTable = (
-			g.tableName 
+			g.tableName
 			+ Optional(g.correlationSpecification)
 			+ Optional(g.tablesample)("tablesample"))
 		g.setGeneratingFunction = (
@@ -737,31 +794,31 @@ def getADQLGrammarCopy(nodes):
 		g.subJoin = '(' + g.joinedTable + ')'
 		g.joinOperand = (
 			g.setGeneratingFunction
-			| g.possiblyAliasedTable 
+			| g.possiblyAliasedTable
 			| g.derivedTable
 			| g.subJoin)
 		g.tableReference = (
-			g.joinedTable 
-			| g.possiblyAliasedTable 
+			g.joinedTable
+			| g.possiblyAliasedTable
 			| g.derivedTable)
 		g.tableReference.setName("table reference")
 
 # JOINs
 		g.columnNameList = (
-			g.columnName 
+			g.columnName
 			+ ZeroOrMore( "," + g.columnName)
 			)
 		g.columnNameList.setName("column name list")
 		g.namedColumnsJoin = (
-			CaselessKeyword("USING") 
+			CaselessKeyword("USING")
 			+ '(' + g.columnNameList("columnNames") + ')')
 		g.joinCondition = CaselessKeyword("ON") - g.searchCondition
 		g.joinSpecification = g.joinCondition | g.namedColumnsJoin
-		g.outerJoinType = (CaselessKeyword("LEFT") 
-			| CaselessKeyword("RIGHT") 
+		g.outerJoinType = (CaselessKeyword("LEFT")
+			| CaselessKeyword("RIGHT")
 			| CaselessKeyword("FULL"))
-		g.joinType = (CaselessKeyword("INNER") 
-			| (g.outerJoinType + CaselessKeyword("OUTER"))
+		g.joinType = (CaselessKeyword("INNER")
+			| (g.outerJoinType + Optional(CaselessKeyword("OUTER")))
 			| CaselessKeyword("CROSS"))  # local extension
 		g.joinOperator = (Optional(CaselessKeyword("NATURAL"))
 				+ Optional(g.joinType)
@@ -772,38 +829,44 @@ def getADQLGrammarCopy(nodes):
 				+ Optional(g.joinSpecification)))
 
 # Detritus in table expressions
-		g.groupByClause = (CaselessKeyword("GROUP") + CaselessKeyword("BY") 
-			+ g.columnReference 
-			+ ZeroOrMore(',' + g.columnReference))("groupby")
-		g.havingClause = (CaselessKeyword("HAVING") 
+		g.groupByClause = (CaselessKeyword("GROUP") + CaselessKeyword("BY")
+			+ g.valueExpression
+			+ ZeroOrMore(',' + g.valueExpression))("groupby")
+		g.havingClause = (CaselessKeyword("HAVING")
 			+ g.searchCondition)("having")
 		g.orderingSpecification = (Regex("(?i)ASC|DESC"))
-		g.sortKey = g.columnName | g.unsignedInteger
+		g.sortKey = g.valueExpression
 		g.sortSpecification = g.sortKey + Optional(g.orderingSpecification)
-		g.orderByClause = (CaselessKeyword("ORDER") 
-			+ CaselessKeyword("BY") + g.sortSpecification 
+		g.orderByClause = (CaselessKeyword("ORDER")
+			+ CaselessKeyword("BY") + g.sortSpecification
 			+ ZeroOrMore( ',' + g.sortSpecification ))("orderBy")
 
 # FROM fragments and such
-		g.fromClause = (Suppress(CaselessKeyword("FROM"))
-			+ g.tableReference 
+		g.fromClause = (Suppress(CaselessKeyword("FROM").setName("FROM"))
+			+ g.tableReference
 			+ ZeroOrMore(Suppress(',') - g.tableReference))("fromClause")
-		g.tableExpression = (g.fromClause 
-			+ Optional(g.whereClause) 
-			+ Optional(g.groupByClause)  
-			+ Optional(g.havingClause) 
+		g.tableExpression = (g.fromClause
+			+ Optional(g.whereClause)
+			+ Optional(g.groupByClause)
+			+ Optional(g.havingClause)
 			+ Optional(g.orderByClause))
 
 # Common Table Expressions
-		g.selectQuery = Forward()
+		g.selectExpression = Forward()
+		g.querySetExpression = Forward()
+		g.querySetTerm = Forward()
+
 		g.withQuery = (g.regularIdentifier("queryName")
 			+ CaselessKeyword("AS")
-			+ '(' + g.selectQuery + ')')
+			+ '(' + g.selectExpression + ')')
 		g.withSpecification = (CaselessLiteral("WITH")
 			+ g.withQuery + ZeroOrMore(',' + g.withQuery) )
 
 # toplevel select clause and set operators
-		g.queryExpression << (g.selectQuery | g.joinedTable)
+		g.selectQuery = (CaselessKeyword("SELECT").setName("SELECT")
+			+ Optional(g.setQuantifier)
+			+ Optional(g.setLimit)
+			+ g.selectList + g.tableExpression)
 
 		g.optionalAll = Optional(CaselessKeyword("ALL"))
 		g.intersectOperator = (
@@ -814,30 +877,24 @@ def getADQLGrammarCopy(nodes):
 				| CaselessKeyword("EXCEPT") )
 			+ g.optionalAll)
 
-		g.selectNoParens = (CaselessKeyword("SELECT") 
-			+ Optional(g.setQuantifier)
-			+ Optional(g.setLimit) 
-			+ g.selectList + g.tableExpression)
-		g.setTerm = (
-			'(' + g.selectQuery + ')'
-			| g.selectNoParens 
-				+ ZeroOrMore( 
-					g.intersectOperator
-					+ g.selectNoParens) )
-		g.firstSetTerm = (
-			g.selectNoParens 
-				+ ZeroOrMore( 
-					g.intersectOperator
-					+ g.selectNoParens) )
+		# I had to rewrite a few of the following ones to avoid pyparse
+		# endless recursions.
+		g.querySetPrimary = MatchFirstWithSaneError([
+			g.selectQuery,
+			'(' + g.selectExpression + ')'])
+		g.querySetTerm << (
+			g.querySetPrimary
+				+ ZeroOrMore(g.intersectOperator + g.querySetExpression))
+		g.querySetExpression << (
+			g.querySetTerm
+				+ ZeroOrMore(g.additiveSetOperator + g.querySetTerm))
 
-		g.selectQuery << (
-			g.firstSetTerm
-			+ ZeroOrMore(
-				g.additiveSetOperator
-				+ g.setTerm )
+		g.selectExpression << (
+			g.querySetExpression
 			+  Optional(g.offsetSpec))
+		g.queryExpression << (g.selectExpression | g.joinedTable)
 
-		g.querySpecification = Optional(g.withSpecification) + g.selectQuery
+		g.querySpecification = Optional(g.withSpecification) + g.selectExpression
 
 		g.statement = g.querySpecification + Optional(White()) + StringEnd()
 
@@ -891,7 +948,7 @@ if __name__=="__main__":  # pragma: no cover
 	enableTree(syms)
 	res = syms["statement"].parseString(
 		"""
-		SELECT MOC(3, point('ICRS', 3, 4)) from spatial
+select * from (select * from z) as q
 		"""
 		, parseAll=True)
 	pprint.pprint(res.asList(), stream=sys.stderr)
