@@ -2,7 +2,7 @@
 Node classes and factories used in ADQL tree processing.
 """
 
-#c Copyright 2008-2021, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -35,8 +35,8 @@ class ReplaceNode(utils.ExecutiveAction):
 	stead.
 
 	This is intended as a special service for ufuncs that want to insert
-	complex, annotatable expressions.  I doubt this is something
-	we should do under other circumstances.
+	complex, annotatable expressions.  We also use that in certain situations
+	when casting.
 	"""
 	def __init__(self, replacingNode):
 		self.replacingNode = replacingNode
@@ -74,11 +74,15 @@ def flatten(arg):
 	"""
 	if isinstance(arg, str):
 		return arg
+
 	elif isinstance(arg, (int, float)):
 		return repr(arg)
+
 	elif isinstance(arg, parsetricks.ParseResults):
 		return " ".join(flatten(c) for c in arg)
+
 #	elif arg is None: import pdb;pdb.Pdb(nosigint=True).set_trace()
+
 	else:
 		return arg.flatten()
 
@@ -159,7 +163,7 @@ class BOMB_OUT(object): pass
 def _uniquify(matches, default, exArgs):
 # helper method for getChildOfX -- see there
 	if len(matches)==0:
-		if default is not BOMB_OUT: 
+		if default is not BOMB_OUT:
 			return default
 		raise common.NoChild(*exArgs)
 	if len(matches)!=1:
@@ -188,7 +192,7 @@ def getChildOfClass(nodeSeq, cls, default=BOMB_OUT):
 
 
 def parseArgs(parseResult):
-	"""returns a sequence of ADQL nodes suitable as function arguments from 
+	"""returns a sequence of ADQL nodes suitable as function arguments from
 	parseResult.
 
 	This is for cleaning up _parseResults["args"], i.e. stuff from the
@@ -204,6 +208,20 @@ def parseArgs(parseResult):
 	return tuple(args)
 
 
+def getStringLiteral(node, description="Argument"):
+	"""ensures that node only contains a constant string literal
+	and returns its value if so.
+
+	The function raises an adql.Error mentioning description otherwise.
+	"""
+	if node.type!='characterStringLiteral':
+		raise common.Error(
+			"%s must be a constant string literal"%description)
+	
+	return node.value
+
+
+
 ######################### Misc helpers related to simple query planning
 
 def _getDescendants(args):
@@ -246,6 +264,8 @@ def _estimateTableSize(args):
 	our estimate.  If it doesn't come, we return None.
 	"""
 	for fi in iterFieldInfos(args):
+		if fi.ignoreTableStats:
+			return
 		for ud in fi.userData:
 			sizeEst = getattr(ud.parent, "nrows", None)
 			if sizeEst is not None:
@@ -272,7 +292,7 @@ def _sortLargeFirst(arg1, arg2):
 		return arg1, arg2
 
 	size1, size2 = _estimateTableSize([arg1]), _estimateTableSize([arg2])
-	
+
 	if size1 is None:
 		if size2 is None:
 			# we know nothing; don't change anything to keep the user in control
@@ -307,11 +327,12 @@ class ADQLNode(utils.AutoNode):
 	is just a concatenation for their flattened children.  This is convenient
 	as long as they are not morphed.
 	
-	To derive actual classes, define 
+	To derive actual classes, define
 	
 		- the _a_<name> class attributes you need,
-		- the type (a nonterminal from the ADQL grammar) 
-		- plus bindings if the class handles more than one symbol,
+		- the type (a nonterminal from the ADQL grammar)
+		- bindings if the class handles more than one symbol (in which case
+		  type is ignored)
 		- a class method _getInitKWs(cls, parseResult); see below.
 		- a method flatten() -> string if you define a parsed ADQLNode.
 		- a method _polish() that is called just before the constructor is
@@ -322,7 +343,7 @@ class ADQLNode(utils.AutoNode):
 	names to values.  You do not need to manually call superclass _getInitKWs,
 	since the fromParseResult classmethod figures out all _getInitKWs in the
 	inheritance tree itself.  It calls all of them in the normal MRO and updates
-	the argument dictionary in reverse order.  
+	the argument dictionary in reverse order.
 	
 	The fromParseResult class method additionally filters out all names starting
 	with an underscore; this is to allow easy returning of locals().
@@ -464,7 +485,7 @@ class ColumnBearingNode(ADQLNode):
 	These are tables, subqueries, etc.  This class is the basis for the
 	annotation of tables and subqueries.
 
-	Their getFieldInfo(name)->fi method gives annotation.FieldInfos 
+	Their getFieldInfo(name)->fi method gives annotation.FieldInfos
 	objects for their columns, None for unknown columns.
 
 	These keep their fieldInfos on a change()
@@ -523,8 +544,8 @@ class TableName(ADQLNode):
 		if self.schema and self.schema.lower()=="tap_upload":
 			self.schema = None
 
-		self.qName = ".".join(flatten(n) 
-			for n in (self.cat, self.schema, self.name) if n) 
+		self.qName = ".".join(flatten(n)
+			for n in (self.cat, self.schema, self.name) if n)
 
 	@classmethod
 	def _getInitKWs(cls, _parseResult):
@@ -551,7 +572,7 @@ class TableName(ADQLNode):
 		"""returns self's qualified name lowercased for regular identifiers,
 		in original capitalisation otherwise.
 		"""
-		return ".".join(self._normalizePart(p) 
+		return ".".join(self._normalizePart(p)
 			for p in [self.cat, self.schema, self.name]
 			if p is not None)
 
@@ -589,9 +610,9 @@ class PlainTableRef(ColumnBearingNode):
 		self.qName = flatten(self.tableName)
 
 	def flatten(self):
-		ot = flatten(self.originalTable)
-		if ot!=self.qName:
-			literal = "%s AS %s"%(ot, flatten(self.tableName))
+		origTable = flatten(self.originalTable)
+		if origTable!=self.qName:
+			literal = "%s AS %s"%(origTable, flatten(self.tableName))
 		else:
 			literal = self.qName
 
@@ -634,7 +655,7 @@ class DerivedTable(ColumnBearingNode):
 	@classmethod
 	def _getInitKWs(cls, _parseResult):
 		tmp = {'tableName': TableName(name=str(_parseResult.get("alias"))),
-			'query': getChildOfClass(_parseResult, SelectQuery),
+			'query': getChildOfClass(_parseResult, SelectExpression),
 		}
 		return tmp
 
@@ -690,7 +711,7 @@ class SetGeneratingFunction(ColumnBearingNode, TransparentMixin):
 
 	def addFieldInfos(self, context):
 		# TODO: Infer types from argument types
-		fieldinfos.FieldInfos(self, context)
+		self.fieldInfos = fieldinfos.FieldInfos(self, context)
 		self.fieldInfos.addColumn(self.name,
 			fieldinfo.FieldInfo("integer", None, None, sqlName=self.name))
 	
@@ -806,7 +827,7 @@ class JoinedTable(ColumnBearingNode):
 		return "NATURAL"
 
 	def getAllTables(self):
-		"""returns all actual tables and subqueries (not sub-joins) 
+		"""returns all actual tables and subqueries (not sub-joins)
 		within this join.
 		"""
 		res = []
@@ -848,7 +869,7 @@ def makeBinaryJoinTree(children):
 	"""takes the parse result for a join and generates a binary tree of
 	JoinedTable nodes from it.
 
-	It's much easier to do this in a separate step than to force a 
+	It's much easier to do this in a separate step than to force a
 	non-left-recursive grammar to spit out the right parse tree in the
 	first place.
 	"""
@@ -898,8 +919,8 @@ class OffsetSpec(ADQLNode):
 		return ""
 
 
-class SelectNoParens(ColumnBearingNode): 
-	type = "selectNoParens"
+class SelectQuery(ColumnBearingNode):
+	type = "selectQuery"
 
 	_a_setQuantifier = None
 	_a_setLimit = None
@@ -991,7 +1012,7 @@ class SetOperationNode(ColumnBearingNode, TransparentMixin):
 	This is UNION, INTERSECT, or EXCEPT.  In all cases, we need to check
 	all contributing sub-expressions have compatible degree.  For now,
 	in violation of SQL1992, we require identical names on all operands --
-	sql92 in 7.10 says 
+	sql92 in 7.10 says
 
 	  [if column names are unequal], the <column name> of the i-th column of TR
 	  is implementation-dependent and different from the <column name> of any
@@ -1005,7 +1026,7 @@ class SetOperationNode(ColumnBearingNode, TransparentMixin):
 	def _assertFieldInfosCompatible(self):
 		"""errors out if operands have incompatible signatures.
 
-		For convenience, if all are compatible, the common signature (ie, 
+		For convenience, if all are compatible, the common signature (ie,
 		fieldInfos) is returned.
 		"""
 		fieldInfos = None
@@ -1013,9 +1034,15 @@ class SetOperationNode(ColumnBearingNode, TransparentMixin):
 			# Skip WithQueries -- they're not part of set operations.
 			if hasattr(child, "fieldInfos") and not isinstance(child, WithQuery):
 				if fieldInfos is None:
-					fieldInfos = child.fieldInfos
+					fieldInfos = child.fieldInfos.copy(self)
 				else:
 					fieldInfos.assertIsCompatible(child.fieldInfos)
+					for (l1, mycol), (l2, theircol) in zip(fieldInfos, child.fieldInfos):
+						if mycol.ucd != theircol.ucd:
+							mycol.ucd = ""
+						if mycol.unit != theircol.unit:
+							mycol.unit = ""
+
 		return fieldInfos
 
 	def addFieldInfos(self, context):
@@ -1040,7 +1067,12 @@ class SetOperationNode(ColumnBearingNode, TransparentMixin):
 
 
 class SetTerm(SetOperationNode):
-	type = "setTerm"
+	type = "querySetTerm"
+	collapsible = True
+
+
+class SetExpression(SetOperationNode):
+	type = "querySetExpression"
 	collapsible = True
 
 
@@ -1051,10 +1083,20 @@ class WithQuery(SetOperationNode):
 	"""
 	type = "withQuery"
 
+	def addFieldInfos(self, context):
+		# overridden because we need to discard statistics here.  We
+		# have code that looks at table sizes (look for "optimize"),
+		# and it looks at columns' parents to do that.  These stats
+		# will be totally off in CTEs.  To avoid dis-optimisations,
+		# we note it's all off here.
+		SetOperationNode.addFieldInfos(self, context)
+		for _, fi in self.fieldInfos:
+			fi.ignoreTableStats = True
+		
 	def _polish(self):
 		self.name = self.children[0]
 		for c in self.children:
-			# this should be a selectQuery, but this we want to be sure
+			# this should be a selectExpression, but this we want to be sure
 			# we don't fail when morphers replace the main query node
 			# (as the pg morpher does)
 			if hasattr(c, "setLimit"):
@@ -1064,7 +1106,7 @@ class WithQuery(SetOperationNode):
 			raise NotImplementedError("WithQuery without select?")
 
 
-class SelectQuery(SetOperationNode):
+class SelectExpression(SetOperationNode):
 	"""A complete query excluding CTEs.
 
 	The main ugly thing here is the set limit; the querySpecification has
@@ -1077,7 +1119,7 @@ class SelectQuery(SetOperationNode):
 	annotation we'll have errored out if set operator arguments aren't
 	reasonably congurent.
 	"""
-	type = "selectQuery"
+	type = "selectExpression"
 
 	_a_setLimit = None
 	_a_offset= None
@@ -1110,7 +1152,7 @@ class SelectQuery(SetOperationNode):
 class QuerySpecification(TransparentNode):
 	"""The toplevel query objects including CTEs.
 
-	Apart from any CTEs, that's just a SelectQuery (which is always the last
+	Apart from any CTEs, that's just a SelectExpression (which is always the last
 	child), and we hand through essentially all attribute access to it.
 	"""
 	type = "querySpecification"
@@ -1132,11 +1174,15 @@ class QuerySpecification(TransparentNode):
 		return getattr(self.children[-1], attrName)
 
 
-class ColumnReference(FieldInfoedNode):
-# normal column references will be handled by the dispatchColumnReference
-# function below, hence the binding is missing here.
+class _BaseColumnReference(FieldInfoedNode):
+	# normal column references will be handled by the dispatchColumnReference
+	# function below; there we look at the columnReference type to figure
+	# out whether something is constant, though, and so multiple
+	# non-terminals share this.
+	# That's why we're unbinding everything here and leave it to
+	# deriving classes to set up bindings.
 	type = "columnReference"
-	bindings = ["geometryValue"]
+	bindings = []
 	_a_refName = None  # if given, a TableName instance
 	_a_name = None
 
@@ -1150,8 +1196,8 @@ class ColumnReference(FieldInfoedNode):
 	def _getInitKWs(cls, _parseResult):
 		names = [_c for _c in _parseResult if _c!="."]
 		names = [None]*(4-len(names))+names
-		refName = TableName(cat=names[0], 
-			schema=names[1], 
+		refName = TableName(cat=names[0],
+			schema=names[1],
 			name=names[2])
 		if not refName:
 			refName = None
@@ -1164,7 +1210,7 @@ class ColumnReference(FieldInfoedNode):
 
 		srcColumn = None
 		if self.fieldInfo.userData:
-			srcColumn = self.fieldInfo.userData[0] 
+			srcColumn = self.fieldInfo.userData[0]
 		if hasattr(srcColumn, "originalName"):
 			# This is a column from a VOTable upload we have renamed to avoid
 			# clashes with postgres-reserved column names.  Update the name
@@ -1183,10 +1229,18 @@ class ColumnReference(FieldInfoedNode):
 		return (self.type, self.name)
 
 
-class ColumnReferenceByUCD(ColumnReference):
+class ColumnReference(_BaseColumnReference):
+	# No bindings: constructed by dispatchColumnReference
+	pass
+
+class GeometryValue(_BaseColumnReference):
+	bindings = ["geometryValue"]
+
+
+class ColumnReferenceByUCD(_BaseColumnReference):
 # these are tricky: As, when parsing, we don't know where the columns
-# might come from, we have to 
-	type = "columnReferenceByUCD"
+# might come from, we have to later figure out where to get our metadata
+# from.
 	bindings = ["columnReferenceByUCD"]
 	_a_ucdWanted = None
 
@@ -1209,7 +1263,7 @@ class ColumnReferenceByUCD(ColumnReference):
 
 		for field in ancestor.fromClause.getAllFields():
 			if fnmatch.fnmatch(field.fieldInfo.ucd, self.ucdWanted):
-				self.fieldInfo = field.fieldInfo
+				self.fieldInfo = field.fieldInfo.change()
 				self.name = self.colName = field.name
 				self.refName = None
 				break
@@ -1217,6 +1271,8 @@ class ColumnReferenceByUCD(ColumnReference):
 			raise utils.NotFoundError(self.ucdWanted, "column matching ucd",
 				"from clause")
 
+		self.fieldInfo.properties["src-expression"] = flatten(self)
+		
 
 @symbolAction("columnReference")
 def dispatchColumnReference(parseResult):
@@ -1245,7 +1301,7 @@ class FromClause(ADQLNode):
 			# just build an artificial join as the table reference
 			tableReference = reduce(lambda left, right:
 				JoinedTable(
-					leftOperand=left, 
+					leftOperand=left,
 					operator=JoinOperator(children=[","]),
 					rightOperand=right), parseResult)
 		return {
@@ -1264,7 +1320,7 @@ class FromClause(ADQLNode):
 		return self.tableReference.getFieldInfo(name)
 
 	def _makeColumnReference(self, sourceTableName, colPair):
-		"""returns a ColumnReference object for a name, colInfo pair from a 
+		"""returns a ColumnReference object for a name, colInfo pair from a
 		table's fieldInfos.
 		"""
 		cr = ColumnReference(name=colPair[0], refName=sourceTableName)
@@ -1354,6 +1410,15 @@ class DerivedColumn(FieldInfoedNode):
 	def _treeRepr(self):
 		return (self.type, self.name)
 
+	def addFieldInfo(self, context):
+		FieldInfoedNode.addFieldInfo(self, context)
+		if self.fieldInfo and self.alias:
+			self.fieldInfo = self.fieldInfo.change()
+			if hasattr(self.alias, "name"): # a QName, presumably
+				self.fieldInfo.properties["src-expression"] = self.alias.name
+			else:
+				self.fieldInfo.properties["src-expression"] = self.alias
+
 
 class QualifiedStar(ADQLNode):
 	type = "qualifiedStar"
@@ -1382,7 +1447,7 @@ class SelectList(ADQLNode):
 			selectFields = None  #noflake: locals returned
 		else:
 			selectFields = list(itertools.chain(#noflake: locals returned
-				*_parseResult.get("fieldSel"))) 
+				*_parseResult.get("fieldSel")))
 		return locals()
 	
 	def flatten(self):
@@ -1417,7 +1482,7 @@ def _guessNumericType(literal):
 	literal.
 
 	I don't want to pull through the literal symbol that matched
-	from grammar in all cases.  Thus, at times I simply guess the type 
+	from grammar in all cases.  Thus, at times I simply guess the type
 	(and yes, I'm aware that -32768 still is a smallint).
 	"""
 	try:
@@ -1429,12 +1494,16 @@ def _guessNumericType(literal):
 		else:
 			type = "bigint"
 	except ValueError:
-		type = "double precision"
+		if literal.lower()=="null":
+			type = None
+		else:
+			# I'm too lazy to spell out all sorts of float.
+			type = "double precision"
 	return type
 
 
 class Factor(FieldInfoedNode, TransparentMixin):
-	"""is a factor within an SQL expression.
+	"""a factor within an SQL expression.
 
 	factors may have only one (direct) child with a field info and copy
 	this.  They can have no child with a field info, in which case they're
@@ -1452,6 +1521,8 @@ class Factor(FieldInfoedNode, TransparentMixin):
 			self.fieldInfo = fieldinfo.FieldInfo(
 				_guessNumericType("".join(self.children)), "", "")
 
+		self.fieldInfo.properties["src-expression"] = flatten(self)
+		
 
 class ArrayReference(FieldInfoedNode, TransparentMixin):
 	type = "arrayReference"
@@ -1478,13 +1549,15 @@ class ArrayReference(FieldInfoedNode, TransparentMixin):
 			tainted=True # array might actually have semantics
 			)
 
+		self.fieldInfo.properties["src-expression"] = flatten(self)
+		
 
 class CombiningFINode(FieldInfoedNode):
 	def addFieldInfo(self, context):
 		infoChildren = self._getInfoChildren()
 		if not infoChildren:
-			if len(self.children)==1: 
-				# probably a naked numeric literal in the grammar, e.g., 
+			if len(self.children)==1:
+				# probably a naked numeric literal in the grammar, e.g.,
 				# in mathFunction
 				self.fieldInfo = fieldinfo.FieldInfo(
 					_guessNumericType(self.children[0]), "", "")
@@ -1496,6 +1569,9 @@ class CombiningFINode(FieldInfoedNode):
 		else:
 			self.fieldInfo = self._combineFieldInfos()
 
+		if self.fieldInfo:
+			self.fieldInfo.properties["src-expression"] = flatten(self)
+		
 
 class Term(CombiningFINode, TransparentMixin):
 	type = "term"
@@ -1508,7 +1584,7 @@ class Term(CombiningFINode, TransparentMixin):
 		fi1 = opd1.fieldInfo
 		while toDo:
 			opr = toDo.pop(0)
-			fi1 = fieldinfo.FieldInfo.fromMulExpression(opr, fi1, 
+			fi1 = fieldinfo.FieldInfo.fromMulExpression(opr, fi1,
 				toDo.pop(0).fieldInfo)
 		return fi1
 
@@ -1551,6 +1627,8 @@ class StringValueExpression(FieldInfoedNode, TransparentMixin):
 			self.fieldInfo = fieldinfo.FieldInfo(
 				"text", "", "")
 
+		self.fieldInfo.properties["src-expression"] = flatten(self)
+
 
 class GenericValueExpression(CombiningFINode, TransparentMixin):
 	"""A container for value expressions that we don't want to look at
@@ -1598,11 +1676,12 @@ class SetFunction(TransparentMixin, FieldInfoedNode):
 	type = "setFunctionSpecification"
 
 	funcDefs = {
-		'AVG': ('%s;stat.mean', None, "double precision"),
-		'MAX': ('%s;stat.max', None, None),
-		'MIN': ('%s;stat.min', None, None),
+		'AVG': ('{u};stat.mean', None, "double precision"),
+		'STDDEV': ('stat.stdev;{u}', None, "double precision"),
+		'MAX': ('stat.max;{u}', None, None),
+		'MIN': ('stat.min;{u}', None, None),
 		'SUM': (None, None, None),
-		'COUNT': ('meta.number;%s', '', "integer"),}
+		'COUNT': ('meta.number;{u}', '', "integer"),}
 
 	def addFieldInfo(self, context):
 		funcName = self.children[0].upper()
@@ -1620,7 +1699,7 @@ class SetFunction(TransparentMixin, FieldInfoedNode):
 			# ucd of a sum is the ucd of the summands?
 			ucd = fi.ucd
 		elif fi.ucd:
-			ucd = ucdPref%(fi.ucd)
+			ucd = ucdPref.format(u=fi.ucd)
 		else:
 			# no UCD given; if we're count, we're meta.number, otherwise we
 			# don't know
@@ -1640,6 +1719,8 @@ class SetFunction(TransparentMixin, FieldInfoedNode):
 		self.fieldInfo = fieldinfo.FieldInfo(
 			newType, unit=newUnit, ucd=ucd, userData=fi.userData, tainted=fi.tainted)
 
+		self.fieldInfo.properties["src-expression"] = flatten(self)
+		
 
 class NumericValueFunction(FunctionNode):
 	"""A numeric function.
@@ -1693,6 +1774,84 @@ class NumericValueFunction(FunctionNode):
 			unit, ucd, *collectUserData(infoChildren))
 		self.fieldInfo.tainted = True
 
+		self.fieldInfo.properties["src-expression"] = flatten(self)
+		
+
+class ScalarArrayFunction(FunctionNode):
+	"""All of these are functions somehow aggregating array elements.
+	
+	For those that we know we try to infer units and UCDs.
+	"""
+	type = "scalarArrayFunction"
+
+	funcDefs = {
+		# the second column was intended for unit handling but isn't used
+		# for now.  Encode unit changes there if they become necessary.
+		'ARR_MIN': ('stat.min;{u}', None, 1),
+		'ARR_MAX': ('stat.max;{u}', None, 1),
+		'ARR_AVG': ('{u};stat.mean', None, 1),
+		'ARR_STDDEV': ('stat.stdev;{u}', None, 1),
+		'ARR_SUM': ('{u};arith.sum', None, 1),
+		'ARR_DOT': (None, None, 2),
+	}
+
+	def addFieldInfo(self, context):
+		ucdchange, unitchange, nPar = self.funcDefs.get(
+			self.funName, (None, None, 1))
+
+		infoChildren = self._getInfoChildren()
+		if len(infoChildren)!=nPar:
+			raise common.Error(
+				"Grammar vs. annotation function arity mismatch!")
+
+		if nPar==1:
+			baseFI = infoChildren[0].fieldInfo
+			mat = fieldinfo.isArray(baseFI.type)
+			if not mat:
+				raise common.Error(
+					"Scalar array function called on non-array?")
+			type = mat.group(1)
+
+		elif nPar==2:
+			type = "double precision"
+			baseFI = fieldinfo.FieldInfo(type, "", "",
+				userData=infoChildren)
+		else:
+			assert False
+
+
+		ucd, unit = "", baseFI.unit
+		if ucdchange is not None and baseFI.ucd:
+			ucd = ucdchange.format(u=baseFI.ucd)
+
+		self.fieldInfo = fieldinfo.FieldInfo(type, unit, ucd,
+			*collectUserData(infoChildren))
+
+
+class ArrayMapFunction(FunctionNode):
+	"""The arr_map extension.
+
+	arg 1 is the expression; any x in it will be the filled with the array
+	elements.  arg 2 is the array to take the elements from.
+	"""
+	type = "arrayMapFunction"
+
+	def addFieldInfo(self, context):
+		self.fieldInfo = fieldinfo.FieldInfo("double precision[]", "", "")
+
+	def _polish(self):
+		# we need to pre-annotate all column references to "x" (our unbound
+		# variable) in our first argument with a literal "x" to keep
+		# our annotation engine from looking at them
+		def preAnnotate(node):
+			for child in node.iterNodeChildren():
+				if isinstance(child, ColumnReference) and child.name=="x":
+					child.fieldInfo = fieldinfo.FieldInfo(
+						"double precision", "", "")
+				preAnnotate(child)
+
+		preAnnotate(self.args[0])
+
 
 class StringValueFunction(FunctionNode):
 	type = "stringValueFunction"
@@ -1701,6 +1860,8 @@ class StringValueFunction(FunctionNode):
 		self.fieldInfo = fieldinfo.FieldInfo("text", "", "",
 			userData=collectUserData(self._getInfoChildren())[0])
 
+		self.fieldInfo.properties["src-expression"] = flatten(self)
+
 
 class TimestampFunction(FunctionNode):
 	type = "timestampFunction"
@@ -1799,6 +1960,25 @@ class CastSpecification(FieldInfoedNode, TransparentMixin):
 			newType = "unicode"
 		return locals()
 
+	geometryConstructors = {
+		"POINT": "Point",
+		"CIRCLE": "Circle",
+		"POLYGON": "Polygon"}
+
+	def _polish(self):
+		# the casts to geometries need to be turned into constructor calls.
+		# We could theoretically do this while morphing, but then annotation
+		# would get a lot more complicated.
+		newNodeName = self.geometryConstructors.get(
+			self.newType.upper(), None)
+		if newNodeName is None:
+			return
+
+		# we only have names in geometryConstructors because we're defined
+		# lexically above the geometries
+		nodeClass = globals()[newNodeName]
+		raise ReplaceNode(nodeClass.fromCastArgument(self.value))
+
 	def addFieldInfo(self, context):
 		# We copy units and UCDs from the subordinate value (if it's there;
 		# NULLs have nothing, of course). That has the somewhat unfortunate
@@ -1829,9 +2009,45 @@ class CoalesceExpression(FieldInfoedNode, TransparentMixin):
 		if subordinates:
 			self.fieldInfo = subordinates[0].fieldInfo.change(tainted=True)
 		else:
-			self.fieldInfo = None
+			# there's probably only nulls here; make it so we produce
+			# the most robust NULL VOTable has.
+			self.fieldInfo = fieldinfo.FieldInfo('real', None, None)
 
 
+class CaseExpression(FieldInfoedNode, TransparentMixin):
+	type = "searchedCase"
+	bindings = ["searchedCase", "simpleCase"]
+
+	def addFieldInfo(self, context):
+		results, nextIsResult = [], False
+		for c in self.children:
+			if c in {"THEN", "ELSE"}:
+				nextIsResult = 1
+			else:
+				if nextIsResult:
+					results.append(c)
+					nextIsResult = False
+
+		for res in results:
+			# we try to find a case that has proper user data (i.e., a column)
+			# attached; anything else will probably be empty info.
+			if res.fieldInfo and res.fieldInfo.userData:
+				self.fieldInfo = res.fieldInfo.change(tainted=True)
+				break
+
+		else:
+			# We probably only have literals.  Get the type from the first
+			# and run.  Should we make sure that all cases have matching types?
+			for res in results:
+				if res.fieldInfo and res.fieldInfo.type:
+					self.fieldInfo = res.fieldInfo.change(tainted=True)
+					break
+
+			else:
+				# we cannot venture any guess what we'll return.  Let's avoid
+				# wild guesses for now and just fail.
+				raise common.Error("CASE statement seems to have no results.")
+
 
 ###################### Geometry and stuff that needs morphing into real SQL
 
@@ -1854,7 +2070,7 @@ class GeometryNode(CoosysMixin, FieldInfoedNode):
 
 	In ADQL 2.1, most of these became polymorphous.  For instance, circles
 	can be constructed with a point as the first (or second, if a coosys
-	is present) argument; that point can also be a column reference.  
+	is present) argument; that point can also be a column reference.
 
 	Also, these will always get morphed in some way (as the database
 	certainly doesn't understand ADQL geometries).  So, we're
@@ -1865,8 +2081,14 @@ class GeometryNode(CoosysMixin, FieldInfoedNode):
 	information.  Some of the actual attributes will be None.
 
 	Flatten is only there for debugging; it'll return invalid SQL.
-	OrigArgs is not for client consumption; clients must go through the 
+	OrigArgs is not for client consumption; clients must go through the
 	symbolic names.
+
+	If you want your geometry to support casts, give it a
+	fromCastArgs(args) method that will, in general, probably replace
+	the node with a GeometryCast node.  You'll then have to teach GeometryCast
+	how to annotate your geometry and add the type name to the
+	geometryConstructors dictionary in CastSpecification.
 	"""
 	_a_origArgs = None
 
@@ -1879,8 +2101,8 @@ class GeometryNode(CoosysMixin, FieldInfoedNode):
 		return {"origArgs": list(_parseResult[1:])}
 
 	def addFieldInfo(self, context):
-		fis = [attr.fieldInfo 
-			for attr in 
+		fis = [attr.fieldInfo
+			for attr in
 				(getattr(self, arg) for arg in self.stcArgs if getattr(self, arg))
 			if attr and attr.fieldInfo]
 		childUserData, childUnits = [], []
@@ -1892,22 +2114,51 @@ class GeometryNode(CoosysMixin, FieldInfoedNode):
 			if fis and fis[0].stc:
 				thisSystem = fis[0].stc
 
+		ignoreTableStats = False
 		for index, fi in enumerate(fis):
 			childUserData.extend(fi.userData)
 			childUnits.append(fi.unit)
+			ignoreTableStats = ignoreTableStats or fi.ignoreTableStats
 			if not context.policy.match(fi.stc, thisSystem):
 				context.errors.append("When constructing %s: Argument %d has"
 					" incompatible STC"%(self.type, index+1))
 
 		self.fieldInfo = fieldinfo.FieldInfo(
 			type=self.sqlType,
-			unit=",".join(childUnits), 
-			ucd="", 
-			userData=tuple(childUserData), 
-			stc=thisSystem)
+			ucd="",
+			unit="",
+			userData=tuple(childUserData),
+			stc=thisSystem,
+			ignoreTableStats=ignoreTableStats)
 		self.fieldInfo.properties["xtype"] = self.xtype
 
 
+class GeometryCast(GeometryNode):
+	"""A cast to a geometry type.
+
+	For these, we defer to functions built into the database.
+	"""
+	_a_argument = None
+	_a_cast_function = None
+
+	resultingTypes = {
+		"cast_to_point": "spoint",
+		"cast_to_circle": "scircle",
+		"cast_to_polygon": "spoly",}
+
+	def addFieldInfo(self, context):
+		self.fieldInfo = fieldinfo.FieldInfo(
+			type=self.resultingTypes[self.cast_function],
+			unit='',
+			ucd='',
+			stc=None)
+	
+	def flatten(self):
+		return "{}({})".format(
+			self.cast_function,
+			self.argument.flatten())
+
+
 class Point(GeometryNode):
 	type = "point"
 	_a_x = _a_y = None
@@ -1925,6 +2176,12 @@ class Point(GeometryNode):
 		x, y = parseArgs(_parseResult["args"]) #noflake: locals returned
 		return locals()
 
+	@classmethod
+	def fromCastArgument(self, arg):
+		raise ReplaceNode(GeometryCast(
+			argument=arg,
+			cast_function="cast_to_point"))
+
 
 class Circle(GeometryNode):
 	"""A circle parsed from ADQL.
@@ -1948,13 +2205,19 @@ class Circle(GeometryNode):
 		if len(args)==2:
 				res["center"], res["radius"] = args[0], args[1]
 		elif len(args)==3:
-			res["center"] = Point(cooSys=_parseResult.get("coordSys", ""), 
+			res["center"] = Point(cooSys=_parseResult.get("coordSys", ""),
 				x=args[0], y=args[1])
 			res["radius"] = args[2]
 		else: # pragma: no cover
 			assert False, "Grammar let through invalid args to Circle"
 		return res
 
+	@classmethod
+	def fromCastArgument(self, arg):
+		raise ReplaceNode(GeometryCast(
+			argument=arg,
+			cast_function="cast_to_circle"))
+
 
 class MOC(GeometryNode):
 	"""a MOC in an ADQL syntax tree.
@@ -2062,7 +2325,7 @@ class Polygon(GeometryNode):
 			for item in res["points"]:
 				if item.type!="point":
 					return res
-			# all points: mutate args to let us fall through to the split coo
+			# all points: mutate args to let us fall through to the split coup
 			# case
 			arg.type = "polygonSplitCooArgs"
 			newArgs = []
@@ -2074,7 +2337,7 @@ class Polygon(GeometryNode):
 			# turn numeric expressions into pairs
 			coos, toDo = [], list(arg.args)
 			while toDo:
-				coos.append(tuple(toDo[:2])) 
+				coos.append(tuple(toDo[:2]))
 				del toDo[:2]
 			res = {"coos": coos}
 
@@ -2083,6 +2346,12 @@ class Polygon(GeometryNode):
 
 		return res
 
+	@classmethod
+	def fromCastArgument(self, arg):
+		raise ReplaceNode(GeometryCast(
+			argument=arg,
+			cast_function="cast_to_polygon"))
+
 	def addFieldInfo(self, name):
 		if self.points is not None:
 			systemSource = self.points
@@ -2109,7 +2378,7 @@ class Polygon(GeometryNode):
 			stc=thisSystem)
 
 
-_regionMakers = [] 
+_regionMakers = []
 def registerRegionMaker(fun):
 	"""adds a region maker to the region resolution chain.
 
@@ -2161,7 +2430,7 @@ class STCSRegion(FieldInfoedNode):
 
 	def addFieldInfo(self, context):
 		# XXX TODO: take type and unit from tapstcObj
-		self.fieldInfo = fieldinfo.FieldInfo("spoly", unit="deg", ucd=None, 
+		self.fieldInfo = fieldinfo.FieldInfo("spoly", unit="deg", ucd=None,
 			stc=tapstc.getSTCForTAP(self.cooSys))
 	
 	def flatten(self): # pragma: no cover
@@ -2192,7 +2461,7 @@ class Distance(FunctionNode):
 
 	def addFieldInfo(self, context):
 		self.fieldInfo = fieldinfo.FieldInfo(type="double precision",
-			unit="deg", ucd="pos.angDistance", 
+			unit="deg", ucd="pos.angDistance",
 			userData=collectUserData(self._getInfoChildren())[0])
 
 	def optimize(self, stack):
@@ -2236,7 +2505,7 @@ class PredicateGeometryFunction(FunctionNode):
 		# optimise the common case of contains(point, circle); both
 		# q3c and pgsphere won't use an index (properly) if the sequence
 		# of the arguments is "wrong".
-		if (self.args[leftInd].type=="point" 
+		if (self.args[leftInd].type=="point"
 				and self.args[rightInd].type=="circle"):
 
 			if _isConstant([self.args[leftInd]]):
@@ -2278,7 +2547,7 @@ class PointFunction(FunctionNode):
 			userData = (childFieldInfo.userData[int(self.funName[-1])-1],)
 		else:
 			userData = childFieldInfo.userData
-		return fieldinfo.FieldInfo("double precision", 
+		return fieldinfo.FieldInfo("double precision",
 			ucd=None, unit="deg", userData=userData)
 
 	def addFieldInfo(self, context):
@@ -2294,7 +2563,7 @@ class Area(FunctionNode):
 
 	def addFieldInfo(self, context):
 		self.fieldInfo = fieldinfo.FieldInfo(type="double precision",
-			unit="deg**2", ucd="phys.angSize", 
+			unit="deg**2", ucd="phys.angSize",
 			userData=collectUserData(self._getInfoChildren())[0])
 
 
@@ -2312,6 +2581,17 @@ def registerNode(node):
 	# adding a node probably changes the grammar bindings, so we
 	# need to clear the cached grammar
 	getTreeBuildingGrammar.cache_clear()
+
+	# if a module registering a node is re-loaded, we have to remove the
+	# node it previously registered, or we'll get a symbol clash.
+	# It's not totally straightforward to figure out what the previous
+	# version was; for now, let's look at type, but given the various
+	# ways nodes might use to bind, that's potentially not enough.
+	for existing in _ADDITIONAL_NODES:
+		if existing.type==node.type:
+			_ADDITIONAL_NODES.remove(existing)
+			break
+
 	_ADDITIONAL_NODES.append(node)
 
 
