@@ -5,7 +5,7 @@ system.
 See the userFunction docstring on how to use these.
 """
 
-#c Copyright 2008-2022, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -23,24 +23,11 @@ from gavo.adql import nodes
 UFUNC_REGISTRY = {}
 
 
-def getStringLiteral(node, description="Argument"):
-	"""ensures that node only contains a constant string literal
-	and returns its value if so.
-
-	The function raises an UfuncError mentioning description otherwise.
-	"""
-	if node.type!='characterStringLiteral':
-		raise common.UfuncError(
-			"%s must be a constant string literal"%description)
-	
-	return node.value
-
-
-def userFunction(name, 
-		signature, 
-		doc, 
-		returntype="double precision", 
-		unit="", 
+def userFunction(name,
+		signature,
+		doc,
+		returntype="double precision",
+		unit="",
 		ucd="",
 		polymorphism=(),
 		additionalNames=[],
@@ -62,12 +49,12 @@ def userFunction(name,
 	case, they'll be passed the (annotated) arguments, and whatever
 	they return will be the unit/ucd.
 
-	polymorphism, if given, must be a sequence of pairs of 
+	polymorphism, if given, must be a sequence of pairs of
 	(signature, documentation) for alternative signatures of the
 	UDF.
 
 	additionalNames, if given, is a list of names the function can
-	also be used as.  This is intended for when you started with 
+	also be used as.  This is intended for when you started with
 	gavo_somefunc and then want to move on to ivo_somefunc without
 	functional change and want to keep legacy queries working.
 
@@ -78,16 +65,16 @@ def userFunction(name,
 	for calling cleanForConn(conn).
 
 	The python function receives a list of arguments; this will in
-	general be ADQL expression trees.  It must return either 
+	general be ADQL expression trees.  It must return either
 	
-	* a string that will go literally into the eventual serialised SQL 
-	  string (so take care to quote; in general, you will use 
-	  nodes.flatten(arg) to flatten individual args); 
+	* a string that will go literally into the eventual serialised SQL
+	  string (so take care to quote; in general, you will use
+	  nodes.flatten(arg) to flatten individual args);
 	* or they may return None, in which case the expression tree
 	  remains unchanged.  This is for when the actual implementation is
 	  in the database.
-	* or they may raise nodes.ReplaceNode(r), where r is a nodes.ADQLNode 
-		instance, which then replaces the user defined function in the parse 
+	* or they may raise nodes.ReplaceNode(r), where r is a nodes.ADQLNode
+		instance, which then replaces the user defined function in the parse
 		tree and will be annotated as usual.
 
 	If you receive bad arguments or something else goes awry, raise
@@ -113,8 +100,10 @@ def userFunction(name,
 	return deco
 
 
-def _makeBooleanizer(funcName, booleanExpr):
-	"""makes and registers a booleanizer for funcName.
+def _makeTwoArgBooleanizer(funcName, booleanExpr):
+	"""makes and registers a 2-argument booleanizer for funcName.
+
+	That's where 1=some_pred(x) should become some [not] op1 opr op2 expression.
 
 	booleanExpr is the postgres expression the function should be booleanized
 	to.  Refer to the two arguments as %(1)s and %(2)s
@@ -124,13 +113,23 @@ def _makeBooleanizer(funcName, booleanExpr):
 			raise common.UfuncError("%s takes exactly two arguments"%funcName)
 		return morphhelpers.addNotToBooleanized(
 			booleanExpr%{
-				'1': nodes.flatten(node.args[0]), 
+				'1': nodes.flatten(node.args[0]),
 				'2': nodes.flatten(node.args[1])},
 			operator, operand)
 
 	morphhelpers.registerBooleanizer(funcName.upper(), _booleanizeThis)
 
 
+def _makeOneArgBooleanizer(funcName):
+	"""makes and registers a one-argument booleanizer for funcName.
+
+	That's for when the morphed ADQL would just be [not] expr, and
+	expr is what the udf code has returned.
+	"""
+	morphhelpers.registerBooleanizer(funcName.upper(),
+		morphhelpers.addNotToBooleanized)
+
+
 @userFunction("gavo_match",
 	"(pattern TEXT, string TEXT) -> INTEGER",
 	"""
@@ -160,7 +159,7 @@ def _hasword(args):
 		raise common.UfuncError("ivo_hasword takes exactly two arguments")
 	return None
 
-_makeBooleanizer("ivo_hasword", 
+_makeTwoArgBooleanizer("ivo_hasword",
 	"(to_tsvector('english', %(1)s) @@ plainto_tsquery('english', %(2)s))")
 
 
@@ -186,7 +185,7 @@ def _nocasematch(args):
 		args[1] = "LOWER(%s)"%nodes.flatten(args[1])
 	return None
 
-_makeBooleanizer("ivo_nocasematch", "(LOWER(%(1)s) like %(2)s)")
+_makeTwoArgBooleanizer("ivo_nocasematch", "(LOWER(%(1)s) like %(2)s)")
 
 
 @userFunction("ivo_hashlist_has",
@@ -205,7 +204,7 @@ def _hashlist_has(args):
 		raise common.UfuncError("ivo_haslist_has takes exactly two arguments")
 	return None
 
-_makeBooleanizer("ivo_hashlist_has", 
+_makeTwoArgBooleanizer("ivo_hashlist_has",
 	"lower(%(2)s) = ANY(string_to_array(lower(%(1)s), '#'))")
 
 @userFunction("ivo_interval_overlaps",
@@ -220,10 +219,17 @@ def _interval_overlaps(args):
 	if len(args)!=4:
 		raise common.UfuncError(
 			"ivo_interval_overlaps takes exactly four arguments")
-	l1, h1, l2, h2 = [nodes.flatten(a) for a in args]
-	return "((%s)>=(%s) AND (%s)>=(%s) AND (%s)<=(%s) AND (%s)<=(%s))::INTEGER"%(
-		h1, l2, h2, l1,
-		l1, h1, l2, h2)
+	l1, h1, l2, h2 = args
+	return nodes.TransparentNode(children=[
+		nodes.Comparison(op1=h1, opr='>=', op2=l2),
+		"AND", nodes.Comparison(op1=h2, opr='>=', op2=l1),
+		"AND", nodes.Comparison(op1=l1, opr='<=', op2=h1),
+		"AND", nodes.Comparison(op1=l2, opr='<=', op2=h2)])
+#	return "((%s)>=(%s) AND (%s)>=(%s) AND (%s)<=(%s) AND (%s)<=(%s))::INTEGER"%(
+#		h1, l2, h2, l1,
+#		l1, h1, l2, h2)
+
+_makeOneArgBooleanizer("ivo_interval_overlaps")
 
 
 @userFunction("ivo_interval_has",
@@ -239,7 +245,7 @@ def _interval_has(args):
 			"ivo_interval_has takes exactly two arguments")
 	return None
 
-_makeBooleanizer("ivo_interval_has", "((%(1)s) <@ (%(2)s))")
+_makeTwoArgBooleanizer("ivo_interval_has", "((%(1)s) <@ (%(2)s))")
 
 
 @userFunction("gavo_to_mjd",
@@ -290,7 +296,7 @@ class HistogramNode(nodes.FunctionNode):
 	Assuming 0-based arrays, result[0] contains the number of underflows (i.e.,
 	val<lower), result[nbins+1] the number of overflows.  Elements 1..nbins
 	are the counts in nbins bins of width (upper-lower)/nbins.  Clients
-	will have to convert back to physical units using some external 
+	will have to convert back to physical units using some external
 	communication, there currently is no (meta-) data as lower and upper in
 	the TAP response.
 	""",
@@ -371,7 +377,7 @@ class _TransformNode(nodes.FunctionNode):
 	to_sys must be literal strings (i.e., they cannot be computed through
 	expressions or be taken from database columns).
 
-	All transforms are just simple rotations, which is only a rough 
+	All transforms are just simple rotations, which is only a rough
 	approximation to the actual relationships between reference systems
 	(in particular between FK4 and ICRS-based ones).  Note that, in particular,
 	the epoch is not changed (i.e., no proper motions are applied).
@@ -386,8 +392,8 @@ def _gavo_transform(args):
 		raise common.UfuncError(
 			"gavo_transform takes exactly three arguments")
 
-	fromSys = getStringLiteral(args[0], "source reference system")
-	toSys = getStringLiteral(args[1], "target reference system")
+	fromSys = nodes.getStringLiteral(args[0], "source reference system")
+	toSys = nodes.getStringLiteral(args[1], "target reference system")
 
 	try:
 		trans = stc.getPGSphereTrafo(fromSys, toSys)
@@ -400,7 +406,7 @@ def _gavo_transform(args):
 		return args[2]
 	else:
 		raise nodes.ReplaceNode(_TransformNode(
-			funName="gavo_transform", 
+			funName="gavo_transform",
 			args=(args[2],),
 			trans=trans,
 			toSys=toSys))
@@ -479,7 +485,7 @@ def _string_agg(args):
 	an object at ra/dec with proper motion pmra/pmde has after epdist years.
 
 	positions must be in degrees, PMs in should be in julian years (i.e., proper
-	motions are expected in degrees/year).  pmra is assumed to contain 
+	motions are expected in degrees/year).  pmra is assumed to contain
 	cos(delta).
 
 	This function goes through the tangential plane.  Since it does not have
@@ -495,7 +501,7 @@ def _string_agg(args):
 def _gavo_apply_pm(args):
 	# we need to make sure we eventually flatten to ivo_apply_pm, as
 	# that's what is defined in //adql.  For that, we have to manipulate
-	# our node, which UserFunction below somwhat stupidly does not allow.
+	# our node, which UserFunction below somewhat stupidly does not allow.
 	# We simply steal its self:
 	utils.stealVar("self").funName = "IVO_APPLY_PM"
 
@@ -528,7 +534,7 @@ def _gavo_apply_pm(args):
 	at out_epoch for these quantities at ref_epoch.
 
 	Essentially, it will apply the proper motion under the assumption of
-	linear motion.  Despite the name of the positional parameters, this is 
+	linear motion.  Despite the name of the positional parameters, this is
 	not restricted to equatorial systems, as long as positions and proper
 	motions are expressed in the same reference frames.
 
@@ -577,7 +583,7 @@ def _epoch_prop(args):
 	with the six parameters at ref_epoch.
 
 	Essentially, it will apply the proper motion under the assumption of
-	linear motion.  Despite the name of the positional parameters, this is 
+	linear motion.  Despite the name of the positional parameters, this is
 	not restricted to equatorial systems, as long as positions and proper
 	motions are expressed in the same reference frames.
 
@@ -591,7 +597,7 @@ def _epoch_prop(args):
 	""",
 	returntype="spoint",
 	polymorphism=[
-		("(ra DOUBLE PRECISION, dec DOUBLE PRECISION," 
+		("(ra DOUBLE PRECISION, dec DOUBLE PRECISION,"
 			" pmra DOUBLE PRECISION, pmdec DOUBLE PRECISION,"
 			" ref_epoch DOUBLE PRECISION, out_epoch DOUBLE PRECISION)"
 			" -> POINT",
@@ -630,10 +636,10 @@ def _epoch_prop_pos(args):
 
 @userFunction("ivo_healpix_index",
 	"(order INTEGER, ra DOUBLE PRECISION, dec DOUBLE PRECISION) -> BIGINT",
-	"""Returns the index of the (nest) healpix with order containing the 
+	"""Returns the index of the (nest) healpix with order containing the
 	spherical point (ra, dec).
 
-	An alternative, 2-argument form 
+	An alternative, 2-argument form
 	
 	ivo_healpix_index(order INTEGER, p POINT) -> BIGINT
 
@@ -688,7 +694,7 @@ def _gavo_getauthority(args):
 	"""returns 1 if matchagainst is term or narrower in the IVOA vocabulary
 	vocname, 0 otherwise.
 
-	This is intended for semantic querying.  For instance, 
+	This is intended for semantic querying.  For instance,
 	gavo_vocmatch('datalink/core', 'calibration', semantics) would be 1
 	if semantics is any of calibration, bias, dark, or flat.
 
@@ -707,8 +713,8 @@ def _gavo_vocmatch(args):
 
 	from gavo.protocols import vocabularies
 	voc = vocabularies.get_vocabulary(
-		getStringLiteral(args[0], "vocabulary name"))
-	term = getStringLiteral(args[1], "term")
+		nodes.getStringLiteral(args[0], "vocabulary name"))
+	term = nodes.getStringLiteral(args[1], "term")
 	if term not in voc["terms"]:
 		raise common.UfuncError("'%s' is not a term in the vocabulary %s"%(
 			term, voc["uri"]))
@@ -720,23 +726,31 @@ def _gavo_vocmatch(args):
 
 	args[1] = "(%s)"%(", ".join(nodes.flatten(n) for n in expanded))
 	raise nodes.ReplaceNode(nodes.FunctionNode(
-		funName='GAVO_VOCMATCH', 
+		funName='GAVO_VOCMATCH',
 		args=(args[2], args[1])))
 
-_makeBooleanizer("gavo_vocmatch", 
+_makeTwoArgBooleanizer("gavo_vocmatch",
 	"%(1)s IN %(2)s")
 
 
 class _SpecconvNode(nodes.FieldInfoedNode):
-	"""A node left by the two-argument gavo_specconv.
+	"""A node left by gavo_specconv.
 
 	It is constructed with the target unit and the expression to
 	convert.  During annotation, it will mutate its children so they
 	are in the target unit.
+
+	We have this complication because at UDF creation time, the columns
+	are not annotated and hence we may not know the unit to convert from.
+	If forceUnit is given (three-argument form), we could do the
+	transformation immediately, but we still use the code in
+	addFieldInfo to avoid too many code paths.  The price is the
+	stupid exprReplaced hack.
 	"""
 	_a_targetUnit = None
 	_a_forceUnit = None
 	_a_expr = None
+	exprReplaced = False
 
 	def _getFromUnit(self, context):
 		"""returns the unit we transform from.
@@ -775,7 +789,7 @@ class _SpecconvNode(nodes.FieldInfoedNode):
 				if hasattr(child, "children"):
 					if replaceTarget(child.children):
 						return True
-
+	
 		if replaceTarget(newExpr.children):
 			self.expr = newExpr
 		else:
@@ -794,12 +808,20 @@ class _SpecconvNode(nodes.FieldInfoedNode):
 				].parseString(exprLit, parseAll=True)[0]
 			
 			self._replaceExpr("REPLACE_ME", exprNode)
+			self.exprReplaced = True
 
-			self.fieldInfo = self.fieldInfo.change(unit=self.targetUnit)
+			self.fieldInfo = self.fieldInfo.change(
+				unit=self.targetUnit,
+				type="double precision")
+			# TODO: We should probably fix the description to indicate
+			# it's some sort of different thing.
 		except base.IncompatibleUnits as msg:
 			raise common.UfuncError(str(msg))
 	
 	def flatten(self):
+		if not self.exprReplaced:
+			raise ValueError("You cannot use ivo_specconv here.  Complain"
+				" to the operators.")
 		return self.expr.flatten()
 
 
@@ -810,7 +832,7 @@ class _SpecconvNode(nodes.FieldInfoedNode):
 	expr has to be in either energy, wavelength, or frequency, and dest_unit
 	must be a VOUnit giving another spectral unit (e.g., MHz, keV, nm, or
 	Angstrom). This is intended to let users express spectral constraints
-	in their preferred unit independently of the choice of unit in the 
+	in their preferred unit independently of the choice of unit in the
 	database.  Examples::
 
 		gavo_specconv(obscore.em_min, "keV") > 300
@@ -821,7 +843,7 @@ class _SpecconvNode(nodes.FieldInfoedNode):
 	argument.
 	""",
 	polymorphism=[(
-		"(expr NUMERIC, expr_unit TEXT dest_unit TEXT) -> NUMERIC",
+		"(expr NUMERIC, expr_unit TEXT, dest_unit TEXT) -> NUMERIC",
 		"""returns expr assumed to be in expr_unit expressed in dest_unit.
 
 		This is a variant of the two-argument gavo_specconv for when
@@ -839,13 +861,13 @@ class _SpecconvNode(nodes.FieldInfoedNode):
 def _gavo_specconv(args):
 	try:
 		if len(args)==2:
-			newNode = _SpecconvNode(expr=args[0], 
-				targetUnit=getStringLiteral(args[1]))
+			newNode = _SpecconvNode(expr=args[0],
+				targetUnit=nodes.getStringLiteral(args[1]))
 
 		elif len(args)==3:
-			newNode = _SpecconvNode(expr=args[0], 
-				targetUnit=getStringLiteral(args[2]),
-				forceUnit=getStringLiteral(args[1]))
+			newNode = _SpecconvNode(expr=args[0],
+				targetUnit=nodes.getStringLiteral(args[2]),
+				forceUnit=nodes.getStringLiteral(args[1]))
 		
 		else:
 			raise common.UfuncError("gavo_specconv takes two or three arguments")
@@ -878,7 +900,7 @@ class UserFunction(nodes.FunctionNode):
 		if self.processedExpression is None:
 			return nodes.FunctionNode.flatten(self)
 		else:
-			return self.processedExpression
+			return nodes.flatten(self.processedExpression)
 
 	def addFieldInfo(self, context):
 		ufunc = self._getFunc()
@@ -889,8 +911,8 @@ class UserFunction(nodes.FunctionNode):
 			else ufunc.adqlUDF_ucd)
 
 		self.fieldInfo = fieldinfo.FieldInfo(
-			ufunc.adqlUDF_returntype, 
-			unit, 
+			ufunc.adqlUDF_returntype,
+			unit,
 			ucd)
 
 
