@@ -1,9 +1,12 @@
 """
-This should evolve into a useful API to GAVO code for more-or-less external
-clients.
+Operator code based on DaCHS (e.g., custom grammars or pages, other helper
+code, or ``<code>`` items in RDs) should just need to ``from gavo import api``
+and then work with what is in here.  See
+http://docs.g-vo.org/DaCHS/ref.html#api-function-reference for what is
+available here.
 """
 
-#c Copyright 2008-2022, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -13,7 +16,7 @@ clients.
 
 # this (and its companion in user/cli.py) works around a race condition in
 # Debian stretch's python-cryptography module.  Remove about 2021.
-from cryptography.hazmat.bindings.openssl.binding import Binding 
+from cryptography.hazmat.bindings.openssl.binding import Binding
 
 from gavo import base
 from gavo import rscdesc
@@ -25,10 +28,10 @@ RD = rscdesc.RD
 
 from gavo.base import (getConfig, setConfig, getBinaryName,
 	getDBConnection, DBError,	IntegrityError,
-	UnmanagedQuerier, AdhocQuerier, 
+	UnmanagedQuerier, AdhocQuerier,
 	getTableConn, getAdminConn, getUntrustedConn,
 	getWritableTableConn, getWritableAdminConn,
-	NoMetaKey, Error, StructureError, ValidationError, LiteralParseError, 
+	NoMetaKey, Error, StructureError, ValidationError, LiteralParseError,
 	ReportableError, NotFoundError, RDNotFound, SourceParseError, DataError,
 	MetaValidationError, BadUnit, BadCode,
 	parseFromString,
@@ -59,9 +62,9 @@ from gavo.rsc import (TableForDef, DBTable, makeData, parseValidating,
 from gavo.rscdef import TableDef, getFlatName, getReferencedElement
 
 from gavo.stc import (dateTimeToJYear, dateTimeToJdn, dateTimeToMJD,
-	jYearToDateTime, jdnToDateTime, mjdToDateTime, parseISODT)
+	jYearToDateTime, jdnToDateTime, mjdToDateTime, parseISODT, isMJD)
 
-from gavo.svcs import (UnknownURI, ForbiddenURI, Authenticate, 
+from gavo.svcs import (UnknownURI, ForbiddenURI, Authenticate,
 	WebRedirect, SeeOther, Core, OutputTableDef, QueryMeta)
 
 from gavo.user.logui import LoggingUI
@@ -69,6 +72,7 @@ from gavo.user.plainui import StingyPlainUI, PlainUI
 
 from gavo.utils import (
 	loadPythonModule, formatISODT, bytify, EqualingRE,
+	document,
 	pyfits, cutoutFITS,
 	safeReplaced,
 	QuotedName,
@@ -81,4 +85,25 @@ from gavo.helpers.testtricks import getXMLTree
 
 from gavo.rscdef.rmkfuncs import *
 
+@document
+def reloadLocal():
+	"""reloads the local namespace.
+
+	This is material an operator defines in $GAVO_CONFIG/local.py.  If that
+	file is missing or unreadable, api.local will be a stub that raises
+	a constant error regardless of what you try to getattr from it.
+	"""
+	global local
+	modpath = base.getConfig("configDir")+"/local"
+	try:
+		local, _ = loadPythonModule(modpath)
+	except (FileNotFoundError, ImportError):
+		class _:
+			def __getattr__(self, name):
+				raise ReportableError(f"No file {modpath}.py found")
+		local = _()
+
+
+reloadLocal()
+
 __version__ = base.getVersion()
