@@ -1,9 +1,9 @@
 """
-Basic code for defining objects in the data center: Structures, their 
+Basic code for defining objects in the data center: Structures, their
 attributes, fundamental VO conventions.
 """
 
-#c Copyright 2008-2022, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -25,7 +25,7 @@ from gavo.base.complexattrs import *
 
 from gavo.base.config import(
 	get as getConfig, set as setConfig,
-	getDBProfile, 
+	getDBProfile,
 	makeFallbackMeta as _makeFallbackMeta)
 
 from gavo.base.events import EventDispatcher
@@ -70,7 +70,7 @@ from gavo.base.parsecontext import (
 	ReferenceListAttribute, resolveId, resolveCrossId, resolveNameBased,
 	getTableDefForTable, assertType)
 
-from gavo.base.sqlsupport import (getDBConnection, 
+from gavo.base.sqlsupport import (getDBConnection,
 	DBError, QueryCanceledError, IntegrityError,
 	AdhocQuerier, UnmanagedQuerier,
 	getUntrustedConn,
@@ -82,7 +82,7 @@ from gavo.base.sqlsupport import (getDBConnection,
 	NonBlockingQuery,
 	onDBInit)
 
-from gavo.base.structure import (Structure, ParseableStructure, 
+from gavo.base.structure import (Structure, ParseableStructure,
 	DataContent, makeStruct, RestrictionMixin)
 
 from gavo.base.typesystems import *
@@ -90,13 +90,14 @@ from gavo.base.typesystems import *
 from gavo.base.valuemappers import (SerManager, ValueMapperFactoryRegistry,
 	VOTNameMaker, registerDefaultMF)
 
-from gavo.base.sqlmunge import (getSQLForField, getSQLKey, 
+from gavo.base.spectral import getSpecExpr, getSpecConverter, getFilterMap
+
+from gavo.base.sqlmunge import (getSQLForField, getSQLKey,
 	joinOperatorExpr)
 
 from gavo.base.unitconv import (
 	computeConversionFactor, parseUnit, computeColumnConversions,
-	getSpecExpr, getSpecConverter,
-	IncompatibleUnits, BadUnit, 
+	IncompatibleUnits, BadUnit,
 	LIGHT_C, PLANCK_H, BOLTZMANN_K)
 
 from gavo.base.xmlstruct import parseFromString, parseFromStream, feedTo
