@@ -2,20 +2,25 @@
 Common code for DaCHS's base package.
 """
 
-#c Copyright 2008-2022, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
 
+
 import warnings
 
-from gavo.utils.excs import *  #noflake: really want those names
-from gavo.utils import excs # make life a bit easier for pyflakes.
-
+from gavo.utils.excs import (  #noflake: exceptions are historically in base	
+	Error,
+	BadCode, DataError, EmptyData, ExecutiveAction, LiteralParseError,
+	MultiplicityError, NotFoundError, ParseException, RDNotFound,
+	ReportableError, RestrictedElement, SkipThis, StructureError,
+	SourceParseError, ValidationError)
+	
 
 class NotGivenType(type):
 	def __str__(self):
-		raise excs.StructureError(
+		raise StructureError(
 			"%s cannot be stringified"%self.__class__.__name__)
 
 	def __repr__(self):
@@ -30,7 +35,7 @@ class NotGiven(object, metaclass=NotGivenType):
 	"""
 
 
-class Ignore(excs.ExecutiveAction):
+class Ignore(ExecutiveAction):
 	"""An executive action causing an element to be not adopted by its
 	parent.
 
@@ -40,7 +45,7 @@ class Ignore(excs.ExecutiveAction):
 	"""
 
 
-class Replace(excs.ExecutiveAction):
+class Replace(ExecutiveAction):
 	"""An executive action replacing the current child with the Exception's
 	argument.
 
@@ -70,7 +75,7 @@ class Parser(object):
 		elif type=="end":
 			return self.end_(ctx, name, value)
 		else:
-			raise excs.StructureError(
+			raise StructureError(
 				"Illegal event type while building: '%s'"%type)
 
 
