@@ -2,7 +2,7 @@
 Definition of DC config options and their management including I/O.
 """
 
-#c Copyright 2008-2022, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -53,8 +53,8 @@ class RelativeURL(StringConfigItem):
 	typedesc = "URL fragment relative to the server's root"
 
 	def _getValue(self):
-		if (self._value.startswith("http://") 
-				or self._value.startswith("https://") 
+		if (self._value.startswith("http://")
+				or self._value.startswith("https://")
 				or self._value.startswith("/")):
 			return self._value
 		return self.parent.get("web", "nevowRoot")+self._value
@@ -321,7 +321,7 @@ _config = Configuration(
 		ExpandedPathConfigItem("rootDir", default="/var/gavo", description=
 			"Path to the root of the DC file (all other paths may be"
 			" relative to this"),
-		RootRelativeConfigItem("configDir", default="etc", 
+		RootRelativeConfigItem("configDir", default="etc",
 			description="Path to the DC's non-ini configuration (e.g., DB profiles)"),
 		RootRelativeConfigItem("inputsDir", default="inputs",
 			description="Path to the DC's data holdings"),
@@ -407,7 +407,7 @@ _config = Configuration(
 				" server's root; this must end with a slash (setting this"
 				" will currently break essentially the entire web interface."
 				" If you must use it, contact the authors and we will fix things.)"),
-		StringConfigItem("realm", default="X-Unconfigured", 
+		StringConfigItem("realm", default="X-Unconfigured",
 			description="Authentication realm to be used (currently,"
 			" only one, server-wide, is supported)"),
 		WebRelativeConfigItem("templateDir", default="templates",
@@ -427,9 +427,9 @@ _config = Configuration(
 		BooleanConfigItem("enableTests", "False",
 			"Enable test pages (don't if you don't know why)"),
 		IntConfigItem("maxPreviewWidth", "300", "Ignored, only present"
-			" for backward compatiblity"),
-		ListConfigItem("graphicMimes", 
-			"image/fits,image/jpeg,application/x-votable+xml;content=datalink", 
+			" for backward compatibility"),
+		ListConfigItem("graphicMimes",
+			"image/fits,image/jpeg,application/x-votable+xml;content=datalink",
 			"Media types considered as graphics (for SIAP, mostly)"),
 		IntConfigItem("maxUploadSize",
 			"20000000",
@@ -510,8 +510,8 @@ _config = Configuration(
 	),
 
 	Section('db', 'Settings concerning database access.',
-		IntConfigItem("indexWorkMem", 
-			"2000", 
+		IntConfigItem("indexWorkMem",
+			"2000",
 			"Megabytes of memory to give to postgres while making indices."
 			"  Set to roughly half your RAM when you have big tables."),
 		StringConfigItem("interface", "psycopg2", "Don't change"),
@@ -526,17 +526,19 @@ _config = Configuration(
 		SetConfigItem("adqlProfiles", "untrustedquery", "Name(s) of profiles that"
 			" get access to tables opened for ADQL"),
 		IntConfigItem("defaultLimit", "100", "Default match limit for DB queries"),
-		ListConfigItem("managedExtensions", 
+		ListConfigItem("managedExtensions",
 			"pg_sphere",
 			"Name(s) of postgres extensions gavo upgrade -e should watch"),
-		BooleanConfigItem("dumpSystemTables", 
+		BooleanConfigItem("dumpSystemTables",
 			"False",
 			"Dump the tables from //users and //system to"
 			" stateDir/system_tables once a day?"),
+		IntConfigItem("poolSize", "2",
+			"Number of connections in DaCHS' postgres connection pools."),
 	),
 	
 	Section('ivoa', 'The interface to the Greater VO.',
-		AuthorityConfigItem("authority", "x-unregistred", 
+		AuthorityConfigItem("authority", "x-unregistred",
 			"The authority id for this DC; this has *no* leading ivo://"),
 		IntConfigItem("dalDefaultLimit", "10000",
 			"Default match limit on SCS/SSAP/SIAP queries"),
@@ -554,12 +556,12 @@ _config = Configuration(
 		EnumeratedConfigItem("sdmVersion", "1",
 			"Obsolete (SDM version 2 is shelved).  Don't use.",
 			options=["1", "2"]),
-		EnumeratedConfigItem("VOSITableDetail", 
+		EnumeratedConfigItem("VOSITableDetail",
 			"max",
 			"Default level of detail to return on the VOSI endpoint (change"
 			" to min when you have more than 100 or tables).",
 			options=["min", "max"]),
-		BooleanConfigItem("registerAlternative", 
+		BooleanConfigItem("registerAlternative",
 			"False",
 			"Give access URLs for the alternative protocol (https when"
 			" serverURL is http and vice versa) as a mirrorURL?  If you're"
@@ -576,14 +578,14 @@ def loadConfig():
 	try:
 		fancyconfig.readConfiguration(_config,
 			os.environ.get("GAVOSETTINGS", "/etc/gavo.rc"),
-			os.environ.get("GAVOCUSTOM", 
+			os.environ.get("GAVOCUSTOM",
 				os.path.join(os.environ.get("HOME", "/no_home"), ".gavorc")))
 	except fancyconfig.ConfigError as ex:
 		# This is usually not be protected by top-level exception catcher
 		sys.exit("Bad configuration item in %s.  %s"%(
 			ex.fileName, str(ex)))
 	
-	# also set XDG directories so astropy and friends look for their 
+	# also set XDG directories so astropy and friends look for their
 	# configuration in DaCHS' directories
 	os.environ["XDG_CONFIG_HOME"] = _config.get("configDir")
 	os.environ["XDG_CACHE_HOME"] = _config.get("cacheDir")
