@@ -6,7 +6,7 @@ This is currently very postgres specific.  If we really wanted to
 support some other database, this would need massive refactoring.
 """
 
-#c Copyright 2008-2022, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -53,7 +53,7 @@ _PG_TIME_UNITS = {
 	"d": 86400.,}
 
 
-# Keep track of wether we have installed our extensions
+# Keep track of whether we have installed our extensions
 # (this is to not require a DB connection on just importing this)
 _PSYCOPG_INITED = False
 
@@ -73,9 +73,9 @@ class SqlSetAdapter(object):
 		qobjs = []
 		for o in self._seq:
 			if isinstance(o, str):
-				qobjs.append(psycopg2.extensions.adapt(str(o)).getquoted()) 
+				qobjs.append(psycopg2.extensions.adapt(str(o)).getquoted())
 			else:
-				qobjs.append(psycopg2.extensions.adapt(o).getquoted()) 
+				qobjs.append(psycopg2.extensions.adapt(o).getquoted())
 		return b'(%s)'%(b", ".join(qobjs))
 
 	__str__ = getquoted
@@ -130,7 +130,7 @@ class SqlArrayAdapter(object):
 				item.encoding = "utf-8"
 				qobjs.append(item.getquoted())
 		else:
-			qobjs = [psycopg2.extensions.adapt(o).getquoted() 
+			qobjs = [psycopg2.extensions.adapt(o).getquoted()
 				for o in self._seq]
 
 		self._addCastIfNecessary(qobjs)
@@ -196,9 +196,9 @@ psycopg2.extensions.register_adapter(set, SqlSetAdapter)
 psycopg2.extensions.register_adapter(frozenset, SqlSetAdapter)
 
 for numpyType, adapter in [
-		("float32", FloatableAdapter), 
-		("float64", FloatableAdapter), 
-		("float96", FloatableAdapter), 
+		("float32", FloatableAdapter),
+		("float64", FloatableAdapter),
+		("float96", FloatableAdapter),
 		("int8", IntableAdapter),
 		("int16", IntableAdapter),
 		("int32", IntableAdapter),
@@ -223,7 +223,7 @@ from gavo.utils import pyfits
 psycopg2.extensions.register_adapter(pyfits.Undefined, NULLAdapter)
 
 from psycopg2 import (OperationalError, #noflake: exported names
-	DatabaseError, IntegrityError, ProgrammingError, 
+	DatabaseError, IntegrityError, ProgrammingError,
 	InterfaceError, DataError, InternalError)
 from psycopg2.extensions import QueryCanceledError #noflake: exported name
 from psycopg2 import Error as DBError
@@ -328,7 +328,7 @@ class GAVOConnection(psycopg2.extensions.connection):
 		"""
 		return self.query(query, args, timeout, True, caseFixer)
 
-	def query(self, query, args={}, timeout=None, 
+	def query(self, query, args={}, timeout=None,
 			yieldDicts=False, caseFixer=None):
 		"""iterates over result tuples for query.
 
@@ -444,7 +444,7 @@ def getDBConnection(profile, debug=debug, autocommitted=False):
 		conn.close = closer
 	else:
 		try:
-			conn = psycopg2.connect(connection_factory=GAVOConnection, 
+			conn = psycopg2.connect(connection_factory=GAVOConnection,
 				**profile.getArgs())
 		except OperationalError as msg:
 			raise utils.ReportableError("Cannot connect to the database server."
@@ -530,7 +530,7 @@ class PostgresQueryMixin(object):
 	"""
 
 	def getPrimaryIndexName(self, tableName):
-		"""returns the name of the index corresponding to the primary key on 
+		"""returns the name of the index corresponding to the primary key on
 		(the unqualified) tableName.
 		"""
 		return ("%s_pkey"%tableName).lower()
@@ -563,16 +563,16 @@ class PostgresQueryMixin(object):
 		This really is a helper for foreignKeyExists.
 		"""
 		colNames = set(n.lower() for n in colNames)
-		res = [r[0] for r in 
+		res = [r[0] for r in
 			self.connection.query("SELECT attnum FROM pg_attribute WHERE"
 				" attrelid=%(relOID)s and attname IN %(colNames)s",
 				locals())]
 		res.sort()
 		return res
 
-	def getForeignKeyName(self, srcTableName, destTableName, srcColNames, 
+	def getForeignKeyName(self, srcTableName, destTableName, srcColNames,
 			destColNames, schema=None):
-		"""returns True if there's a foreign key constraint on srcTable's 
+		"""returns True if there's a foreign key constraint on srcTable's
 		srcColNames using destTableName's destColNames.
 
 		Warning: names in XColNames that are not column names in the respective
@@ -583,7 +583,7 @@ class PostgresQueryMixin(object):
 		try:
 			srcOID = self.getOIDForTable(srcTableName, schema)
 			srcColInds = self._getColIndices( #noflake: used in locals()
-				srcOID, srcColNames) 
+				srcOID, srcColNames)
 			destOID = self.getOIDForTable(destTableName, schema)
 			destColInds = self._getColIndices( #noflake: used in locals()
 				destOID, destColNames)
@@ -602,12 +602,12 @@ class PostgresQueryMixin(object):
 		else:
 			raise ValueError("Non-existing or ambiguous foreign key")
 
-	def foreignKeyExists(self, srcTableName, destTableName, srcColNames, 
+	def foreignKeyExists(self, srcTableName, destTableName, srcColNames,
 			destColNames, schema=None):
 		try:
 			_ = self.getForeignKeyName(   #noflake: ignored value
-				srcTableName, destTableName, 
-				srcColNames, destColNames, 
+				srcTableName, destTableName,
+				srcColNames, destColNames,
 				schema)
 			return True
 		except ValueError:
@@ -640,7 +640,7 @@ class PostgresQueryMixin(object):
 		cursor = self.connection.cursor()
 		try:
 			cursor.execute("select * from %s limit 0"%tableName)
-			return [(col.name, self._resolveTypeCode(col.type_code)) for col in 
+			return [(col.name, self._resolveTypeCode(col.type_code)) for col in
 				cursor.description]
 		finally:
 			cursor.close()
@@ -663,7 +663,7 @@ class PostgresQueryMixin(object):
 		"""returns True if there role is known to the database.
 		"""
 		matches = list(self.connection.query(
-			"SELECT usesysid FROM pg_user WHERE usename=%(role)s", 
+			"SELECT usesysid FROM pg_user WHERE usename=%(role)s",
 			locals()))
 		return len(matches)!=0
 	
@@ -697,10 +697,10 @@ class PostgresQueryMixin(object):
 		res = list(
 			self.connection.query("""SELECT table_name, table_type FROM
 				information_schema.tables WHERE (
-						table_schema=%(schemaName)s 
+						table_schema=%(schemaName)s
 						OR table_type='LOCAL TEMPORARY')
 					AND table_name=%(tableName)s""", {
-					'tableName': tableName.lower(), 
+					'tableName': tableName.lower(),
 					'schemaName': schema.lower()}))
 
 		if not res:
@@ -713,10 +713,10 @@ class PostgresQueryMixin(object):
 					self.connection.query(
 						"""SELECT matviewname, 'MATERIALIZED VIEW' AS table_type
 						FROM pg_matviews
-						WHERE 
-							schemaname=%(schemaName)s 
+						WHERE
+							schemaname=%(schemaName)s
 							AND matviewname=%(tableName)s""", {
-						'tableName': tableName.lower(), 
+						'tableName': tableName.lower(),
 						'schemaName': schema.lower()}))
 
 			if not res:
@@ -743,7 +743,7 @@ class PostgresQueryMixin(object):
 			"BASE TABLE": "TABLE",
 			"LOCAL TEMPORARY": "TABLE"}[tableType]
 		self.connection.execute("DROP %s %s %s"%(
-			dropQualification, 
+			dropQualification,
 			tableName,
 			"CASCADE" if cascade else ""))
 		
@@ -814,7 +814,7 @@ class PostgresQueryMixin(object):
 
 class StandardQueryMixin(object):
 	"""is a mixin containing various useful queries that should work
-	agains all SQL systems.
+	against all SQL systems.
 
 	This mixin expects a parent that mixes is QuerierMixin (that, for now,
 	also mixes in StandardQueryMixin, so you won't need to mix this in).
@@ -858,7 +858,7 @@ class StandardQueryMixin(object):
 					self.connection.execute(
 						"GRANT %s ON %s TO %s"%(shouldPrivs[role], objectName, role))
 				else:
-					utils.sendUIEvent("Warning", 
+					utils.sendUIEvent("Warning",
 						"Request to grant privileges to non-existing"
 						" database user %s dropped"%role)
 		for role in set(shouldPrivs)&set(foundPrivs):
@@ -990,13 +990,13 @@ class AdhocQuerier(QuerierMixin):
 class NonBlockingQuery:
 	"""a query run in a pseudo-nonblocking way.
 
-	While psycopg2 can do proper async, that doesn't play well with 
+	While psycopg2 can do proper async, that doesn't play well with
 	about everything else DaCHS is doing so far.  So, here's a quick
 	way to allow long-running queries that users can still interrupt.
 	The ugly secret is that it's based on threads.
 
 	This should not be used within the server.  We might want to port
-	the async taprunner (which runs outside of the server) to using this, 
+	the async taprunner (which runs outside of the server) to using this,
 	though.
 
 	To use it, construct it with conn, query and perhaps args and
@@ -1047,7 +1047,7 @@ class NonBlockingQuery:
 	def cleanup(self, timeout=None):
 		"""tries to reap the thread (i.e., join it).
 
-		If the thread hasn't terminated within timeout seconds, a 
+		If the thread hasn't terminated within timeout seconds, a
 		sqlsupport.Error is raised.
 		"""
 		self.thread.join(timeout=timeout)
@@ -1087,18 +1087,18 @@ def getDBMeta(key):
 
 JOIN_FUNCTION_BODY = """
 SELECT (
-    (   
+    (
        ((q3c_ang2ipix($3,$4)>=(q3c_nearby_it($1,$2,$5,0))) AND (q3c_ang2ipix($3,$4)<=(q3c_nearby_it($1,$2,$5,1))))
     OR ((q3c_ang2ipix($3,$4)>=(q3c_nearby_it($1,$2,$5,2))) AND (q3c_ang2ipix($3,$4)<=(q3c_nearby_it($1,$2,$5,3))))
     OR ((q3c_ang2ipix($3,$4)>=(q3c_nearby_it($1,$2,$5,4))) AND (q3c_ang2ipix($3,$4)<=(q3c_nearby_it($1,$2,$5,5))))
     OR ((q3c_ang2ipix($3,$4)>=(q3c_nearby_it($1,$2,$5,6))) AND (q3c_ang2ipix($3,$4)<=(q3c_nearby_it($1,$2,$5,7))))
-    ) AND                           
-    (    
+    ) AND
+    (
        ((q3c_ang2ipix($1,$2)>=(q3c_nearby_it($3,$4,$5,0))) AND (q3c_ang2ipix($1,$2)<=(q3c_nearby_it($3,$4,$5,1))))
     OR ((q3c_ang2ipix($1,$2)>=(q3c_nearby_it($3,$4,$5,2))) AND (q3c_ang2ipix($1,$2)<=(q3c_nearby_it($3,$4,$5,3))))
     OR ((q3c_ang2ipix($1,$2)>=(q3c_nearby_it($3,$4,$5,4))) AND (q3c_ang2ipix($1,$2)<=(q3c_nearby_it($3,$4,$5,5))))
     OR ((q3c_ang2ipix($1,$2)>=(q3c_nearby_it($3,$4,$5,6))) AND (q3c_ang2ipix($1,$2)<=(q3c_nearby_it($3,$4,$5,7))))
-    )                               
+    )
     )
     AND q3c_sindist($1,$2,$3,$4)<POW(SIN(RADIANS($5)/2),2)
 ' LANGUAGE SQL IMMUTABLE;
@@ -1131,7 +1131,7 @@ def _initPsycopg():
 
 	This will always open an admin connection.
 	"""
-# collect all DB setup in this function or use onDBInit.  XXX TODO: 
+# collect all DB setup in this function or use onDBInit.  XXX TODO:
 # in particular, the Box mess from coords (if we still want it)
 	global _PSYCOPG_INITED
 
@@ -1147,7 +1147,7 @@ def _initPsycopg():
 		# Add symmetrised q3c_joins if q3c is in use and the functions are
 		# not already defined
 		# TODO: Delete this when q3c is fixed.
-		funcs = set(r[0] for r in 
+		funcs = set(r[0] for r in
 			conn.query("SELECT DISTINCT proname FROM pg_proc"
 			" WHERE proname IN ('q3c_join', 'q3c_join_symmetric')"))
 		if funcs==frozenset(["q3c_join"]):
@@ -1161,7 +1161,7 @@ def _initPsycopg():
       	rightra real, rightdec real,
       	radius double precision) RETURNS boolean AS '"""+JOIN_FUNCTION_BODY)
 
-		GAVOConnection.extensionFunctions = frozenset(r[0] for r in 
+		GAVOConnection.extensionFunctions = frozenset(r[0] for r in
 			conn.query(
      		"SELECT proname FROM pg_proc WHERE"
      		"  proname in ('epoch_prop', 'q3c_ang2ipix',"
@@ -1252,25 +1252,38 @@ def _cleanupAfterDBError(ex, conn, pool, poolLock):
 	# in it, so we don't want to re-use it
 	try:
 		pool[0].putconn(conn, close=True)
-	except InterfaceError:  
+	except InterfaceError:
 		# Connection already closed
 		pass
 	except Exception as msg:
-		utils.sendUIEvent("Error", 
+		utils.sendUIEvent("Error",
 			"Disaster: %s while force-closing connection"%msg)
 
 
-def _makeConnectionManager(profileName, minConn=2, maxConn=20,
-		autocommitted=True):
+def _makeConnectionManager(profileName, autocommitted=True, singleton=False):
 	"""returns a context manager for a connection pool for profileName
 	connections.
+
+	With singleton=True, only one connection will be created rather than
+	[db]poolSize ones.
 	"""
 	pool = []
 	poolLock = threading.Lock()
 
 	def makePool():
+		if singleton:
+			minConn = 1
+		else:
+			minConn = config.get("db", "poolSize")
+
 		with poolLock:
-			pool.append(CustomConnectionPool(minConn, maxConn, profileName,
+			pool.append(CustomConnectionPool(
+				minConn,
+				# I don't think there's any point in maxConn at all the
+				# way psycopg pools are done right now, so all I care about
+				# here is that it won't get in our way.
+				200,
+				profileName,
 				autocommitted))
 
 	def getConnFromPool():
@@ -1309,11 +1322,11 @@ def _makeConnectionManager(profileName, minConn=2, maxConn=20,
 
 getUntrustedConn = _makeConnectionManager("untrustedquery")
 getTableConn = _makeConnectionManager("trustedquery")
-getAdminConn = _makeConnectionManager("admin")
+getAdminConn = _makeConnectionManager("admin", singleton=True)
 
-getWritableUntrustedConn = _makeConnectionManager("untrustedquery", 
-	autocommitted=False)
-getWritableTableConn = _makeConnectionManager("trustedquery", 
-	autocommitted=False)
-getWritableAdminConn = _makeConnectionManager("admin", 
-	autocommitted=False)
+getWritableUntrustedConn = _makeConnectionManager("untrustedquery",
+	autocommitted=False, singleton=True)
+getWritableTableConn = _makeConnectionManager("trustedquery",
+	autocommitted=False, singleton=True)
+getWritableAdminConn = _makeConnectionManager("admin",
+	autocommitted=False, singleton=True)
