@@ -17,11 +17,12 @@ linuga franca of our type systems is SQL+extensions as laid
 down in utils.typeconversions.
 """
 
-#c Copyright 2008-2022, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
 
+
 import re
 
 import numpy
@@ -139,11 +140,11 @@ class ToNumpyConverter(FromSQLConverter):
 		"integer": numpy.int32,
 		"bigint": numpy.int64,
 		"real": numpy.float32,
-		"boolean": numpy.bool,
+		"boolean": numpy.bool_,
 		"double precision": numpy.float64,
-		"text": numpy.str,
-		"unicode": numpy.str,
-		"char": numpy.str,
+		"text": numpy.str_,
+		"unicode": numpy.str_,
+		"char": numpy.str_,
 		"date": numpy.float32,
 		"timestamp": numpy.float64,
 		"time": numpy.float32,
@@ -159,10 +160,10 @@ class ToPythonBase(FromSQLConverter):
 	into python values.
 
 	These return the identity for most "complex" types that do not have
-	plain literals.  
+	plain literals.
 
 	What is returned here is a name of a function turning a single literal
-	into an object of the desired type; all those reside within base.literals.  
+	into an object of the desired type; all those reside within base.literals.
 
 	All such functions should be transparent to None (null value) and to
 	objects that already are of the desired type.
@@ -181,7 +182,7 @@ class ToPythonBase(FromSQLConverter):
 		"timestamp": "parseDefaultDatetime",
 		"time": "parseDefaultTime",
 		"spoint": "parseSPoint",
-		"scircle": "parseSimpleSTCS", 
+		"scircle": "parseSimpleSTCS",
 		"spoly": "parseSimpleSTCS",
 		"sbox": "identity",  # hmha, there's no STC-S for this kind of box...
 		"smoc": "identity",  # use pgsphere.SMoc.fromASCII
@@ -216,7 +217,7 @@ class ToPythonCodeConverter(ToPythonBase):
 	processing needs to be done.
 
 	The values of the map are simple string interpolation templates, with a
-	single %s for the name of the variable to be converted.  
+	single %s for the name of the variable to be converted.
 
 	The code assumes whatever executes those literals has done the equvialent
 	of gavo.base.literals import * or use gavo.base.literals.defaultParsers()
@@ -376,7 +377,7 @@ def sqltypeToPG(type):
 	'TEXT[23]'
 	>>> sqltypeToPG("char(5)[4][7]")
 	'TEXT[4][7]'
-	>>> 
+	>>>
 	"""
 	if type.upper().startswith("UNICODE"):
 		type = "TEXT"+type[7:]
