@@ -13,7 +13,7 @@ Unit tests for this are at
 http://svn.ari.uni-heidelberg.de/svn/gavo/python/trunk/tests/unitconvtest.py
 """
 
-#c Copyright 2008-2022, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -41,69 +41,69 @@ BOLTZMANN_K = 1.380649e-23
 # would be necessary
 PLAIN_UNITS = units = {
 	"a": (3600*24*365.25, "s"), # This is the SI/julian year!
-	"A": (1, "A"),				# *Ampere* not Angstrom 
+	"A": (1, "A"),				# *Ampere* not Angstrom
 	"adu": (1, "adu"),
 	"\xc5": (1e-10, "m"),
 	"Angstrom": (1e-10, "m"),
 	"angstrom": (1e-10, "m"),
 	"AU": (1.495978701e11, "m"), # as per ESA/SP-1200
-	"arcmin": (math.pi/180./60., "rad"), 
-	"arcsec": (math.pi/180./3600., "rad"), 
+	"arcmin": (math.pi/180./60., "rad"),
+	"arcsec": (math.pi/180./3600., "rad"),
 	"barn": (1, "barn"),  # 1e-28 m2
-	"beam": (1, "beam"), 
-	"bit": (1, "bit"), 
-	"bin": (1, "bin"), 
+	"beam": (1, "beam"),
+	"bit": (1, "bit"),
+	"bin": (1, "bin"),
 	"byte": (1, "byte"),  # I don't think we ever want to unify bit and byte
 	"C": (1, "C"),        # A.s
-	"cd": (1, "cd"), 
-	"ct": (1, "ct"), 
-	"count": (1, "ct"), 
-	"chan": (1, "chan"), 
+	"cd": (1, "cd"),
+	"ct": (1, "ct"),
+	"count": (1, "ct"),
+	"chan": (1, "chan"),
 	"D": (1e-19/3., "D"), # C.m
-	"d": (3600*24, "s"), 
-	"deg": (math.pi/180., "rad"), 
-	"erg": (1e-7, "J"), 
-	"eV": (1.6021766208e-19, "J"), 
+	"d": (3600*24, "s"),
+	"deg": (math.pi/180., "rad"),
+	"erg": (1e-7, "J"),
+	"eV": (1.6021766208e-19, "J"),
 	"F": (1, "F"),        # C/V
-	"g": (1e-3, "kg"), 
-	"G": (1e-4, "T"), 
-	"h": (3600., "s"), 
+	"g": (1e-3, "kg"),
+	"G": (1e-4, "T"),
+	"h": (3600., "s"),
 	"H": (1, "H"),        # Wb/A
 	"Hz": (1, "Hz"),      # s-1
 	"J": (1, "J"), 				# kg m^2/s^2
 	"Jy": (1, "Jy"),			# 1e-26 W/m2/Hz
-	"K": (1, "K"), 
-	"lm": (1, "lm"), 
-	"lx": (1, "lx"), 
-	"lyr": (2627980686828.0, "m"), 
-	"m": (1, "m"), 
+	"K": (1, "K"),
+	"lm": (1, "lm"),
+	"lx": (1, "lx"),
+	"lyr": (2627980686828.0, "m"),
+	"m": (1, "m"),
 	"mag": (1, "mag"),    # correlate that with, erm, lux?
-	"mas": (math.pi/180./3.6e6, "rad"), 
-	"min": (60, "s"), 
-	"mol": (1, "mol"), 
+	"mas": (math.pi/180./3.6e6, "rad"),
+	"min": (60, "s"),
+	"mol": (1, "mol"),
 	"N": (1, "N"),        # kg.m/s2
 	"Ohm": (1, "Ohm"),    # V/A
 	"Pa": (1, "Pa"),      # N/m^2
-	"pc": (3.0857e16, "m"), 
-	"ph": (1, "ph"), 
-	"photon": (1, "ph"), 
-	"pix": (1, "pix"), 
-	"pixel": (1, "pix"), 
+	"pc": (3.0857e16, "m"),
+	"ph": (1, "ph"),
+	"photon": (1, "ph"),
+	"pix": (1, "pix"),
+	"pixel": (1, "pix"),
 	"R": (1, "R"),        # Rayleigh
-	"rad": (1, "rad"), 
-	"Ry": (2.17989e-18, "J"), 
-	"s": (1, "s"), 
+	"rad": (1, "rad"),
+	"Ry": (2.17989e-18, "J"),
+	"s": (1, "s"),
 	"S": (1, "S"),        # A/V
 	"solLum": (3.826e26, "W"),
 	"solMass": (1.989e30, "kg"),
-	"solRad": (6.9559e8, "m"), 
-	"sr": (1, "sr"), 
+	"solRad": (6.9559e8, "m"),
+	"sr": (1, "sr"),
 	"T": (1, "T"),        # V.s/m2
 	"u": (1.66053886e-27, "kg"),
-	"V": (1, "V"), 
+	"V": (1, "V"),
 	"voxel": (1, "voxel"),
 	"W": (1, "W"),        # kg.m2/s3 or A.V -- that's going to be a tough one
-	"Wb": (1, "Wb"), 
+	"Wb": (1, "Wb"),
 	"yr": (3600*24*365.25, "s"), # This is the SI/julian year!
 }
 
@@ -123,9 +123,9 @@ NON_PREFIXABLE = frozenset([
 	"voxel"])
 
 
-PREFIXES = prefixes = {"d": 1e-1, "c": 1e-2, "m":1e-3, "u":1e-6, 
+PREFIXES = prefixes = {"d": 1e-1, "c": 1e-2, "m":1e-3, "u":1e-6,
 	"n":1e-9, "p":1e-12, "f":1e-15, "a":1e-18, "z":1e-21, "y":1e-24,
-	"da": 10., "h":1e2, "k":1e3, "M":1e6, "G":1e9, "T":1e12, "P":1e15, 
+	"da": 10., "h":1e2, "k":1e3, "M":1e6, "G":1e9, "T":1e12, "P":1e15,
 	"E":1e18, "Z":1e21, "Y":1e24, "": 1}
 
 
@@ -162,7 +162,7 @@ class _Node(object):
 	All these must implement fromToks methods that can be called
 	as pyparsing callbacks.
 	"""
-	@classmethod 
+	@classmethod
 	def fromToks(cls, s, p, toks):
 		raise NotImplementedError("%s objects have no implementation of"
 			" fromToks (i.e., they are broken)."%cls.__name__)
@@ -176,10 +176,14 @@ class UnitNode(_Node):
 	Check out the unit, prefix, and isUnknown attributes.  The prefix
 	can be turned to a factor using the PREFIXES dictionary.  An empty
 	prefix (factor 1) is represented by an empty string.
+
+	There is also isValid; that's isUnknown except if an unknown unit is
+	quoted.
 	"""
 	def __init__(self, unit, prefix=""):
 		self.unit = unit
 		self.isUnknown = self.unit not in PLAIN_UNITS
+		self.isValid = not self.isUnknown
 		self.prefix = prefix
 
 	@classmethod
@@ -226,7 +230,7 @@ class UnitNode(_Node):
 		if self.isUnknown:
 			# these don't hurt if they're on both sides of an equation
 			return PREFIXES[self.prefix], {self.unit: 1}
-			return 
+
 		factor, basic = PLAIN_UNITS[self.unit]
 		return PREFIXES[self.prefix]*factor, {basic: 1}
 
@@ -246,6 +250,8 @@ class FunctionApplication(_Node):
 		self.term = term
 		self.isUnknown = (self.funcName not in self._pythonFunc
 			) or self.term.isUnknown
+		self.isValid = (self.funcName in self._pythonFunc
+			) and self.term.isValid
 
 	@classmethod
 	def fromToks(cls, s, p, toks):
@@ -270,10 +276,10 @@ class FunctionApplication(_Node):
 	def getSI(self):
 		factor, powers = self.term.getSI()
 		if self.funcName=="sqrt":
-			powers = dict((key, value/2.) 
+			powers = dict((key, value/2.)
 				for key, value in powers.items())
 		else:
-			powers = dict(((self.funcName, key), value) 
+			powers = dict(((self.funcName, key), value)
 				for key, value in powers.items())
 		return self._pythonFunc[self.funcName](factor), powers
 
@@ -284,6 +290,7 @@ class QuotedUnitNode(_Node):
 	def __init__(self, unit):
 		self.unit = unit
 		self.isUnknown = True
+		self.isValid = True
 
 	@classmethod
 	def fromToks(cls, s, p, toks):
@@ -307,6 +314,7 @@ class Factor(_Node):
 	def __init__(self, unit, power):
 		self.unit, self.power = unit, power
 		self.isUnknown = self.unit.isUnknown
+		self.isValid = self.unit.isValid
 	
 	@classmethod
 	def fromToks(cls, s, p, toks):
@@ -367,6 +375,7 @@ class Term(_Node):
 		assert operator in {'.', '/'}
 		self.op1, self.operator, self.op2 = op1, operator, op2
 		self.isUnknown = self.op1.isUnknown or self.op2.isUnknown
+		self.isValid = self.op1.isValid and self.op2.isValid
 	
 	@classmethod
 	def fromToks(cls, s, p, toks):
@@ -395,7 +404,7 @@ class Term(_Node):
 		return "%s%s%s"%(op1Lit, self.operator, op2Lit)
 
 	def __repr__(self):
-		return "T(%s ; %s ; %s)"%(repr(self.op1), 
+		return "T(%s ; %s ; %s)"%(repr(self.op1),
 			repr(self.operator), repr(self.op2))
 
 	def getFactorWithUnit(self, unit):
@@ -431,13 +440,14 @@ class Term(_Node):
 
 
 class Expression(_Node):
-	"""The root node of an expression tree. 
+	"""The root node of an expression tree.
 	
 	This contains a term and optionally a scale factor.
 	"""
 	def __init__(self, term, scaleFactor):
 		self.term, self.scaleFactor = term, scaleFactor
 		self.isUnknown = self.term.isUnknown
+		self.isValid = self.term.isValid
 
 	@classmethod
 	def fromToks(cls, s, p, toks):
@@ -468,7 +478,7 @@ class Expression(_Node):
 
 
 def _buildTerm(s, pos, toks):
-	"""a parseAction for terms, making trees out of parse lists 
+	"""a parseAction for terms, making trees out of parse lists
 	left-associatively.
 
 	This will also normalise products of the same units to powers.
@@ -497,7 +507,7 @@ class getUnitGrammar(utils.CachedResource):
 	"""
 	@classmethod
 	def impl(cls):
-		from gavo.utils.parsetricks import (Word, Literal, Regex, 
+		from gavo.utils.parsetricks import (Word, Literal, Regex,
 			Optional, ZeroOrMore, alphas,
 			Suppress, Forward, pyparsingWhitechars)
 
@@ -522,11 +532,11 @@ class getUnitGrammar(utils.CachedResource):
 			power_operator = Literal('**')
 			multiplication_operator = Literal(".")
 			division_operator = Literal("/")
-			numeric_power = (integer 
-				| OPEN_P + integer + CLOSE_P 
-				| OPEN_P + FLOAT + CLOSE_P 
+			numeric_power = (integer
+				| OPEN_P + integer + CLOSE_P
+				| OPEN_P + FLOAT + CLOSE_P
 				| OPEN_P + integer + '/'
-					+ UNSIGNED_INTEGER.addParseAction(lambda s, p, t: t[0]+".") 
+					+ UNSIGNED_INTEGER.addParseAction(lambda s, p, t: t[0]+".")
 					+ CLOSE_P)
 			numeric_power.setParseAction(evalAll)
 
@@ -535,8 +545,8 @@ class getUnitGrammar(utils.CachedResource):
 				).setName("scale_factor"
 				).setParseAction(evalAll)
 
-			any_unit_atom = unit_atom | quoted_unit_atom 
-			factor = (any_unit_atom 
+			any_unit_atom = unit_atom | quoted_unit_atom
+			factor = (any_unit_atom
 				+ Optional( Suppress(power_operator) + numeric_power )
 				).setName("factor"
 				).addParseAction(Factor.fromToks)
@@ -549,10 +559,10 @@ class getUnitGrammar(utils.CachedResource):
 
 			unit_expression = (
 				Suppress(OPEN_P) + complete_expression + Suppress(CLOSE_P)
-				| ( factor 
+				| ( factor
 					^ function_application )).setName("unit expression")
 
-			product_of_units = (unit_expression 
+			product_of_units = (unit_expression
 					+ ZeroOrMore(multiplication_operator + unit_expression)
 				).setName("units term"
 				).setParseAction(_buildTerm)
@@ -561,7 +571,7 @@ class getUnitGrammar(utils.CachedResource):
 				product_of_units + Optional(division_operator + unit_expression) )
 			complete_expression.setParseAction(Term.fromToks)
 
-			input = (Optional(scale_factor) 
+			input = (Optional(scale_factor)
 				+ complete_expression
 				).setName("unit expression"
 				).setParseAction(Expression.fromToks)
@@ -667,59 +677,6 @@ def computeColumnConversions(newColumns, oldColumns):
 	return res
 
 
-#################### special handling for spectral unit handling
-
-def toStdSpecUnit(fromUnit):
-	"""returns a pair of (standard unit, factor) to turn fromUnit into
-	one of J, Hz, or m.
-
-	Unconvertable units raise a UfuncError.
-	"""
-	for stdUnit in ["J", "Hz", "m"]:
-		try:
-			 factor = computeConversionFactor(fromUnit, stdUnit)
-		except IncompatibleUnits:
-			continue
-		return stdUnit, factor
-	
-	raise IncompatibleUnits(
-		f"{fromUnit} is not a spectral unit understood here")
-
-
-SPEC_CONVERSION = {
-	("J", "m"): "h*c/(({expr})*{f})",
-	("J", "Hz"): "({expr})*{f}/h",
-	("J", "J"): "({expr})*{f}",
-	("Hz", "m"): "c/({expr})/{f}",
-	("Hz", "Hz"): "{f}*({expr})",
-	("Hz", "J"): "h*{f}*({expr})",
-	("m", "m"): "{f}*({expr})",
-	("m", "Hz"): "c/({expr})/{f}",
-	("m", "J"): "h*c/({expr})/{f}",}
-
-
-def getSpecExpr(fromUnit, toUnit):
-	"""returns an expression that turns a value in fromUnit to one in toUnit.
-
-	The value in fromUnit is represented by a {} so you can just .format
-	the expression it is in into the resulting string.
-	"""
-	fromStd, fromFactor = toStdSpecUnit(fromUnit)
-	toStd, toFactor = toStdSpecUnit(toUnit)
-	convExpr = SPEC_CONVERSION[fromStd, toStd
-			].replace("h", str(PLANCK_H)
-			).replace("c", str(LIGHT_C))
-	return "{}/{}".format(
-		convExpr.format(f=fromFactor, expr="{}"),
-		toFactor)
-
-
-def getSpecConverter(fromUnit, toUnit):
-	"""Returns a function converting spectral values in fromUnit to toUnit.
-	"""
-	return eval("lambda x: "+getSpecExpr(fromUnit, toUnit).format("x"))
-
-
 if __name__=="__main__":
 	getUnitGrammar.enableDebuggingOutput()
 	g = getUnitGrammar()
