@@ -5,14 +5,14 @@ twisted templates and rend.Pages.
 See porting guide in the README.
 """
 
-#c Copyright 2008-2022, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
 
 
 import io
-import itertools 
+import itertools
 import sys
 
 from xml.sax import make_parser, handler
@@ -32,7 +32,7 @@ NAMESPACE_MAPPING = {NEVOW_NS: TWISTED_NS}
 
 # ELEMENT_MAPPING is only consulted if the namespace is in
 # NAMESPACE_MAPPING, so we can be a bit cavalier here.
-ELEMENT_MAPPING = { 
+ELEMENT_MAPPING = {
     "invisible": "transparent",
 }
 
@@ -54,7 +54,7 @@ class MyToStan(_ToStan):
         self.prefixMap = _NSContext()
         self.prefixMap[NEVOW_NS] = "nevow"
         self.prefixMap[TWISTED_NS] = "nevow"
-    
+
     def startPrefixMapping(self, prefix, uri):
         # overridden to swallow attempts to map prefixes for our
         # standard namespaces
@@ -79,7 +79,7 @@ class MyToStan(_ToStan):
         if ns in NAMESPACE_MAPPING:
             ns = NAMESPACE_MAPPING[ns]
             name = ELEMENT_MAPPING.get(name, name)
-        
+
         # twisted web blindly discards attributes when constructing
         # n:attr; we need n:data, at least, so I need to copy and fix
         # the code
@@ -87,7 +87,7 @@ class MyToStan(_ToStan):
             if not self.stack or (None, 'name') not in attrs:
                 raise AssertionError(
                     '<attr> usage error')
-            
+
             fixedAttrs = {}
             for (ns, attName), val in list(attrs.items()):
                 if ns:
@@ -96,7 +96,7 @@ class MyToStan(_ToStan):
                 else:
                     fixedAttrs[attName] = val
 
-            el = Tag('', 
+            el = Tag('',
                 render=attrs.get((TWISTED_NS, 'render')),
                 attributes=fixedAttrs,
                 filename=self.sourceFilename,
@@ -184,11 +184,11 @@ class Passthrough(Raw):
 
     def getContent(self):
         return self.content
-    
+
 template.tags.xml = Passthrough
 
 
-def iterChildren(tag, 
+def iterChildren(tag,
         stopRenders=frozenset(["sequence", "mapping"]),
         stopAtData=False):
     """yields the Tag-typed descendents of tag preorder.
@@ -206,14 +206,14 @@ def iterChildren(tag,
 
             yield t
 
-            if (t.render is None or t.render not in stopRenders 
+            if (t.render is None or t.render not in stopRenders
                     ) and not (stopAtData and "nevow:data" in t.attributes):
                 for c in iterChildren(t, stopRenders, stopAtData):
                     yield c
 
 
-def locatePatterns(tag, 
-        searchPatterns=[], 
+def locatePatterns(tag,
+        searchPatterns=[],
         stopRenders=["sequence", "mapping"]):
     """returns all descendents of tags for which nevow:pattern is in
     searchPatterns.
@@ -258,16 +258,16 @@ class CommonRenderers(object):
     def sequence(self, request, tag):
         toProcess = tag.slotData
 
-        patterns = locatePatterns(tag, 
-            ["item", "empty", "separator", "header", "footer"], 
+        patterns = locatePatterns(tag,
+            ["item", "empty", "separator", "header", "footer"],
             ["sequence"])
         if not patterns["item"]:
             patterns["item"] = [template.tags.transparent]
         tagIterator = iter(patterns["item"])
 
-        newTag = tag.clone(True).clear() 
+        newTag = tag.clone(True).clear()
         newTag.render = None
-   
+
         if patterns["header"]:
             newTag[patterns["header"]]
 
@@ -279,7 +279,7 @@ class CommonRenderers(object):
                 tagIterator = iter(patterns["item"])
                 nextTag = next(tagIterator)
             newTag(nextTag.clone(True, item))
-        
+
         if newTag.children or not patterns["empty"]:
             if patterns["footer"]:
                 newTag[patterns["footer"]]
@@ -325,16 +325,16 @@ class CommonRenderers(object):
             parts = name.split()
             method = template.renderer.get(self, parts[0], None)
             if method is None:
-                raise Exception("Missing render method: (%s %s)"%(
-                    self, name))
+                raise Exception("Missing render method on %s: %s"%(
+                    repr(self), name))
             method = method(" ".join(parts[1:]))
 
         else:
             method = template.renderer.get(self, name, None)
-        
+
         if method is None:
-            raise Exception("Missing render method: (%s %s)"%(
-                self, name))
+            raise Exception("Missing render method on %s: %s"%(
+                repr(self), name))
 
         return method
 
@@ -364,7 +364,7 @@ class CommonRenderers(object):
         if " " in name:
             parts = name.split()
             return getattr(self, "data_"+parts[0])(" ".join(parts[1:]))
-        
+
         return getattr(self, "data_"+name)
 
 
@@ -387,10 +387,10 @@ def elementFromTag(tag, rendererFactory):
     class _Root(rendererFactory, template.Element):
         def __init__(self, child):
             self.child = child
-        
+
         def render(self, request):
             return self.child
-    
+
     return _Root(tag)
 
 
@@ -408,7 +408,7 @@ class TemplatedPage(resource.Resource, CommonRenderers):
     This will look for a ``gavo_useDoctype`` attribute on the
     template and use an XHTML doctype if it's not present.
 
-    You can override ``handleError(failure, request)`` to override the 
+    You can override ``handleError(failure, request)`` to override the
     error document (this will finish the request, too).
 
     If you need extra callbacks, define your own render method and use
@@ -432,7 +432,7 @@ class TemplatedPage(resource.Resource, CommonRenderers):
                 if name.startswith("__"):
                     raise AttributeError(name)
                 return getattr(self, name)
-           
+
             # we need to redirect lookupRenderMethod so Element
             # can deal with render="foo arg".
             def lookupRenderMethod(el_self, *args):
@@ -457,8 +457,8 @@ class TemplatedPage(resource.Resource, CommonRenderers):
 
     def renderDocument(self, request):
         """returns a deferred firing when the document is written.
-        
-        Use this method rather than render if you need to introduce 
+
+        Use this method rather than render if you need to introduce
         extra callbacks.
 
         This does *not* finish the request by itself.
@@ -496,7 +496,7 @@ class Redirect(resource.Resource):
     def __init__(self, destURL, code=302):
         self.destURL = str(destURL)
         self.code = 302
-    
+
     def render(self, request):
         # todo: see if destURL is relative and fix based on request?
     		request.setHeader("location", self.destURL)
@@ -523,7 +523,7 @@ def flattenSync(element, request=None):
             " containing deferreds.")
     elif hasattr(result, "raiseException"):
         result.raiseException()
-    
+
     return result
 
 
