@@ -176,9 +176,9 @@ class TextArea(object):
             self.rows = rows
 
     def _renderTag(self, request, key, value, readonly):
-        tag=T.textarea(name=key, 
-            id=render_cssid(key), 
-            cols=str(self.cols), 
+        tag=T.textarea(name=key,
+            id=render_cssid(key),
+            cols=str(self.cols),
             rows=str(self.rows))[str(value or '')]
         if readonly:
             tag(class_='readonly', readonly='readonly')
@@ -222,9 +222,9 @@ class TextAreaList(object):
 
     def _renderTag(self, request, key, values, readonly):
         value = '\n'.join(values)
-        tag=T.textarea(name=key, 
-            id=render_cssid(key), 
-            cols=str(self.cols), 
+        tag=T.textarea(name=key,
+            id=render_cssid(key),
+            cols=str(self.cols),
             rows=str(self.rows))[str(value or '')]
         if readonly:
             tag(class_='readonly', readonly='readonly')
@@ -282,7 +282,7 @@ class CheckedPassword(object):
         else:
             values = ('', '')
         return [
-            T.input(type='password', name=key, id=render_cssid(key), 
+            T.input(type='password', name=key, id=render_cssid(key),
                 value=str(values[0])),
             T.br,
             T.label(for_=render_cssid(key, 'confirm'))[' Confirm '],
@@ -292,12 +292,12 @@ class CheckedPassword(object):
     def renderImmutable(self, request, key, args, errors):
         values = ('', '')
         return [
-            T.input(type='password', name=key, id=render_cssid(key), 
+            T.input(type='password', name=key, id=render_cssid(key),
                 value=str(values[0]), class_='readonly', readonly='readonly'),
             T.br,
             T.label(for_=render_cssid(key, 'confirm'))[' Confirm '],
             T.input(type='password', name=key, id=render_cssid(key, 'confirm'),
-                    value=str(values[1]), class_='readonly', 
+                    value=str(values[1]), class_='readonly',
                     readonly='readonly')
         ]
 
@@ -573,14 +573,14 @@ class DatePartsSelect(object):
     The default entry format is the US (month, day, year) but can be switched to
     the more common (day, month, year) by setting the dayFirst attribute to
     True.
-    
+
     The start and end year can be passed through but default to 1970 and 2070.
-    
+
     The months default to non-zero prefixed numerics but can be passed as a list
     of label, value pairs
 
     default can be whitespace-separated parts here.
-    """    
+    """
 
     dayFirst = False
     days = [ (d,d) for d in range(1,32) ]
@@ -625,7 +625,7 @@ class DatePartsSelect(object):
                 options.append( T.option(value=str(value)[0])[
                     iformal.ILabel(value[1]).label()] )
         yearTag = T.select(name=namer('year'))[ options ]
-        
+
         options = []
         if self.noneOption is not None:
             options.append( T.option(value=str(self.noneOption[0]))[
@@ -639,7 +639,7 @@ class DatePartsSelect(object):
                 options.append( T.option(value=str(value[0]))[
                     iformal.ILabel(value[1]).label()] )
         monthTag = T.select(name=namer('month'))[ options ]
-        
+
         options = []
         if self.noneOption is not None:
             options.append( T.option(value=str(self.noneOption[0]))[
@@ -653,7 +653,7 @@ class DatePartsSelect(object):
                 options.append( T.option(value=str(value[0]))[
                     iformal.ILabel(value[1]).label()] )
         dayTag = T.select(name=namer('day'))[ options ]
-        
+
         if readonly:
             tags = (yearTag, monthTag, dayTag)
             for tag in tags:
@@ -685,7 +685,7 @@ class DatePartsSelect(object):
     def processInput(self, request, key, args, default=None):
         namer = self._namer(key)
         # Get the form field values as a (y,m,d) tuple
-        ymd = [getStringFromArgs(args, namer(part)).strip() 
+        ymd = [getStringFromArgs(args, namer(part)).strip()
             for part in ('year', 'month', 'day')]
         # Remove parts that were not entered.
         ymd = [p for p in ymd if p]
@@ -703,8 +703,8 @@ class DatePartsSelect(object):
                 raise validation.FieldValidationError("Invalid date")
         ymd = iformal.IDateTupleConvertible(self.original).toType(ymd)
         return self.original.validate(ymd)
-    
-    
+
+
 @implementer( iformal.IWidget )
 class DatePartsInput(object):
     """
@@ -714,16 +714,16 @@ class DatePartsInput(object):
     The default entry format is the US (month, day, year) but can be switched to
     the more common (day, month, year) by setting the dayFirst attribute to
     True.
-    
+
     By default the widget is designed to only accept unambiguous years, i.e.
     the user must enter 4 character dates.
-    
+
     Many people find it convenient or even necessary to allow a 2 character
     year. This can be allowed by setting the twoCharCutoffYear attribute to an
     integer value between 0 and 99. Anything greater than or equal to the cutoff
     year will be considered part of the 20th century (1900 + year); anything
     less the cutoff year is a 21st century (2000 + year) date.
-    
+
     A typical twoCharCutoffYear value is 70 (i.e. 1970). However, that value is
     somewhat arbitrary. It's the year that time began according to the PC, but
     it doesn't mean much to your non-techie user.
@@ -753,11 +753,11 @@ class DatePartsInput(object):
         return _
 
     def _renderTag(self, request, year, month, day, namer, readonly):
-        yearTag = T.input(type="text", name=namer('year'), 
+        yearTag = T.input(type="text", name=namer('year'),
             value=str(year or ""), size="4")
-        monthTag = T.input(type="text", name=namer('month'), 
+        monthTag = T.input(type="text", name=namer('month'),
             value=str(month or ""), size="2")
-        dayTag = T.input(type="text", name=namer('day'), 
+        dayTag = T.input(type="text", name=namer('day'),
             value=str(day or ""), size="2")
         if readonly:
             tags = (yearTag, monthTag, dayTag)
@@ -790,7 +790,7 @@ class DatePartsInput(object):
     def processInput(self, request, key, args, default=None):
         namer = self._namer(key)
         # Get the form field values as a (y,m,d) tuple
-        ymd = [getStringFromArgs(args, namer(part)).strip() 
+        ymd = [getStringFromArgs(args, namer(part)).strip()
             for part in ('year', 'month', 'day')]
         # Remove parts that were not entered.
         ymd = [p for p in ymd if p]
@@ -848,9 +848,9 @@ class MMYYDatePartsInput(object):
         return _
 
     def _renderTag(self, request, year, month, namer, readonly):
-        yearTag = T.input(type="text", name=namer('year'), 
+        yearTag = T.input(type="text", name=namer('year'),
             value=str(year), size="2")
-        monthTag = T.input(type="text", name=namer('month'), 
+        monthTag = T.input(type="text", name=namer('month'),
             value=str(month), size="2")
         if readonly:
             tags=(yearTag, monthTag)
@@ -885,7 +885,7 @@ class MMYYDatePartsInput(object):
 
     def processInput(self, request, key, args, default=None):
         namer = self._namer(key)
-        value = [getStringFromArgs(args, namer(part)).strip() 
+        value = [getStringFromArgs(args, namer(part)).strip()
             for part in ('year', 'month')]
         value = [p for p in value if p]
         if not value:
@@ -913,7 +913,7 @@ class CheckboxMultiChoice(template.Element):
     """
     Multiple choice list, rendered as a list of checkbox fields.
 
-    lots enters blanks between the items (which allows a bit more 
+    lots enters blanks between the items (which allows a bit more
     flexibility in styling the stuff).
 
     emphasized can be a set of item titles that should additionally
@@ -924,7 +924,7 @@ class CheckboxMultiChoice(template.Element):
 
     options = None
 
-    def __init__(self, original, options=None, lots=False, 
+    def __init__(self, original, options=None, lots=False,
             emphasized=frozenset()):
         self.original = original
         self.lots = lots
@@ -942,8 +942,10 @@ class CheckboxMultiChoice(template.Element):
                optLabel = iformal.ILabel(item).label()
                optValue = converter.fromType(optValue)
                optid = render_cssid(key, n)
-               checkbox = T.input(type='checkbox', name=key, 
-                    value=str(optValue),
+               checkbox = T.input(type='checkbox', name=key,
+                    # ouch: at least some browsers don't quote the + in
+                    # value attributes properly.  Hack around it:
+                    value=str(optValue).replace("+", "%2b"),
                     id=optid+["-box"], class_="multichoice",
                     onchange="Forms.Util.updateLabel(this)")
                if disabled:
@@ -1003,7 +1005,9 @@ class CheckboxMultiChoice(template.Element):
 
 @implementer( iformal.IWidget )
 class FileUpload(object):
-
+    """NOTE: You have use something like twistedpatch.MPRequest as
+    your site's requestFactory to make this work.
+    """
     def __init__(self, original):
         self.original = original
 
@@ -1021,20 +1025,15 @@ class FileUpload(object):
         return self._renderTag(request, key, True)
 
     def processInput(self, request, key, args, default=None):
-        # default is ignored here.
+       # default is ignored here.
 
-        if request.fields is None or key not in request.fields:
+        if key not in request.files:
             return self.original.validate(None)
-        fileitem = request.fields[key]
-        if not fileitem.file.getvalue():
-            # empty input; this happens in modern cgi for when no
-            # file has been passed in
-            return self.original.validate(None)
-
-        name = fileitem.filename
+        fileitem = request.files[key][0]
+        name = fileitem.file_name
         if isinstance(name, bytes):
             name = name.decode("utf-8", "replace")
-        value = (name, fileitem.file)
+        value = (name, fileitem.file_object)
 
         value = iformal.IFileConvertible(self.original).fromType(value)
         return self.original.validate(value)
@@ -1060,7 +1059,7 @@ class Hidden(object):
         if value is None:
             value = ""
 
-        return T.input(type=self.inputType, name=key, id=render_cssid(key), 
+        return T.input(type=self.inputType, name=key, id=render_cssid(key),
             value=str(value))
 
     def renderImmutable(self, request, key, args, errors):
@@ -1073,7 +1072,7 @@ class Hidden(object):
 
 
 __all__ = [
-    'Checkbox', 'CheckboxMultiChoice', 'CheckedPassword', 
+    'Checkbox', 'CheckboxMultiChoice', 'CheckedPassword',
     'Password', 'SelectChoice', 'TextArea', 'TextInput', 'DatePartsInput',
     'DatePartsSelect', 'MMYYDatePartsInput', 'Hidden', 'RadioChoice',
     'SelectOtherChoice', 'FileUpload', 'TextAreaList',
