@@ -3,13 +3,13 @@ Generating VOTables from internal data representations.
 
 This is glue code to the more generic GAVO votable library.  In particular,
 it governs the application of base.SerManagers and their column descriptions
-(which are what is passed around as colDescs in this module to come up with 
+(which are what is passed around as colDescs in this module to come up with
 VOTable FIELDs and the corresponding values.
 
 You should access this module through formats.votable.
 """
 
-#c Copyright 2008-2022, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -72,7 +72,7 @@ class VOTableContext(utils.IdManagerMixin):
 	If VO-DML processing is enabled, the context also manages models declared;
 	that's the modelsUsed dictionary, mapping prefix -> dm.Model instances
 	"""
-	def __init__(self, mfRegistry=valuemappers.defaultMFRegistry, 
+	def __init__(self, mfRegistry=valuemappers.defaultMFRegistry,
 			tablecoding='binary', version=None, acquireSamples=True,
 			suppressNamespace=False, overflowElement=None):
 		self.mfRegistry = mfRegistry
@@ -92,16 +92,23 @@ class VOTableContext(utils.IdManagerMixin):
 		# end up in the tree so they get only included once when
 		# referenced by multiple other annotations.
 		self.groupIdsInTree = set()
-		self.modelsUsed = {}
+		self.modelsDeclared = set()
 		self.rootVODML = V.VODML()
 		self.vodmlTemplates = V.TEMPLATES()
 		self.rootVODML[self.vodmlTemplates]
 
+		# While we're producing both note-style STC1 utypes and ad-hoc
+		# COOSYS refs from <stc>, make them share COOSYS elements based
+		# on the id of the underlying STC ast.  Remove this when we've
+		# dumped STC1 utypes.
+		self.coosysByAST = {}
+
 	def addVODMLPrefix(self, prefix):
 		"""arranges the DM with prefix to be included in modelsUsed.
 		"""
-		if prefix not in self.modelsUsed:
-			self.modelsUsed[prefix] = dm.getModelForPrefix(prefix)
+		if prefix not in self.modelsDeclared:
+			self.rootVODML[dm.getModelForPrefix(prefix).getVOT(self, None)]
+			self.modelsDeclared.add(prefix)
 
 	def addVODMLMaterial(self, stuff):
 		"""adds VODML annotation to this VOTable.
@@ -157,7 +164,7 @@ class VOTableContext(utils.IdManagerMixin):
 	def currentTable(self):
 		"""the DaCHS table object from which things are currently built.
 
-		If no builder has declared a table being built (using buildingFromTable), 
+		If no builder has declared a table being built (using buildingFromTable),
 		it's a value error.
 		"""
 		if not self._tableStack:
@@ -227,7 +234,7 @@ class VOTableContext(utils.IdManagerMixin):
 		"""
 		try:
 			votEl.ID = self.getIdFor(rdEl)
-		except base.NotFoundError: 
+		except base.NotFoundError:
 			# the param is not referenced and thus needs no ID
 			pass
 
@@ -275,7 +282,7 @@ def _iterDatalinkResources(ctx, dataSet):
 				# can't do it.
 				from gavo.protocols import datalink
 				yield datalink.makeDatalinkServiceDescriptor(
-					ctx, service, table.tableDef, 
+					ctx, service, table.tableDef,
 					base.getMetaText(svcMeta, "idColumn"))
 
 				# Temporary hack: for now, if something wants to have immediate
@@ -331,7 +338,7 @@ def _iterUniqueInfosFromMeta(
 def _iterResourceMeta(ctx, dataSet):
 	"""adds resource metadata to the Resource parent.
 	"""
-	yield V.DESCRIPTION[base.getMetaText(dataSet, "description", 
+	yield V.DESCRIPTION[base.getMetaText(dataSet, "description",
 		macroPackage=dataSet.dd.rd, propagate=False)]
 	for el in  itertools.chain(
 			_iterInfoInfos(dataSet), _iterWarningInfos(dataSet)):
@@ -342,7 +349,7 @@ def _iterResourceMeta(ctx, dataSet):
 	yield from _iterUniqueInfosFromMeta(ctx, sources,
 		"source",
 		"citation",
-		lambda value: "meta.bib.bibcode" 
+		lambda value: "meta.bib.bibcode"
 			if utils.couldBeABibcode(value)
 			else "meta.bib",
 		"This resource contains data associated with the publication {value}")
@@ -472,7 +479,7 @@ def defineField(ctx, element, colDesc):
 				content_type=rscCol.getProperty("targetType"),
 				title=rscCol.getProperty("targetTitle", "Link"))]
 
-	element(**dict((key, colDesc.get(key) or None) 
+	element(**dict((key, colDesc.get(key) or None)
 			for key in _voFieldCopyKeys))[
 		V.DESCRIPTION[colDesc["description"]],
 		_makeValuesForColDesc(colDesc),
@@ -513,6 +520,19 @@ def _makeVOTParam(ctx, param):
 
 	el = V.PARAM()
 	defineField(ctx, el, valuemappers.AnnotatedColumn(param))
+
+	# id management is a particular pain for params at this moment,
+	# because DM annotation uses the tableDef's params, whereas
+	# we see the copied params here.  We should stop copying
+	# the params in rsc.common.ParamMixin.
+	try:
+		el.ID = ctx.getIdFor(param.originalParam)
+	except (base.NotFoundError, AttributeError):
+		try:
+			el.ID = ctx.getIdFor(param)
+		except base.NotFoundError:
+			pass
+
 	if content is None:
 		el.value = ""
 	else:
@@ -540,7 +560,8 @@ STC_FRAMES_TO_COOSYS = {
 	'FK4': 'eq_FK4',
 	'ECLIPTIC': 'ecl_FK5',  # neglecting the odds there's actually ecl_FK4 data
 	'GALACTIC_II': 'galactic',
-	'SUPERGALACTIC': 'supergalactic'}
+	'SUPERGALACTIC': 'supergalactic',
+	None: None}
 
 # these are utypes for which column refs should be ref-ed to the COOSYS.
 COLUMN_REF_UTYPES = [
@@ -607,6 +628,10 @@ def _makeCOOSYSFromSTC1(utypeMap, serManager):
 		"stc:astrocoordsystem.spaceframe.coordrefframe", None)
 	coosys(system=STC_FRAMES_TO_COOSYS.get(stcFrame, stcFrame))
 
+	if "references-from-coosys" in base.getConfig("future"):
+		coosys(refposition=utypeMap.get(
+			"stc:astrocoordsystem.spaceframe.referenceposition", None))
+
 	for utype in COLUMN_REF_UTYPES:
 		if utype in utypeMap:
 			val = utypeMap[utype]
@@ -619,7 +644,7 @@ def _makeCOOSYSFromSTC1(utypeMap, serManager):
 
 
 def _iterSTC(ctx, tableDef, serManager):
-	"""adds STC groups for the systems to votTable fetching data from 
+	"""adds STC groups for the systems to votTable fetching data from
 	tableDef.
 	"""
 	def getColumnFor(colRef):
@@ -628,7 +653,7 @@ def _iterSTC(ctx, tableDef, serManager):
 		except KeyError:
 			# in ADQL processing, names are lower-cased, and there's not
 			# terribly much we can do about it without breaking other things.
-			# Hence, let's try and see whether our target is there with 
+			# Hence, let's try and see whether our target is there with
 			# case normalization:
 			return serManager.getColumnByName(colRef.dest.lower())
 	
@@ -641,8 +666,9 @@ def _iterSTC(ctx, tableDef, serManager):
 			# "Note-style" STC only supported in 1.2 and higher
 			yield container
 		# legacy COOSYS specification supported everywhere
-		ctx.getEnclosingResource()[
-			_makeCOOSYSFromSTC1(utypeMap, serManager)]
+		coosysEl = _makeCOOSYSFromSTC1(utypeMap, serManager)
+		ctx.coosysByAST[str(id(ast))] = coosysEl
+		ctx.getEnclosingResource()[coosysEl]
 
 
 def _addStupidRefByAnnotation(ctx, annotation, serManager, destId):
@@ -663,45 +689,128 @@ def _addStupidRefByAnnotation(ctx, annotation, serManager, destId):
 			destId)
 
 
-def _addLegacySYSFromSTC2(ctx, tableDef, serManager):
-	"""adds COOSYS and TIMESYS elements from stc2 annotation to the enclosing
-	RESOURCE.
+# mapping from our votable-stc role names to COOSYS utypes
+_STC_UTYPE_MAPPING = {
+	"longitude": "votable:LonLatPoint-lon",
+	"latitude": "votable:LonLatPoint-lat",
+	"pm_longitude": "votable:ProperMotion-lon",
+	"pm_latitude": "votable:ProperMotion-lat",
+	"distance": "votable:LonLatPoint-dist",
+	"rv": "votable:ProperMotion-rv",
+}
+
+
+if "references-from-coosys" in base.getConfig("future"):
+	def _makeLegacyRefForAnn(ctx, annotation, utype):
+		"""returns a PARAMref or FIELDref for annotation with the utype
+		passed.
+
+		The target of annotation will receive an id in ctx.  If the
+		"""
+		valId = ctx.getOrMakeIdFor(
+			annotation.value, suggestion=annotation.value.name)
+
+		if isinstance(annotation, dm.ColumnAnnotation):
+			return V.FIELDref(utype=utype, ref=valId)
+		elif isinstance(annotation, dm.ParamAnnotation):
+			return V.PARAMref(utype=utype, ref=valId)
+		else:
+			base.ui.notifyWarning(f"Unknown annotation {annotation}")
+			# fall through to returning None, which the serialiser will swallow
+else:
+	def _makeLegacyRefForAnn(ctx, annotation, utype):
+		return None
+
+
+def _guessTimeOrigin(frame, timeCol):
+	"""returns a plausible value for a TIMESYS' timeorigin attribute
+	from a frame definition and a time column.
+
+	This is a depressing mess; thanks, DM WG.
 	"""
-	for ann in tableDef.iterAnnotationsOfType("stc2:Coords"):
+	if "time0" in frame:
+		# that's the only sane option
+		return frame.get("time0")
+
+	if timeCol:
+		timeCol = timeCol.weakref()
+	if not timeCol:
+		# A timesys without location is rather bogus anyway.
+		return None
+
+	# let's guess time origin based on units and such.
+	if timeCol.unit=="d":
+		# assuming JD or MJD.  Since people didn't give a time0, try to guess.
+		if "mjd" in timeCol.name:
+			return "MJD-origin"
+		# JD is a silly default, but what can I do? SIGH!
+		return "JD-origin"
+
+	if timeCol.unit=="s":
+		# if that's really not a unix timestamp, people need to
+		# give a time0.
+		return "2440587.5"
+	
+	# we ought to assert unit="yr" here
+	return None
+
+
+def _addLegacySYSFromVOTableSTC(ctx, tableDef, serManager):
+	"""adds COOSYS and TIMESYS elements from votable-stc annotation to
+	the enclosing RESOURCE.
+	"""
+	for ann in tableDef.iterAnnotationsOfType("votable:Coords"):
 		if "time" in ann:
 			timeFrame = ann["time"].get("frame", {})
+	
 			timesysEl = V.TIMESYS(
-				timescale=timeFrame.get("timescale"),
-				refposition=timeFrame.get("refPosition"),
-				timeorigin=timeFrame.get("time0"))
+				timescale=timeFrame.get("timescale", "UNKNOWN"),
+				refposition=timeFrame.get("refPosition", "UNKNOWN"),
+				timeorigin=_guessTimeOrigin(timeFrame, ann["time"].get("location")))
 			timesysId = serManager.makeIdFor(timesysEl, "ts")
 			timesysEl(ID=timesysId)
 			ctx.getEnclosingResource()[				
 				timesysEl]
-			_addStupidRefByAnnotation(
-				ctx, 
-				ann["time"]["location"],
-				serManager, 
-				timesysId)
+			if "location" in ann["time"]:
+				_addStupidRefByAnnotation(
+					ctx,
+					ann["time"]["location"],
+					serManager,
+					timesysId)
 
 		if "space" in ann:
 			spaceFrame = ann["space"]["frame"]
-			coosys = V.COOSYS()
-			sysId = serManager.makeIdFor(coosys, "system")
+			coosys = None
+			if "astid" in ann:
+				# the annotation was built from an <stc> declaration, and
+				# _iterSTC may already have created the coosys element, in
+				# which case we want to re-use it.
+				coosys = ctx.coosysByAST.get(str(ann["astid"]))
+
+			if coosys is None:
+				# no such coosys created yet
+				coosys = V.COOSYS()
+				ctx.getEnclosingResource()[coosys]
+
+			sysId = serManager.getOrMakeIdFor(coosys, "system")
 			# we probably should be more careful with literals vs. references
 			# for the orientation, too
 			coosys(ID=sysId,
-				system=spaceFrame.get("orientation"))
+				system=STC_FRAMES_TO_COOSYS[spaceFrame.get("orientation")],
+				refposition=spaceFrame.get("refPosition"))
 			epVal = spaceFrame.get("epoch")
 			if isinstance(epVal, str):
 				coosys(epoch=epVal)
+			elif epVal:
+				coosys[_makeLegacyRefForAnn(
+					ctx, epVal, "votable:CustomRefLocation-epoch")]
 
-			# this would probably need to recurse into children.  Let's see
-			# how the DM will look like in the end
 			for child in ann["space"].iterChildRoles():
 				_addStupidRefByAnnotation(ctx, child, serManager, sysId)
-
-			ctx.getEnclosingResource()[coosys]
+				if (child.name in _STC_UTYPE_MAPPING and
+						isinstance(child, dm.ParamLikeAnnotation)):
+					coosys[_makeLegacyRefForAnn(
+						ctx, child, _STC_UTYPE_MAPPING[child.name])]
 
 
 _PHOT_CAL_PARAMS = {
@@ -723,7 +832,7 @@ def _addLegacyPhotCal(ctx, tableDef, serManager):
 	"""adds PhotCal groups to the enclosing RESOURCE.
 	"""
 	for ann in tableDef.iterAnnotationsOfType("phot:PhotCal"):
-		photGroup = V.GROUP(name="photcal") 
+		photGroup = V.GROUP(name="photcal")
 		photGroup(ID=serManager.makeIdFor(photGroup, "phot_def"))
 		for child in ann.iterChildRoles():
 			if child.name=="value":
@@ -761,7 +870,7 @@ def _iterNotes(serManager):
 
 
 def _makeRef(baseType, ref, container, serManager):
-	"""returns a new node of baseType reflecting the group.TypedRef 
+	"""returns a new node of baseType reflecting the group.TypedRef
 	instance ref.
 
 	container is the destination of the reference.  For columns, that's
@@ -828,8 +937,11 @@ def makeTable(ctx, table, isMeta=False):
 		# have the stupid ref to COOSYS
 		result[_iterSTC(ctx, table.tableDef, sm)]
 
-		# same for STC2 (except here the FIELDs need reliable IDs)
-		_addLegacySYSFromSTC2(ctx, table.tableDef, sm)
+		# same for votable-STC (except here the FIELDs need reliable IDs)
+		try:
+			_addLegacySYSFromVOTableSTC(ctx, table.tableDef, sm)
+		except Exception as ex:
+			base.ui.notifyError(f"Could not serialise VOTable COOSYS/TIMESYS: {ex}")
 
 		# photometry hack
 		_addLegacyPhotCal(ctx, table.tableDef, sm)
@@ -839,9 +951,9 @@ def makeTable(ctx, table, isMeta=False):
 				utype=base.getMetaText(table, "utype", macroPackage=table.tableDef,
 				propagate=False))[
 			# _iterGroups must run before _iterFields and _iterParams since it
-			# may need to add ids to the respective items.  XSD-correct ordering of 
+			# may need to add ids to the respective items.  XSD-correct ordering of
 			# the elements is done by xmlstan.
-			V.DESCRIPTION[base.getMetaText(table, "description", 
+			V.DESCRIPTION[base.getMetaText(table, "description",
 				macroPackage=table.tableDef, propagate=False)],
 			_iterGroups(ctx, table.tableDef, sm),
 			_iterFields(ctx, sm),
@@ -867,6 +979,14 @@ def _makeResource(ctx, data):
 	"""
 	resType = base.getMetaText(data, "_type")
 	res = V.RESOURCE()
+
+	# For now, DaCHS will only have one RESOURCE that can contain VODML
+	# annotation.  In case that ever changes, ctx will have to keep track
+	# of what "top-level" RESOURCE we're annotating (unless MIVOT gets
+	# sanitised).
+	if ctx.produceVODML:
+		res[V.RESOURCE(type="meta")[ctx.rootVODML]]
+
 	with ctx.activeContainer(res):
 		res(type=resType,
 				utype=base.getMetaText(data, "utype"))[
@@ -926,17 +1046,10 @@ def makeVOTable(data, ctx=None, **kwargs):
 	vot[dlResources]
 
 	if ctx.produceVODML:
-		if ctx.modelsUsed:
-			# if we declare any models, we'll need vo-dml
-			ctx.addVODMLPrefix("vo-dml")
+		vot._fixedTagMaterial += ' xmlns:mivot="{}"'.format(
+			utils.getPrefixInfo("mivot")[0])
 
-		ctx.rootVODML[[
-				model.getVOT(ctx, None)
-			for model in list(ctx.modelsUsed.values())]]
-
-		vot[ctx.rootVODML]
-
-	if ctx.suppressNamespace:  
+	if ctx.suppressNamespace:
 		# use this for "simple" table with nice element names
 		vot._fixedTagMaterial = ""
 
@@ -947,7 +1060,6 @@ def makeVOTable(data, ctx=None, **kwargs):
 		rootHacks = [vot._fixedTagMaterial]+[
 			item.getContent() for item in rootAttrs]
 		vot._fixedTagMaterial = " ".join(s for s in rootHacks if s)
-	
 
 	return vot
 
@@ -981,7 +1093,7 @@ def format(data, outputFile, **ctxargs):
 # used for construction of the formats.common interface
 	return writeAsVOTable(data, outputFile, VOTableContext(**ctxargs))
 
-common.registerDataWriter("votable", format, 
+common.registerDataWriter("votable", format,
 	base.votableType, "Default VOTable", ".vot",
 	tapId="ivo://ivoa.net/std/TAPRegExt#output-votable-binary")
 common.registerDataWriter("votableb2", functools.partial(
@@ -992,37 +1104,37 @@ common.registerDataWriter("votableb2", functools.partial(
 	"votable/b2",
 	tapId="ivo://ivoa.net/std/TAPRegExt#output-votable-binary2")
 common.registerDataWriter("votabletd", functools.partial(
-	format, tablecoding="td"), 
+	format, tablecoding="td"),
 	"application/x-votable+xml;serialization=TABLEDATA", "Tabledata VOTable",
 	".vottd",
 	"text/xml",
 	"votable/td",
 	tapId="ivo://ivoa.net/std/TAPRegExt#output-votable-td")
 common.registerDataWriter("votabletd1.1", functools.partial(
-	format, tablecoding="td", version=(1,1)), 
-	"application/x-votable+xml;serialization=TABLEDATA;version=1.1", 
+	format, tablecoding="td", version=(1,1)),
+	"application/x-votable+xml;serialization=TABLEDATA;version=1.1",
 	"Tabledata VOTable version 1.1",
 	".vot1",
 	"text/xml")
 common.registerDataWriter("votable1.1", functools.partial(
-	format, tablecoding="binary", version=(1,1)), 
-	"application/x-votable+xml;version=1.1", 
+	format, tablecoding="binary", version=(1,1)),
+	"application/x-votable+xml;version=1.1",
 	"Tabledata VOTable version 1.1",
 	".vot1",
 	"text/xml")
 common.registerDataWriter("votabletd1.2", functools.partial(
-	format, tablecoding="td", version=(1,2)), 
-	"application/x-votable+xml;serialization=TABLEDATA;version=1.2", 
+	format, tablecoding="td", version=(1,2)),
+	"application/x-votable+xml;serialization=TABLEDATA;version=1.2",
 	"Tabledata VOTable version 1.2",
 	".vot2",
 	"text/xml")
 common.registerDataWriter("vodml", functools.partial(
-	format, tablecoding="td", version=(1,5)), 
+	format, tablecoding="td", version=(1,5)),
 	"application/x-votable+xml;serialization=TABLEDATA;version=1.5",
 	"VOTable version 1.5, tabledata",
 	".vot5")
 common.registerDataWriter("vodmlb", functools.partial(
-	format, version=(1,5)), 
-	"application/x-votable+xml;version=1.5", 
+	format, version=(1,5)),
+	"application/x-votable+xml;version=1.5",
 	"VOTable version 1.5",
 	".vot5")
