@@ -6,7 +6,7 @@ rscdef.builtingrammars.GRAMMAR_REGISTRY (we don't want to import all
 the mess in this package just to make that).
 """
 
-#c Copyright 2008-2022, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -62,8 +62,8 @@ class FilteredInputFile(object):
 		self.filterCommand, self.origFile = filterCommand, origFile
 
 		processArgs = {"shell": True,
-			"stdin": subprocess.PIPE, 
-			"stdout": subprocess.PIPE, 
+			"stdin": subprocess.PIPE,
+			"stdout": subprocess.PIPE,
 			"close_fds": True}
 		if silent:
 			processArgs["stderr"] = utils.devnull()
@@ -99,7 +99,7 @@ class FilteredInputFile(object):
 		if data==b"":
 			# source file exhausted, tell child we're done
 			self.toChild.close()
-			self.toChild = None 
+			self.toChild = None
 		else:
 			self.toChild.write(data)
 
@@ -185,7 +185,7 @@ class Rowfilter(procdef.ProcApp):
 	"NoneType is not iterable" error that's a bit hard to understand.
 
 	Here, you can only access whatever comes from the grammar.  You can
-	access grammar keys in late parameters as row[key] or, if key is 
+	access grammar keys in late parameters as row[key] or, if key is
 	like an identifier, as @key.
 	"""
 	name_ = "rowfilter"
@@ -220,7 +220,7 @@ def compileRowfilter(filters):
 
 
 class SourceFieldApp(rscdef.ProcApp):
-	"""A procedure application that returns a dictionary added to all 
+	"""A procedure application that returns a dictionary added to all
 	incoming rows.
 	
 	Use this to programmatically provide information that can be computed
@@ -245,7 +245,7 @@ class MapKeys(base.Structure):
 	mapKeys is necessary in grammars like keyValueGrammar or fitsProdGrammar.
 	In these, the source files themselves give key names.  Within the GAVO
 	DC, keys are required to be valid python identifiers (i.e., match
-	``[A-Za-z\_][A-Za-z\_0-9]*``).  If keys coming in do not have this form, 
+	``[A-Za-z\_][A-Za-z\_0-9]*``).  If keys coming in do not have this form,
 	mapping can force proper names.
 
 	mapKeys could also be used to make incoming names more suitable for
@@ -256,7 +256,7 @@ class MapKeys(base.Structure):
 	_content = base.DataContent(description="Simple mappings in the form"
 		"<dest>:<src>{,<dest>:<src>}")
 	_mappings = base.DictAttribute("maps", keyName="dest", description=
-		"Map source names given in content to the name given in dest.", 
+		"Map source names given in content to the name given in dest.",
 		itemAttD=base.UnicodeAttribute("map"), inverted=True,
 		copyable=True)
 
@@ -306,13 +306,13 @@ class RowIterator(object):
 	in finding out why something went wrong (file name, line number, etc.)
 
 	This default implementation works for when source is a sequence
-	of dictionaries.  You will, in general, want to override 
+	of dictionaries.  You will, in general, want to override
 	_iteRows and getLocator, plus probably __init__ (to prepare external
 	resources) and getParameters (if you have them; make sure to update
 	any parameters you have with self.sourceRow as shown in the default
 	getParameters implementation).
 
-	RowIterators are supposed to be self-destructing, i.e., they should 
+	RowIterators are supposed to be self-destructing, i.e., they should
 	release any external resources they hold when _iterRows runs out of
 	items.
 
@@ -591,10 +591,10 @@ class GrammarMacroMixin(base.StandardMacroMixin):
 			+"+getFlatName(accref)")
 
 	def macro_splitPreviewPath(self, ext):
-		"""returns an expression for the split standard path for a custom 
+		"""returns an expression for the split standard path for a custom
 		preview.
 
-		As standardPreviewPath, except that the directory hierarchy of the data 
+		As standardPreviewPath, except that the directory hierarchy of the data
 		files will be reproduced in previews.  For ext, you should typically pass
 		the extension appropriate for the preview (like {.png} or {.jpeg}).
 		
@@ -648,7 +648,7 @@ class GrammarMacroMixin(base.StandardMacroMixin):
 class Grammar(base.Structure, GrammarMacroMixin):
 	"""An abstract grammar.
 
-	Grammars are configured via their structure parameters.  Their 
+	Grammars are configured via their structure parameters.  Their
 	parse(sourceToken) method returns an object that iterates over rawdicts
 	(dictionaries mapping keys to (typically) strings) that can then be fed
 	through rowmakers; it also has a method getParameters that returns
@@ -669,9 +669,9 @@ class Grammar(base.Structure, GrammarMacroMixin):
 	name_ = "grammar"
 
 	_encoding = base.UnicodeAttribute("enc", default=None, description=
-		"Encoding of strings coming in from source.", copyable=True)
-	_rowfilters = base.StructListAttribute("rowfilters", 
-		description="Row filters for this grammar.", 
+		"Encoding of the source file(s).", copyable=True)
+	_rowfilters = base.StructListAttribute("rowfilters",
+		description="Row filters for this grammar.",
 		childFactory=Rowfilter, copyable=True)
 	_ignoreOn = base.StructAttribute("ignoreOn", default=None, copyable=True,
 		description="Conditions for ignoring certain input records.  These"
@@ -704,7 +704,7 @@ class Grammar(base.Structure, GrammarMacroMixin):
 		return self._compiledSourceFields(sourceToken, data)
 
 	def parse(self, sourceToken, targetData=None):
-		ri = self.rowIterator(self, sourceToken, 
+		ri = self.rowIterator(self, sourceToken,
 			sourceRow=self.getSourceFields(sourceToken, targetData))
 		if self.rowfilters:
 			ri.rowfilter = compileRowfilter(self.rowfilters)
