@@ -1,5 +1,5 @@
 """
-A grammar supporting C language boosters (or possibly other mechanisms 
+A grammar supporting C language boosters (or possibly other mechanisms
 bypassing internal dbtable).
 
 These actually bypass most of our machinery and should only be used if
@@ -9,7 +9,7 @@ the rest of the DC software.
 Currently, only one kind of DirectGrammar is supported: C boosters.
 """
 
-#c Copyright 2008-2022, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -75,7 +75,7 @@ class CBooster(object):
 		if dd.grammar.type=="fits":
 			f.write("LDFLAGS += -lcfitsio\n")
 
-		if dd.grammar.type=="hdf5vaex":
+		if dd.grammar.type.startswith("hdf5"):
 			flags = subprocess.check_output(
 				["pkg-config", "--cflags", "hdf5"]).strip().decode("ascii")
 			if flags:
@@ -172,7 +172,7 @@ class DirectGrammar(base.Structure, base.RestrictionMixin):
 
 	notify = True
 
-	_cbooster = rscdef.ResdirRelativeAttribute("cBooster", 
+	_cbooster = rscdef.ResdirRelativeAttribute("cBooster",
 		default=base.Undefined,
 		description="resdir-relative path to the booster C source.",
 		copyable=True)
@@ -206,8 +206,8 @@ class DirectGrammar(base.Structure, base.RestrictionMixin):
 		" booster.",
 		copyable=True)
 
-	_type = base.EnumeratedUnicodeAttribute("type", default="col", 
-		validValues=["col", "bin", "fits", "split", "hdf5vaex"],
+	_type = base.EnumeratedUnicodeAttribute("type", default="col",
+		validValues=["col", "bin", "fits", "split", "hdf5vaex", "hdf5rootarrs"],
 		description="Make code for a booster parsing by column indices (col),"
 			" by splitting along separators (split), by reading fixed-length"
 			" binary records (bin), for from FITS binary tables (fits).",
@@ -222,7 +222,7 @@ class DirectGrammar(base.Structure, base.RestrictionMixin):
 		copyable=True)
 
 	_mapKeys = base.StructAttribute("mapKeys", childFactory=common.MapKeys,
-		default=None, copyable=True, 
+		default=None, copyable=True,
 		description="For certain input formats (FITS, HDF5), map input"
 			" column names to database column name.  For instance, to generate"
 			" code for a FITS column flx to end up in the DB column flux,"
@@ -241,7 +241,7 @@ class DirectGrammar(base.Structure, base.RestrictionMixin):
 				raise base.StructureError("DirectGrammars reading from binary need"
 					" a recordSize attribute")
 		if self.mapKeys is not None:
-			if self.type not in {"fits", "hdf5vaex"}:
+			if self.type not in {"fits", "hdf5vaex", "hdf5rootarrs"}:
 				raise base.StructureError("mapKeys is not allowed for this"
 					" kind of booster.")
 
@@ -541,7 +541,7 @@ class BinCodeGenerator(_CodeGenerator):
 			pline = "%s(%s, *(double*)(inputLine+ ), '<nil>');"
 		else:
 			pline = "%s %s"
-		return ["/* %s (%s) */"%(item.description, t), 
+		return ["/* %s (%s) */"%(item.description, t),
 			pline%(_getMakeMacro(item), getNameForItem(item))]
 
 	def getPreamble(self):
@@ -599,6 +599,7 @@ class _NumpyMetaCodeGenerator(_CodeGenerator):
 		"ulonglong": "unsigned long long",
 		"uint64": "uint64_t",
 		"single": "float",
+		"float16": "float",
 		"float32": "float",
 		"double": "double",
 		"float64": "double",
@@ -690,12 +691,12 @@ class FITSCodeGenerator(_NumpyMetaCodeGenerator):
 				][1]
 
 			return [
-				"/* %s (%s) */"%(item.description, item.type), 
+				"/* %s (%s) */"%(item.description, item.type),
 				"if (nulls[%d][rowIndex]) {"%fitsIndex,
 				"  MAKE_NULL(%s);"%getNameForItem(item),
 				"} else {",
 				"	 %s(%s, ((%s*)(data[%d]))[rowIndex]);"%(
-					self.makers[item.type], 
+					self.makers[item.type],
 					getNameForItem(item),
 					castTo,
 					fitsIndex),
@@ -705,7 +706,7 @@ class FITSCodeGenerator(_NumpyMetaCodeGenerator):
 			# no FITS table source column
 			return ["MAKE_NULL(%s); /* %s(%s, FILL IN VALUE); */"%(
 				getNameForItem(item),
-				_getMakeMacro(item), 
+				_getMakeMacro(item),
 				getNameForItem(item))]
 
 	def getPreamble(self):
@@ -806,7 +807,7 @@ FITSColDesc COL_DESCS[%(nCols)d] = %(colDescs)s;
 		die("FITS tables cannot be read from stdin.");
 	}
 
-	FITSCATCH(fits_movabs_hdu(fitsInput, %(extension)s+1, 
+	FITSCATCH(fits_movabs_hdu(fitsInput, %(extension)s+1,
 		&hdutype, &status));
 	assert(hdutype==BINARY_TBL);
 
@@ -815,7 +816,7 @@ FITSColDesc COL_DESCS[%(nCols)d] = %(colDescs)s;
 	for (i=0; i<%(nCols)d; i++) {
 		if (COL_DESCS[i].cSize==0) {
 			/* Column not used */
-			continue; 
+			continue;
 
 		} else if (COL_DESCS[i].fitsType==TSTRING) {
 			char *stringHoldings = NULL;
@@ -838,7 +839,7 @@ FITSColDesc COL_DESCS[%(nCols)d] = %(colDescs)s;
 		if (!(nulls[i] = malloc(nRows*sizeof(char)*COL_DESCS[i].arraysize))) {
 			die("out of memory");
 		}
-		FITSCATCH(fits_read_colnull(fitsInput, COL_DESCS[i].fitsType, 
+		FITSCATCH(fits_read_colnull(fitsInput, COL_DESCS[i].fitsType,
 			COL_DESCS[i].index, 1, 1,
      	nRows*COL_DESCS[i].arraysize, data[i], nulls[i], &ignored, &status));
 	}"""%infoDict
@@ -851,22 +852,24 @@ FITSColDesc COL_DESCS[%(nCols)d] = %(colDescs)s;
 		+COMMON_MAIN_FOOT)
 
 
-def HDF5vaexCodeGenerator(*args, **kwargs):
-	"""a factory function for an HDF5 code generator.
+def getHDF5Generator(name):
+	"""a factory for HDF5 generators classes.
 
-	This is built like this because directhdf5 imports this module and 
+	This is built like this because directhdf5 imports this module and
 	we need to avoid circular imports.  If this kind of construct becomes
 	more common, we should pull basic code generation stuff into a
 	boostercommon module.  Meanwhile, this is a convenient place to
 	produce a useful error message for when h5py is not available.
 	"""
-	try:
-		from gavo.grammars import directhdf5
-	except ImportError:
-		raise base.ReportableError("HDF5 boosters can only be built if h5py"
-			" (Debian: python3-h5py) is installed.")
-	return directhdf5.HDF5vaexCodeGenerator(*args, **kwargs)
-	
+	def _(*args, **kwargs):
+		try:
+			from gavo.grammars import directhdf5
+		except ImportError:
+			raise base.ReportableError("HDF5 boosters can only be built if h5py"
+				" (Debian: python3-h5py) is installed.")
+		return getattr(directhdf5, name)(*args, **kwargs)
+	return _
+
 
 def getCodeGen(grammar, tableDef):
 	"""returns the code generator suitable for making code for grammar.
@@ -876,7 +879,8 @@ def getCodeGen(grammar, tableDef):
 		"split": SplitCodeGenerator,
 		"fits": FITSCodeGenerator,
 		"col": ColCodeGenerator,
-		"hdf5vaex": HDF5vaexCodeGenerator,
+		"hdf5vaex": getHDF5Generator("HDF5vaexCodeGenerator"),
+		"hdf5rootarrs": getHDF5Generator("HDF5rootarrCodeGenerator"),
 	}[grammar.type](grammar, tableDef)
 
 
@@ -915,7 +919,11 @@ def indent(stringList, indentChar):
 def buildSource(grammar, td):
 	"""returns (possibly incomplete) C source for a booster to read into td.
 	"""
-	codeGen = getCodeGen(grammar, td)
+	try:
+		codeGen = getCodeGen(grammar, td)
+	except KeyError:
+		raise base.NotFoundError(grammar.type, "Booster type",
+			"DaCHS' booster type registry")
 
 	code = codeGen.getPreamble()
 	code.extend(getEnum(td, grammar))
