@@ -6,7 +6,7 @@ WARNING: This messes up some global state.  DO NOT import into modules
 doing regular work.  testtricks is the module for that kind for stuff.
 """
 
-#c Copyright 2008-2022, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -43,15 +43,15 @@ except ImportError:
 #
 # sudo -u postgres -s `id -nu`
 #
-# ). You should be able to tear both ~/_gavo_test and the database 
-# down, and this should automatically recreate everything.  That's 
+# ). You should be able to tear both ~/_gavo_test and the database
+# down, and this should automatically recreate everything.  That's
 # an integration test for DaCHS, too.
 #
 # This must be run before anything else from gavo is imported because
 # it manipulates the config stuff; this, in turn, runs as soon as
 # base is imported.
 
-# The following forces tests to be run from the tests directory.  
+# The following forces tests to be run from the tests directory.
 # Reasonable, I'd say.
 #
 # All the custom setup can be suppressed by setting a GAVO_OOTTEST
@@ -70,7 +70,7 @@ else:
 	TEST_BASE = os.getcwd()
 	originalEnvironment = os.environ.copy()
 	os.environ["GAVOCUSTOM"] = "/invalid"
-	os.environ["GAVOSETTINGS"] = os.path.join(TEST_BASE, 
+	os.environ["GAVOSETTINGS"] = os.path.join(TEST_BASE,
 		"test_data", "test-gavorc")
 	if not os.path.exists(os.environ["GAVOSETTINGS"]):
 		warnings.warn("testhelpers imported from non-test directory.  This"
@@ -85,7 +85,7 @@ else:
 		initdachs.createFSHierarchy(dsn, "test")
 
 		with open(os.path.join(
-				base.getConfig("configDir"), "defaultmeta.txt"), "a", 
+				base.getConfig("configDir"), "defaultmeta.txt"), "a",
 				encoding="utf-8") as f:
 			f.write("!organization.description: Mein wüster Club\n")
 			f.write("!contact.email: invalid@wherever.else\n")
@@ -110,7 +110,7 @@ else:
 			# and that, as used in DaCHS, may create a thread.  That means
 			# risking a deadlock while python is importing.
 			try:
-				subprocess.check_call(["createdb", "--template=template0", 
+				subprocess.check_call(["createdb", "--template=template0",
 					"--encoding=UTF-8", "--locale=C", dbname])
 
 				initdachs.initDB(dsn)
@@ -182,15 +182,15 @@ class FakeSimbad(object):
 
 # Here's the deal on TestResource: When setting up complicated stuff for
 # tests (like, a DB table), define a TestResource for it.  Override
-# the make(dependents) method returning something and the clean(res) 
+# the make(dependents) method returning something and the clean(res)
 # method to destroy whatever you created in make().
 #
 # Then, in VerboseTests, have a class attribute
 # resource = [(name1, res1), (name2, res2)]
 # giving attribute names and resource *instances*.
 # There's an example in adqltest.py
-# 
-# If you use this and you have a setUp of your own, you *must* call 
+#
+# If you use this and you have a setUp of your own, you *must* call
 # the superclass's setUp method.
 
 import testresources
@@ -296,7 +296,7 @@ class ForkingSubprocess(subprocess.Popen):
 							 p2cread, p2cwrite,
 							 c2pread, c2pwrite,
 							 errread, errwrite,
-							 restore_signals, 
+							 restore_signals,
                # post-3.7, some additional parameters were added, which
                # fortunately we don't need.
 							 *ignored_args):
@@ -451,7 +451,7 @@ class VerboseTest(testresources.ResourcedTestCase):
 	"""A TestCase with a couple of convenient assert methods.
 	"""
 	def assertEqualForArgs(self, callable, result, *args):
-		self.assertEqual(callable(*args), result, 
+		self.assertEqual(callable(*args), result,
 			"Failed for arguments %s.  Expected result is: %s, result found"
 			" was: %s"%(str(args), repr(result), repr(callable(*args))))
 
@@ -479,7 +479,7 @@ class VerboseTest(testresources.ResourcedTestCase):
 		except:
 			raise
 		else:
-			raise self.failureException(msg or 
+			raise self.failureException(msg or
 				"%s not raised (function returned %s)"%(
 					str(exception), repr(value)))
 
@@ -511,7 +511,7 @@ class VerboseTest(testresources.ResourcedTestCase):
 		denom = abs(a+b)
 		self.assertTrue(abs(a-b)/denom<ratio, msg)
 
-	def assertOutput(self, toExec, argList, expectedStdout=None, 
+	def assertOutput(self, toExec, argList, expectedStdout=None,
 			expectedStderr="", expectedRetcode=0, input=None,
 			stdoutStrings=None):
 		"""checks that execName called with argList has the given output and return
@@ -535,10 +535,10 @@ class VerboseTest(testresources.ResourcedTestCase):
 				pass
 
 		if isinstance(toExec, str):
-			p = subprocess.Popen([toExec]+argList, executable=toExec, 
+			p = subprocess.Popen([toExec]+argList, executable=toExec,
 				stdout=subprocess.PIPE, stdin=subprocess.PIPE, stderr=subprocess.PIPE)
 		else:
-			p = ForkingSubprocess(["test harness"]+argList, executable=toExec, 
+			p = ForkingSubprocess(["test harness"]+argList, executable=toExec,
 				stdout=subprocess.PIPE, stdin=subprocess.PIPE, stderr=subprocess.PIPE)
 		out, err = p.communicate(input=utils.bytify(input))
 		retcode = p.wait()
@@ -578,8 +578,9 @@ class VerboseTest(testresources.ResourcedTestCase):
 			raise
 
 	def assertEqualIgnoringAliases(self, result, expectation):
-		pat = re.escape(expectation).replace("ASWHATEVER", "AS [a-z]+")+"$"
-		if not re.match(pat, result):
+		pat = re.escape(re.sub(r"\s+", " ", expectation)
+			).replace("ASWHATEVER", "AS [a-z]+")+"$"
+		if not re.match(pat, re.sub(r"\s+", " ", result)):
 			raise AssertionError("%r != %r"%(result, expectation))
 
 
@@ -600,8 +601,8 @@ def cleanXML(aString):
 	aString can be bytes, in which case it will be interpreted as ASCII.
 	cleanXML returns a string in any case.
 	"""
-	return re.sub("\s+", " ", 
-		_xmlJunkPat.sub('', 
+	return re.sub("\s+", " ",
+		_xmlJunkPat.sub('',
 			utils.debytify(aString))).strip(
 				).replace(" />", "/>").replace(" >", ">")
 
@@ -665,13 +666,13 @@ def computeWCSKeys(pos, size, cutCrap=False):
 		"NAXIS1": imgPix[0],
 		"NAXIS2": imgPix[1],
 		"NAXIS": 2,
-		"CTYPE1": 'RA---TAN-SIP', 
+		"CTYPE1": 'RA---TAN-SIP',
 		"CTYPE2": 'DEC--TAN-SIP',
 		"LONPOLE": 180.}
 	if not cutCrap:
 		res.update({"imageTitle": "test image at %s"%repr(pos),
 			"instId": None,
-			"dateObs":55300+pos[0], 
+			"dateObs":55300+pos[0],
 			"refFrame": None,
 			"wcs_equinox": None,
 			"bandpassId": None,
@@ -711,7 +712,7 @@ def getTestData(dataId):
 
 
 def hasUDF(udfName):
-	"""returns true if this system has an ADQL user defined function named 
+	"""returns true if this system has an ADQL user defined function named
 	udfName.
 
 	This is intended to be used as in::
