@@ -2,7 +2,7 @@
 Helpers for trial-based tests, in particular retrieving pages.
 """
 
-#c Copyright 2008-2022, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -45,7 +45,7 @@ class ArchiveTest(RenderTest):
 	def assertResponseIsValid(self, res):
 		errs = testtricks.getXSDErrors(res[0], True)
 		if errs:
-			with open("remote.data", "w") as f:
+			with open("remote.data", "wb") as f:
 				f.write(res[0])
 			raise AssertionError(errs)
 
@@ -103,6 +103,8 @@ class FakeRequest(testing.FakeRequest, web.Request):
 	pass
 testing.FakeRequest = FakeRequest
 
+# for convenience, expose FakeFile, too
+FakeFile = testing.FakeFile
 
 def runSvcWith(service, renderer, args):
 	"""runs svc through renderers, passing a dict args.
