@@ -2,7 +2,7 @@
 Code to bind the adql library to the data center software.
 """
 
-#c Copyright 2008-2022, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -88,7 +88,7 @@ def _makeColumnFromFieldInfo(ctx, colName, fi):
 			# if we're reasonably sure there's no intervening AS
 			colName = fi.userData[0].name
 
-	else: 
+	else:
 		res = base.makeStruct(svcs.OutputField, name=colName)
 
 	res.name = ctx.getName(colName)
@@ -130,7 +130,7 @@ def _makeColumnFromFieldInfo(ctx, colName, fi):
 	# sure that a query yields defined results for all of them.
 	# Tough luck if our artificial value is already taken by the table
 	# (remedy: select a suitable null value in the column metadata)
-	if (res.type in _artificialNULLs 
+	if (res.type in _artificialNULLs
 			and (
 				not (res.values and res.values.nullLiteral)
 				or fi.tainted)):
@@ -138,7 +138,7 @@ def _makeColumnFromFieldInfo(ctx, colName, fi):
 		if res.values:
 			res.feedObject("values", res.values.change(nullLiteral=nullLiteral))
 		else:
-			res.feedObject("values", base.makeStruct(rscdef.Values, 
+			res.feedObject("values", base.makeStruct(rscdef.Values,
 				nullLiteral=nullLiteral))
 
 	# unconditionally do away with tableheads inherited from columns
@@ -157,7 +157,7 @@ def _getTableDescForOutput(parsedTree):
 	parsed and annotated ADQL query parsedTree.
 	"""
 	ctx = TDContext()
-	columns = [_makeColumnFromFieldInfo(ctx, *fi) 
+	columns = [_makeColumnFromFieldInfo(ctx, *fi)
 		for fi in parsedTree.fieldInfos.seq]
 
 	# if this is a simple one-table query, take the metadata and params
@@ -187,14 +187,14 @@ def _getTableDescForOutput(parsedTree):
 	params = []
 	for tableName in fromNames:
 		try:
-			params.extend(p.copy(parent=None) 
+			params.extend(p.copy(parent=None)
 				for p in mth.getTableDefForTable(tableName).params)
 		except Exception:
 			# don't fail just because a funny thing is in fromNames
 			pass
 
 	resTable = base.makeStruct(
-		rscdef.TableDef, 
+		rscdef.TableDef,
 		columns=columns,
 		params=params,
 		id=parsedTree.suggestAName())
@@ -260,7 +260,7 @@ class DaCHSFieldInfoGetter(adql.FieldInfoGetter):
 		self.mth = base.caches.getMTH(None)
 		for td in tdsForUploads:
 			self.addExtraFieldInfos(
-				td.id, 
+				td.id,
 				[(_getADQLName(f), makeFieldInfo(f, sqlName=f.name)) for f in td])
 
 	def getInfosFor(self, tableName):
@@ -275,7 +275,7 @@ def _addTableMeta(translated, query, tree, table):
 	result table.
 	"""
 	table.makeOriginal("info")
-	table.addMeta("info", "", infoName="server", 
+	table.addMeta("info", "", infoName="server",
 		infoValue=base.getConfig("web", "serverURL"))
 	table.addMeta("info", "", infoName="sql_query", infoValue=translated)
 	table.addMeta("info", "", infoName="query", infoValue=query)
@@ -294,18 +294,18 @@ def _addTableMeta(translated, query, tree, table):
 
 		try:
 			sourceTD = mth.getTableDefForTable(tableName)
-			table.addMeta("info", 
+			table.addMeta("info",
 				base.getMetaText(sourceTD.rd, "description", ""),
-				infoName="src_res", 
+				infoName="src_res",
 				infoValue="Contains traces from resource %s"%(sourceTD.rd.sourceId))
 			for m in sourceTD.iterMeta("rights", propagate=True):
 				table.addMeta("info",
 					m.getContent("text"),
-					infoName="copyright", 
+					infoName="copyright",
 					infoValue="%s copyright or license"%(sourceTD.rd.sourceId))
-			table.addMeta("info", 
+			table.addMeta("info",
 				base.getMetaText(sourceTD, "description", "", propagate=False),
-				infoName="src_table", 
+				infoName="src_table",
 				infoValue="Contains traces from table %s"%(
 					sourceTD.getQName()))
 
@@ -323,12 +323,12 @@ def _addTableMeta(translated, query, tree, table):
 					try:
 						destCol = table.tableDef.getColumnByName(idColumn.name)
 						table.addMeta("_associatedDatalinkService", None)
-						table.addMeta("_associatedDatalinkService.idColumn", 
+						table.addMeta("_associatedDatalinkService.idColumn",
 							destCol.name)
 						serviceId = m.getMeta("serviceId").getContent()
 						if "#" not in serviceId:
 							serviceId = "%s#%s"%(sourceTD.rd.sourceId, serviceId)
-						table.addMeta("_associatedDatalinkService.serviceId", 
+						table.addMeta("_associatedDatalinkService.serviceId",
 							serviceId)
 					except base.NotFoundError:
 						# User hasn't selected the column with the id.  No problem.
@@ -339,12 +339,12 @@ def _addTableMeta(translated, query, tree, table):
 
 
 def _updateMatchLimits(tree, maxrec, hardLimit):
-	"""instruments the ADQL tree for the user row set limit maxrec 
+	"""instruments the ADQL tree for the user row set limit maxrec
 	and the system row set limit hard limit.
 
 	maxrec is a match limit from the protocol level, as opposed to the
 	setLimit from the ADQL TOP clause.  The rules of interaction between
-	the two are documented inline below (it's messy).  
+	the two are documented inline below (it's messy).
 	
 	This returns the overflow set limit.  If exactly this many rows
 	are returned from he query, and overflow indicator should be set.
@@ -384,7 +384,7 @@ def _updateMatchLimits(tree, maxrec, hardLimit):
 		return maxrec+1
 
 
-def morphADQL(query, metaProfile=None, tdsForUploads=[], 
+def morphADQL(query, metaProfile=None, tdsForUploads=[],
 		maxrec=None, hardLimit=None):
 	"""returns an postgres query and an (empty) result table for the
 	ADQL in query.
@@ -423,7 +423,7 @@ def query(adqlQuery, timeout=15, metaProfile=None, tdsForUploads=[],
 	Don't use it anywhere else.
 	"""
 	with base.getWritableUntrustedConn() as connection:
-		qtable = runTAPQuery(adqlQuery, timeout, 
+		qtable = runTAPQuery(adqlQuery, timeout,
 			connection, tdsForUploads, externalLimit, False)
 
 		# instantiate the qtable; we want it all in one go
@@ -447,6 +447,11 @@ def runTAPQuery(query, timeout, connection, tdsForUploads, maxrec,
 		autoClose=True):
 	"""executes a TAP query and returns the result in a data instance.
 	"""
+	# ugly convenience hack: because people these days regularly paste
+	# in 0xa0 for blanks and this kind of thing is almost impossible to spot,
+	# I just map them to blanks for now
+	query = query.replace("\xa0", " ")
+
 	postgresParams = [
 		# cursor tuple fraction only kicks in if we're actually using
 		# cursors (which for psycopg2 only happens with named cursors).
@@ -470,7 +475,7 @@ def runTAPQuery(query, timeout, connection, tdsForUploads, maxrec,
 				('enable_seqscan', 'no'))
 
 		resetTo = connection.configure(postgresParams)
-
+	
 		result = rsc.QueryTable(tableTrunk.tableDef, pgQuery, connection,
 			autoClose=autoClose)
 		result.meta_ = tableTrunk.meta_
@@ -517,7 +522,7 @@ class ADQLCore(svcs.Core, base.RestrictionMixin):
 		queryString = inRow["query"]
 		base.ui.notifyInfo("Incoming ADQL query: %s"%queryString)
 		try:
-			res = query(queryString, 
+			res = query(queryString,
 				timeout=queryMeta["timeout"], hardLimit=100000,
 				externalLimit=queryMeta["dbLimit"])
 # XXX Warning: We're returning the db connection to the connection
