@@ -4,7 +4,7 @@ The datalink core and its numerous helper classes.
 More on this in "Datalink Cores" in the reference documentation.
 """
 
-#c Copyright 2008-2022, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -35,23 +35,33 @@ from gavo.votable import V, modelgroups
 
 from twisted.web import static
 
+LINKS_STANDARD_ID = "ivo://ivoa.net/std/datalink#links-1.1"
+SYNC_STANDARD_ID = "ivo://ivoa.net/std/soda#sync-1.0"
+ASYNC_STANDARD_ID = "ivo://ivoa.net/std/soda#async-1.0"
 
 MS = base.makeStruct
 
 
 class DatalinkDescriptorMixin:
-	"""some generally useful methods that various code
-	expects datalink descriptors to have.
+	"""some generally useful attributes that various code expects datalink
+	descriptors to have.
+
+	These are the makeLink/makeLinkFromFile methods, but also data (where
+	SODA processors accumulate the data they will serve), forSemantics
+	(the semantics we are trying to produce) and contentQualifier
+	(the sort of data at the other end; only touch this if you expect
+	your link to be sent through SAMP).
 	"""
 	data = None
 	forSemantics = DEFAULT_SEMANTICS
+	contentQualifier = None
 
 	def makeLink(self, url, **kwargs):
 		"""returns a LinkDef for this descriptor for url.
 
 		kwargs are passed on to LinkDef and include, in particular,
-		semantics, contentType, contentLength, description, and
-		localSemantics.
+		semantics, contentType, contentLength, description,
+		localSemantics, and contentQualifier.
 		"""
 		if not "semantics" in kwargs:
 			kwargs["semantics"] = self.forSemantics
@@ -59,7 +69,7 @@ class DatalinkDescriptorMixin:
 
 	def makeLinkFromFile(self, localPath, description, semantics=None,
 			service=None, contentType=None, suppressMissing=False,
-			localSemantics=None):
+			localSemantics=None, contentQualifier=None):
 		"""returns a LinkDef for a local file.
 
 		Arguments are as for LinkDef.fromFile, except you don't have
@@ -78,8 +88,9 @@ class DatalinkDescriptorMixin:
 			semantics = self.forSemantics
 
 		return LinkDef.fromFile(localPath, description, semantics,
-			service=service, contentType=contentType, 
-			suppressMissing=suppressMissing, localSemantics=localSemantics)
+			service=service, contentType=contentType,
+			suppressMissing=suppressMissing, localSemantics=localSemantics,
+			contentQualifier=contentQualifier)
 
 	@contextlib.contextmanager
 	def currentSemantics(self, sem):
@@ -93,6 +104,13 @@ class DatalinkDescriptorMixin:
 		finally:
 			self.forSemantics = oldSemantics
 
+	def cleanup(self):
+		"""this method is called at the end of a dlget run.
+
+		Use it to, for instance, close any files you still have open.  By
+		default, it does nothing.
+		"""
+
 
 class ProductDescriptor(DatalinkDescriptorMixin):
 	"""An encapsulation of information about some "product" (i.e., file).
@@ -103,13 +121,17 @@ class ProductDescriptor(DatalinkDescriptorMixin):
 	It also has an attribute data defaulting to None.  DataGenerators
 	set it, DataFilters potentially change it.
 
-	If you inherit from this method and you have a way to guess the
+	There is also a method cleanup() that is called at the end of dlget
+	processing; the default implementation does nothing, so there is no
+	need to up-call when you override it from here.
+
+	If you inherit from this class and you have a way to guess the
 	size of what the descriptor describes, override the estimateSize()
 	method.  The default will return a file size if accessPath points
 	to an existing file, None otherwise.
 	"""
 
-	def __init__(self, pubDID, accref, accessPath, mime, 
+	def __init__(self, pubDID, accref, accessPath, mime,
 			owner=None, embargo=None, sourceTable=None, datalink=None,
 			preview=None, preview_mime=None):
 		self.pubDID = pubDID
@@ -148,21 +170,25 @@ class FITSProductDescriptor(ProductDescriptor):
 	"""A SODA descriptor for FITS files.
 
 	On top of the normal product descriptor, this has an attribute hdr
-	containing a copy of the image header, and a method 
-	changingAxis (see there). 
+	containing a copy of the image header, and a method
+	changingAxis (see there).
 
 	There's also an attribute dataIsPristine that must be set to false
 	if changes have been made.  The formatter will spit out the original
 	data otherwise, ignoring your changes.
 
-	Finally, there's a slices attribute provided explained in 
+	Finally, there's a slices attribute provided explained in
 	soda#fits_doWCSCutout that can be used by data functions running before
 	it to do cutouts.
 
+	Append any files you have open during dlget processing to self.toClose.
+	The cleanup method makes sure they are closed before continuing.
+
 	The FITSProductDescriptor is constructed like a normal ProductDescriptor.
 	"""
 
 	def __init__(self, *args, **kwargs):
+		self.toClose = []
 		qnd = kwargs.pop("qnd", True)
 		ProductDescriptor.__init__(self, *args, **kwargs)
 
@@ -189,7 +215,7 @@ class FITSProductDescriptor(ProductDescriptor):
 	def changingAxis(self, axisIndex, parName):
 		"""must be called before cutting out along axisIndex.
 
-		axIndex is a FITS (1-based) axis index axIndex, parName the name of the 
+		axIndex is a FITS (1-based) axis index axIndex, parName the name of the
 		parameter that causes the cutout.
 		
 		This will simply return if nobody has called changingAxis with that index
@@ -202,6 +228,10 @@ class FITSProductDescriptor(ProductDescriptor):
 				" has been modified before."%axisIndex, parName)
 		self._axesTouched.add(axisIndex)
 
+	def cleanup(self):
+		for closable in self.toClose:
+			closable.close()
+
 
 class DLFITSProductDescriptor(FITSProductDescriptor):
 	"""A SODA descriptor for FITS files with datalink product paths.
@@ -216,7 +246,7 @@ class DLFITSProductDescriptor(FITSProductDescriptor):
 		FITSProductDescriptor.__init__(self, *args, **kwargs)
 
 
-def getFITSDescriptor(pubDID, accrefPrefix=None, 
+def getFITSDescriptor(pubDID, accrefPrefix=None,
 		cls=FITSProductDescriptor, qnd=False):
 	"""returns a datalink descriptor for a FITS file.
 
@@ -260,7 +290,7 @@ class _TemporaryFile(_File):
 class DescriptorGenerator(rscdef.ProcApp):
 	"""A procedure application for making product descriptors for PUBDIDs
 
-	Despite the name, a descriptor generator has to *return* (not yield) 
+	Despite the name, a descriptor generator has to *return* (not yield)
 	a descriptor instance.  While this could be anything, it is recommended
 	to derive custom classes from prodocols.datalink.ProductDescrpitor, which
 	exposes essentially the columns from DaCHS' product table as attributes.
@@ -288,7 +318,7 @@ class DescriptorGenerator(rscdef.ProcApp):
 	the default (//soda#fromStandardPubDID) should do.
 
 	If you need to derive custom descriptor classes, you can see the base
-	class under the name ProductDescriptor; there's also 
+	class under the name ProductDescriptor; there's also
 	FITSProductDescriptor and DatalinkFault in each proc's namespace.
 	If your Descriptor does not actually refer to something in the
 	product table, it is likely that you want to set the descriptor's
@@ -310,16 +340,16 @@ class DescriptorGenerator(rscdef.ProcApp):
 
 
 class LinkDef:
-	"""A definition of a datalink related document.
+	"""a definition of a datalink related document.
 
 	These are constructed at least with:
 
 	- the pubDID (as a string)
 	- the access URL (as a string)
 
-	In addition, we accept the remaining column names from 
+	In addition, we accept the remaining column names from
 	`//datalink#dlresponse`_ as keyword arguments, except we use
-	camel case here, i.e., 
+	camel case here, i.e.,
 	``description``, ``semantics``, ``contentType``, ``contentLength``,
 	``localSemantics``.
 
@@ -329,20 +359,22 @@ class LinkDef:
 	whenever you can; that will improve error messages.
 	"""
 	def __init__(self, pubDID, accessURL,
-			serviceType=None, 
+			serviceType=None,
 			errorMessage=None,
-			description=None, 
+			description=None,
 			semantics=DEFAULT_SEMANTICS,
-			contentType=None, 
+			contentType=None,
 			contentLength=None,
-			localSemantics=None):
+			localSemantics=None,
+			contentQualifier=None):
 		ID = pubDID #noflake: used in locals()
 		del pubDID
 		self.dlRow = locals()
 
 	@classmethod
-	def fromFile(cls, localPath, description, semantics, 
-			service, contentType=None, localSemantics=None, 
+	def fromFile(cls, localPath, description, semantics,
+			service, contentType=None, localSemantics=None,
+			contentQualifier=None,
 			suppressMissing=False):
 		"""constructs a LinkDef based on a local file.
 		
@@ -354,7 +386,7 @@ class LinkDef:
 		must allow the static renderer and have a staticData property.  It should
 		normally be the datalink service itself, which in a metaMaker
 		is accessible as self.parent.parent.  It is, however, legal
-		to reference other suitable services (use self.parent.rd.getById or 
+		to reference other suitable services (use self.parent.rd.getById or
 		base.resolveCrossId)
 
 		If you pass suppressMissing=True, a link to a non-existing file
@@ -380,20 +412,21 @@ class LinkDef:
 			svcPath = utils.getRelativePath(localPath, staticPath)
 		except ValueError:
 			return LinkDef(pubDID, errorMessage="FatalFault: Linked item"
-				" not accessible through the given service", 
-				semantics=semantics, description=description, 
+				" not accessible through the given service",
+				semantics=semantics, description=description,
 				localSemantics=localSemantics)
 
 		ext = os.path.splitext(localPath)[-1]
-		contentType = (contentType 
+		contentType = (contentType
 			or static.File.contentTypes.get(ext, "application/octet-stream"))
 
-		return cls(pubDID, 
+		return cls(pubDID,
 			service.getURL("static")+"/"+svcPath,
 			description=description, semantics=semantics,
-			contentType=contentType, 
+			contentType=contentType,
 			contentLength=os.path.getsize(localPath),
-			localSemantics=localSemantics)
+			localSemantics=localSemantics,
+			contentQualifier=contentQualifier)
 
 	def asDict(self):
 		"""returns the link definition in a form suitable for ingestion
@@ -408,7 +441,70 @@ class LinkDef:
 			"semantics": self.dlRow["semantics"],
 			"content_type": self.dlRow["contentType"],
 			"content_length": self.dlRow["contentLength"],
-			"local_semantics": self.dlRow["localSemantics"]}
+			"local_semantics": self.dlRow["localSemantics"],
+			"content_qualifier": self.dlRow["contentQualifier"]}
+
+
+class ProcLinkDef:
+	"""a definition of a datalink processing service other than self's dlget.
+
+	Whenever you datalink service allows a dlget, a descriptor for that
+	will be generated based on the keys returned by the metamaker.  Sometimes,
+	you want to provide a second processing service within a single datalink
+	document.  In that case, yield a ProcLinkDef objects constructed
+	with a dlget-allowing datalink service in its second argument.  As usual
+	for LinkDefs, the first argument must be the DID for which to generate
+	the service.
+
+	The description of this link will be the description of that service.
+	"""
+	def __init__(self, pubDID, dlSvc):
+		if (not isinstance(dlSvc, svcs.Service)
+				or not "dlget" in dlSvc.allowed):
+			raise base.StructureError("ProcLinkDef must be constructed"
+				" with a dlget-renderable service.")
+		self.pubDID = pubDID
+		self.svcId = None
+		self.description = base.getMetaText(dlSvc, "description",
+				default="Another interactive service on this dataset.")
+
+		# this is for stealing by adaptForRenderer
+		args = {"ID": [self.pubDID]} #noflake: for stealing
+		dlget = svcs.getRenderer("dlget")
+		dlgetCore = dlSvc.core.adaptForRenderer(
+			dlget, {"dbLimit": None})
+
+		self.serviceDescriptor = _ServiceDescriptor(
+			self.pubDID,
+			dlgetCore.inputKeys,
+			"dlget",
+			dlSvc)
+
+	def asDict(self):
+		"""returns a datalink row for this service.
+
+		This is called by //datalink#make_response for the table row.
+		"""
+		if self.svcId is None:
+			raise base.Error("You need to arrange for asVOT to be called first"
+				f" on this {self.__class__}.")
+		return {
+			"ID": self.pubDID,
+			"service_def": self.svcId,
+			"description": self.description,
+			"semantics": "#proc"}
+
+	def asVOT(self, ctx):
+		"""returns a VOTable fragment for the service descriptor
+		
+		This called by runForMeta.  It currently constructs an actual
+		Datalink core for every pubDID, which is *relatively* expensive.
+		If you find your datalink service is prohibitively slow, we may
+		want to re-think this.
+		"""
+		self.svcId = ctx.getOrMakeIdFor(
+			self.serviceDescriptor, suggestion="ext_svc")
+		return self.serviceDescriptor.asVOT(ctx)
 
 
 class _ServiceDescriptor(object):
@@ -418,9 +514,10 @@ class _ServiceDescriptor(object):
 	Basically, these collect input keys, a pubDID, as well as any other
 	data we might need in service definition.
 	"""
-	def __init__(self, pubDID, inputKeys, rendName, description):
+	def __init__(self, pubDID, inputKeys, rendName, dlSvc):
 		self.pubDID, self.inputKeys = pubDID, inputKeys
-		self.rendName, self.description = rendName, description
+		self.rendName, self.dlSvc = rendName, dlSvc
+
 		if self.pubDID:
 		# if we're fixed to a specific pubDID, reflect that in the ID
 		# field -- this is how clients know which dataset to pull
@@ -431,14 +528,26 @@ class _ServiceDescriptor(object):
 					ik.set(pubDID)
 					self.inputKeys[index] = ik
 
-	def asVOT(self, ctx, accessURL, linkIdTo=None):
+	@property
+	def description(self):
+		return base.getMetaText(self.dlSvc,
+			"description",
+			default="An interactive service on this dataset.")
+
+	@property
+	def title(self):
+		return base.getMetaText(self.dlSvc,
+			"title",
+			default="Processing service with id {}".format(self.dlSvc.id))
+
+	def asVOT(self, ctx, linkIdTo=None):
 		"""returns VOTable stanxml for a description of this service.
 
 		This is a RESOURCE as required by Datalink.
 
 		linkIdTo is used to support data access descriptors embedded
 		in discovery queries.  It is the id of the column containing
-		the identifiers.  SSA can already provide this. 
+		the identifiers.  SSA can already provide this.
 		"""
 		paramsByName, stcSpecs = {}, set()
 		for param in self.inputKeys:
@@ -448,21 +557,26 @@ class _ServiceDescriptor(object):
 
 		def getIdFor(colRef):
 			colRef.toParam = True
-			return ctx.getOrMakeIdFor(paramsByName[colRef.dest], 
+			return ctx.getOrMakeIdFor(paramsByName[colRef.dest],
 				suggestion=colRef.dest)
-
-		res = V.RESOURCE(ID=ctx.getOrMakeIdFor(self, suggestion="proc_svc"), 
-			name="proc_svc", type="meta", utype="adhoc:service")[
+	
+		res = V.RESOURCE(ID=ctx.getOrMakeIdFor(self, suggestion="proc_svc"),
+				name=self.dlSvc.id,
+				type="meta",
+				utype="adhoc:service")[
 			V.DESCRIPTION[self.description],
+			V.INFO(name="title", value=self.title),
 			[modelgroups.marshal_STC(ast, getIdFor)[0]
 				for ast in stcSpecs],
-			V.PARAM(arraysize="*", datatype="char", 
+			V.PARAM(arraysize="*", datatype="char",
 				name="accessURL", ucd="meta.ref.url",
-				value=accessURL)]
+				value=self.dlSvc.getURL(self.rendName))]
 
-		standardId = {
-			"dlasync": "ivo://ivoa.net/std/SODA#async-1.0",
-			"dlget": "ivo://ivoa.net/std/SODA#sync-1.0"}.get(self.rendName)
+		standardId = base.getMetaText(self.dlSvc, "standardID", default=None)
+		if standardId is None:
+			standardId = {
+				"dlasync": ASYNC_STANDARD_ID,
+				"dlget": SYNC_STANDARD_ID}.get(self.rendName)
 		if standardId:
 			res[
 				V.PARAM(arraysize="*", datatype="char",
@@ -480,7 +594,7 @@ class _ServiceDescriptor(object):
 		
 			if ik.hasProperty("defaultForForm"):
 				param[
-					V.LINK(action="rdf", content_role="#pre-set", 
+					V.LINK(action="rdf", content_role="#pre-set",
 						value=str(ik.getProperty("defaultForForm")))]
 			inputParams[param]
 
@@ -516,8 +630,8 @@ class MetaMaker(rscdef.ProcApp):
 	(of course, you should give more metadata -- ucds, better description,
 	etc) in production).
 
-	Note that InputKey-returning MetaMakers cannot rely 
-	on descriptor.pubDID to actually have 
+	Note that InputKey-returning MetaMakers cannot rely
+	on descriptor.pubDID to actually have
 	a value; in particular SSA may construct cores to produce "direct"
 	processing datalink descriptors.  When you need a pubDID, just return if
 	descriptor.pubDID is None.
@@ -551,6 +665,8 @@ class MetaMaker(rscdef.ProcApp):
 	- Values -- the class to make for input parameters' values attributes
 	- Options -- used by Values
 	- LinkDef -- a class to define further links within datalink services.
+	- ProcLinkDef -- a class to reference dlget services outside of the current
+	  service
 	- DatalinkFault -- a container of datalink error generators
 	- soda -- the soda module.
 	"""
@@ -564,6 +680,7 @@ class MetaMaker(rscdef.ProcApp):
 		"Values": rscdef.Values,
 		"Option": rscdef.Option,
 		"LinkDef": LinkDef,
+		"ProcLinkDef": ProcLinkDef,
 		"DatalinkFault": DatalinkFault,
 		"soda": soda,
 	}
@@ -583,7 +700,7 @@ class DataFunction(rscdef.ProcApp):
 
 	All these operate on the data attribute of the product descriptor.
 	The first data function plays a special role: It *must* set the data
-	attribute (or raise some appropriate exception), or a server error will 
+	attribute (or raise some appropriate exception), or a server error will
 	be returned to the client.
 
 	What is returned depends on the service, but typcially it's going to
@@ -606,10 +723,10 @@ class DataFunction(rscdef.ProcApp):
 		- FormatNow -- exception to raise to go directly to the formatter
 		- DeliverNow -- exception to raise to skip all further formatting
 			and just deliver what's currently in descriptor.data
-		- File(path, type) -- if you just want to return a file on disk, pass 
-			its path and media type to File and assign the result to 
-			descriptor.data.  
-		- TemporaryFile(path,type) -- as File, but the disk file is 
+		- File(path, type) -- if you just want to return a file on disk, pass
+			its path and media type to File and assign the result to
+			descriptor.data.
+		- TemporaryFile(path,type) -- as File, but the disk file is
 			unlinked after use
 		- makeData -- the rsc.makeData function
 		- soda -- the protocols.soda module
@@ -623,8 +740,8 @@ class DataFunction(rscdef.ProcApp):
 		"FormatNow": FormatNow,
 		"DeliverNow": DeliverNow,
 		"File": _File,
-		"TemporaryFile": _TemporaryFile, 
-		"makeData": rsc.makeData, 
+		"TemporaryFile": _TemporaryFile,
+		"makeData": rsc.makeData,
 		"soda": soda,
 		"DataFromURL": products.RemoteProduct.fromURL,
 	}
@@ -638,7 +755,7 @@ class DataFormatter(rscdef.ProcApp):
 	a pair of (mime-type, bytes), which is understood by most renderers.
 
 	When no dataFormatter is given for a core, it will return descriptor.data
-	directly.  This can work with the datalink renderer itself if 
+	directly.  This can work with the datalink renderer itself if
 	descriptor.data will work as a nevow resource (i.e., has a renderHTTP
 	method, as our usual products do).  Consider, though, that renderHTTP
 	runs in the main event loop and thus most not block for extended
@@ -651,9 +768,9 @@ class DataFormatter(rscdef.ProcApp):
 	In addition to the usual names available to ProcApps, data formatters have:
 		- Page -- base class for resources with renderHTTP methods.
 		- IRequest -- the nevow interface to make Request objects with.
-		- File(path, type) -- if you just want to return a file on disk, pass 
-			its path and media type to File and return the result. 
-		- TemporaryFile(path, type) -- as File, but the disk file is unlinked 
+		- File(path, type) -- if you just want to return a file on disk, pass
+			its path and media type to File and return the result.
+		- TemporaryFile(path, type) -- as File, but the disk file is unlinked
 			after use
 		- soda -- the protocols.soda module
 	"""
@@ -665,20 +782,20 @@ class DataFormatter(rscdef.ProcApp):
 		"Page": nevowc.TemplatedPage,
 		"IRequest": lambda arg:arg, # compatibility wrapper
 		"File": _File,
-		"TemporaryFile": _TemporaryFile, 
+		"TemporaryFile": _TemporaryFile,
 		"soda": soda,
 	}
 
 
 class DatalinkCoreBase(svcs.Core, base.ExpansionDelegator):
-	"""Basic functionality for datalink cores.  
+	"""Basic functionality for datalink cores.
 
 	This is pulled out of the datalink core proper as it is used without
 	the complicated service interface sometimes, e.g., by SSAP.
 	"""
 
 	_descriptorGenerator = base.StructAttribute("descriptorGenerator",
-		default=base.NotGiven, 
+		default=base.NotGiven,
 		childFactory=DescriptorGenerator,
 		description="Code that takes a PUBDID and turns it into a"
 			" product descriptor instance.  If not given,"
@@ -717,18 +834,20 @@ class DatalinkCoreBase(svcs.Core, base.ExpansionDelegator):
 		" front of the proc apps that interpret them.",
 		copyable=True)
 
+	# This tells the context grammar to fail if someone tries to pass
+	# in parameters we don't have input keys for.
 	rejectExtras = True
 
 	def completeElement(self, ctx):
 		if self.descriptorGenerator is base.NotGiven:
-			self.descriptorGenerator = MS(DescriptorGenerator, 
+			self.descriptorGenerator = MS(DescriptorGenerator,
 				procDef=base.resolveCrossId("//soda#fromStandardPubDID"))
 
 		if self.dataFormatter is base.NotGiven:
-			self.dataFormatter = MS(DataFormatter, 
+			self.dataFormatter = MS(DataFormatter,
 				procDef=base.caches.getRD("//soda").getById("trivialFormatter"))
 		
-		self.inputKeys.append(MS(svcs.InputKey, name="ID", type="text", 
+		self.inputKeys.append(MS(svcs.InputKey, name="ID", type="text",
 			ucd="meta.id;meta.main",
 			multiplicity="multiple",
 			std=True,
@@ -749,12 +868,12 @@ class DatalinkCoreBase(svcs.Core, base.ExpansionDelegator):
 		and this core.
 		"""
 		linkDefs, inputKeys, errors = [], self.inputKeys[:], []
-	
+
 		for metaMaker in self.metaMakers:
 			with descriptor.currentSemantics(metaMaker.semantics):
 				try:
 					for item in metaMaker.compile(self)(self, descriptor):
-						if isinstance(item, LinkDef):
+						if isinstance(item, (LinkDef, ProcLinkDef)):
 							linkDefs.append(item)
 						elif isinstance(item, DatalinkFault):
 							errors.append(item)
@@ -779,7 +898,7 @@ class DatalinkCoreBase(svcs.Core, base.ExpansionDelegator):
 
 		That's #that and #preview unless some special conditions apply.
 		"""
-		if (not isinstance(descriptor, ProductDescriptor) 
+		if (not isinstance(descriptor, ProductDescriptor)
 				or hasattr(descriptor, "suppressAutoLinks")):
 			return
 
@@ -792,20 +911,22 @@ class DatalinkCoreBase(svcs.Core, base.ExpansionDelegator):
 				mediaType = "image/fits"
 			else:
 				mediaType = formats.guessMediaType(descriptor.accref),
-			yield LinkDef(descriptor.pubDID, 
+			yield LinkDef(descriptor.pubDID,
 				service.getURL("dlget")+"?ID=%s"%urllib.parse.quote(
 					descriptor.pubDID),
 				description="The full dataset.",
 				contentType=mediaType,
 				contentLength=descriptor.estimateSize(),
+				contentQualifier=getattr(descriptor, "contentQualifier", None),
 				semantics="#this")
 
 		else:
-			yield LinkDef(descriptor.pubDID, 
+			yield LinkDef(descriptor.pubDID,
 				products.makeProductLink(descriptor.accref),
 				description="The full dataset.",
 				contentType=descriptor.mime,
 				contentLength=descriptor.estimateSize(),
+				contentQualifier=getattr(descriptor, "contentQualifier", None),
 				semantics="#this")
 
 		if getattr(descriptor, "preview", None):
@@ -817,16 +938,15 @@ class DatalinkCoreBase(svcs.Core, base.ExpansionDelegator):
 				# It's either AUTO or a local path; in both cases, let
 				# the products infrastructure worry about it.
 				previewLink = products.makeProductLink(
-					products.RAccref(descriptor.accref, 
+					products.RAccref(descriptor.accref,
 						inputDict={"preview": True}))
 
 			yield LinkDef(descriptor.pubDID,
-				previewLink, 
+				previewLink,
 				description="A preview for the dataset.",
-				contentType=descriptor.previewMime, 
+				contentType=descriptor.previewMime,
 				semantics="#preview")
 
-
 	def getDatalinksResource(self, ctx, service):
 		"""returns a VOTable RESOURCE element with the data links.
 
@@ -841,9 +961,9 @@ class DatalinkCoreBase(svcs.Core, base.ExpansionDelegator):
 
 		internalLinks.extend(
 			LinkDef(
-				s.pubDID, 
+				s.pubDID,
 				None,
-				serviceType=ctx.getOrMakeIdFor(s, suggestion="procsvc"), 
+				serviceType=ctx.getOrMakeIdFor(s),
 				description=s.description,
 				semantics="#proc")
 			for s in self.datalinkEndpoints)
@@ -937,14 +1057,12 @@ class DatalinkCore(DatalinkCoreBase):
 					if len(inputKeys)>1 and rendName in allowedForSvc:
 						endpoints.append(
 							_ServiceDescriptor(
-								descriptor.pubDID, 
-								inputKeys, 
+								descriptor.pubDID,
+								inputKeys,
 								rendName,
-								base.getMetaText(self.parent, 
-									"dlget.description", 
-									default="An interactive service on this dataset.")))
+								self.parent))
 
-			if (renderer.name=="dlmeta" 
+			if (renderer.name=="dlmeta"
 					and maxRec is not None
 					and len(linkDefs)+len(errors)>=maxRec):
 				descriptors[curInd+1:] = []
@@ -958,23 +1076,23 @@ class DatalinkCore(DatalinkCoreBase):
 			pass # RESPONSEFORMAT and friends added through pql#DALIPars
 
 		else:
-			# we're a data generating core;  inputKeys are the core's plus 
-			# possibly those of actual processors.  Right now, we assume they're
-			# all the same, so we take the last one as representative
-			#
-			# TODO: this restricts the use of the core to dlget and dlasync
-			# (see endpoint creation above).  It's not clear that's what we
-			# want, as e.g. form may work fine as well.
+			# in dlget, we don't actually know what to do with multiple
+			# descriptors.  But we will fault out if the last (and usually
+			# only one) has a problem.
 			if not descriptors:
 				raise base.ValidationError("ID is mandatory with dlget",
 					"ID")
+			# If there are multiple endpoints, we take the parameters from
+			# the last one.  If there are none, we keep the ID of the
+			# main service interface.
 			if endpoints:
-				inputKeys.extend(endpoints[-1].inputKeys)
+				inputKeys = endpoints[-1].inputKeys
 			if isinstance(descriptors[-1], DatalinkFault):
 				descriptors[-1].raiseException()
 
-		res = self.change(inputTable=MS(svcs.InputTD, 
-			inputKeys=inputKeys, exclusive=True))
+		res = self.change(inputTable=MS(svcs.InputTD,
+			inputKeys=inputKeys, exclusive=True),
+			inputKeys=inputKeys)
 
 		# again dispatch on meta or data, this time as regards what to run.
 		if renderer.name=="dlmeta":
@@ -1004,10 +1122,10 @@ class DatalinkCore(DatalinkCoreBase):
 		It is particularly important to never let service cache the
 		cores returned for the dl* renderers; hence to "nocache" magic.
 
-		This tries to generate all datalink-relevant metadata in one go 
+		This tries to generate all datalink-relevant metadata in one go
 		and avoid calling the descriptorGenerator(s) more than once per
 		pubDID.  It therefore adds datalinkLinks, datalinkEndpoints,
-		and datalinkDescriptors attributes.  These are used later
+		and descriptors attributes.  These are used later
 		in either metadata generation or data processing.
 
 		An additional complication is MAXREC; when adaptForDescriptors
@@ -1016,7 +1134,7 @@ class DatalinkCore(DatalinkCoreBase):
 		MAXREC is ignored; we don't really do multi-ID dlget right
 		now, anyway.
 
-		The latter will in general use only the last pubDID passed in.  
+		The latter will in general use only the last pubDID passed in.
 		Therefore, this last pubDID determines the service interface
 		for now.  Perhaps we should be joining the inputKeys in some way,
 		though, e.g., if we want to allow retrieving multiple datasets
@@ -1055,7 +1173,7 @@ class DatalinkCore(DatalinkCoreBase):
 
 				descriptors.append(desc)
 			except svcs.RedirectBase:
-				# let through redirects even for dlmeta; these are rendered 
+				# let through redirects even for dlmeta; these are rendered
 				# further up in the call chain.
 				raise
 
@@ -1084,23 +1202,41 @@ class DatalinkCore(DatalinkCoreBase):
 		the datalink rows produced by service.
 		"""
 		for dlSvc in self.datalinkEndpoints:
-			yield dlSvc.asVOT(ctx, service.getURL(dlSvc.rendName))
+			yield dlSvc.asVOT(ctx)
+
+		for linkDef in self.datalinkLinks:
+			# normal LinkDefs can also contribute VOTable material directly;
+			# presumably, that'll always be RESOURCE blocks of dlget services,
+			# but who knows: perhaps we'll one day want to return an extra
+			# info or something?
+			if hasattr(linkDef, "asVOT"):
+				yield linkDef.asVOT(ctx)
 
 	def runForMeta(self, service, inputTable, queryMeta):
 		"""returns a rendered VOTable containing the datalinks.
 		"""
 		try:
 			ctx = votablewrite.VOTableContext(tablecoding="td")
-			vot = V.VOTABLE[
-					self.getDatalinksResource(ctx, service), 
-					self._iterAccessResources(ctx, service)]
+
+			# we'll need to serialise the access resources first, as ctx has to
+			# produce their id-s before their table rows are produced.
+			accessResources = list(
+				self._iterAccessResources(ctx, service))
+
+			dlres = self.getDatalinksResource(ctx, service)
 
 			if self.overflowed:
-				vot[V.INFO(name="QUERY_STATUS", value="OVERFLOW")[
+				dlres[V.INFO(name="QUERY_STATUS", value="OVERFLOW")[
 					"Results cropped after hitting the MAXREC you passed in."]]
 
 			else:
-				vot[V.INFO(name="QUERY_STATUS", value="OK")]
+				dlres[V.INFO(name="QUERY_STATUS", value="OK")]
+
+			dlres[V.INFO(name="standardID",
+				value=LINKS_STANDARD_ID)[
+				"Written by DaCHS %s %s"%(base.getVersion(), self.__class__.__name__)]]
+
+			vot = V.VOTABLE[dlres, accessResources]
 
 			if "text/html" in queryMeta["accept"]:
 				# we believe it's a web browser; let it do stylesheet magic
@@ -1144,6 +1280,7 @@ class DatalinkCore(DatalinkCoreBase):
 			res = self.dataFormatter.compile(self)(descriptor, args)
 			return res
 		finally:
+			self.descriptors[-1].cleanup()
 			self.finalize()
 	
 	def finalize(self):
@@ -1168,23 +1305,22 @@ def makeDatalinkServiceDescriptor(ctx, service, tableDef, columnName):
 	What's returned is gavo.votable elements.
 
 	ctx is a votablewrite VOTableContext that manages the IDs of the elements
-	involved, service is the datalink service as a svc.Service instance, 
+	involved, service is the datalink service as a svc.Service instance,
 	tableDef is the rscdef.TableDef instance with the rows the datalink service
 	operates on, and columnName names the column within table to take
 	the datalink's ID parameter from.
 	"""
-	return V.RESOURCE(type="meta", utype="adhoc:service", 
-			name=base.getMetaText(service, "name", default="links", propagate=False))[
+	return V.RESOURCE(type="meta", utype="adhoc:service", name=service.id)[
 		V.DESCRIPTION[base.getMetaText(service, "description",
 			default="A Datalink service to retrieve the data set"
 			" as well as additional related files, plus possibly services"
 			" for server-side processing.", propagate=False)],
 		V.PARAM(name="standardID", datatype="char", arraysize="*",
-			value="ivo://ivoa.net/std/DataLink#links-1.0"),
+			value=LINKS_STANDARD_ID),
 		V.PARAM(name="accessURL", datatype="char", arraysize="*",
 			value=service.getURL("dlmeta")),
 		V.GROUP(name="inputParams")[
-			V.PARAM(name="ID", datatype="char", arraysize="*", 
+			V.PARAM(name="ID", datatype="char", arraysize="*",
 				ref=ctx.getOrMakeIdFor(
 					tableDef.getColumnByName(columnName), suggestion=columnName),
 				ucd="meta.id;meta.main")]]
