@@ -12,8 +12,8 @@ The "user-visible" part are just accrefs, as modelled by the RAccref
 -- they can contain instructions for cutouts or scaling, hence the additional
 structure.
 
-Using the product table and some logic in this module, such accrefs 
-are turned into subclasses of ProductBase.  
+Using the product table and some logic in this module, such accrefs
+are turned into subclasses of ProductBase.
 
 These have mime types and know how to generate their data through their
 synchronous iterData methods.  They must also work as t.w resources and thus
@@ -23,7 +23,7 @@ it if we don't, and for now it doesn't seem we'll support a different framework
 in the foreseeable future.
 """
 
-#c Copyright 2008-2022, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -81,8 +81,8 @@ def makePreviewFromFITS(product):
 			inFile = product.getFile()
 
 		with utils.fitsLock():
-			pixels = numpy.array([row 
-				for row in fitstools.iterScaledRows(inFile, 
+			pixels = numpy.array([row
+				for row in fitstools.iterScaledRows(inFile,
 					destSize=PREVIEW_SIZE)])
 	else:
 		raise NotImplementedError("TODO: Fix fitstools.iterScaledRows"
@@ -109,7 +109,7 @@ def makePreviewWithPIL(product):
 _PIL_COMPATIBLE_MIMES = frozenset(['image/jpeg', 'image/png'])
 
 def computePreviewFor(product):
-	"""returns a deferred returning image/jpeg bytes containing a 
+	"""returns a deferred returning image/jpeg bytes containing a
 	preview of product.
 
 	This only works for a select subset of products.  You're usually
@@ -144,7 +144,7 @@ class PreviewCacheManager(object):
 	so provided a preview has been generated successfully.
 
 	A cache file is touched when it is used, so you can clean up rarely used
-	cache files by deleting all files in the preview cache older than some 
+	cache files by deleting all files in the preview cache older than some
 	limit.
 	"""
 	cachePath = base.getConfig("web", "previewCache")
@@ -207,19 +207,19 @@ class ProductBase(object):
 	See the module docstring for the big picture.
 
 	The constructor arguments of RAccrefs depend on what they are.
-	The common interface is the the class method 
+	The common interface is the the class method
 	fromRAccref(rAccref, authGroups=None).
 	It returns None if the RAccref is not for a product of the
-	respective sort, the product otherwise.  
+	respective sort, the product otherwise.
 
 	authGroups is a set of groups authorised for the user when
 	controlling access to embargoed products.  This is the main
 	reason you should never hand out products yourself but always
 	expose the to the user through the product core.
 
-	The actual constructor requires a RAccref, which is exposed as the 
-	rAccref attribute.  Do not use the productsRow attribute from rAccref, 
-	though, as constructors may want to manipulate the content of the 
+	The actual constructor requires a RAccref, which is exposed as the
+	rAccref attribute.  Do not use the productsRow attribute from rAccref,
+	though, as constructors may want to manipulate the content of the
 	product row (e.g., in NonExistingProduct).  Access the product
 	row as self.pr in product classes.
 
@@ -272,7 +272,7 @@ class ProductBase(object):
 		return str(self)
 	
 	def __eq__(self, other):
-		return (isinstance(other, self.__class__) 
+		return (isinstance(other, self.__class__)
 			and self.rAccref==other.rAccref)
 	
 	def __ne__(self, other):
@@ -284,7 +284,7 @@ class ProductBase(object):
 
 	@classmethod
 	def setupRealFile(cls, openMethod):
-		"""changes cls such that read and close work an an actual file-like 
+		"""changes cls such that read and close work an an actual file-like
 		object rather than the inefficient iterData.
 
 		openMethod has to be an instance method of the class returning
@@ -484,7 +484,7 @@ class RemotePreview(RemoteProduct):
 
 
 class UnauthorizedProduct(FileProduct):
-	"""A local file that is not delivered to the current client. 
+	"""A local file that is not delivered to the current client.
 
 	iterData returns the data for the benefit of preview making.
 	However, there is a render method, so the product renderer will
@@ -495,7 +495,7 @@ class UnauthorizedProduct(FileProduct):
 	@classmethod
 	def fromRAccref(cls, rAccref, authGroups=None):
 		dbRow = rAccref.productsRow
-		if (dbRow["embargo"] is None 
+		if (dbRow["embargo"] is None
 				or dbRow["embargo"]<datetime.date.today()):
 			return None
 		if authGroups is None or dbRow["owner"] not in authGroups:
@@ -587,7 +587,7 @@ class InvalidProduct(NonExistingProduct):
 class CutoutProduct(ProductBase):
 	"""A class representing cutouts from FITS files.
 	
-	This only works for local FITS files with two axes.  For everything 
+	This only works for local FITS files with two axes.  For everything
 	else, use datalink.
 	
 	We assume the cutouts are smallish -- they are, right now, not
@@ -611,7 +611,7 @@ class CutoutProduct(ProductBase):
 	def _getCutoutHDU(self):
 		ra, dec, sra, sdec = [self.rAccref.params[k] for k in self._myKeys]
 		with utils.fitsLock():
-			hdus = pyfits.open(self.rAccref.localpath, 
+			hdus = pyfits.open(self.rAccref.localpath,
 				do_not_scale_image_data=True,
 				memmap=True)
 			try:
@@ -620,7 +620,7 @@ class CutoutProduct(ProductBase):
 					numpy.round(skyWCS.wcs_world2pix([
 						(ra-sra/2., dec-sdec/2.),
 						(ra+sra/2., dec+sdec/2.)], 1)), numpy.int32)
-				res = fitstools.cutoutFITS(hdus[0], 
+				res = fitstools.cutoutFITS(hdus[0],
 					(skyWCS.longAxis, min(pixelFootprint[:,0]), max(pixelFootprint[:,0])),
 					(skyWCS.latAxis, min(pixelFootprint[:,1]), max(pixelFootprint[:,1])))
 			finally:
@@ -629,7 +629,7 @@ class CutoutProduct(ProductBase):
 		return res
 
 	def iterData(self):
-		# we need to write into a BytesIO (rather than to request) because 
+		# we need to write into a BytesIO (rather than to request) because
 		# pyfits wants to seek
 		res = self._getCutoutHDU()
 		bytes = BytesIO()
@@ -742,8 +742,8 @@ class ProductCore(svcs.DBCore):
 
 	The input data consists of accref; you can use the string form
 	of RAccrefs, and if you renderer wants, it can pass in ready-made
-	RAccrefs.  You can pass accrefs in through both an accref 
-	param and table rows.  
+	RAccrefs.  You can pass accrefs in through both an accref
+	param and table rows.
 	
 	The accref param is the normal way if you just want to retrieve a single
 	image, the table case is for building tar files and such.  There is one core
@@ -820,7 +820,7 @@ class RAccref(object):
 	a question mark.  This representation is be parsed by the fromString
 	function.
 
-	RAccrefs have a (read only) property productsRow attribute -- that's 
+	RAccrefs have a (read only) property productsRow attribute -- that's
 	a dictionary containing the row for accres from //products#products
 	if that exists.  If it doesn't, accessing the property will raise
 	an NotFoundError.
@@ -851,7 +851,7 @@ class RAccref(object):
 				inputDict[key] = value[-1]
 
 		# Save old URLs: if no (real) path was passed, try to get it
-		# from key.  Remove this ca. 2014, together with 
+		# from key.  Remove this ca. 2014, together with
 		# RaccrefTest.(testPathFromKey|testKeyMandatory)
 		if not path.strip("/").strip():
 			if "key" in inputDict:
@@ -886,7 +886,7 @@ class RAccref(object):
 		qSep = keyString.rfind("?")
 		if qSep!=-1:
 			return cls.fromPathAndArgs(
-				unquoteProductKey(keyString[:qSep]), 
+				unquoteProductKey(keyString[:qSep]),
 				urllib.parse.parse_qs(keyString[qSep+1:]))
 
 		return cls(unquoteProductKey(keyString))
@@ -929,7 +929,7 @@ class RAccref(object):
 		return str(self)
 
 	def __eq__(self, other):
-		return (isinstance(other, RAccref) 
+		return (isinstance(other, RAccref)
 			and self.accref==other.accref
 			and self.params==other.params)
 
@@ -953,7 +953,7 @@ class RAccref(object):
 		try:
 			return self._localpathCache
 		except AttributeError:
-			self._localpathCache = os.path.join(base.getConfig("inputsDir"), 
+			self._localpathCache = os.path.join(base.getConfig("inputsDir"),
 				self.productsRow["accessPath"])
 		return self._localpathCache
 
@@ -1033,7 +1033,7 @@ def formatProductLink(val, useHost):
 	must be an accref into the product table).
 	"""
 	if val:
-		# type check to allow cut-out or scaled accrefs (which need 
+		# type check to allow cut-out or scaled accrefs (which need
 		# makeProductLink in any case)
 		if isinstance(val, str) and utils.looksLikeURLPat.match(val):
 			return val
@@ -1051,8 +1051,7 @@ def _productMapperFactory(colDesc):
 	like a URL to begin with.
 	"""
 	if not (
-			colDesc["name"]=="accref"
-			or colDesc["utype"]=="ssa:Access.Reference"
+			colDesc["utype"]=="ssa:Access.Reference"
 			or colDesc["ucd"]=="VOX:Image_AccessReference"
 			or colDesc["displayHint"].get("type")=="product"):
 		return
