@@ -2,7 +2,7 @@
 TAP: schema maintenance, job/parameter definition incl. upload and UWS actions.
 """
 
-#c Copyright 2008-2022, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -72,7 +72,7 @@ def getSupportedLanguages():
 	This is SUPPORTED_LANGUAGES in a format suitable for the
 	TAP capabilities element.
 
-	Each tuple returned is made up of 
+	Each tuple returned is made up of
 	(name, description, [(version, ivo-id)...]).
 	"""
 	for name, desc in SUPPORTED_LANGUAGES.items():
@@ -113,7 +113,7 @@ def _insertRDIntoTAP_SCHEMA(rd, connection):
 	else:
 		return
 	tapRD = base.caches.getRD(RD_ID)
-	for ddId in ["importTablesFromRD", "importDMsFromRD", "importColumnsFromRD", 
+	for ddId in ["importTablesFromRD", "importDMsFromRD", "importColumnsFromRD",
 			"importFkeysFromRD", "importGroupsFromRD"]:
 		dd = tapRD.getById(ddId)
 		rsc.makeData(dd, forceSource=rd, parseOptions=rsc.parseValidating,
@@ -123,12 +123,12 @@ def _insertRDIntoTAP_SCHEMA(rd, connection):
 	# with adql-hidden tables, when moving RDs, etc.  Such leftovers
 	# are trouble because they're in TAP_SCHEMA but not in /tables any more.
 	connection.execute("""
-		DELETE FROM tap_schema.schemas 
+		DELETE FROM tap_schema.schemas
 		WHERE
 			schema_name IN (
-				SELECT schema_name 
-				FROM tap_schema.schemas 
-					LEFT OUTER JOIN tap_schema.tables USING (schema_name) 
+				SELECT schema_name
+				FROM tap_schema.schemas
+					LEFT OUTER JOIN tap_schema.tables USING (schema_name)
 				WHERE table_name IS NULL)""")
 
 
@@ -267,22 +267,25 @@ def mangleUploads(request):
 
 	The underlying trouble is that the core cannot see the request any more,
 	and thus could not resolve param: uploads.
+
+	Since this is different in TAP than in DALI, we have extra code here
+	vs. dali.mangleUploads.
 	"""
 	uploadSpec = request.strargs.pop("upload", [])
 	uploadSpec = ";".join(uploadSpec)
-	if not uploadSpec: 
+	if not uploadSpec:
 		return
-	
+
 	parsed = []
 	for tableName, upload in parseUploadString(uploadSpec):
 		if upload.startswith("param:"):
 			try:
-				upload = request.fields[upload[6:]]
+				upload = request.files[upload[6:]][0]
 				parsed.append(
-					(tableName, (upload.filename, upload.file)))
+					(tableName, (upload.file_name, upload.file_object)))
 			except KeyError:
 				raise base.ui.logOldExc(
-					base.ValidationError(f"No inline upload '{upload[6:]}' found", 
+					base.ValidationError(f"No inline upload '{upload[6:]}' found",
 						"UPLOAD"))
 			except AttributeError:
 				raise base.ui.logOldExc(
@@ -420,8 +423,8 @@ class TAPCore(svcs.Core):
 			<inputKey name="format" type="text"
 				description="Deprecated alias of RESPONSEFORMAT"/>
 			<inputKey name="upload" type="raw"
-				description="A TAP-compliant upload; roughly: 
-					(tablename,source-uri), where source-uri can also have 
+				description="A TAP-compliant upload; roughly:
+					(tablename,source-uri), where source-uri can also have
 					a param: scheme.">
 			</inputKey>
 			<FEED source="//pql#DALIPars">
