@@ -9,11 +9,11 @@ Primarily for testing an alternative interface rabRun exists that takes that
 takes jobid, and parameters.
 
 The tap runner takes the job to EXECUTING shortly before sending the
-query to the DB server.  When done, the job's state is one of COMPLETED, 
+query to the DB server.  When done, the job's state is one of COMPLETED,
 ABORTED or ERROR.
 """
 
-#c Copyright 2008-2022, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -118,7 +118,7 @@ def _makeDataFor(resultTable):
 	resData = rsc.wrapTable(resultTable)
 	resData.addMeta("info", "Query successful",
 		infoName="QUERY_STATUS", infoValue="OK")
-	resData.addMeta("_type", "results")
+	resData.setMeta("_type", "results")
 	resData.overflowLimit = resultTable.tableDef.overflowLimit
 	return resData
 
@@ -140,7 +140,7 @@ def writeResultTo(format, res, outF):
 			votable.V.INFO(name="QUERY_STATUS", value="OVERFLOW"))
 		ctx = votablewrite.VOTableContext(
 			tablecoding=enc,
-			acquireSamples=False, 
+			acquireSamples=False,
 			overflowElement=oe)
 		votablewrite.writeAsVOTable(res, outF, ctx)
 	else:
@@ -222,7 +222,7 @@ def getQTableFromJob(parameters, job, queryProfile, timeout):
 	except: # Don't fail just because we can't kill workers
 		base.ui.notifyError(
 			f"Could not obtain PID for the worker, job {job.jobId}")
-	tdsForUploads = _ingestUploads(job, parameters["upload"], 
+	tdsForUploads = _ingestUploads(job, parameters["upload"],
 		connectionForQuery)
 
 	return adqlglue.runTAPQuery(query, timeout, connectionForQuery,
@@ -243,7 +243,7 @@ def runTAPJobNoState(parameters, jobId, queryProfile, timeout):
 	if base.getConfig("ivoa", "votDefaultEncoding")=="td":
 		defaultFormat = "votable/td"
 
-	rawFormat = (parameters.get("responseformat") 
+	rawFormat = (parameters.get("responseformat")
 		or parameters.get("format")
 		or defaultFormat)
 	format = formats.getKeyFor(rawFormat)
@@ -255,8 +255,8 @@ def runTAPJobNoState(parameters, jobId, queryProfile, timeout):
 
 		destF = job.openResult(
 			formats.getMIMEFor(
-				format, 
-				rawFormat), 
+				format,
+				rawFormat),
 				"result")
 		writeResultTo(format, res, destF)
 		destF.close()
@@ -291,7 +291,7 @@ def runTAPJob(jobId, queryProfile="untrustedquery"):
 def runSyncTAPJob(jobId, queryMeta=None):
 	"""executes a TAP job synchronously.
 
-	When this is done, the job will be in an end state, i.e., ERROR, 
+	When this is done, the job will be in an end state, i.e., ERROR,
 	COMPLETED or perhaps ABORTED.
 
 	You must call tap.WORKER_SYSTEM.destroy(jobId) when done yourself.
@@ -344,7 +344,7 @@ def _killWorker(jobId):
 
 
 def joinInterruptibly(t, jobId):
-	while True: 
+	while True:
 		t.join(timeout=0.5)
 		if not t.is_alive():
 			return
@@ -384,7 +384,7 @@ def main():
 	"""
 	from gavo import rscdesc #noflake: cache registration
 
-	# there's a problem in CLI behaviour in that if anything goes wrong in 
+	# there's a problem in CLI behaviour in that if anything goes wrong in
 	# main, a job that may have been created will remain QUEUED forever.
 	# There's little we can do about that, though, since we cannot put
 	# a job into ERROR when we don't know its id or cannot get it from the DB.
