@@ -13,7 +13,7 @@ using common.getResType; this means the resType meta is tried first,
 using resob.resType as a fallback.
 """
 
-#c Copyright 2008-2022, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -22,10 +22,10 @@ using resob.resType as a fallback.
 # DataCollection mess: In rev 3769, we experimentally pushed out
 # CatalogService records instead of DataCollections, trying to have
 # capabilities for them.  That didn't turn out well even though
-# we didn't do that for SIA and frieds: All-VO discovery of a 
-# given service type is a very typcial use case.  So, we backed
+# we didn't do that for SIA and frieds: All-VO discovery of a
+# given service type is a very typical use case.  So, we backed
 # that out again in rev. 3883, though the support code remains.
-# 
+#
 # Then, in rev. 3891, we went back to CatalogService records, only
 # this time we only pushed out capabilities with "auxiliary" standard ids.
 
@@ -110,7 +110,7 @@ _vrResourceBuilder = meta.ModelBasedBuilder([
 		('contentLevel', SF(VOR.contentLevel)),] + [
 		# old-style relationship terms from VOResource 1.0 f
 		# plan: after a while, make the update mapping in here
-			(None, SF(lambda t=vorTerm: 
+			(None, SF(lambda t=vorTerm:
 				VOR.relationship[VOR.relationshipType[t]]), [
 			(metaName, SF(VOR.relatedResource), (), {
 					"ivoId": "ivoId"})])
@@ -123,7 +123,7 @@ _vrResourceBuilder = meta.ModelBasedBuilder([
 				("uses", "Cites")]
 		] + [
 		# new-style relationship terms from VOResource 1.1
-			(None, SF(lambda term=term: 
+			(None, SF(lambda term=term:
 				VOR.relationship[VOR.relationshipType[term]]), [
 			(term[0].lower()+term[1:], SF(VOR.relatedResource), (), {
 					"ivoId": "ivoId"})])
@@ -167,7 +167,8 @@ _oaiHeaderBuilder = meta.ModelBasedBuilder([
 
 _orgMetaBuilder = meta.ModelBasedBuilder([
 	('facility', SF(VOR.facility)),# XXX TODO: look up ivo-ids?
-	('instrument', SF(VOR.instrument)),
+	('instrument', SF(VOR.instrument), (),
+		{"altIdentifier": "altIdentifier", "ivoId": "ivoId"}),
 ])
 
 
@@ -251,7 +252,7 @@ def getOAIHeaderElementForRestup(restup):
 		OAI.identifier[identifiers.computeIdentifierFromRestup(restup)],
 		OAI.datestamp[restup["recTimestamp"].strftime(utils.isoTimestampFmt)],
 		[
-			OAI.setSpec[setName] 
+			OAI.setSpec[setName]
 				for setName in servicelist.getSetsForResource(restup)]]
 
 
@@ -346,11 +347,11 @@ def getDCResourceElement(resob, setNames=_defaultSet):
 	return getResourceElement(resob, setNames, getDCMetadataElement)
 
 
-def getDCListRecordsElement(resobs, setNames, 
+def getDCListRecordsElement(resobs, setNames,
 		makeRecord=getDCResourceElement):
 	"""returns stanxml for ListRecords in dublin core format.
 
-	resobs is a sequence of res objects.  
+	resobs is a sequence of res objects.
 	makeRecord(resob, setNames) -> stanxml is a function that returns
 	an OAI.record element.  For ivo_vor metadata prefixes, this is overridden.
 	by getVOListRecordsElement.
@@ -440,7 +441,7 @@ class ServiceResourceMaker(ResourceMaker):
 
 	def _makeResource(self, service, setNames):
 		res = ResourceMaker._makeResource(self, service, setNames)[
-			_rightsBuilder.build(service)] 
+			_rightsBuilder.build(service)]
 		return res[[
 					capabilities.getCapabilityElement(pub)
 				for pub in service.getPublicationsForSet(setNames)]]
@@ -512,7 +513,7 @@ class StandardsResourceMaker(ResourceMaker):
 			_standardsMetaBuilder.build(registry)]
 
 
-class DocResourceMaker(DataServiceResourceMaker):
+class DocResourceMaker(CatalogServiceResourceMaker):
 	resourceClass = DOC.Document
 	resType = "document"
 
@@ -535,8 +536,8 @@ class DataCollectionResourceMaker(ResourceMaker):
 		"""returns xmlstan for schemas within metaCarrier.
 
 		metaCarrier has to provide all the VOR metadata.  schemas is a
-		sequence of triples of (rd, tables); rd is used to define a 
-		VODataService schema, tables is a sequence of TableDefs that 
+		sequence of triples of (rd, tables); rd is used to define a
+		VODataService schema, tables is a sequence of TableDefs that
 		define the tables within that schema.
 		"""
 		res = ResourceMaker._makeResource(self, metaCarrier, setNames)[[
@@ -570,8 +571,8 @@ class DataResourceMaker(DataCollectionResourceMaker):
 			self, dd, [(dd.rd, set(dd.iterTableDefs()))], setNames)
 
 
-_getResourceMaker = utils.buildClassResolver(ResourceMaker, 
-	list(globals().values()), instances=True, 
+_getResourceMaker = utils.buildClassResolver(ResourceMaker,
+	list(globals().values()), instances=True,
 	key=lambda obj: obj.resType)
 
 
@@ -595,7 +596,7 @@ def getVOResourceElement(resob, setNames=_defaultSet):
 
 
 def getVOListRecordsElement(resobs, setNames):
-	return getDCListRecordsElement(resobs, setNames, 
+	return getDCListRecordsElement(resobs, setNames,
 		getVOResourceElement)
 
 
