@@ -1,4 +1,4 @@
-""" 
+"""
 VOResource capability/interface elements.
 
 The basic mapping from our RD elements to VOResource elements is that
@@ -7,7 +7,7 @@ Thus, in the module we mostly deal with publication objects.  If you
 need the service object, use publication.parent.
 """
 
-#c Copyright 2008-2022, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -125,8 +125,8 @@ class InterfaceMaker(object):
 		return self._makeInterface(publication)
 
 
-# Use this for when a standard protocol just wants a vs:ParamHTTP interface.
 class StandardParamHTTP(VS.ParamHTTP):
+# Use this for when a standard protocol just wants a vs:ParamHTTP interface.
 		_a_role = "std"
 
 
@@ -303,8 +303,8 @@ class GetProductInterface(WebBrowserInterface):
 	renderer = "get"
 
 
-_getInterfaceMaker = utils.buildClassResolver(InterfaceMaker, 
-	list(globals().values()), instances=True, 
+_getInterfaceMaker = utils.buildClassResolver(InterfaceMaker,
+	list(globals().values()), instances=True,
 	key=lambda obj: obj.renderer,
 	default=InterfaceWithParams())
 
@@ -322,11 +322,11 @@ class CapabilityMaker(object):
 	basic functionality).
 
 	In particular, you will typically want to override capabilityClass
-	with a stanxml element spitting out the right standardIds.  
+	with a stanxml element spitting out the right standardIds.
 		
-	Additionally, if the capability should also appear in data collections 
+	Additionally, if the capability should also appear in data collections
 	served by a service with the capability, also define auxiliaryId (that's
-	an IVOID like ivo://ivoa.net/std/TAP#aux).  These are used in 
+	an IVOID like ivo://ivoa.net/std/TAP#aux).  These are used in
 	getCapabilityElement.
 
 	CapabilityMakers are used by calling them.
@@ -336,7 +336,7 @@ class CapabilityMaker(object):
 	auxiliaryId = None
 
 	def _getInterfaceElement(self, publication):
-		"""returns the appropriate interface definition(s) for service and 
+		"""returns the appropriate interface definition(s) for service and
 		renderer.
 		"""
 		if publication.auxiliary and publication.render=="dali":
@@ -349,7 +349,7 @@ class CapabilityMaker(object):
 	
 	def _makeCapability(self, publication):
 		return self.capabilityClass[
-			VOR.description[base.getMetaText(publication, "description", 
+			VOR.description[base.getMetaText(publication, "description",
 				propagate=False, macroPackage=publication.parent)],
 			self._getInterfaceElement(publication)]
 
@@ -360,7 +360,7 @@ class CapabilityMaker(object):
 class PlainCapabilityMaker(CapabilityMaker):
 	"""A capability maker for generic VR.capabilities.
 
-	These essentially just set standardId. in addition to what
+	These essentially just set standardId, in addition to what
 	the plain capabilities do.
 	"""
 	standardId = None
@@ -398,7 +398,7 @@ class SIACapabilityMaker(CapabilityMaker):
 				service.getMeta("sia.maxFileSize", default=None),
 			],
 			SIA.maxRecords[
-				service.getMeta("sia.maxRecords", 
+				service.getMeta("sia.maxRecords",
 					default=str(base.getConfig("ivoa", "dalHardLimit"))),
 			],
 			SIA.testQuery[
@@ -451,7 +451,7 @@ class SSACapabilityMaker(CapabilityMaker):
 
 		cap[
 			SSAP.dataSource[service.getMeta("ssap.dataSource", raiseOnFail=True)],
-			SSAP.creationType[service.getMeta("ssap.creationType", 
+			SSAP.creationType[service.getMeta("ssap.creationType",
 				default="archival")],
 			SSAP.supportedFrame["ICRS"],
 			SSAP.maxSearchRadius["90"],
@@ -459,7 +459,7 @@ class SSACapabilityMaker(CapabilityMaker):
 			SSAP.defaultMaxRecords[str(base.getConfig("ivoa", "dalDefaultLimit"))],
 			SSAP.maxAperture["90"],
 			SSAP.testQuery[
-				SSAP.queryDataCmd[base.getMetaText(service, "ssap.testQuery", 
+				SSAP.queryDataCmd[base.getMetaText(service, "ssap.testQuery",
 					raiseOnFail=True)]],
 		]
 		return cap
@@ -473,16 +473,16 @@ class SLAPCapabilityMaker(CapabilityMaker):
 		service = publication.parent
 		return CapabilityMaker._makeCapability(self, publication)[
 			SLAP.complianceLevel[
-				service.getMeta("slap.complianceLevel", default="full")], 
+				service.getMeta("slap.complianceLevel", default="full")],
 			SSAP.dataSource[service.getMeta("slap.dataSource", raiseOnFail=True)],
 			SSAP.testQuery[
-				SSAP.queryDataCmd[base.getMetaText(service, "slap.testQuery", 
+				SSAP.queryDataCmd[base.getMetaText(service, "slap.testQuery",
 					raiseOnFail=True)]],
 		]
 
 
 _tapModelBuilder = meta.ModelBasedBuilder([
-	('supportsModel', meta.stanFactory(TR.dataModel), (), 
+	('supportsModel', meta.stanFactory(TR.dataModel), (),
 		{"ivoId": "ivoId"})])
 
 class TAPCapabilityMaker(CapabilityMaker):
@@ -512,9 +512,11 @@ class TAPCapabilityMaker(CapabilityMaker):
 						"select dmname, dmivorn from tap_schema.supportedmodels")]]
 
 			udfs = []
-			for udf in ufunctions.UFUNC_REGISTRY.values():
+			# the extra set() in the next line is to de-dupe UDFs that have
+			# legacy aliases (e.g., gavo_specconv vs. ivo_specconv)
+			for udf in set(ufunctions.UFUNC_REGISTRY.values()):
 				udfs.append({
-						"signature": udf.adqlUDF_signature, 
+						"signature": udf.adqlUDF_signature,
 						"doc": udf.adqlUDF_doc})
 				udfs.extend(udf.adqlUDF_additionalSignatures)
 
@@ -612,9 +614,9 @@ class TAPCapabilityMaker(CapabilityMaker):
 					for langName, description, versions
 						in tap.getSupportedLanguages()],
 				[TR.outputFormat(ivoId=ivoId)[
-						TR.mime[mime], 
+						TR.mime[mime],
 							[TR.alias[alias] for alias in aliases]]
-					for mime, aliases, description, ivoId 
+					for mime, aliases, description, ivoId
 						in tap.getSupportedOutputFormats()],
 				[TR.uploadMethod(ivoId="ivo://ivoa.net/std/TAPRegExt#%s"%proto)
 					for proto in tap.UPLOAD_METHODS],
@@ -664,6 +666,11 @@ class ExamplesCapabilityMaker(PlainCapabilityMaker):
 	renderer = "examples"
 	standardId = "ivo://ivoa.net/std/DALI#examples"
 
+class HiPSCapabilityMaker(PlainCapabilityMaker):
+	renderer = "hips"
+	standardId = "ivo://ivoa.net/std/hips#hips-1.0"
+	interfaceClass = StandardParamHTTP
+
 class SOAPCapabilityMaker(CapabilityMaker):
 	renderer = "soap"
 
@@ -702,7 +709,7 @@ class DatalinkCapabilityMaker(CapabilityMaker):
 	renderer = "dlmeta"
 
 	class capabilityClass(VOR.capability):
-		_a_standardID = "ivo://ivoa.net/std/DataLink#links-1.0"
+		_a_standardID = "ivo://ivoa.net/std/DataLink#links-1.1"
 
 
 class SODACapabilityMaker(CapabilityMaker):
@@ -740,8 +747,8 @@ class EditionCapabilityMaker(CapabilityMaker):
 				base.getMetaText(publication, "locTitle", None)]]
 
 
-_getCapabilityMaker = utils.buildClassResolver(CapabilityMaker, 
-	list(globals().values()), instances=True, 
+_getCapabilityMaker = utils.buildClassResolver(CapabilityMaker,
+	list(globals().values()), instances=True,
 	key=lambda obj: obj.renderer)
 
 
