@@ -2,7 +2,7 @@
 The schema and XML namespaces for OAI/VOR documents.
 """
 
-#c Copyright 2008-2022, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -201,6 +201,7 @@ class VOR:
 	class ResourceName(VORElement):
 		_a_ivoId = None
 		_name_a_ivoId = "ivo-id"
+		_a_altIdentifier = None
 
 	class identifier(VORElement): pass
 
@@ -216,7 +217,7 @@ class VOR:
 	
 	class date(VORElement):
 		_a_role = None
-	
+
 	class version(VORElement): pass
 	
 	class contact(VORElement): pass
@@ -225,7 +226,7 @@ class VOR:
 
 	class facility(VORElement): pass
 
-	class instrument(VORElement): pass
+	class instrument(ResourceName): pass
 	
 	class name(VORElement): pass
 	
@@ -421,7 +422,6 @@ class VS:
 
 	class tableset(VSElement):
 		_additionalPrefixes = xsiPrefix
-		_mayBeEmpty = True
 		_childSequence = ["schema"]
 	
 	class schema(VSElement):
@@ -468,7 +468,7 @@ class VS:
 
 	class temporal(VSElement): pass
 
-	class format(VSElement): 
+	class format(VSElement):
 		_a_isMIMEType = None
 	
 	class accessURL(VSElement): pass
@@ -752,7 +752,7 @@ class TR(object):
 	class alias(TRElement): pass
 	class description(TRElement): pass
 	class executionDuration(TRElement): pass
-	class mime(TRElement): pass 
+	class mime(TRElement): pass
 	class name(TRElement): pass
 	class parameter(TRElement): pass
 	class protocol(TRElement): pass
