@@ -12,7 +12,7 @@ the content of the service table is not actually used to build resource
 records.
 """
 
-#c Copyright 2008-2022, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -67,7 +67,7 @@ def makeBaseRecord(res, keepTimestamp=False):
 	ivoid = base.getMetaText(res, "identifier")
 	auth = urllib.parse.urlparse(ivoid).netloc
 
-	if (auth=="x-unregistred" 
+	if (auth=="x-unregistred"
 			and not "yes"==base.getMetaText(res, "i-am-sure", "")):
 		# allow this if people have no ivo_managed publications
 		if list(res.getPublicationsForSet(
@@ -94,7 +94,7 @@ def makeBaseRecord(res, keepTimestamp=False):
 	rec["deleted"] = False
 	rec["recTimestamp"] = datetime.datetime.utcnow()
 	rec["description"] = base.getMetaText(res, "description")
-	rec["authors"] = "; ".join(m.getExpandedContent(res) 
+	rec["authors"] = "; ".join(m.getExpandedContent(res)
 		for m in res.iterMeta("creator.name", propagate=True))
 	rec["dateUpdated"] = base.getMetaText(res, "_dataUpdated", default=None)
 
@@ -125,7 +125,7 @@ def iterAuthorsAndSubjects(resource, sourceRD, resId):
 	# for authors, we support a special notation, separating individual
 	# authors with semicolons.
 	for authors in resource.iterMeta("creator.name", propagate="True"):
-		authors = [s.strip() for s in 
+		authors = [s.strip() for s in
 			authors.getExpandedContent(resource).split(";")]
 		for author in authors:
 			if not author.startswith("et al"):
@@ -136,7 +136,7 @@ def iterAuthorsAndSubjects(resource, sourceRD, resId):
 
 
 def iterSvcRecs(service, keepTimestamp=False):
-	"""iterates over records suitable for importing into the service list 
+	"""iterates over records suitable for importing into the service list
 	for service.
 	"""
 	if not service.publications:
@@ -159,9 +159,9 @@ def iterSvcRecs(service, keepTimestamp=False):
 			continue
 
 		try:
-			browsable = service.isBrowseableWith(pub.render)
+			browseable = service.isBrowseableWith(pub.render)
 		except AttributeError:  # service is not a ServiceBasedPage
-			browsable = False
+			browseable = False
 
 		pubService = service
 		if pub.service:
@@ -173,7 +173,7 @@ def iterSvcRecs(service, keepTimestamp=False):
 			"renderer": pub.render,
 			"accessURL":  pubService.getURL(pub.render, absolute=False),
 			"referenceURL": base.getMetaText(pubService, "referenceURL"),
-			"browseable": browsable,
+			"browseable": browseable,
 			"deleted": False}
 		yield ("interfaces", intfRec)
 
@@ -182,7 +182,7 @@ def iterSvcRecs(service, keepTimestamp=False):
 			intfRec["setName"] = setName
 			yield ("sets", intfRec)
 
-	for pair in iterAuthorsAndSubjects(service, 
+	for pair in iterAuthorsAndSubjects(service,
 			rec["sourceRD"], rec["resId"]):
 		yield pair
 
@@ -201,7 +201,7 @@ def iterResRecs(res, keepTimestamp=False):
 	yield ("resources", rec)
 	yield ("sets", rec)
 
-	for pair in iterAuthorsAndSubjects(res, 
+	for pair in iterAuthorsAndSubjects(res,
 			rec["sourceRD"], rec["resId"]):
 		yield pair
 
@@ -232,7 +232,7 @@ def iterDataRecs(res, keepTimestamp=False):
 				"browseable": False,
 				"deleted": False})
 
-	for pair in iterAuthorsAndSubjects(res, 
+	for pair in iterAuthorsAndSubjects(res,
 			rec["sourceRD"], rec["resId"]):
 		yield pair
 
@@ -241,6 +241,8 @@ class RDRscRecIterator(grammars.RowIterator):
 	"""A RowIterator yielding resource records for inclusion into the
 	service list for the services defined in the source token RD.
 	"""
+	notify = False
+
 	def _iterRows(self):
 		if self.grammar.unpublish:
 			return
@@ -292,16 +294,16 @@ def getDeletedIdentifiersUpdater(conn, rd):
 	We don't want to change resources.rectimestamp there; this would,
 	for instance, bump old deleted records.
 
-	What we can do is see if there's new deleted records after and rd 
+	What we can do is see if there's new deleted records after and rd
 	is through and update their rectimestamp.  We don't need to like it.
 	"""
-	oldDeletedRecords = set(r[0] for r in 
+	oldDeletedRecords = set(r[0] for r in
 		conn.query("select ivoid from dc.resources"
 			" where sourcerd=%(rdid)s and deleted",
 			{"rdid": rd.sourceId}))
 
 	def bumpNewDeletedRecords():
-		newDeletedRecords = set(r[0] for r in 
+		newDeletedRecords = set(r[0] for r in
 			conn.query("select ivoid from dc.resources"
 				" where sourcerd=%(rdid)s and deleted",
 				{"rdid": rd.sourceId}))
@@ -329,9 +331,9 @@ class RDRscRecGrammar(grammars.Grammar):
 	unpublish = False
 
 
-def updateServiceList(rds, 
-		metaToo=False, 
-		connection=None, 
+def updateServiceList(rds,
+		metaToo=False,
+		connection=None,
 		onlyWarn=True,
 		keepTimestamp=False,
 		unpublish=False):
@@ -400,7 +402,7 @@ def _purgeFromServiceTables(rdId, conn):
 
 	This is not for user code that should rely on the tables doing the
 	right thing (e.g., setting the deleted flag rather than deleting rows).
-	Test code that is not in contact with the actual registry might want 
+	Test code that is not in contact with the actual registry might want
 	this, though (until postgres grows nested transactions).
 	"""
 	cursor = conn.cursor()
@@ -462,7 +464,7 @@ def findAllRDs():
 				rds.append(os.path.splitext(
 					utils.getRelativePath(os.path.join(dir, file), inputsDir))[0])
 
-	for name in pkg_resources.resource_listdir('gavo', 
+	for name in pkg_resources.resource_listdir('gavo',
 			"resources/inputs/__system__"):
 		if not name.endswith(".rd"):  # ignore VCS files (and possibly others:-)
 			continue
@@ -496,10 +498,10 @@ def parseCommandLine():
 	import argparse
 	parser = argparse.ArgumentParser(
 		description="Publish services from an RD")
-	parser.add_argument("-k", "--keep-timestamps", 
+	parser.add_argument("-k", "--keep-timestamps",
 		help="Preserve the time stamp of the last record modification."
 		" This may sometimes be desirable with minor updates to an RD"
-		" that don't justify a re-publication to the VO.", 
+		" that don't justify a re-publication to the VO.",
 		action="store_true", dest="keepTimestamp")
 	parser.add_argument("-u", "--unpublish", help="Unpublish all"
 		" resources coming from this RD",
@@ -523,7 +525,7 @@ def main():
 	elif len(args.rd)==1 and args.rd[0]=="ALL_RECURSE":
 		args.rd = findAllRDs()
 
-	updateServiceList(getRDs(args.rd), 
+	updateServiceList(getRDs(args.rd),
 		keepTimestamp=args.keepTimestamp,
 		unpublish=args.unpublish)
 	base.tryRemoteReload("__system__/services")
