@@ -2,7 +2,7 @@
 Making data out of descriptors and sources.
 """
 
-#c Copyright 2008-2022, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -203,13 +203,13 @@ class Data(base.MetaMixin, common.ParamMixin):
 
 	``Data``, in essence, is the instantiation of a ``DataDescriptor``.
 
-	It is what ``makeData`` returns.  In typical one-table situations, 
+	It is what ``makeData`` returns.  In typical one-table situations,
 	you just want to call the ``getPrimaryTable()`` method to obtain the
 	table built.
 	"""
-	def __init__(self, 
-			dd, 
-			tables, 
+	def __init__(self,
+			dd,
+			tables,
 			parseOptions=common.parseNonValidating,
 			overrideMakes=None):
 		base.MetaMixin.__init__(self)  # we're not a structure
@@ -235,8 +235,8 @@ class Data(base.MetaMixin, common.ParamMixin):
 		res = cls(dd, controlledTables, parseOptions)
 		for make in dd.makes:
 			controlledTables[make.table.id
-				] = make.create(connection, 
-					parseOptions, 
+				] = make.create(connection,
+					parseOptions,
 					tables.TableForDef,
 					parent=res)
 		return res
@@ -249,8 +249,8 @@ class Data(base.MetaMixin, common.ParamMixin):
 		for make in dd.makes:
 			controlledTables[make.table.id
 				] = tables.TableForDef(
-					make.table, 
-					create=False, 
+					make.table,
+					create=False,
 					connection=connection,
 					make=make)
 		data = cls(dd, controlledTables, parseOptions)
@@ -258,8 +258,8 @@ class Data(base.MetaMixin, common.ParamMixin):
 		return data
 
 	@classmethod
-	def createWithTable(cls, 
-			dd, 
+	def createWithTable(cls,
+			dd,
 			tableDef,
 			parseOptions=common.parseNonValidating):
 		"""builds a table for tableDef with this data item.
@@ -278,7 +278,7 @@ class Data(base.MetaMixin, common.ParamMixin):
 				" has thought about it; it would probably be harmless).")
 
 		controlledTables = {tableDef.id: tables.TableForDef(tableDef)}
-		return cls(dd, controlledTables, parseOptions, 
+		return cls(dd, controlledTables, parseOptions,
 			overrideMakes=[dd.makes[0].change(table=tableDef)])
 
 	def dbCatalogChanged(self):
@@ -286,11 +286,11 @@ class Data(base.MetaMixin, common.ParamMixin):
 		this class.
 		"""
 		return any([
-			isinstance(t, dbtable.DBTable) 
+			isinstance(t, dbtable.DBTable)
 			and t.newlyCreated for t in list(self.tables.values())])
 
 	def validateParams(self):
-		"""raises a ValidationError if any required parameters within 
+		"""raises a ValidationError if any required parameters within
 		this data's tables are still None.
 		"""
 		for t in self:
@@ -343,7 +343,7 @@ class Data(base.MetaMixin, common.ParamMixin):
 			try:
 				return self.getPrimaryTable().getParam(paramName, default)
 			except (base.DataError, base.NotFoundError):
-				# param not in primary table, or no primary table: 
+				# param not in primary table, or no primary table:
 				# raise first exception
 				pass
 			raise
@@ -435,8 +435,8 @@ def processSource(data, source, feeder, opts, connection=None):
 
 	If this builds database tables, you must pass in a connection object.
 
-	If opts.keepGoing is True,the system will continue importing 
-	even if a particular source has caused an error.  In that case, 
+	If opts.keepGoing is True,the system will continue importing
+	even if a particular source has caused an error.  In that case,
 	everything contributed by the bad source is rolled back (this will
 	only work when filling database tables).
 	"""
@@ -447,7 +447,7 @@ def processSource(data, source, feeder, opts, connection=None):
 	else: # recover from bad sources, be more careful
 		if connection is None:
 			raise base.ReportableError("Can only ignore source errors"
-				" when filling database tables.", 
+				" when filling database tables.",
 				hint="The -c flag on dachs imp and its friends builds on database"
 				" savepoints.  You can thus only meaningfully use it when your"
 				" table has onDisk='True'.")
@@ -462,12 +462,12 @@ def processSource(data, source, feeder, opts, connection=None):
 			base.ui.notifyError("Error while importing source %s; changes from"
 				" this source will be rolled back, processing will continue."
 				" (%s)"%(
-					utils.makeSourceEllipsis(source), 
+					utils.makeSourceEllipsis(source),
 					utils.safe_str(ex)))
 
 
 class _TableCornucopeia(object):
-	"""a scaffolding class instances of which return something (eventually 
+	"""a scaffolding class instances of which return something (eventually
 	table-like) for all keys it is asked for.
 	"""
 	def __getitem__(self, key):
@@ -554,8 +554,8 @@ def makeData(dd, parseOptions=common.parseNonValidating,
 
 	if parseOptions.buildDependencies:
 		makeDependentsFor(
-			[dd], 
-			parseOptions, 
+			[dd],
+			parseOptions,
 			connection,
 			res.dbCatalogChanged())
 
@@ -565,7 +565,7 @@ def makeData(dd, parseOptions=common.parseNonValidating,
 class DDDependencyGraph(object):
 	"""a graph giving the dependency structure between DDs.
 
-	This is constructed with a list of DDs.  
+	This is constructed with a list of DDs.
 	
 	From it, you can get a build sequence (least-depending thing build first) or
 	a destroy sequence (most-depending things built first).
@@ -618,7 +618,7 @@ def makeDependentsFor(dds, parseOptions, connection, sysCatChanged):
 		buildSequence = DDDependencyGraph(dds).getBuildSequence()
 	except ValueError as ex:
 		raise utils.logOldExc(base.ReportableError("Could not sort"
-			" dependent DDs topologically (use  --hints to learn more).", 
+			" dependent DDs topologically (use  --hints to learn more).",
 			hint="This is most likely because there's a cyclic dependency."
 			" Please check your dependency structure.  The original message"
 			" is: %s"%utils.safe_str(ex)))
@@ -655,13 +655,16 @@ def makeDataById(ddId, parseOptions=common.parseNonValidating,
 	return makeData(dd, parseOptions=parseOptions, connection=connection)
 
 
-def wrapTable(table, rdSource=None):
+def wrapTable(table, rdSource=None, resTypeDefault="results"):
 	"""returns a Data instance containing only table (or table if it's already
 	a data instance).
 
 	If table has no rd, you must pass rdSource, which must be an object having
 	and rd attribute (rds, tabledefs, etc, work).
 
+	resTypeDefault will be used as the new data item's _type meta.  If you
+	want to override that later, use setMeta("_type"...) rather than addMeta.
+
 	This will grab info meta from the table.
 	"""
 	if hasattr(table, "dd"):
@@ -683,6 +686,7 @@ def wrapTable(table, rdSource=None):
 	newDD.setMetaParent(table.tableDef.rd)
 
 	res = Data(newDD, tables={table.tableDef.id: table})
+	res.setMeta("_type", resTypeDefault)
 
 	for infoMeta in table.iterMeta("info"):
 		res.addMeta("info", infoMeta)
@@ -705,5 +709,5 @@ def makeCombinedData(baseDD, tablesForRoles):
 		makes=[MS(rscdef.Make, table=t.tableDef, rowmaker=None, role=role)
 			for role, t in tablesForRoles.items()])
 	newDD.meta_ = baseDD._metaAttr.getCopy(baseDD, newDD, None)
-	return Data(newDD, tables=dict((t.tableDef.id, t) 
+	return Data(newDD, tables=dict((t.tableDef.id, t)
 		for t in list(tablesForRoles.values())))
