@@ -12,7 +12,7 @@ different TableDefs for all these aspects, but then I'd have a plethora
 of TableDef elements, which I think is worse than a factory function.
 """
 
-#c Copyright 2008-2022, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -93,10 +93,10 @@ class _Feeder(object):
 	  of crazy exceptions.
 	- flush() -> None -- flush out all data that may be cached to the table
 	  (this is done automatically on a successful exit)
-	- reset() -> None -- discard any data that may still wait to be 
+	- reset() -> None -- discard any data that may still wait to be
 	  flushed to the table
 
-	At the end of the controlled block, the importFinished or importFailed 
+	At the end of the controlled block, the importFinished or importFailed
 	methods or the parent table are called depending on whether all is
 	well or an exception happened.  If importFinished raises and
 	exception, it is handed on to importFailed and re-raised if importFailed
@@ -202,9 +202,9 @@ class BaseTable(base.MetaMixin, common.ParamMixin):
 	  is defined, a ValueError is raised, if the key is not present, a
 	  KeyError.  An atomic primary key is accessed through its value,
 	  for compound primary keys a tuple must be passed.
-	- ``getFeeder(**kwargs)`` -> feeder object -- returns an object with add and 
+	- ``getFeeder(**kwargs)`` -> feeder object -- returns an object with add and
 	  exit methods.  See feeder above.
-	- ``importFinished(nAffected)`` -> None -- called when a feeder exits 
+	- ``importFinished(nAffected)`` -> None -- called when a feeder exits
 	  successfully
 	- ``importFailed(*excInfo)`` -> boolean -- called when feeding has failed;
 	  when returning True, the exception that has caused the failure
@@ -242,6 +242,14 @@ class BaseTable(base.MetaMixin, common.ParamMixin):
 	getRow = _makeFailIncomplete("getRow")
 	getFeeder = _makeFailIncomplete("getFeeder")
 
+	def getPrimaryTable(self):
+		"""returns the table itself.
+
+		This is just there so that code can blindly say .getPrimaryTable()
+		not worrying whether what it has is a data item or a table.
+		"""
+		return self
+
 	def addTuple(self, tupRow):
 		self.addRow(self.tableDef.makeRowFromTuple(tupRow))
 
@@ -303,7 +311,7 @@ class InMemoryTable(BaseTable):
 	def getLimits(self):
 		"""returns a limits instance for this table.
 
-		This is a characterisation of the ranges of things in this table, 
+		This is a characterisation of the ranges of things in this table,
 		pretty much as what dachs info does; if you fix things here, you probably
 		want to fix things there, too.
 		"""
