@@ -2,7 +2,7 @@
 Description of columns (and I/O fields).
 """
 
-#c Copyright 2008-2022, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -37,9 +37,9 @@ class TypeNameAttribute(base.AtomicAttribute):
 
 	def parse(self, value):
 		try:
-			typesystems.sqltypeToPython(value)
+			typesystems.sqltypeToVOTable(value)
 		except base.Error:
-			raise base.ui.logOldExc(base.LiteralParseError(self.name_, value, 
+			raise base.ui.logOldExc(base.LiteralParseError(self.name_, value,
 				hint="A supported SQL type was expected here.  If in doubt,"
 				" check base/typeconversions.py, in particular ToPythonCodeConverter."))
 		return value
@@ -146,7 +146,7 @@ class TableManagedAttribute(base.AttributeDef):
 			yield ("value", self.name_, _AttBox(val))
 
 	def getCopy(self, instance, newParent, ctx):
-		# these never get copied; the values are potentially shared 
+		# these never get copied; the values are potentially shared
 		# between many objects, so the must not be changed anyway.
 		return getattr(instance, self.name_)
 
@@ -177,7 +177,7 @@ class DMRolesAttribute(base.AttributeDef):
 	typeDesc_ = "read-only list of roles played by this column in DMs"
 
 	def __init__(self, name, description="Undocumented"):
-		base.AttributeDef.__init__(self, 
+		base.AttributeDef.__init__(self,
 			name, default=base.Computed, description=description,
 			copyable=True)
 
@@ -229,7 +229,7 @@ class DisplayHintAttribute(base.AtomicAttribute):
 	typeDesc_ = "Display hint"
 
 	def __init__(self, name, description, **kwargs):
-		base.AtomicAttribute.__init__(self, name, default=_roEmptyDict, 
+		base.AtomicAttribute.__init__(self, name, default=_roEmptyDict,
 			description=description, **kwargs)
 
 	def parse(self, value):
@@ -238,7 +238,7 @@ class DisplayHintAttribute(base.AtomicAttribute):
 		try:
 			return dict([f.split("=") for f in value.split(",")])
 		except (ValueError, TypeError):
-			raise base.ui.logOldExc(base.LiteralParseError(self.name_, value, 
+			raise base.ui.logOldExc(base.LiteralParseError(self.name_, value,
 				hint="DisplayHints have a format like tag=value{,tag=value}"))
 
 	def unparse(self, value):
@@ -255,7 +255,7 @@ class Option(base.Structure):
 	name_ = "option"
 
 	_title = base.UnicodeAttribute("title", default=base.NotGiven,
-		description="A Label for presentation purposes; defaults to val.", 
+		description="A Label for presentation purposes; defaults to val.",
 		copyable=True)
 	_val = base.DataContent(copyable=True, description="The value of"
 		" the option; this is what is used in, e.g., queries and the like.")
@@ -300,19 +300,19 @@ class Values(base.Structure):
 	"""
 	name_ = "values"
 
-	_min = base.UnicodeAttribute("min", default=None, 
+	_min = base.UnicodeAttribute("min", default=None,
 		description="Minimum acceptable"
 		" value as a datatype literal", copyable=True)
-	_max = base.UnicodeAttribute("max", default=None, 
+	_max = base.UnicodeAttribute("max", default=None,
 		description="Maximum acceptable"
 		" value as a datatype literal", copyable=True)
-	_options = base.StructListAttribute("options", 
+	_options = base.StructListAttribute("options",
 		childFactory=Option,
 		description="List of acceptable values (if set)", copyable=True)
-	_default = base.UnicodeAttribute("default", default=None, 
+	_default = base.UnicodeAttribute("default", default=None,
 		description="A default"
 		" value (currently only used for options).", copyable=True)
-	_nullLiteral = base.UnicodeAttribute("nullLiteral", default=None, 
+	_nullLiteral = base.UnicodeAttribute("nullLiteral", default=None,
 		description=
 		"An appropriate value representing a NULL for this column in VOTables"
 		" and similar places.  You usually should only set it for integer"
@@ -335,21 +335,21 @@ class Values(base.Structure):
 		default=False,
 		copyable=True)
 
-	_percentile03 = base.UnicodeAttribute("percentile03", 
+	_percentile03 = base.UnicodeAttribute("percentile03",
 		default=None,
 		description="Value at the 3rd percentile of the distribution of this"
 			" column.",
 		copyable=True)
-	_percentile97 = base.UnicodeAttribute("percentile97", 
+	_percentile97 = base.UnicodeAttribute("percentile97",
 		default=None,
 		description="Value at the 97rd percentile of the distribution of this"
 			" column.",
 		copyable=True)
-	_median = base.UnicodeAttribute("median", 
+	_median = base.UnicodeAttribute("median",
 		default=None,
 		description="Median of the distribution of this column.",
 		copyable=True)
-	_fillFactor = base.UnicodeAttribute("fillFactor", 
+	_fillFactor = base.UnicodeAttribute("fillFactor",
 		default=None,
 		description="Ratio of non-NULL values to the number of rows in the"
 			" embedding table.",
@@ -363,7 +363,7 @@ class Values(base.Structure):
 	def fromOptions(cls, labels):
 		"""returns Values with the elements of labels as valid options.
 		"""
-		return base.makeStruct(cls, 
+		return base.makeStruct(cls,
 			options=[base.makeStruct(Option, content_=l) for l in labels])
 
 	@property
@@ -392,13 +392,18 @@ class Values(base.Structure):
 			with base.getTableConn() as conn:
 				for row in conn.query(self.parent.parent.expand(
 						"SELECT DISTINCT %s"%(self.fromdb))):
+					if row[0] is None:
+						# a NULL here would be about the same as required=False;
+						# dealing with it properly in constraints is hard.  Let's
+						# see how well we get away with ignoring it.
+						continue
 					self._options.feedObject(self, base.makeStruct(Option,
 						content_=row[0]))
 		except base.DBError: # Table probably doesn't exist yet, ignore.
 			base.ui.notifyWarning("Values fromdb '%s' failed, ignoring"%self.fromdb)
 
 	def onParentComplete(self):
-		"""converts options, the nullLiteral, and the default from 
+		"""converts options, the nullLiteral, and the default from
 		string literals to python values.
 		"""
 		dataField = self.parent
@@ -440,7 +445,7 @@ class Values(base.Structure):
 				("fill_factor", "fillFactor"),
 			]:
 			if srcKey in statRow:
-				if (getattr(self, destKey, None) is None 
+				if (getattr(self, destKey, None) is None
 						and statRow[srcKey] is not None):
 					self.feed(destKey, str(statRow[srcKey]), ctx)
 
@@ -482,7 +487,7 @@ class Values(base.Structure):
 class ColumnBase(base.Structure, base.MetaMixin):
 	"""A base class for columns, parameters, output fields, etc.
 
-	Actually, right now there's far too much cruft in here that 
+	Actually, right now there's far too much cruft in here that
 	should go into Column proper or still somewhere else.  Hence:
 	XXX TODO: Refactor.
 
@@ -496,11 +501,12 @@ class ColumnBase(base.Structure, base.MetaMixin):
 		"datatype for the column (SQL-like type system)",
 		copyable=True, before="unit")
 	_unit = base.UnicodeAttribute("unit", default="", description=
-		"Unit of the values", copyable=True, before="ucd",
-		strip=True)
+		"Unit of the values.  Use VOUnits syntax and use single quotes when"
+		" you use custom units (you should avoid that).",
+		copyable=True, before="ucd", strip=True)
 	_ucd = base.UnicodeAttribute("ucd", default="", description=
 		"UCD of the column", copyable=True, before="description")
-	_description = base.NWUnicodeAttribute("description", 
+	_description = base.NWUnicodeAttribute("description",
 		default="", copyable=True,
 		description="A short (one-line) description of the values in this column.")
 	_tablehead = base.UnicodeAttribute("tablehead", default=None,
@@ -509,18 +515,18 @@ class ColumnBase(base.Structure, base.MetaMixin):
 	_utype = base.UnicodeAttribute("utype", default=None, description=
 		"utype for this column", copyable=True)
 	_required = base.BooleanAttribute("required", default=False,
-		description="Record becomes invalid when this column is NULL", 
+		description="Record becomes invalid when this column is NULL",
 		copyable=True)
-	_displayHint = DisplayHintAttribute("displayHint", 
+	_displayHint = DisplayHintAttribute("displayHint",
 		description="Suggested presentation; the format is "
 			" <kw>=<value>{,<kw>=<value>}, where what is interpreted depends"
 			" on the output format.  See, e.g., documentation on HTML renderers"
 			" and the formatter child of outputFields.", copyable=True)
 	_verbLevel = base.IntAttribute("verbLevel", default=20,
-		description="Minimal verbosity level at which to include this column", 
+		description="Minimal verbosity level at which to include this column",
 		copyable=True)
 	_values = base.StructAttribute("values", default=None,
-		childFactory=Values, description="Specification of legal values", 
+		childFactory=Values, description="Specification of legal values",
 		copyable=True)
 	_fixup = base.UnicodeAttribute("fixup", description=
 		"A python expression the value of which will replace this column's"
@@ -692,7 +698,7 @@ class ColumnBase(base.Structure, base.MetaMixin):
 		}
 	
 	def getDDL(self):
-		"""returns an SQL fragment describing this column ready for 
+		"""returns an SQL fragment describing this column ready for
 		inclusion in a DDL statement.
 		"""
 		type = base.sqltypeToPG(self.type)
@@ -749,7 +755,7 @@ class Column(ColumnBase):
 
 	- std -- set to 1 to tell the tap schema importer to have the column's
 	  std column in TAP_SCHEMA 1 (it's 0 otherwise).
-	- statisticsTarget -- an integer to be set as this column's 
+	- statisticsTarget -- an integer to be set as this column's
 	  statistics-gathering target.  Set this to something between 100 and
 	  10000 on postgres if you have large tables and columns with strongly
 	  non-uniform distributions.  Set to -1 to revert to the system default.
@@ -760,7 +766,7 @@ class Column(ColumnBase):
 	  the discrete values allowed (currently only supported for strings).
 	- targetType -- for a column containing a URL, the media type of the
 	  resource pointed at.  This is for producing extra annotation for
-	  Aladin and friends as per 
+	  Aladin and friends as per
 	  http://mail.ivoa.net/pipermail/dal/2018-May/008017.html
 	- targetTitle -- if you give targetType, use this to set the link
 		title (defaults to "Link").
@@ -784,7 +790,7 @@ class Column(ColumnBase):
 		super().validate()
 		# Now check if we can serialize the column safely in VOTables.
 		# I only want to hear about this when the column may end up in
-		# a VOTable; 
+		# a VOTable;
 		if self.type in EXPLICIT_NULL_TYPES:
 			if not self.required and not (
 					self.values and self.values.nullLiteral):
@@ -794,7 +800,7 @@ class Column(ColumnBase):
 						" Column %s may be null but has no explicit"
 						" null value."%(pos, self.name))
 				except (ValueError, AttributeError):
-					# This is stealVar's ValueError, we have no context in stack (or 
+					# This is stealVar's ValueError, we have no context in stack (or
 					# it's a context var not from our parsing code).
 					# Seems we're not parsing from a file, so the user probably
 					# can't help it anyway.  Don't complain.
@@ -885,7 +891,7 @@ class ParamBase(ColumnBase):
 		self.__contentStore = val
 	
 	def __get_content(self):
-		if (self.__contentStore is base.NotGiven 
+		if (self.__contentStore is base.NotGiven
 				and self._valueCache is not base.Undefined):
 			self.__contentStore = self._unparse(self._valueCache)
 		return self.__contentStore
@@ -934,7 +940,7 @@ class ParamBase(ColumnBase):
 		"""
 		if self.type in self.unprocessedTypes:
 			return "(Unrepresentable %s)"%self.type
-		return self.content_ 
+		return self.content_
 
 	def set(self, val):
 		"""sets this parameter's value.
@@ -970,7 +976,7 @@ class ParamBase(ColumnBase):
 		elif literal=="__NULL__" or literal=="":
 			value = None
 
-		elif ((self.type=="text" or self.type=="unicode") 
+		elif ((self.type=="text" or self.type=="unicode")
 				and literal=="__EMPTY__"):
 			value = ""
 
@@ -1052,7 +1058,7 @@ class Param(ParamBase):
 
 	This is like a column, except that it conceptually applies to all
 	rows in the table.  In VOTables, params will be rendered as
-	PARAMs.  
+	PARAMs.
 	
 	While we validate the values passed using the DaCHS default parsers,
 	at least the VOTable params will be literal copies of the string
@@ -1060,7 +1066,7 @@ class Param(ParamBase):
 
 	You can obtain a parsed value from the value attribute.
 
-	Null value handling is a bit tricky with params.  An empty param (like 
+	Null value handling is a bit tricky with params.  An empty param (like
 	``<param name="x"/>)`` is always NULL (None in python).
 	In order to allow setting NULL even where syntactially something has
 	to stand, we also turn any __NULL__ to None.
@@ -1070,7 +1076,7 @@ class Param(ParamBase):
 
 		<param name="x" type="integer"><values nullLiteral="-1"/>-1</params>
 	
-	For arrays, floats, and strings, the interpretation of values is 
+	For arrays, floats, and strings, the interpretation of values is
 	undefined.  Following VOTable practice, we do not tell empty strings and
 	NULLs apart; for internal usage, there is a little hack: __EMPTY__ as literal
 	does set an empty string.  This is to allow defaulting of empty strings -- in
@@ -1106,7 +1112,7 @@ class Param(ParamBase):
 		expansion.
 		"""
 		if (isinstance(val, str)
-				and "\\" in val 
+				and "\\" in val
 				and hasattr(self.parent, "expand")):
 			val = self.parent.expand(val)
 		return ParamBase.set(self, val)
@@ -1118,3 +1124,10 @@ class Param(ParamBase):
 		# ends up in the target VOTable.  Or do we want to enforce
 		# params to be within the table?
 		return dm.ParamAnnotation(roleName, self, instance)
+	
+	def copy(self, parent, ctx=None):
+		# overridden so we can set originalParent, which in turn we need
+		# in DM serialisation to VOTables
+		newInstance = super().copy(parent, ctx)
+		newInstance.originalParam = self
+		return newInstance
