@@ -2,7 +2,7 @@
 Common items used by resource definition objects.
 """
 
-#c Copyright 2008-2022, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -43,7 +43,7 @@ class RDAttribute(base.AttributeDef):
 	
 	def iterParentMethods(self):
 		def _getRD(self):
-			if getattr(self, "parent", None) is None: 
+			if getattr(self, "parent", None) is None:
 				# not yet adopted, we may want to try again later
 				return None
 			try:
@@ -76,7 +76,7 @@ class ResdirRelativeAttribute(base.FunctionRelativePathAttribute):
 	The parent needs an RDAttribute.
 	"""
 	def __init__(self, name, default=None, description="Undocumented", **kwargs):
-		base.FunctionRelativePathAttribute.__init__(self, name, 
+		base.FunctionRelativePathAttribute.__init__(self, name,
 			baseFunction=self.getResdir,
 			default=default, description=description, **kwargs)
 
@@ -93,7 +93,7 @@ class ResdirRelativeAttribute(base.FunctionRelativePathAttribute):
 class ProfileListAttribute(base.AtomicAttribute):
 	"""An attribute containing a comma separated list of profile names.
 
-	There's the special role name "defaults" for whatever default this 
+	There's the special role name "defaults" for whatever default this
 	profile list was constructed with.
 	"""
 	typeDesc_ = "Comma separated list of profile names"
@@ -120,7 +120,7 @@ class ProfileListAttribute(base.AtomicAttribute):
 		return pNames
 	
 	def unparse(self, value):
-# It would be nice to reconstruct "defaults" here, but right now it's 
+# It would be nice to reconstruct "defaults" here, but right now it's
 # certainly not worth the effort.
 		return ", ".join(value)
 
@@ -136,15 +136,15 @@ class PrivilegesMixin(base.StructCallbacks):
 	Access is managed on the level of database profiles.  Thus, the names
 	here are not directly role names in the database.
 	
-	We have two types of privileges: "All" means at least read  and write, 
+	We have two types of privileges: "All" means at least read  and write,
 	and "Read" meaning at least read and lookup.
 	"""
-	_readProfiles = ProfileListAttribute("readProfiles", 
+	_readProfiles = ProfileListAttribute("readProfiles",
 		default=base.getConfig("db", "queryProfiles"),
 		description="A (comma separated) list of profile names through"
 			" which the object can be read.",
 		defaultSource="[db]queryProfiles")
-	_allProfiles = ProfileListAttribute("allProfiles", 
+	_allProfiles = ProfileListAttribute("allProfiles",
 		default=base.getConfig("db", "maintainers"),
 		description="A (comma separated) list of profile names through"
 			" which the object can be written or administred.",
@@ -211,7 +211,7 @@ class Registration(base.Structure, base.MetaMixin):
 	auxiliary = True
 
 	def _completeMetadataFromResRow(self, resRow):
-		"""fiddles publication dates from a dc.resources row for the parent 
+		"""fiddles publication dates from a dc.resources row for the parent
 		table or data item.
 
 		(see rscdef.rdjinj for where this comes from).
@@ -222,7 +222,7 @@ class Registration(base.Structure, base.MetaMixin):
 		# the RD is more current.
 
 	def completeElement(self, ctx):
-		if (ctx is not None 
+		if (ctx is not None
 				and self.id
 				and self.parent.rd is not None):
 			self._completeMetadataFromResRow(
@@ -274,7 +274,7 @@ class ColumnList(list):
 	"""
 	def __init__(self, *args):
 		list.__init__(self, *args)
-		self.nameIndex = dict([(c.name, ct) for ct, c in enumerate(self)])
+		self.redoIndex()
 		self.withinId = "unnamed table"
 
 	def __contains__(self, fieldName):
@@ -282,14 +282,22 @@ class ColumnList(list):
 
 	def __eq__(self, other):
 		if isinstance(other, ColumnList):
-			myFields = set([f.name for f in self 
+			myFields = set([f.name for f in self
 				if f.name not in self.internallyUsedFields])
-			otherFields = set([f.name for f in other 
+			otherFields = set([f.name for f in other
 				if f.name not in self.internallyUsedFields])
 			return myFields==otherFields
 		elif other==[] and len(self)==0:
 			return True
 		return False
+	
+	def redoIndex(self):
+		"""creates a mapping of names to list indexes.
+
+		You must call this when you dare to munge this manually (which you
+		shouldn't).
+		"""
+		self.nameIndex = dict([(c.name, ct) for ct, c in enumerate(self)])
 
 	def deepcopy(self, newParent):
 		"""returns a deep copy of self.
@@ -373,7 +381,7 @@ class ColumnList(list):
 		for item in self:
 			if item.utype and item.utype.lower()==utype:
 				return item
-		raise base.NotFoundError(utype, what="column with utype", 
+		raise base.NotFoundError(utype, what="column with utype",
 			within=self.withinId)
 
 	def getColumnsByUCD(self, ucd):
@@ -405,7 +413,7 @@ class ColumnList(list):
 		for ucd in ucds:
 			try:
 				return self.getColumnByUCD(ucd)
-			except base.StructureError:  
+			except base.StructureError:
 				# probably just no column with this UCD, try next
 				pass
 		raise base.StructureError("No unique column for any of %s"%", ".join(ucds))
@@ -427,7 +435,17 @@ class ColumnListAttribute(base.StructListAttribute):
 			raise base.StructureError("Can only replace fields of the same"
 				" name in a ColumnList")
 		getattr(instance, self.name_).append(newStruct)
-	
+
+	def feedObject(self, instance, obj):
+		# we really want columns and params to have the proper parents
+		if isinstance(obj, list):
+			for child in obj:
+				self.feedObject(instance, child)
+		else:
+			if obj.parent and obj.parent is not instance:
+				obj = obj.copy(instance)
+			super().feedObject(instance, obj)
+
 
 class NamePathAttribute(base.AtomicAttribute):
 	"""defines an attribute NamePath used for resolution of "original"
@@ -494,8 +512,8 @@ def replaceProcDefAt(src, dictName="vars"):
 def getReferencedElement(refString, forceType=None, **kwargs):
 	"""returns the element for the DaCHS reference ``refString``.
 
-	``refString`` has the form ``rdId[#subRef]``; ``rdId`` can be 
-	filesystem-relative, but the RD referenced must be below ``inputsDir`` 
+	``refString`` has the form ``rdId[#subRef]``; ``rdId`` can be
+	filesystem-relative, but the RD referenced must be below ``inputsDir``
 	anyway.
 
 	You can pass a structure class into ``forceType``, and a ``StructureError``
@@ -529,7 +547,7 @@ def getStandardPubDID(path):
 
 	The publisher dataset identifier (PubDID) is important in protocols like
 	SSAP and obscore.  If you use this function, the PubDID will be your
-	authority, the path component ~, and the inputs-relative path of 
+	authority, the path component ~, and the inputs-relative path of
 	the input file as the parameter.
 
 	``path`` can be relative, in which case it is interpreted relative to
@@ -537,7 +555,7 @@ def getStandardPubDID(path):
 
 	You *can* define your PubDIDs in a different way, but you'd then need
 	to provide a custom descriptorGenerator to datalink services (and
-	might need other tricks).  If your data comes from plain files, use 
+	might need other tricks).  If your data comes from plain files, use
 	this function.
 
 	In a rowmaker, you'll usually use the \\standardPubDID macro.
@@ -584,7 +602,7 @@ def getAccrefFromStandardPubDID(pubdid,
 	"""returns an accref from a standard DaCHS PubDID.
 
 	This is basically the inverse of getStandardPubDID.  It will raise
-	NotFound if pubdid "looks like a URI" (implementation detail: has  a colon 
+	NotFound if pubdid "looks like a URI" (implementation detail: has  a colon
 	in the first 10 characters) and does not start with ivo://<authority>/~?.
 	If it's not a URI, we assume it's a local accref and just return it.
 
@@ -598,8 +616,8 @@ def getAccrefFromStandardPubDID(pubdid,
 		return pubdid
 
 	if not pubdid.startswith(authBase):
-		raise base.NotFoundError(pubdid, 
-			"The authority in the dataset identifier", 
+		raise base.NotFoundError(pubdid,
+			"The authority in the dataset identifier",
 			"the authorities managed here")
 	return pubdid[len(authBase):]
 
@@ -608,7 +626,7 @@ def getAccrefFromStandardPubDID(pubdid,
 def getInputsRelativePath(absPath, liberalChars=True):
 	"""returns absath relative to the DaCHS inputsDir.
 
-	If ``absPath`` is not below ``inputsDir``, a ``ValueError`` results.  On 
+	If ``absPath`` is not below ``inputsDir``, a ``ValueError`` results.  On
 	``liberalChars``, we see the `function getRelativePath`_.
 
 	In rowmakers and rowfilters, you'll usually use the macro
