@@ -6,7 +6,7 @@ defining tests for a given piece of data.	These things can then be
 run while (or rather, after) executing dachs val.
 """
 
-#c Copyright 2008-2022, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -58,7 +58,7 @@ def _loadCreds():
 	"""
 	res = {}
 	try:
-		with open(os.path.join(os.environ["HOME"], ".gavo", "test.creds"), 
+		with open(os.path.join(os.environ["HOME"], ".gavo", "test.creds"),
 				"rb") as f:
 			for ln in f:
 				authKey, user, pw = ln.strip().split()
@@ -82,7 +82,7 @@ def getAuthFor(authKey):
 		base64.b64encode(b"%s:%s"%(user, pw))).strip()}
 
 
-def doHTTPRequest(scheme, method, host, path, query, 
+def doHTTPRequest(scheme, method, host, path, query,
 		payload, headers, timeout):
 	"""creates the HTTP request and retrieves the result.
 	"""
@@ -127,7 +127,7 @@ class Keywords(argparse.Action):
 	no keywords are defined or all keywords are present in other (after
 	case folding).
 
-	This doubles as an argparse action and as such is "self-parsing" if you 
+	This doubles as an argparse action and as such is "self-parsing" if you
 	will.
 	"""
 	def __init__(self, *args, **kwargs):
@@ -317,14 +317,14 @@ class DataURL(base.Structure):
 		" you like.",
 		copyable=True)
 	
-	_httpMethod = base.UnicodeAttribute("httpMethod", 
+	_httpMethod = base.UnicodeAttribute("httpMethod",
 		description="Request method; usually one of GET or POST",
 		default="GET")
 
-	_httpPost = common.ResdirRelativeAttribute("postPayload", 
+	_httpPost = common.ResdirRelativeAttribute("postPayload",
 		default=base.NotGiven,
 		description="Path to a file containing material that should go"
-		" with a POST request (conflicts with additional parameters).", 
+		" with a POST request (conflicts with additional parameters).",
 		copyable=True)
 
 	_parset = base.EnumeratedUnicodeAttribute("parSet",
@@ -333,7 +333,7 @@ class DataURL(base.Structure):
 		validValues=["form", "TAP"],
 		copyable=True)
 
-	_httpHeaders = base.DictAttribute("httpHeader", 
+	_httpHeaders = base.DictAttribute("httpHeader",
 		description="Additional HTTP headers to pass.",
 		copyable=True)
 
@@ -434,7 +434,7 @@ class DataURL(base.Structure):
 		status, respHeaders, content = doHTTPRequest(
 			scheme, str(self.httpMethod),
 			host, path, query, payload, headers, timeout)
-		
+	
 		while self.httpHonorRedirects and status in [301, 302, 303]:
 			scheme, host, path, _, query, _ = urllib.parse.urlparse(
 				getHeaderValue(respHeaders, "location"))
@@ -613,6 +613,7 @@ class RegTest(procdef.ProcApp, unittest.TestCase):
 		"v1": "http://www.ivoa.net/xml/VOTable/v1.1",
 		"o": "http://www.openarchives.org/OAI/2.0/",
 		"h": "http://www.w3.org/1999/xhtml",
+		"m": "http://www.ivoa.net/xml/mivot",
 	}
 
 	@utils.document
@@ -623,7 +624,9 @@ class RegTest(procdef.ProcApp, unittest.TestCase):
 		prefixes statically mapped; there's currently v2 (VOTable
 		1.2), v1 (VOTable 1.1), v (whatever VOTable version
 		is the current DaCHS default), h (the namespace of the
-		XHTML elements DaCHS generates), and o (OAI-PMH 2.0).
+		XHTML elements DaCHS generates), m (the provisional MIVOT namespace)
+		and o (OAI-PMH 2.0).
+
 		If you need more prefixes, hack the source and feed back
 		your changes (or just add to self.XPATH_NAMESPACE_MAP
 		locally).
@@ -794,8 +797,8 @@ class TestStatistics(object):
 		self.timeSum += runTime
 #XXX TODO: Payload can use a lot of memory -- I'm nuking it for now
 # -- maybe use an on-disk database to store this and allow later debugging?
-		self.runs.append((runTime, status, title, 
-			None, #str(payload), 
+		self.runs.append((runTime, status, title,
+			None, #str(payload),
 			srcRD))
 		self.lastTimestamp = time.time()
 
@@ -821,7 +824,7 @@ class TestStatistics(object):
 			if status!="OK":
 				failures.setdefault(srcRD, []).append("%s %s"%(status, title))
 
-		return "\n".join("From %s:\n	%s\n\n"%(srcRD, 
+		return "\n".join("From %s:\n	%s\n\n"%(srcRD,
 				"\n	".join(badTests))
 			for srcRD, badTests in failures.items())
 
@@ -847,7 +850,7 @@ class TestRunner(object):
 # those, the individual tests must be serialized, which happens using the magic
 # followUp attribute on the tests.
 
-	def __init__(self, suites, serverURL=None, 
+	def __init__(self, suites, serverURL=None,
 			verbose=True, dumpNegative=False, tags=None,
 			timeout=45, failFile=None, nRepeat=1,
 			execDelay=0, nThreads=8, printTitles=False,
@@ -888,7 +891,7 @@ class TestRunner(object):
 		"""constructs a TestRunner for a single RegTest
 		"""
 		return cls([base.makeStruct(RegTestSuite, tests=[test],
-				parent_=test.parent.parent)], 
+				parent_=test.parent.parent)],
 			**kwargs)
 
 	def _makeTestList(self, suites):
@@ -923,7 +926,7 @@ class TestRunner(object):
 			sys.stderr.write(" <%s> "%test.title)
 			sys.stderr.flush()
 
-		newThread = threading.Thread(target=self.runOneTest, 
+		newThread = threading.Thread(target=self.runOneTest,
 			args=(test, self.threadId, self.execDelay))
 		newThread.description = test.description
 		newThread.setDaemon(True)
@@ -944,7 +947,7 @@ class TestRunner(object):
 		To support sequential execution, if test has a followUp attribute,
 		this followUp is queued after the test has run.
 
-		If the execDelay argument is non-zero, the thread delays its execution 
+		If the execDelay argument is non-zero, the thread delays its execution
 		by that many seconds.
 		"""
 		if execDelay:
@@ -976,7 +979,7 @@ class TestRunner(object):
 
 				f = io.StringIO()
 				traceback.print_exc(file=f)
-				self.resultsQueue.put(("ERROR", test, ex, f.getvalue(), 
+				self.resultsQueue.put(("ERROR", test, ex, f.getvalue(),
 					time.time()-startTime))
 
 		finally:
@@ -996,7 +999,7 @@ class TestRunner(object):
 				test.title, test.getDataSource()))
 			print(">>>>", payload)
 		elif state=="ERROR":
-			print("**** Internal Failure: %s -- %s\n"%(test.title, 
+			print("**** Internal Failure: %s -- %s\n"%(test.title,
 				test.url.httpURL))
 			print(traceback)
 
@@ -1069,7 +1072,7 @@ def urlToURL():
 	while True:
 		parts = urllib.parse.urlparse(input())
 		print("<url %s>%s</url>"%(
-			" ".join('%s="%s"'%(k,v[0]) 
+			" ".join('%s="%s"'%(k,v[0])
 				for k,v in urllib.parse.parse_qs(parts.query).items()),
 			parts.path))
 
@@ -1134,18 +1137,18 @@ def parseCommandLine(args=None):
 		action="store", dest="timeout", type=int, default=15, metavar="SECONDS")
 	parser.add_argument("-D", "--dump-to", help="Dump the content of"
 		" last failing test to FILE", metavar="FILE",
-		action="store", type=str, dest="failFile", 
+		action="store", type=str, dest="failFile",
 		default=None)
 	parser.add_argument("-w", "--wait", help="Wait SECONDS before executing"
-		" a request", metavar="SECONDS", action="store", 
+		" a request", metavar="SECONDS", action="store",
 		dest="execDelay", type=int, default=0)
 	parser.add_argument("-u", "--serverURL", help="URL of the DaCHS root"
 		" at the server to test",
-		action="store", type=str, dest="serverURL", 
+		action="store", type=str, dest="serverURL",
 		default=base.getConfig("web", "serverURL"))
 	parser.add_argument("-n", "--number-par", help="Number of requests"
 		" to be run in parallel",
-		action="store", type=int, dest="nThreads", 
+		action="store", type=int, dest="nThreads",
 		default=8)
 	parser.add_argument("--seed", help="Seed the RNG with this number."
 		"  Note that this doesn't necessarily make the execution sequence"
