@@ -2,11 +2,9 @@
 Functions available to rowmaker procs.
 
 Rowmaker procs are compiled in the namespace defined by this module.
-
-Maybe we should allow additional modules to be specified in gavorc?
 """
 
-#c Copyright 2008-2022, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -14,6 +12,7 @@ Maybe we should allow additional modules to be specified in gavorc?
 
 import base64
 import datetime
+import linecache
 import math
 import os
 import pprint #noflake: exported name
@@ -71,7 +70,7 @@ class IgnoreThisRow(base.ExecutiveAction):
 	Note: To skip an entire source, raise SkipThis (usually in a rowfilter
 	or so).
 
-	Also note that the non-code way to skip things, `Triggers`_, is 
+	Also note that the non-code way to skip things, `Triggers`_, is
 	preferred when you don't already use code.
 	"""
 
@@ -161,7 +160,7 @@ def parseDate(literal, format="%Y-%m-%d"):
 
 @utils.document
 def parseTimestamp(literal, format="%Y-%m-%dT%H:%M:%S"):
-	"""returns a ``datetime.datetime`` object from a literal parsed according 
+	"""returns a ``datetime.datetime`` object from a literal parsed according
 	to the strptime-similar format.
 
 	A ``ValueError`` is raised if literal doesn't match format (actually,
@@ -208,7 +207,7 @@ def makeTimestamp(date, time):
 def parseAngle(literal, format, sepChar=None):
 	"""converts the various forms angles might be encountered to degrees.
 
-	format is one of hms, dms, fracHour.  For sexagesimal/time angles, 
+	format is one of hms, dms, fracHour.  For sexagesimal/time angles,
 	you can pass a sepChar (default: split at blanks) that lets you
 	specify what separates hours/degrees, minutes, and seconds.
 
@@ -237,7 +236,7 @@ def computeMean(val1, val2):
 
 	>>> computeMean(1.,3)
 	2.0
-	>>> computeMean(datetime.datetime(2000, 10, 13), 
+	>>> computeMean(datetime.datetime(2000, 10, 13),
 	...   datetime.datetime(2000, 10, 12))
 	datetime.datetime(2000, 10, 12, 12, 0)
 	"""
@@ -285,7 +284,7 @@ def scale(val, factor, offset=0):
 @utils.document
 def parseWithNull(literal, baseParser, nullLiteral=base.Undefined,
 		default=None, checker=None):
-	"""returns default if literal is ``nullLiteral``, else 
+	"""returns default if literal is ``nullLiteral``, else
 	``baseParser(literal)``.
 
 	If ``checker`` is non-None, it must be a callable returning ``True`` if its
@@ -321,7 +320,7 @@ def requireValue(val, fieldName):
 	for ``fieldName`` will be raised.
 	"""
 	if val is None:
-		raise base.ValidationError("Value is required but was not provided", 
+		raise base.ValidationError("Value is required but was not provided",
 			fieldName)
 	return val
 
@@ -330,7 +329,7 @@ def genLimitKeys(inputKey):
 	"""yields _MAX and _MIN inputKeys from a single input key.
 
 	This also tries to sensibly fix descriptions and ucds.
-	This is mainly for datalink metaMakers; condDescs may use a 
+	This is mainly for datalink metaMakers; condDescs may use a
 	similar thing, but that's not exposed to RDs.
 
 	Don't use this function any more.  It will go away soon.
@@ -342,10 +341,10 @@ def genLimitKeys(inputKey):
 		"  Fix your RD to use proper DALI-intervals (ask dachs-support"
 		" if you don't know what to do.)")
 
-	yield inputKey.change(name=name+"_MIN", 
+	yield inputKey.change(name=name+"_MIN",
 		ucd=ucd and "par.min;"+ucd,
 		description=description.rstrip(".")+", lower limit")
-	yield inputKey.change(name=name+"_MAX", 
+	yield inputKey.change(name=name+"_MAX",
 		ucd=ucd and "par.max;"+ucd,
 		description=description.rstrip(".")+", upper limit")
 
@@ -379,20 +378,33 @@ def makeProc(funcName, code, setupCode, parent, **moreNames):
 	funcNs["parent"] = parent
 	funcNs.update(moreNames)
 	if setupCode.strip():
+		src = setupCode.rstrip()
 		try:
-			exec(setupCode.rstrip(), funcNs)
+			uniqueName = "<setup code in {}>".format(parent.getSourcePosition())
+			compiled = compile(src, uniqueName, 'exec')
+			linecache.cache[uniqueName] = (
+				len(src), None, src.split("\n"), uniqueName)
+			exec(compiled, funcNs)
 		except (SyntaxError, TypeError) as ex:
 			raise base.ui.logOldExc(
-				base.BadCode(setupCode, "setup code", ex, 
+				base.BadCode(setupCode, "setup code", ex,
 					pos=parent.getSourcePosition()))
 		except NameError as ex:
 			raise base.ui.logOldExc(
-				base.BadCode(setupCode, "setup code", ex, 
+				base.BadCode(setupCode, "setup code", ex,
 					hint="This typically happens when you forget to put"
-					" quotes around string values.", 
+					" quotes around string values.",
 					pos=parent.getSourcePosition()))
-	return utils.compileFunction(code.rstrip(), funcName, funcNs, 
-		debug=base.DEBUG)
+
+	uniqueName = None
+	if parent and getattr(parent, "name_", None):
+		uniqueName = "<{} at {}>".format(parent.name_, parent.getSourcePosition())
+	return utils.compileFunction(
+		code.rstrip(),
+		funcName,
+		funcNs,
+		debug=base.DEBUG,
+		uniqueName=uniqueName)
 
 
 def _test():  # pragma: no cover
