@@ -5,7 +5,7 @@ See the __init__.py docstring for a bit more on the general
 architecture.
 """
 
-#c Copyright 2008-2022, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -59,7 +59,7 @@ class Publication(base.Structure, base.ComputedMetaMixin):
 	_render = base.UnicodeAttribute("render", default=base.Undefined,
 		description="The renderer the publication will point at.",
 		copyable=True)
-	_sets = base.StringSetAttribute("sets", 
+	_sets = base.StringSetAttribute("sets",
 		description="Comma-separated list of sets this service will be"
 			" published in.  Predefined are: local=publish on front page,"
 			" ivo_managed=register with the VO registry.  If you leave it"
@@ -87,7 +87,7 @@ class Publication(base.Structure, base.ComputedMetaMixin):
 			self.sets)
 
 	def _completeMetadataFromResRow(self, resRow):
-		"""fiddles publication dates from a dc.resources row for the parent 
+		"""fiddles publication dates from a dc.resources row for the parent
 		service.
 
 		(see rscdef.rdjinj for where this comes from).
@@ -106,7 +106,7 @@ class Publication(base.Structure, base.ComputedMetaMixin):
 			self.service = self.parent
 		self.setMetaParent(self.service)
 
-		if (ctx is not None 
+		if (ctx is not None
 				# RD-less services are probably testing-only, and they certainly
 				# won't be registered.
 				and self.rd is not None):
@@ -162,7 +162,7 @@ class CustomPageFunction(base.Structure, base.RestrictionMixin):
 	"""
 	_name = base.UnicodeAttribute("name", default=base.Undefined,
 		description="Name of the render or data function (use this in the"
-			" n:render or n:data attribute in custom templates).", 
+			" n:render or n:data attribute in custom templates).",
 			copyable=True, strip=True)
 	_code = base.DataContent(description="Function body of the renderer"
 		" or data function; the"
@@ -261,7 +261,7 @@ class CustomDF(CustomPageFunction):
 	
 	You can use the request to fetch request parameters.  Within DaCHS,
 	in addition to the clumsy request.args (mapping bytes to bytes), there
-	is also request.strargs (mapping strings to strings).  So, access a 
+	is also request.strargs (mapping strings to strings).  So, access a
 	query parameter ``order`` like this::
 
 		sortOrder = ctx.request.strargs.get("order", ["authors"])
@@ -271,10 +271,10 @@ class CustomDF(CustomPageFunction):
 
 class CoreAttribute(base.ReferenceAttribute):
 	def __init__(self):
-		base.ReferenceAttribute.__init__(self, "core", 
+		base.ReferenceAttribute.__init__(self, "core",
 			description="The core that does the computations for this service."
 			"  Instead of a reference, you can use an immediate element"
-			" of some registered core.", 
+			" of some registered core.",
 			forceType=core.Core, copyable=True, aliases=list(core.CORE_REGISTRY.keys()))
 	
 	def _makeChild(self, name, parent):
@@ -289,7 +289,7 @@ def getDALIServiceKeys():
 	in here, be sure to make copies before touching either the list or
 	the items.
 	"""
-	nullSvc = base.parseFromString(Service, 
+	nullSvc = base.parseFromString(Service,
 		"""<service><nullCore/><FEED source="//pql#DALIPars"/></service>""")
 
 	res = []
@@ -301,7 +301,7 @@ def getDALIServiceKeys():
 	return res
 
 
-class Service(base.Structure, base.ComputedMetaMixin, 
+class Service(base.Structure, base.ComputedMetaMixin,
 		base.StandardMacroMixin, rscdef.IVOMetaMixin):
 	"""A service definition.
 
@@ -330,7 +330,7 @@ class Service(base.Structure, base.ComputedMetaMixin,
 		' nevow templates for this service; use key "form" to replace the Form'
 		" renderer's standard template; qp uses resulttable and resultline"
 		" depending on whether there's many result lines or just one."
-		"  Start the path with two slashes to access system templates.", 
+		"  Start the path with two slashes to access system templates.",
 		itemAttD=rscdef.ResdirRelativeAttribute(
 			"template", description="resdir-relative path to a nevow template"
 			" used for the function given in key."), copyable=True)
@@ -401,7 +401,7 @@ class Service(base.Structure, base.ComputedMetaMixin,
 		# style of dali or pql.
 		for rendName in self.allowed:
 			try:
-				ps = renderers.getRenderer(rendName).parameterStyle 
+				ps = renderers.getRenderer(rendName).parameterStyle
 			except base.NotFoundError:
 				# letting this exc through could lead to confusing error messages
 				# on cross-RD reference, so we just warn:
@@ -446,7 +446,7 @@ class Service(base.Structure, base.ComputedMetaMixin,
 				page = modNs.MainPage
 			except ImportError:
 				raise base.ui.logOldExc(
-					base.LiteralParseError("customPage", self.customPage, 
+					base.LiteralParseError("customPage", self.customPage,
 					hint="This means that an exception was raised while DaCHS"
 						" tried to import the renderer module.  If DaCHS ran"
 						" with --debug, the original traceback is available"
@@ -508,7 +508,7 @@ class Service(base.Structure, base.ComputedMetaMixin,
 		This method ventures a guess.  You can override this decision by setting
 		the resType meta item.
 		"""
-		if (self.core.outputTable.columns 
+		if (self.core.outputTable.columns
 			or "dali" in self.allowed):
 			self.resType = "catalogService"
 		else: # no output table defined, we're a plain service
@@ -524,36 +524,36 @@ class Service(base.Structure, base.ComputedMetaMixin,
 
 		# All actual services get VOSI caps
 		if not isinstance(self.core, core.getCore("nullCore")):
-			yield base.makeStruct(Publication, 
-					render="availability", 
+			yield base.makeStruct(Publication,
+					render="availability",
 					sets=vosiSet,
 					parent_=self)
-			yield base.makeStruct(Publication, 
-					render="capabilities", 
+			yield base.makeStruct(Publication,
+					render="capabilities",
 					sets=vosiSet,
 					parent_=self)
-			yield base.makeStruct(Publication, 
-					render="tableMetadata", 
+			yield base.makeStruct(Publication,
+					render="tableMetadata",
 					sets=vosiSet,
 					parent_=self)
 
-		# things querying tables get a TAP relationship if 
+		# things querying tables get a TAP relationship if
 		# their table is adql-queriable
 		if isinstance(self.core, core.getCore("dbCore")):
 			if self.core.queriedTable.adql:
-				tapService = base.resolveCrossId("//tap#run") 
-				yield base.makeStruct(Publication, 
+				tapService = base.resolveCrossId("//tap#run")
+				yield base.makeStruct(Publication,
 						render="dali",
 						sets=vosiSet,
-						auxiliary=True, 
+						auxiliary=True,
 						service=tapService,
 						parent_=self)
 
 		# things with examples meta get an examples capability
 		try:
 			self.getMeta("_example", raiseOnFail=True)
-			yield base.makeStruct(Publication, 
-					render="examples", 
+			yield base.makeStruct(Publication,
+					render="examples",
 					sets=utils.AllEncompassingSet(),
 					parent_=self)
 		except base.NoMetaKey:
@@ -593,7 +593,7 @@ class Service(base.Structure, base.ComputedMetaMixin,
 		if isinstance(self.core, core.getCore("dbCore")):
 			if self.core.queriedTable.adql:
 				tapService = base.resolveCrossId("//tap#run")
-				self.addMeta("isServedBy", 
+				self.addMeta("isServedBy",
 					base.getMetaText(tapService, "title"),
 					ivoId=base.getMetaText(tapService, "identifier"))
 
@@ -637,7 +637,7 @@ class Service(base.Structure, base.ComputedMetaMixin,
 		# the or in the list comprehension is because I can't see a way
 		# to make AllEmcompassingSet work on the right side of the
 		# operand, but it can occur on both sides.
-		result = [pub for pub in self.publications 
+		result = [pub for pub in self.publications
 			if pub.sets & names or names & pub.sets]
 
 		# for ivo_managed, also return a datalink endpoints if they're
@@ -683,7 +683,7 @@ class Service(base.Structure, base.ComputedMetaMixin,
 		return result
 
 	def getURL(self, rendName, absolute=True, canonical=False, **kwargs):
-		"""returns the full canonical access URL of this service together 
+		"""returns the full canonical access URL of this service together
 		with renderer.
 
 		rendName is the name of the intended renderer in the registry
@@ -708,7 +708,7 @@ class Service(base.Structure, base.ComputedMetaMixin,
 	# used by getBrowserURL; keep external higher than form as long as
 	# we have mess like Potsdam CdC.
 	_browserScores = {"form": 10, "external": 12, "fixed": 15,
-		"custom": 3, "img.jpeg": 2, "static": 1}
+		"custom": 8, "img.jpeg": 7, "static": 5, "hips": 3, "examples": 2}
 
 	def getBrowserURL(self, fq=True):
 		"""returns a published URL that's suitable for a web browser or None if
@@ -729,7 +729,7 @@ class Service(base.Structure, base.ComputedMetaMixin,
 			return None
 		
 	def isBrowseableWith(self, rendName):
-		"""returns true if rendering this service through rendName results 
+		"""returns true if rendering this service through rendName results
 		in something pretty in a web browser.
 		"""
 		try:
@@ -758,10 +758,10 @@ class Service(base.Structure, base.ComputedMetaMixin,
 		This is used by table/@adql and the publish element on data.
 		"""
 		if data.registration:
-			self.addMeta("isServiceFor", 
+			self.addMeta("isServiceFor",
 				base.getMetaText(data, "title", default="Anonymous"),
 				ivoId=base.getMetaText(data, "identifier"))
-			data.addMeta("isServedBy", 
+			data.addMeta("isServedBy",
 				base.getMetaText(self, "title"),
 				ivoId=base.getMetaText(self, "identifier"))
 
@@ -813,7 +813,7 @@ class Service(base.Structure, base.ComputedMetaMixin,
 			columnSource = self.getAllOutputFields()
 
 		fields = self._getFilteredColumns(
-			[f for f in columnSource 
+			[f for f in columnSource
 				if f.displayHint.get("noxml")!="true"],
 			queryMeta,
 			verbLevel)
@@ -873,7 +873,7 @@ class Service(base.Structure, base.ComputedMetaMixin,
 		# best choice for DaCHS, but changing this now to VOTable would introduce
 		# needless breakage).
 		renderer = queryMeta.get("renderer")
-		format = queryMeta.get("format", 
+		format = queryMeta.get("format",
 			getattr(renderer, "defaultOutputFormat", None)) or "HTML"
 
 		if format in self.htmlLikeFormats:
@@ -886,7 +886,7 @@ class Service(base.Structure, base.ComputedMetaMixin,
 
 		This is what the core gives, and this is what will be declared
 		to the registry.  Depending on the output format, the verbosity
-		level and perhaps other user settings, the actuall columns produced
+		level and perhaps other user settings, the actual columns produced
 		will be different.
 		"""
 		return self.core.outputTable.columns
@@ -896,7 +896,7 @@ class Service(base.Structure, base.ComputedMetaMixin,
 	def getCoreFor(self, renderer, queryMeta):
 		"""returns a core tailored for renderer.
 
-		See svcs.core's module docstring. 
+		See svcs.core's module docstring.
 		
 		The argument can be a renderer or a renderer name.
 		"""
@@ -961,59 +961,62 @@ class Service(base.Structure, base.ComputedMetaMixin,
 	def adaptCoreRes(self, coreRes, queryMeta):
 		"""adapts a core result to the service's output table if appropriate.
 
+		Frankly, this was an early mis-design that complicates matters
+		unnecessarily, but I can't really drop it as long as some interesting
+		things are based on service's outputTables.
+
 		The adaptation works like this:
 
 		(0) if coreRes isn't a table, or that table has a noPostprocess
 		attribute, return coreRes and let the renderer worry about it.
 
-		(1) if names and units of newColumns are a subset of coreRes.columns,
-		return a table with the rows from coreRes and the columns from
-		newColumns.
-
-		(1a) if coreRes has a noPostprocess attribute, proceed to (4)
+		(1) if names and units of newColumns the same as coreRes.columns,
+		accept coreRes as the new table
 
 		(2) if names of newColumns are a subset of coreRes.columns match
 		but one or more units don't, set up a conversion routine and create
 		new rows, combining them with newColumns to the result.
 
-		(3) else raise an error.
+		(3) else raise an error
 
-		(4) Finally, stick the whole thing into a data container.
+		(4) eventually, wrap everything up in a rsc.Data instance for
+		compatibility with cores actually returning data items.
 		"""
 		if not isinstance(coreRes, rsc.BaseTable):
 			return coreRes
 
-		newColumns = self.getCurOutputFields(queryMeta)
+		newTable = coreRes
 
 		if hasattr(coreRes, "noPostprocess"):
-			colDiffs = None
-			newTd = coreRes.tableDef
-
+			colDiffs, swallowColumns = None, None
 		else:
+			newColumns = self.getCurOutputFields(queryMeta)
+			swallowColumns = {c.name for c in coreRes.tableDef
+				}-{c.name for c in newColumns}
 			colDiffs = base.computeColumnConversions(
 				newColumns, coreRes.tableDef.columns)
-			newTd = coreRes.tableDef.copy(coreRes.tableDef.parent)
-			newTd.columns = newColumns
 
-		if not colDiffs:
-			newTable = table.InMemoryTable(newTd, rows=coreRes.rows)
-			newTable.meta_ = coreRes.meta_
-			newTable._params = coreRes._params
-
-		else: # we need to do work
+		if colDiffs or swallowColumns:
+			newTd = coreRes.tableDef.change(columns=newColumns)
 			newTd.copyMetaFrom(coreRes.tableDef)
-			rmk = rscdef.RowmakerDef(None)
-			for col in newColumns:
-				exprStart = ""
-				if col.name in colDiffs:
-					exprStart = "%s*"%colDiffs[col.name]
-				rmk.feedObject("map", rmkdef.MapRule(rmk, dest=col.name,
-					content_="%svars[%s]"%(exprStart, repr(col.name))
-					).finishElement(None))
-			newTable = table.InMemoryTable(newTd, validate=False)
-			mapper = rmk.finishElement(None).compileForTableDef(newTd)
-			for r in coreRes:
-				newTable.addRow(mapper(r, newTable))
+
+			if colDiffs:
+				rmk = rscdef.RowmakerDef(None)
+				for col in newColumns:
+					exprStart = ""
+					if col.name in colDiffs:
+						exprStart = "%s*"%colDiffs[col.name]
+					rmk.feedObject("map", rmkdef.MapRule(rmk, dest=col.name,
+						content_="%svars[%s]"%(exprStart, repr(col.name))
+						).finishElement(None))
+				newTable = table.InMemoryTable(newTd, validate=False)
+				mapper = rmk.finishElement(None).compileForTableDef(newTd)
+				for r in coreRes:
+					newTable.addRow(mapper(r, newTable))
+
+			else:
+				newTable = rsc.TableForDef(newTd, rows=coreRes.rows)
+
 			newTable._params = coreRes._params
 
 		return rsc.wrapTable(newTable, rdSource=coreRes.tableDef)
