@@ -1,12 +1,12 @@
 """
 Some standard cores for services.
 
-A core receives and "input table" (usually just containing the query 
+A core receives and "input table" (usually just containing the query
 parameters in the doc rec) and returns an output thing, usually a table
 (but see the src package docstring).
 """
 
-#c Copyright 2008-2022, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -77,12 +77,12 @@ class CondDesc(base.Structure):
 	"""
 	name_ = "condDesc"
 
-	_inputKeys = rscdef.ColumnListAttribute("inputKeys", 
-		childFactory=inputdef.InputKey, 
+	_inputKeys = rscdef.ColumnListAttribute("inputKeys",
+		childFactory=inputdef.InputKey,
 		description="One or more InputKeys defining the condition's input.",
 		copyable=True)
 
-	_silent = base.BooleanAttribute("silent", 
+	_silent = base.BooleanAttribute("silent",
 		default=False,
 		description="Do not produce SQL from this CondDesc.  This"
 			" can be used to convey meta information to the core.  However,"
@@ -91,28 +91,34 @@ class CondDesc(base.Structure):
 			" to silent CondDescs.",
 		copyable=True)
 
-	_required = base.BooleanAttribute("required", 
+	_required = base.BooleanAttribute("required",
 		default=False,
 		description="Reject queries not filling the InputKeys of this CondDesc",
 		copyable=True)
 
-	_fixedSQL = base.UnicodeAttribute("fixedSQL", 
+	_inputOptional = base.BooleanAttribute("inputOptional",
+		default=False,
+		description="Call the phrase maker even if no input to our input keys"
+			" is there (useful to provide built-in defaults).",
+		copyable=True)
+
+	_fixedSQL = base.UnicodeAttribute("fixedSQL",
 		default=None,
 		description="Always insert this SQL statement into the query.  Deprecated.",
 		copyable=True)
 
-	_buildFrom = base.ReferenceAttribute("buildFrom", 
+	_buildFrom = base.ReferenceAttribute("buildFrom",
 		description="A reference to a column or an InputKey to define"
 			" this CondDesc",
 		default=None)
 
-	_phraseMaker = base.StructAttribute("phraseMaker", 
+	_phraseMaker = base.StructAttribute("phraseMaker",
 		default=None,
-		description="Code to generate custom SQL from the input keys", 
-		childFactory=PhraseMaker, 
+		description="Code to generate custom SQL from the input keys",
+		childFactory=PhraseMaker,
 		copyable=True)
 
-	_combining = base.BooleanAttribute("combining", 
+	_combining = base.BooleanAttribute("combining",
 		default=False,
 		description="Allow some input keys to be missing when others are given?"
 			" (you want this for pseudo-condDescs just collecting random input"
@@ -209,6 +215,11 @@ class CondDesc(base.Structure):
 
 	def _isActive(self, inPars):
 		"""returns True if the dict inPars contains input to all our input keys.
+
+		This is different from whether there is input to these keys; keys with
+		no input are in inParse with a value of None.  That means that
+		if _isActive is false, the core for some reason hasn't included
+		our input keys and hence we shouldn't run.
 		"""
 		for f in self.inputKeys:
 			if f.name not in inPars:
@@ -223,6 +234,8 @@ class CondDesc(base.Structure):
 		"""
 		if not self._isActive(inPars):
 			return False
+		if self.inputOptional:
+			return True
 		keysFound, keysMissing = [], []
 		for f in self.inputKeys:
 			if inPars.get(f.name) is None:
@@ -237,13 +250,13 @@ class CondDesc(base.Structure):
 		if not self.required and not keysFound:
 			return False
 		if self.required:
-			raise base.ValidationError("is mandatory but was not provided.", 
+			raise base.ValidationError("is mandatory but was not provided.",
 				colName=keysMissing[0].name)
 
 		# we're optional, but a value was given and others are missing
 		if not self.combining:
 			raise base.ValidationError("When you give a value for %s,"
-				" you must give value(s) for %s, too"%(keysFound[0].getLabel(), 
+				" you must give value(s) for %s, too"%(keysFound[0].getLabel(),
 						", ".join(k.name for k in keysMissing)),
 					colName=keysMissing[0].name)
 		return True
@@ -267,7 +280,7 @@ class CondDesc(base.Structure):
 		which will do because core.adaptForRenderer stops adapting if
 		the condDescs are stable).
 
-		The adaptors may also raise a Replace exception and return a 
+		The adaptors may also raise a Replace exception and return a
 		full CondDesc; this is done, e.g., for spoints for the form
 		renderer, since they need two input keys and a completely modified
 		phrase.
@@ -339,7 +352,7 @@ class TableBasedCore(core.Core):
 			if cd.group:
 				groups.append(cd.group.change(
 					paramRefs=[MS(rscdef.ParameterReference, dest=ik.name)
-						for ik in cd.inputKeys], 
+						for ik in cd.inputKeys],
 					parent_=None))
 		self.inputTable = MS(inputdef.InputTD,
 			inputKeys=iks,
@@ -359,7 +372,7 @@ class TableBasedCore(core.Core):
 		# if no outputTable has been given, make it up from the columns
 		# of the queried table unless a prototype is defined (which is
 		# handled by core itself).
-		if (self.outputTableXML is None 
+		if (self.outputTableXML is None
 				and self.outputTable is base.NotGiven
 				and self.queriedTable):
 			self.outputTable = outputdef.OutputTableDef.fromTableDef(
@@ -391,7 +404,7 @@ class TableBasedCore(core.Core):
 			del resultTable.rows[-1]
 		queryMeta["Matched"] = len(resultTable.rows)
 		if isOverflowed:
-			resultTable.addMeta("_warning", 
+			resultTable.addMeta("_warning",
 				"The query limit was reached.  Increase it"
 				" to retrieve more matches.  Note that unsorted truncated queries"
 				" are not reproducible (i.e., might return a different result set"
@@ -423,7 +436,7 @@ class TableBasedCore(core.Core):
 class FancyQueryCore(TableBasedCore, base.RestrictionMixin):
 	"""A core executing a pre-specified query with fancy conditions.
 
-	Unless you select \*, you *must* define the outputTable here; 
+	Unless you select \*, you *must* define the outputTable here;
 	Weird things will happen if you don't.
 
 	The queriedTable attribute is ignored.
@@ -433,10 +446,10 @@ class FancyQueryCore(TableBasedCore, base.RestrictionMixin):
 	_query = base.UnicodeAttribute("query", description="The query to"
 		" execute.  It must contain exactly one %s where the generated"
 		" where clause is to be inserted.  Do not write WHERE yourself."
-		" All other percents must be escaped by doubling them.", 
+		" All other percents must be escaped by doubling them.",
 		default=base.Undefined,
 		copyable=True)
-	_timeout = base.FloatAttribute("timeout", default=5., 
+	_timeout = base.FloatAttribute("timeout", default=5.,
 		description="Seconds until the query is aborted",
 		copyable=True)
 
@@ -451,7 +464,7 @@ class FancyQueryCore(TableBasedCore, base.RestrictionMixin):
 				res = rsc.TableForDef(
 					self.outputTable,
 					rows=list(conn.queryToDicts(
-						self.query%fragment, 
+						self.query%fragment,
 						pars,
 						timeout=self.timeout,
 						caseFixer=self.outputTable.caseFixer)))
@@ -466,7 +479,7 @@ class DBCore(TableBasedCore):
 	"""A core performing database queries on one table or view.
 
 	DBCores ask the service for the desired output schema and adapt their
-	output.  The DBCore's output table, on the other hand, lists all fields 
+	output.  The DBCore's output table, on the other hand, lists all fields
 	available from the queried table.
 	"""
 	name_ = "dbCore"
@@ -524,10 +537,11 @@ class DBCore(TableBasedCore):
 	def _makeResultTableDef(self, service, inputTable, queryMeta):
 		"""returns an OutputTableDef object for querying our table with queryMeta.
 		"""
-		return base.makeStruct(outputdef.OutputTableDef,
+		resTD = base.makeStruct(outputdef.OutputTableDef,
 			parent_=self.queriedTable.parent, id="result",
 			onDisk=False, columns=self.getQueryCols(service, queryMeta),
 			params=self.queriedTable.params)
+		return resTD
 
 	def run(self, service, inputTable, queryMeta):
 		"""does the DB query and returns an InMemoryTable containing
@@ -564,7 +578,7 @@ class FixedQueryCore(core.Core, base.RestrictionMixin):
 	"""
 	name_ = "fixedQueryCore"
 
-	_timeout = base.FloatAttribute("timeout", default=15., 
+	_timeout = base.FloatAttribute("timeout", default=15.,
 		description="Seconds until the query is aborted", copyable=True)
 	_query = base.UnicodeAttribute("query", default=base.Undefined,
 		description="The query to be executed.  You must define the"
