@@ -2,7 +2,7 @@
 A wrapper script suitable for starting the server.
 """
 
-#c Copyright 2008-2022, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -147,13 +147,13 @@ def manageResourceLimits():
 	"""
 	try:
 		configuredLimit = base.getConfig("web", "serverFDLimit")
-		resource.setrlimit(resource.RLIMIT_NOFILE, 
+		resource.setrlimit(resource.RLIMIT_NOFILE,
 			(configuredLimit, configuredLimit))
 		_, hard = resource.getrlimit(resource.RLIMIT_CORE)
 		resource.setrlimit(resource.RLIMIT_CORE, (hard, hard))
 
 		# for an actual coredump in production systems (which setuid)
-		# you will usually also need 
+		# you will usually also need
 		# echo 2 >/proc/sys/fs/suid_dumpable
 		# and this (needs python3-prctl installed):
 		# import prctl; prctl.set_dumpable(1)
@@ -181,7 +181,7 @@ def _dropPrivileges():
 		try:
 			try:
 				os.setgid(grp.getgrnam(base.getConfig("group"))[2])
-			except Exception as ex: 
+			except Exception as ex:
 				# don't fail because of setgid failure (should I rather?)
 				warnings.warn("Could not sgid to gavo group (%s)."%(str(ex)))
 			os.setuid(uid)
@@ -200,12 +200,12 @@ def _dropPrivileges():
 
 def daemonize(logFile, callable):
 	# We translate TERMs to INTs to ensure finally: code is executed
-	signal.signal(signal.SIGTERM, 
+	signal.signal(signal.SIGTERM,
 		lambda a,b: os.kill(os.getpid(), signal.SIGINT))
 	pid = os.fork()
 	if pid == 0:
-		os.setsid() 
-		pid = os.fork() 
+		os.setsid()
+		pid = os.fork()
 		if pid==0:
 			os.close(0)
 			os.close(1)
@@ -272,7 +272,7 @@ def _preloadRDs():
 
 	Errors while loading those are logged but are not fatal to the server.
 
-	This must be run from the reactor if preloadAllRDs is set, as it 
+	This must be run from the reactor if preloadAllRDs is set, as it
 	uses deferreds.
 	"""
 	for rdId in base.getConfig("web", "preloadRDs"):
@@ -329,7 +329,7 @@ def _loadCertificate(srcFile):
 
 	# now build the options from one key, a cert for it, and any number
 	# of other intermediate stuff
-	if len(keys)!=1: 
+	if len(keys)!=1:
 		raise base.ReportableError("Only one secret key expected in %s"%srcFile)
 	privateFingerprint = keys[0].keyHash()
 
@@ -368,8 +368,8 @@ def _perhapsEnableSSL(factory, sslPort=443):
 
 	try:
 		reactor.listenSSL(
-			sslPort, 
-			factory, 
+			sslPort,
+			factory,
 			_loadCertificate(certPath),
 			interface=base.getConfig("web", "bindAddress"))
 		base.LISTENING_TO_HTTPS = True
@@ -383,7 +383,7 @@ def _startServer(managePID=True):
 	"""
 	try:
 		reactor.listenTCP(
-			int(base.getConfig("web", "serverPort")), 
+			int(base.getConfig("web", "serverPort")),
 			root.site,
 			interface=base.getConfig("web", "bindAddress"))
 		_perhapsEnableSSL(root.site)
@@ -403,11 +403,11 @@ def _startServer(managePID=True):
 		PIDManager.setPID()
 	try:
 		setupServer(root)
-		signal.signal(signal.SIGHUP, lambda sig, stack: 
+		signal.signal(signal.SIGHUP, lambda sig, stack:
 			reactor.callLater(0, _reloadConfig))
 		reactor.callLater(1, _preloadRDs)
 		if base.getConfig("web", "preloadpublishedrds"):
-			task.deferLater(reactor, 
+			task.deferLater(reactor,
 				1, threads.deferToThread, _preloadPublishedRDs)
 		reactor.run()
 	finally:
@@ -467,7 +467,9 @@ def _waitForServerExit(timeout=5):
 def _stopServer():
 	pid = PIDManager.getPID()
 	if pid is None:  # No server running, nothing to do
-		base.ui.notifyWarning("No running DaCHS server found.")
+		base.ui.notifyWarning("No running DaCHS server found."
+			"  If you are trying to restart a DaCHS server on a systemd machine:"
+			" use `sudo systemctl restart dachs` instead.")
 		return
 
 	try:
@@ -542,10 +544,11 @@ class ExitPage(twresource.Resource):
 	" all kinds of stuff to the terminal")
 def debug(args):
 	log.startLogging(sys.stderr)
+	root.site.webLog = _configureTwistedLog()
 	base.DEBUG = True
 	root.root.child_exit = ExitPage()
 	reactor.listenTCP(
-		int(base.getConfig("web", "serverPort")), 
+		int(base.getConfig("web", "serverPort")),
 		root.site,
 		interface=base.getConfig("web", "bindAddress"))
 	_perhapsEnableSSL(root.site, 40443)
@@ -616,8 +619,8 @@ def updateCertificate(args):
 		pass # let's hope the challenge dir is already in place
 
 	# The letsencrypt interaction is done by acme-tiny
-	cert = subprocess.check_output(["acme-tiny", 
-		"--account-key", "./account.key", "--csr", "dachs.csr", 
+	cert = subprocess.check_output(["acme-tiny",
+		"--account-key", "./account.key", "--csr", "dachs.csr",
 		"--acme-dir", acmeChallengeDir])
 	
 	# finally, glue together the signature with intermediary certificates
@@ -634,7 +637,7 @@ def updateCertificate(args):
 		f.write(intermediate+b"\n")
 		f.write(secretKey+b"\n")
 
-	subprocess.check_call(["dachs", "serve", "restart", 
+	subprocess.check_call(["dachs", "serve", "restart",
 		"--if-unloaded-within=300"])
 
 
