@@ -5,7 +5,7 @@ This comprises helpers and wrappers that do not need gavo.base but for some
 reason or another should be within the dc package.
 """
 
-#c Copyright 2008-2022, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -26,8 +26,8 @@ from gavo.utils.autonode import AutoNode
 
 from gavo.utils.codetricks import (silence, ensureExpression, compileFunction,
 	loadPythonModule, DeferredImport,
-	memoized, identity, runInSandbox, document, 
-	buildClassResolver, CachedGetter, CachedResource, intToFunnyWord, 
+	memoized, identity, runInSandbox, document,
+	buildClassResolver, CachedGetter, CachedResource, intToFunnyWord,
 	IdManagerMixin,
 	addDefaults, iterDerivedClasses, iterDerivedObjects, iterConsecutivePairs,
 	importModule, loadInternalObject, printFrames, memoizeOn, forgetMemoized,
@@ -37,7 +37,8 @@ from gavo.utils.codetricks import (silence, ensureExpression, compileFunction,
 	stealVar,
 	AllEncompassingSet,
 	bytelist, intlist, floatlist, complexlist,
-	EqualingRE, NocaseString)
+	EqualingRE, NocaseString,
+	NS)
 from builtins import list
 
 from gavo.utils.excs import *
@@ -53,9 +54,9 @@ from gavo.utils.fitstools import (readPrimaryHeaderQuick, pyfits,
 from gavo.utils.mathtricks import *
 
 from gavo.utils.misctricks import (Undefined, QuotedName, getfirst,
-	logOldExc, sendUIEvent, 
+	logOldExc, sendUIEvent,
 	getWithCache,
-	rstxToHTML, rstxToHTMLWithWarning, 
+	rstxToHTML, rstxToHTMLWithWarning,
 	couldBeABibcode,
 	parseKVLine, makeKVLine,
 	StreamBuffer, CaseSemisensitiveDict,
@@ -63,7 +64,7 @@ from gavo.utils.misctricks import (Undefined, QuotedName, getfirst,
 	getCleanBytes, getDirtyBytes,
 	RateLimiter)
 
-from gavo.utils.ostricks import (safeclose, 
+from gavo.utils.ostricks import (safeclose,
 	urlopenRemote, setUserAgent,
 	fgetmtime, cat, ensureDir, safeReplaced,
 	Arg, exposedFunction, makeCLIParser, StatusDisplay)
@@ -80,7 +81,7 @@ from gavo.utils.stanxml import (ElementTree, xmlrender, xmlwrite,
 from gavo.utils.texttricks import (formatSize, formatFloat,
 	makeEllipsis, makeLeftEllipsis, makeSourceEllipsis,
 	floatRE, dateRE, datetimeRE, identifierPattern, looksLikeURLPat,
-	datetimeToRFC2616, 
+	datetimeToRFC2616,
 	parseDefaultDatetime, parseDefaultDate, parseDefaultTime,
 	parseAccept,
 	isoTimestampFmt, isoTimestampFmtNoTZ, parseISODT, formatISODT,
@@ -89,7 +90,7 @@ from gavo.utils.texttricks import (formatSize, formatFloat,
 	fixIndentation, parsePercentExpression, hmsToDeg, dmsToDeg,
 	makeIAUId,
 	fracHoursToDeg, degToHms, degToDms, hoursToHms,
-	getRelativePath, parseAssignments, 
+	getRelativePath, parseAssignments,
 	NameMap, formatSimpleTable, replaceXMLEntityRefs,
 	ensureOneSlash, getRandomString,
 	safe_str, defuseFileName, bytify, debytify, iterSimpleText)
