@@ -1,9 +1,9 @@
 """
-Functions dealing with compilation and introspection of python and 
+Functions dealing with compilation and introspection of python and
 external code.
 """
 
-#c Copyright 2008-2022, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -73,7 +73,7 @@ class CachedGetter(object):
 	
 	def __call__(self):
 		if (self.isAlive is not None
-				and self.cache is not None 
+				and self.cache is not None
 				and not self.isAlive(self.cache)):
 			self.cache = None
 
@@ -85,7 +85,7 @@ class CachedGetter(object):
 				self.cache = self.getter(*self.args, **self.kwargs)
 
 				# If the cache is immortal, do away with the stuff needed
-				# for its creation 
+				# for its creation
 				if self.isAlive is None:
 					del self.args
 					del self.kwargs
@@ -102,7 +102,7 @@ class CachedResource(object):
 	"""
 	cache = None
 
-	@classmethod 
+	@classmethod
 	def __new__(cls, arg):
 		if cls.cache is None:
 			cls.cache = cls.impl()
@@ -164,7 +164,7 @@ class IdManagerMixin(object):
 		- getForId(id) -- returns the object belonging to an id that has
 			been handed out before.  Raises a NotFoundError for unknown ids.
 
-		- cloneFrom(other) -- overwrites the self's id management dictionaries 
+		- cloneFrom(other) -- overwrites the self's id management dictionaries
 			with those from other.  You want this if two id managers must work
 			on the same document.
 	"""
@@ -179,8 +179,8 @@ class IdManagerMixin(object):
 			return self.__objectToId, self.__idsToObject
 
 	def _fixSuggestion(self, suggestion, invMap):
-		for i in itertools.count():
-			newId = suggestion+str(i)
+		for i in itertools.count(start=2):
+			newId = f"{suggestion}-{i:02d}"
 			if newId not in invMap:
 				return newId
 
@@ -196,7 +196,7 @@ class IdManagerMixin(object):
 		if id(ob) in map:
 			return None
 
-		if suggestion is not None: 
+		if suggestion is not None:
 			if suggestion in invMap:
 				newId = self._fixSuggestion(suggestion, invMap)
 			else:
@@ -286,7 +286,7 @@ class Infimum(_Comparer):
 	>>> -2333<Infimum
 	False
 	"""
-	cmpRes = True 
+	cmpRes = True
 
 
 class Supremum(_Comparer):
@@ -310,7 +310,7 @@ class AllEncompassingSet(set):
 	"""a set that contains everything.
 
 	Ok, so this doesn't exist.  Yes, I've read my Russell.  You see, this
-	is a restricted hack for a reason.  And even the docstring is 
+	is a restricted hack for a reason.  And even the docstring is
 	contradictory.
 
 	Sort-of.  This now works for intersection and containing.
@@ -367,8 +367,8 @@ def buildClassResolver(baseClass, objects, instances=False,
 	"""returns a function resolving classes deriving from baseClass
 	in the sequence objects by their names.
 
-	This is used to build registries of Macros and RowProcessors.  The
-	classes in question have to have a name attribute.
+	This is used to build registries of, for instance, Macros and RowProcessors.
+	The classes in question have to have a name attribute.
 
 	objects would usually be something like globals().values()
 
@@ -387,6 +387,7 @@ def buildClassResolver(baseClass, objects, instances=False,
 		clsKey = key(cls)
 		if clsKey is not None:
 			registry[clsKey] = cls
+
 	def resolve(name, registry=registry):
 		try:
 			return registry[name]
@@ -394,6 +395,7 @@ def buildClassResolver(baseClass, objects, instances=False,
 			if default is not None:
 				return default
 			raise
+
 	resolve.registry = registry
 	return resolve
 
@@ -424,7 +426,7 @@ def silence(errToo=False):
 			sys.stdout.close()
 			sys.stdout = realstdout
 			if errToo:
-				sys.stderr = realstderr 
+				sys.stderr = realstderr
 
 
 @contextlib.contextmanager
@@ -503,7 +505,7 @@ def runInSandbox(setUp, func, tearDown, *args, **kwargs):
 	owd = os.getcwd()
 	# within DaCHS, this should be within tempDir, but we don't bother to
 	# get access to DaCHS' config.  So there.
-	wd = tempfile.mkdtemp("sandbox")  
+	wd = tempfile.mkdtemp("sandbox")
 	try:
 		if setUp:
 			setUp(wd, *args, **kwargs)
@@ -516,74 +518,100 @@ def runInSandbox(setUp, func, tearDown, *args, **kwargs):
 	return result
 
 
-class _FunctionCompiler(object):
-	"""A singleton to keep compileFunction's state somewhat localised.
+def _instrumentForDebugging(func, funcName, src):
+	"""returns a callable for a callable func with instrumentation for
+	debugging.
 
-	The state currently is a counter used to build unique ids for
-	stuff compiled.
+	src must be the source of func.
+
+	Specifically, when there's an exception when calling func, this will
+	be logged as a warning.
 	"""
-	compiledCount = 0
+	debugLocals = {}
+	embSrc = "\n".join([
+		"from gavo.utils import excs",
+		"def compileFunctionDebugWrapper(*args, **kwargs):",
+		"  try:",
+		"    return %s(*args, **kwargs)"%funcName,
+		"  except (excs.ExecutiveAction, AssertionError):",
+		"    raise",
+		"  except:",
+		'    notify("Failing source:\\n%s"%src)',
+		"    raise"])
+	debugLocals["src"] = src
+	debugLocals["notify"] = lambda msg: misctricks.sendUIEvent("Warning", msg)
+	debugLocals[funcName] = func
+	exec(embSrc+"\n", debugLocals)
+	return debugLocals["compileFunctionDebugWrapper"]
 
-	@classmethod
-	def _compile(cls, src, funcName, useGlobals=None, debug=False):
-		"""runs src through exec and returns the item funcName from the resulting
-		namespace.
 
-		This is typically used to define functions, like this:
+def compileFunction(src,
+		funcName,
+		useGlobals=None,
+		debug=False,
+		uniqueName=None):
+	"""runs src through exec and returns the funcName from the resulting
+	namespace.
 
-		>>> resFunc = compileFunction("def f(x): print(x)", "f")
-		>>> resFunc(1); resFunc("abc")
-		1
-		abc
-		"""
-		src = src+"\n"
+	This takes care to preserve src in the line cache so it is available
+	in tracebacks or in the debugger.
 
-		locals = {}
-		if useGlobals is None:
-			useGlobals = globals()
+	useGlobals can be a namespace; if not passed, the globals of the
+	utils.codestricks is used.
 
-		uniqueName = "<generated code %s>"%cls.compiledCount
-		cls.compiledCount += 1
+	If debug=True is passed in, additional code is produced to give halfway
+	useful tracebacks.
 
-		try:
-			code = compile(src, uniqueName, 'exec')
-			exec(code, useGlobals, locals)
-		except Exception as ex:
-			misctricks.sendUIEvent("Warning", "The code that failed to compile was:"
-				"\n%s"%src)
-			raise misctricks.logOldExc(excs.BadCode(src, "function", ex))
-		func = locals[funcName]
+	uniqueName, if given, is the identifier for the code.  If passed in,
+	no automatic cleanup for the linecache is done under the assumption
+	that reloads (or whatever) will overwrite the linecache.  Otherwise
+	code is autonumbered (which is not really desirable for user-provided
+	code, as they won't know where their failing code comes from).
+	
+	This is typically used to define functions, like this:
 
-		# this makes our compiled lines available to the traceback writer.
-		# we might want to do sys.excepthook = traceback.print_exception
-		# somewhere so the post mortem dumper uses this, too.  Let's see
-		# if it's worth the added risk of breaking things.
-		linecache.cache[uniqueName] = len(src), None, src.split("\n"), uniqueName
-		func._cleanup = weakref.ref(func, 
+	>>> resFunc = compileFunction("def f(x): print(x)", "f")
+	>>> resFunc(1); resFunc("abc")
+	1
+	abc
+	"""
+	if not hasattr(compileFunction, "autoNumber"):
+		compileFunction.autoNumber = 0
+
+	doCleanup = False
+	if uniqueName is None:
+		uniqueName = "<generated code %s>"%compileFunction.autoNumber
+		compileFunction.autoNumber += 1
+		doCleanup = True
+	src = src+"\n"
+
+	locals = {}
+	if useGlobals is None:
+		useGlobals = globals()
+
+	try:
+		code = compile(src, uniqueName, 'exec')
+		exec(code, useGlobals, locals)
+	except Exception as ex:
+		misctricks.sendUIEvent("Warning", "The code that failed to compile was:"
+			"\n%s"%src)
+		raise misctricks.logOldExc(excs.BadCode(src, "function", ex))
+	func = locals[funcName]
+
+	# this makes our compiled lines available to the traceback writer.
+	# we might want to do sys.excepthook = traceback.print_exception
+	# somewhere so the post mortem dumper uses this, too.  Let's see
+	# if it's worth the added risk of breaking things.
+	linecache.cache[uniqueName] = len(src), None, src.split("\n"), uniqueName
+	if doCleanup:
+		func._cleanup = weakref.ref(func,
 			lambda _, key=uniqueName: linecache and linecache.cache.pop(key, None))
 
-		if debug:
-			debugLocals = {}
-			embSrc = "\n".join([
-				"from gavo.utils import excs",
-				"def compileFunctionDebugWrapper(*args, **kwargs):",
-				"  try:",
-				"    return %s(*args, **kwargs)"%funcName,
-				"  except (excs.ExecutiveAction, AssertionError):",
-				"    raise",
-				"  except:",
-				'    notify("Failing source:\\n%s"%src)',
-				"    raise"])
-			debugLocals["src"] = src
-			debugLocals["notify"] = lambda msg: misctricks.sendUIEvent("Warning", msg)
-			debugLocals[funcName] = func
-			exec(embSrc+"\n", debugLocals)
-			return debugLocals["compileFunctionDebugWrapper"]
-				
+	if debug:
+		return _instrumentForDebugging(func, funcName, src)
+	else:
 		return func
 
-compileFunction = _FunctionCompiler._compile
-
 
 def ensureExpression(expr, errName="unknown"):
 	"""raises a LiteralParserError if expr is not a parseable python expression.
@@ -651,7 +679,7 @@ def loadPythonModule(fqName, relativeTo=None):
 	The python path is temporarily amended with the path part of the
 	source module.
 
-	If the module is in /var/gavo/inputs/foo/bar/mod.py, Python will know 
+	If the module is in /var/gavo/inputs/foo/bar/mod.py, Python will know
 	the module as foo_bar_mod (the last two path components are always added).
 	This is to keep Python from using the module when someone writes
 	import mod.
@@ -738,9 +766,9 @@ def identity(x):
 
 
 def intToFunnyWord(anInt, translation=bytes.maketrans(
-		b"-0123456789abcdef", 
+		b"-0123456789abcdef",
 		b"zaeiousmnthwblpgd")):
-	"""returns a sometimes funny (but unique) byte string from an arbitrary 
+	"""returns a sometimes funny (but unique) byte string from an arbitrary
 	integer.
 
 	>>> intToFunnyWord(3829901938)
@@ -779,7 +807,7 @@ def memoizeOn(onObject, generatingObject, generatingFunction, *args):
 	This is for caching things that adapt to onObjects; see procdefs
 	and rowmakers for examples why this is useful.
 	"""
-	cacheName = "_cache%s%s"%(generatingObject.__class__.__name__, 
+	cacheName = "_cache%s%s"%(generatingObject.__class__.__name__,
 		str(id(generatingObject)))
 	if getattr(onObject, cacheName, None) is None:
 		setattr(onObject, cacheName, generatingFunction(*args))
@@ -884,7 +912,7 @@ class NocaseString(str):
 	it does a linear search.
 
 	Normal DaCHS code doesn't need this because of various hacks in
-	contextgrammar and elsewhere.  If you're touching request.fields manually,
+	contextgrammar and elsewhere.  If you're touching request.args manually,
 	you'll have to look at this, though.
 
 	Case insensitivity is evil.  Let's get rid of it and then get rid of
@@ -914,6 +942,14 @@ class floatlist(list): pass
 class complexlist(list): pass
 
 
+class NS(object):
+	"""An object that has its kwargs as attributes.
+	"""
+	def __init__(self, **kwargs):
+		for k,v in kwargs.items():
+			setattr(self, k, v)
+
+
 if __name__=="__main__":  # pragma: no cover
 	import doctest
 	doctest.testmod()
