@@ -5,7 +5,7 @@ It's unclear if pyfits coming from astropy is now threadsafe, but I'm hoping
 it might be, so for now I'm making fitslock a no-op.
 """
 
-#c Copyright 2008-2022, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -38,10 +38,10 @@ try:
 	# temporary measure to shut up astropy's configuration parser.
 	builtins._ASTROPY_SETUP_ = True
 	# the following is the original import of pyfits, and only it should be used
-	from astropy.io import fits as pyfits  
+	from astropy.io import fits as pyfits
 	from astropy.utils.exceptions import AstropyDeprecationWarning
 	warnings.filterwarnings('ignore', category=AstropyDeprecationWarning)
-except ImportError:  
+except ImportError:
 	# pyfits is not installed; don't die, since the rest of gavo.utils
 	# will still work.
 	pyfits = misctricks.NotInstalledModuleStub( #noflake: exported name
@@ -106,7 +106,7 @@ def padCard(input, length=CARD_SIZE):
 
 
 def readHeaderBytes(f, maxHeaderBlocks=80):
-	"""returns the bytes beloning to a FITS header starting at the current
+	"""returns the bytes belonging to a FITS header starting at the current
 	position within the file f.
 
 	If the header is not complete after reading maxHeaderBlocks blocks,
@@ -150,7 +150,7 @@ def readPrimaryHeaderQuick(f, maxHeaderBlocks=80):
 def parseCards(aString):
 	"""returns a list of pyfits Cards parsed from aString.
 
-	This will raise a ValueError if aString's length is not divisible by 80.  
+	This will raise a ValueError if aString's length is not divisible by 80.
 	It may also raise pyfits errors for malformed cards.
 
 	Empty (i.e., all-whitespace) cards are ignored.  If an END card is
@@ -191,7 +191,7 @@ def replacePrimaryHeader(inputFile, newHeader, targetFile, bufSize=100000):
 	"""writes a FITS to targetFile having newHeader as the primary header,
 	where the rest of the data is taken from inputFile.
 
-	inputFile must be a file freshly opened for binary reading, targetFile one 
+	inputFile must be a file freshly opened for binary reading, targetFile one
 	freshly opened for binary writing.
 
 	This function is (among other things) a workaround for pyfits' misfeature of
@@ -281,11 +281,11 @@ def _enforceHeaderConstraints(cardList, cardSequence):
 
 	This may raise a FITSError if mandatory cards are missing.
 
-	This will only work correctly for image FITSes with less than five 
+	This will only work correctly for image FITSes with less than five
 	dimensions.
 
 	On cardSequence, see sortHeaders (except that this function always
-	requries sequences of pairs).
+	requires sequences of pairs).
 	"""
 # I can't use pyfits.verify for this since cardList may not refer to
 # a data set that's actually in memory
@@ -339,7 +339,7 @@ def sortHeaders(header, commentFilter=None, historyFilter=None,
 	As a shortcut, a sequence item may be something else then a tuple;
 	it will then be combined with a False to make one.
 
-	These days, if you think you need this, have a look at 
+	These days, if you think you need this, have a look at
 	fitstricks.makeHeaderFromTemplate first.
 	"""
 	commentCs, historyCs, realCs = [], [], []
@@ -389,25 +389,25 @@ def openGz(fitsName, tempDir=None):
 	f.close()
 	hdus = pyfits.open(pathname)
 	hdus.readall()
-	os.unlink(pathname) 
+	os.unlink(pathname)
 	return hdus
 
 
 def writeGz(hdus, fitsName, compressLevel=5, mode=0o664):
-	"""writes and gzips hdus into fitsName.  As a side effect, hdus will be 
+	"""writes and gzips hdus into fitsName.  As a side effect, hdus will be
 	closed.
 
-	Appearently, not even recent versions of pyfits support writing of
+	Apparently, not even recent versions of pyfits support writing of
 	zipped files (which is a bit tricky, admittedly).  So, we'll probably
 	have to live with this kludge for a while.
 	"""
 	handle, pathname = tempfile.mkstemp(suffix="fits")
 	with codetricks.silence():
-		hdus.writeto(pathname, clobber=True)
+		hdus.writeto(pathname, overwrite=True)
 	os.close(handle)
 	rawFitsData = open(pathname, "rb").read()
 	os.unlink(pathname)
-	handle, pathname = tempfile.mkstemp(suffix="tmp", 
+	handle, pathname = tempfile.mkstemp(suffix="tmp",
 		dir=os.path.dirname(fitsName))
 	os.close(handle)
 	dest = gzip.open(pathname, "w", compressLevel)
@@ -420,23 +420,18 @@ def writeGz(hdus, fitsName, compressLevel=5, mode=0o664):
 def openFits(fitsName):
 		"""returns the hdus for fName.
 
-		(gzip detection is tacky, and we should look at the magic).
-
 		This is simulating legacy pyfits behaviour in that it will use
 		memmap I/O if it thinks that works.  You'll probably want to use
-		astrpy directly these days if this gives you a headache.
+		astropy directly these days if this gives you a headache.
 		"""
-		if os.path.splitext(fitsName)[1].lower()==".gz":
-			return openGz(fitsName)
-		else:
-			hdus = pyfits.open(fitsName, memmap=True)
-			# memmapping will bomb later if data needs to be scaled, so we
-			# guess now if that's going to happen
-			if (hdus[0].header.get("BZERO", 0)!=0
-					or hdus[0].header.get("BSCALE", 1)!=1
-					or "BLANK" in hdus[0].header):
-				hdus = pyfits.open(fitsName, memmap=False)
-			return hdus
+		hdus = pyfits.open(fitsName, memmap=True)
+		# memmapping will bomb later if data needs to be scaled, so we
+		# guess now if that's going to happen
+		if (hdus[0].header.get("BZERO", 0)!=0
+				or hdus[0].header.get("BSCALE", 1)!=1
+				or "BLANK" in hdus[0].header):
+			hdus = pyfits.open(fitsName, memmap=False)
+		return hdus
 
 
 class PlainHeaderManipulator:
@@ -445,7 +440,7 @@ class PlainHeaderManipulator:
 
 	This class exists because pyfits insists on scaling scaled image data
 	on reading it.  While we can scale back on writing, this is something
-	I'd rather not do.  So, I have this base class to facilate the 
+	I'd rather not do.  So, I have this base class to facilate the
 	HeaderManipulator that can handle gzipped fits files as well.
 	"""
 	def __init__(self, fName):
@@ -559,7 +554,7 @@ def cutoutFITS(hdu, *cuts):
 	hdu is a primary FITS hdu.  cuts is a list of cut specs, each of which is
 	a triple (axis, lower, upper).  axis is between 1 and naxis, lower and
 	upper a 1-based pixel coordinates of the limits, and "border" pixels
-	are included.  Specifications outside of the image are legal and will 
+	are included.  Specifications outside of the image are legal and will
 	be cropped back.  Open limits are supported via a specification of
 	None.
 
@@ -571,10 +566,10 @@ def cutoutFITS(hdu, *cuts):
 	array.
 
 	WCS CRPIXes in hdu's header will be updated.  Axes and specified will
-	not be touched.  It is an error to specifiy cuts for an axis twice 
+	not be touched.  It is an error to specify cuts for an axis twice
 	(behaviour is undefined).
 
-	Note that this will lose all extensions the orginal FITS file might have
+	Note that this will lose all extensions the original FITS file might have
 	had.
 	"""
 	slices, newHeader = cutoutHeader(hdu.header, *cuts)
@@ -636,11 +631,11 @@ def shrinkWCSHeader(oldHeader, factor):
 
 
 NUM_CODE = {
-		8: 'uint8', 
-		16: '>i2', 
-		32: '>i4', 
-		64: '>i8', 
-		-32: '>f4', 
+		8: 'uint8',
+		16: '>i2',
+		32: '>i4',
+		64: '>i8',
+		-32: '>f4',
 		-64: '>f8'}
 
 def _makeDecoder(hdr):
@@ -657,7 +652,7 @@ def _makeDecoder(hdr):
 	if bzero!=0 or bscale!=1:
 		def read(f):
 			return numpy.asarray(
-				numpy.frombuffer(f.read(nBytes), numType, rowLength), 
+				numpy.frombuffer(f.read(nBytes), numType, rowLength),
 					'float32')*bscale+bzero
 	else:
 		def read(f):  #noflake: conditional definition
@@ -717,7 +712,7 @@ def fixImageExtind(hdus, extInd):
 def _iterSetupCompatible(inFile, hdr, extInd=0):
 	"""helps iterScaledRows for when _iterSetupFast will not work.
 
-	Using extInd, you can select a different extension.  extInd=0 
+	Using extInd, you can select a different extension.  extInd=0
 	will automatically select extension 1 if that's a compressed image
 	HDU.
 	"""
@@ -792,12 +787,12 @@ def iterScaledRows(inFile, factor=None, destSize=None, hdr=None, slow=False,
 
 
 def iterScaledBytes(inFileName, factor, extraCards={}):
-	"""iterates over the bytes for a simple FITS file generated by scaling 
+	"""iterates over the bytes for a simple FITS file generated by scaling
 	down the 2D image inFileName by factor.
 
 	factor must be an integer between 1 and something reasonable.
 
-	This function acquires the FITS lock itself.  
+	This function acquires the FITS lock itself.
 	"""
 	with fitsLock():
 		with open(inFileName, "rb") as f:
@@ -842,7 +837,7 @@ class WCSAxis(object):
 	simple and robust.  It doesn't know many of the finer points of WCS,
 	though, and in particular it's 1D only.
 
-	However, for the purposes of cutouts it probably should do for the 
+	However, for the purposes of cutouts it probably should do for the
 	overwhelming majority of non-spatial FITS axes.
 
 	The default pixel coordinates are handled in the FITS sense here,
@@ -889,7 +884,7 @@ class WCSAxis(object):
 		return self.physToPix(physCoo)-1
 
 	def getLimits(self):
-		"""returns the minimal and maximal physical values this axis 
+		"""returns the minimal and maximal physical values this axis
 		takes within the image.
 		"""
 		limits = self.pixToPhys(1), self.pixToPhys(self.axisLength)
@@ -922,10 +917,10 @@ class WCSAxis(object):
 
 		guessedName = "%s_%d"%(guessedName, axisIndex)
 
-		return cls(guessedName, 
+		return cls(guessedName,
 			get("CRVAL", 0), get("CRPIX", 0), get("CDELT", 1),
-			get("CTYPE", "UNKNOWN").strip(), 
-			get("CUNIT", "").strip() or None, 
+			get("CTYPE", "UNKNOWN").strip(),
+			get("CUNIT", "").strip() or None,
 			get("NAXIS", 1))
 
 
