@@ -2,7 +2,7 @@
 Math-related helper functions.
 """
 
-#c Copyright 2008-2022, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -16,10 +16,10 @@ DEG = math.pi/180
 ARCSEC = DEG/3600
 
 def findMinimum(f, left, right, minInterval=3e-8):
-	"""returns an estimate for the minimum of the single-argument function f 
+	"""returns an estimate for the minimum of the single-argument function f
 	on (left,right).
 
-	minInterval is a fourth of the smallest test interval considered.  
+	minInterval is a fourth of the smallest test interval considered.
 
 	For constant functions, a value close to left will be returned.
 
@@ -102,8 +102,8 @@ def dotprod3(seq1, seq2, inds=(0,1,2)):
 
 
 def crossprod3(seq1, seq2):
-	return (seq1[1]*seq2[2]-seq1[2]*seq2[1], 
-		seq1[2]*seq2[0]-seq1[0]*seq2[2], 
+	return (seq1[1]*seq2[2]-seq1[2]*seq2[1],
+		seq1[2]*seq2[0]-seq1[0]*seq2[2],
 		seq1[0]*seq2[1]-seq1[1]*seq2[0])
 
 
@@ -202,6 +202,24 @@ def spherDist(vec1, vec2):
 	return math.acos(dotprod3(vec1, vec2))
 
 
+def floorInt(flt:float):
+	"""returns the largest integer smaller than flt, except if flt is a
+	special float, in which case flt is returned.
+	"""
+	if math.isfinite(flt):
+		return int(math.floor(flt))
+	return flt
+
+
+def ceilInt(flt:float):
+	"""returns the smallest integer largest than flt, except if flt is a
+	special float, in which case flt is returned.
+	"""
+	if math.isfinite(flt):
+		return int(math.ceil(flt))
+	return flt
+
+
 if __name__=="__main__": # pragma: no cover
 	import doctest
 	doctest.testmod()
