@@ -12,7 +12,7 @@ and we fall back to the built-in pyparsing (which is 2.4) if your pyparsing
 is too old.
 """
 
-#c Copyright 2008-2022, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -24,7 +24,7 @@ import contextlib
 import os
 import threading
 
-if True:
+if False:
 	from gavo.imp.pyparsing import *
 else:
 	import pyparsing
@@ -33,6 +33,36 @@ else:
 	else:
 		from pyparsing import *
 
+
+class MatchFirstWithSaneError(pyparsing.MatchFirst):
+	# upstream MatchFirst munges the error messages, which sometimes
+	# really is distastrous for us.  We monkeypatch this until
+	# https://github.com/pyparsing/pyparsing/issues/464 is somehow
+	# resolved
+  def parseImpl(self, instring, loc, doActions=True):
+        maxExcLoc = -1
+        maxException = None
+        for e in self.exprs:
+            try:
+                ret = e._parse(instring, loc, doActions)
+                return ret
+            except pyparsing.ParseException as err:
+                if err.loc > maxExcLoc:
+                    maxException = err
+                    maxExcLoc = err.loc
+            except IndexError:
+                if len(instring) > maxExcLoc:
+                    maxException = pyparsing.ParseException(instring, len(instring), e.errmsg, self)
+                    maxExcLoc = len(instring)
+
+        # only got here if no expression matched, raise exception for match that made it the furthest
+        else:
+            if maxException is not None:
+                raise maxException
+            else:
+                raise pyparsing.ParseException(instring, loc, "no defined alternatives to match", self)
+
+
 ####################### Pyparsing hacks
 #
 # (1) When building grammars, always do so using the pyparsingWhitechars
@@ -63,7 +93,7 @@ def pyparsingWhitechars(whiteChars):
 	Since whitespace apparently can only be set globally for pyparsing,
 	we provide this c.m.  Since it is possible that grammars will be
 	compiled in threads (e.g., as a side effect of getRD), this is
-	protected by a lock.  This, in turn, means that this can 
+	protected by a lock.  This, in turn, means that this can
 	potentially block for a fairly long time.
 
 	Bottom line: When compiling pyparsing grammars, *always* set
