@@ -2,11 +2,12 @@
 Formatting, text manipulation, string constants, and such.
 """
 
-#c Copyright 2008-2022, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
 
+
 import calendar
 import datetime
 import functools
@@ -111,7 +112,7 @@ def getFileStem(fPath):
 
 	The base name is what remains if you take the base name and split off
 	extensions.  The extension here starts with the last dot in the file name,
-	except up to one of some common compression extensions (.gz, .xz, .bz2, 
+	except up to one of some common compression extensions (.gz, .xz, .bz2,
 	.Z, .z) is stripped off the end if present before determining the extension.
 
 	>>> getFileStem("/foo/bar/baz.x.y")
@@ -140,7 +141,7 @@ def formatSimpleTable(data, stringify=True, titles=None):
 
 	If you pass titles, it must be a sequence of strings; they are then
 	used as table headers; the shorter of data[0] and titles will determine
-	the number fo columns displayed.
+	the number of columns displayed.
 	"""
 	if stringify:
 		data = [[str(v) for v in row] for row in data]
@@ -166,7 +167,7 @@ def getRelativePath(fullPath, rootPath, liberalChars=True):
 	exception otherwise.
 
 	Pass ``liberalChars=False`` to make this raise a ValueError when
-	URL-dangerous characters (blanks, amperands, pluses, non-ASCII, and 
+	URL-dangerous characters (blanks, amperands, pluses, non-ASCII, and
 	similar) are present in the result.  This is mainly for products.
 	"""
 	if not fullPath.startswith(rootPath):
@@ -191,7 +192,7 @@ def resolvePath(rootPath, relPath):
 			"Full path %s does not start with resource root %s"%(fullPath, rootPath))
 	if not os.path.exists(fullPath):
 		raise ValueError(
-			"Invalid path %s. This should not happend."%(fullPath))
+			"Invalid path %s. This should not happen."%(fullPath))
 	return fullPath
 
 
@@ -340,7 +341,7 @@ def hmsToDeg(hms, sepChar=None):
 
 @codetricks.document
 def dmsToDeg(dmsAngle, sepChar=None):
-	"""returns the degree minutes seconds-specified dmsAngle as a 
+	"""returns the degree minutes seconds-specified dmsAngle as a
 	float in degrees.
 
 	>>> "%3.8f"%dmsToDeg("45 30.6")
@@ -386,9 +387,9 @@ def fracHoursToDeg(fracHours):
 
 
 @codetricks.document
-def degToHms(deg, 
-		sepChar=" ", 
-		secondFracs=3, 
+def degToHms(deg,
+		sepChar=" ",
+		secondFracs=3,
 		truncate=False):
 	"""converts a float angle in degrees to an time angle (hh:mm:ss.mmm).
 
@@ -422,7 +423,7 @@ def degToHms(deg,
 	"""
 	if truncate:
 		return angle_formats.hours_to_string(
-			deg/360*24, sep=(sepChar, sepChar), 
+			deg/360*24, sep=(sepChar, sepChar),
 			precision=secondFracs+8, pad=True)[:-8].rstrip(".")
 
 	return angle_formats.hours_to_string(
@@ -433,7 +434,7 @@ def degToHms(deg,
 def hoursToHms(decimal_hours, sepChar=":", secondFracs=0):
 	"""returns a time span in hours in sexagesmal time (h:m:s).
 
-	The optional argments are as for degToHms.
+	The optional arguments are as for degToHms.
 
 	>>> hoursToHms(0)
 	'00:00:00'
@@ -453,9 +454,9 @@ def hoursToHms(decimal_hours, sepChar=":", secondFracs=0):
 
 
 @codetricks.document
-def degToDms(deg, 
-		sepChar=" ", 
-		secondFracs=2, 
+def degToDms(deg,
+		sepChar=" ",
+		secondFracs=2,
 		preserveLeading=False,
 		truncate=False,
 		addSign=True):
@@ -536,12 +537,12 @@ def parseRFC2616Date(s):
 
 
 # The following timegm implementation is due to Frederik Lundh
-def _d(y, m, d, days=(0,31,59,90,120,151,181,212,243,273,304,334,365)): 
+def _d(y, m, d, days=(0,31,59,90,120,151,181,212,243,273,304,334,365)):
 		return (((y - 1901)*1461)/4 + days[m-1] + d + (
 			(m > 2 and not y % 4 and (y % 100 or not y % 400)) and 1))
 
-def timegm(tm, epoch=_d(1970,1,1)): 
-		year, month, day, h, m, s = tm[:6] 
+def timegm(tm, epoch=_d(1970,1,1)):
+		year, month, day, h, m, s = tm[:6]
 		return (_d(year, month, day) - epoch)*86400 + h*3600 + m*60 + s
 
 
@@ -616,7 +617,7 @@ def parseISODT(literal, useTime=False):
 
 	else:
 		return datetime.datetime(int(parts["year"]), int(parts["month"]),
-			int(parts["day"]), int(parts["hour"]), int(parts["minute"]), 
+			int(parts["day"]), int(parts["hour"]), int(parts["minute"]),
 			int(parts["seconds"]), int(float(parts["secFracs"])*1000000))
 
 
@@ -810,7 +811,7 @@ def _iterSimpleTextNoContinuation(f):
 
 @codetricks.document
 def iterSimpleText(f):
-	"""iterates over ``(physLineNumber, line)`` in f with some usual 
+	"""iterates over ``(physLineNumber, line)`` in f with some usual
 	conventions for simple data files.
 
 	You should use this function to read from simple configuration and/or
@@ -897,7 +898,9 @@ def defuseFileName(fName, replaceSlash=True):
 	a string that's guaranteed to be ASCII only.  With replaceSlash=False,
 	it can also double as a reasonable asciification.
 	"""
-	res = quopri.encodestring(fName.encode("utf-8")
+	if isinstance(fName, str):
+		fName = fName.encode("utf-8")
+	res = quopri.encodestring(fName
 		).translate(DEFUSE_NONPRINTABLE
 		).decode("ascii")
 	if replaceSlash:
