@@ -22,11 +22,11 @@ This module contains a base class and the VOTable type system conversion,
 as the VOTable module (that should not depend on base) depends on it.
 The remaining actual converters are in base.typesystems, as they may depend
 on details of base.  Even the SQL converters should be taken from there
-when code can rely on gavo.base; this module should be considered an 
+when code can rely on gavo.base; this module should be considered an
 implementation detail.
 """
 
-#c Copyright 2008-2022, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -157,7 +157,12 @@ class ToVOTableConverter(FromSQLConverter):
 			# we may leave a leading * there, which upstream would have
 			# to fix (oh, madness)
 
-			t, l, xtype = self.simpleMap[type]
+			try:
+				t, l, xtype = self.simpleMap[type]
+			except KeyError:
+				raise excs.StructureError(
+					f"Cannot infer a VOTable type for '{type}'")
+
 			if l is None:
 				newLength = length
 			else:
