@@ -2,29 +2,22 @@
 Common code for the nevow based interface.
 """
 
-#c Copyright 2008-2022, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
 
 
-import cgi
-import io
 import json
 import re
-import urllib.request, urllib.parse, urllib.error
-from email.message import Message
+from urllib import parse as urlparse
 
+from multipart import multipart
 from twisted.python import failure
 from twisted.web import resource
 from twisted.web import server
 from twisted.web import static
 from twisted.web import template
-try:
-	from twisted.python.compat import _PY3, _PY37PLUS
-except ImportError:
-	# python3.5 in stretch
-	_PY3, _PY37PLUS = True, False
 from twisted.web.template import tags as T
 from twisted.web import http
 from twisted.web.http import parse_qs
@@ -36,8 +29,6 @@ from gavo.base import meta
 from gavo.protocols import creds
 from gavo.protocols import dali
 
-# a magic sentinel for the FieldStorage hack (see below)
-_FSHACK_MAGIC = "__FSHACK_MAGIC_KEY__"
 
 # monkeypatching static's mime magic
 static.File.contentTypes[".ascii"] = "application/octet-stream"
@@ -102,7 +93,7 @@ class HTMLMetaBuilder(meta.MetaBuilder):
 	def endKey(self, atom):
 		children = [c for c in self.resultTree.pop() if c]
 		# see enterValue on why we're doing this
-		if (self.resultTree[-1] 
+		if (self.resultTree[-1]
 				and self.resultTree[-1][-1] is None):
 			return
 
@@ -139,8 +130,8 @@ class doctypedStan(template.TagLoader):
 	work if the template will be rendered through nevowc.TemplatedPage.
 	"""
 
-	DOCTYPE = (b'<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN"'
-		b' "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">')
+	DOCTYPE = ('<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN"'
+		' "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">')
 
 	def __init__(self, rootEl):
 		rootEl(xmlns="http://www.w3.org/1999/xhtml")
@@ -196,12 +187,12 @@ class CommonRenderers(object):
 		res = tag[
 			T.meta(**{"http-equiv": "Content-type",
 				"content": "text/html;charset=UTF-8"}),
-			T.link(rel="stylesheet", 
+			T.link(rel="stylesheet",
 				href=base.makeSitePath("/static/css/formal.css"),
 				type="text/css"),
 			T.link(rel="stylesheet", href=base.makeSitePath(
 				"/static/css/gavo_dc.css"), type="text/css"),
-			T.script(src=base.makeSitePath("/static/js/jquery-gavo.js"), 
+			T.script(src=base.makeSitePath("/static/js/jquery-gavo.js"),
 				type="text/javascript"),
 			T.script(src=base.makeSitePath("/static/js/gavo"+JSEXT),
 				type="text/javascript"),
@@ -218,7 +209,7 @@ class CommonRenderers(object):
 		"""renders data as a url-escaped string.
 		"""
 		data = tag.slotData
-		return urllib.parse.quote(data)
+		return urlparse.quote(data)
 	
 	@template.renderer
 	def getconfig(self, request, tag):
@@ -286,56 +277,6 @@ class JSONQuery(resource.Resource):
 		return json.dumps(res).encode("utf-8")
 
 
-####################### Start twisted request overhaul
-#
-# Python 3 twisted has had quite a few severe shortcomings which I
-# hacked around for a while.  In the end, people said I might as
-# well just override requestReceived in a way convenient for me.
-#
-# This is what happens here, together with some support code.
-#
-# twisted.web uses cgi.parse_multipart that, in python3, internally uses
-# cgi.FieldStorage.  I really can't live without FieldStorage in DaCHS,
-# what with all the uploads that are being hurled around in various ways.
-# But I don't want to construct the FieldStorage a second time because we
-# may have fairly substantial amounts of upload and because it sucks.
-# However, the parse_multipart thing is in the midst of a large chunk of
-# code in twisted.web.server, and the request.args interface is hard
-# to change anyway.  So, I instead hack cgi.parse_multipart to be what
-# it is in python3.7; it will now return its internal FieldStorage in
-# a magic key in the attribute that I pick out in requestReceived.
-# While I'm at it, I also remove file uploads from request.args, which makes
-# my life easier in some other ways.
-
-def parse_multipart(fp, pdict, encoding="utf-8", errors="replace"):
-	"""Parse multipart input from an input file fp.
-
-	pdict is a dictionary containing other parameters of content-type header,
-	encoding and errors is the request encoding and error handler, passed to
-  FieldStorage
-
-	This is the basically python 3.7 implementation of this, except I'm
-	returning the internal FieldStorage in the dictionary (key None).
-	"""
-# RFC 2026, Section 5.1 : The "multipart" boundary delimiters are always
-# represented as 7bit US-ASCII.
-	boundary = pdict['boundary'].decode('ascii')
-	ctype = "multipart/form-data; boundary={}".format(boundary)
-	headers = Message()
-	headers.set_type(ctype)
-	headers['Content-Length'] = pdict['CONTENT-LENGTH']
-	fs = cgi.FieldStorage(fp, 
-		headers=headers, 
-		encoding=encoding,
-	 	errors=errors,
-		environ={'REQUEST_METHOD': 'POST'})
-	res = {k: fs.getlist(k)[:] for k in fs}
-	res[_FSHACK_MAGIC] = fs
-	return res
-
-cgi.parse_multipart = parse_multipart
-
-
 class BinaryItem:
 	"""A sentinel marking request.args items that couldn't be utf-8 decoded
 
@@ -358,52 +299,6 @@ class BinaryItem:
 			utils.makeEllipsis(repr(self.original)))
 
 
-class _UploadLimiter(object):
-	"""A wrapper class for upload destinations that limits writes to
-	limit bytes and sends a 413 if the limit is reached.
-	"""
-	def __init__(self, original, request, limit):
-		self.original, self.request, self.limit = original, request, limit
-		self.writtenSoFar = 0
-
-		# at least twisted 16 doesn't have clientproto while doing the
-		# upload; I'll fix this here heuristically
-		if request.clientproto.startswith(b"("):
-			request.clientproto = getattr(request.channel, "_version", b"HTTP/1.0")
-		# ...about the same for the client attribute
-		if not hasattr(request, "client"):
-			request.client = getattr(request.channel, "client", b"(unknown client)")
-		self.discard = False
-	
-	def __getattr__(self, attrName):
-		return getattr(self.original, attrName)
-
-	def write(self, bytes):
-		if self.discard:
-			return
-
-		self.writtenSoFar += len(bytes)
-		if self.writtenSoFar>=self.limit:
-			# I wonder if there's a way to stop the client without
-			# brutally closing the connection (which would kill our
-			# error messages).
-			self.discard = True
-
-		else:
-			self.original.write(bytes)
-
-	def finish(self):
-		"""break cycles and return our original.
-
-		This is mainly a courtesy to the garbage collector.
-		"""
-		del self.request
-		if self.discard:
-			raise svcs.RequestEntityTooLarge(
-				f"Would only accept {self.limit} bytes of upload here.")
-		return self.original
-
-
 class Request(server.Request):
 	"""a custom request class used in DaCHS' application server.
 
@@ -411,21 +306,19 @@ class Request(server.Request):
 
 	* you can write str; it's going to be utf-8 encoded automatically,
 	  which out to be about the right thing almost always in DaCHS.
-	* we enforce a limit to the size of the upload payload.
 	* There's a popSegments method for use during resource dispatch (it
 	  gets all remaining segments and pushes them to prepath at the same
 	  time).  popSegments also returns (utf-8-decoded) strings rather than
 	  bytes.
-	* There's the FieldStorage hack (see below).
 	* There's strargs, which is a dict mapping strings to lists of strings.
-	  This is being created at first call can then doesn't follow 
+	  This is being created at first call can then doesn't follow
 	  request.args any more.  DaCHS code should only use strargs unless
 	  you're actually dealing with non-utf-8 data (and then you should
-	  probably use request.fields).
+	  probably use request.files).
 	"""
 	def __init__(self, *args, **kwargs):
 		server.Request.__init__(self, *args, **kwargs)
-		self.fields = None
+		self.files, self.args = {}, {}
 		self.finished = False
 		self.notifyFinish().addBoth(self._markFinished)
 	
@@ -450,19 +343,15 @@ class Request(server.Request):
 			base.ui.notifyError("In gotLength: %s"%msg)
 
 		if isSync:
-			maxUploadSize = base.getConfig("web", "maxSyncUploadSize")
+			self.maxUploadSize = base.getConfig("web", "maxSyncUploadSize")
 		else:
-			maxUploadSize = base.getConfig("web", "maxUploadSize")
+			self.maxUploadSize = base.getConfig("web", "maxUploadSize")
 
 		# The following is a shortcut to break out as soon as possible
 		# when the uploading program declares a content length (though
 		# by twisted's architecture we can't exit in this place anyway)
-		if length and length>maxUploadSize:
-			maxUploadSize = 0
-
-		if self.content:
-			self.content = _UploadLimiter(
-				self.content, self, maxUploadSize)
+		if length and length>self.maxUploadSize:
+			self.maxUploadSize = 0
 
 	def write(self, payload):
 		if self.finished or getattr(self, "channel", None) is None:
@@ -532,7 +421,7 @@ class Request(server.Request):
 		match segments.  If you're devious, you might exhaust the
 		prepath in this way; that's your fault, then.
 
-		segments may contain strings or bytes and will be destroyed in the 
+		segments may contain strings or bytes and will be destroyed in the
 		process.
 		"""
 		postSegments = []
@@ -601,21 +490,9 @@ class Request(server.Request):
 
 		Use the strargs property to get this.
 		"""
-		if self.fields is None:
-			# GET request: there only is args, and we need to encode things
-			def iterParams():
-				for k, v in self.args.items():
-					yield k.decode("iso-8859-1", "ignore"), v
-
-		else:
-			# POST Request: field storage has basically all we need.
-			def iterParams():
-				for k in self.fields:
-					yield k, self.fields.getlist(k)
-				
 		res = utils.CaseSemisensitiveDict()
-		for key, values in iterParams():
-			res[key] = mapped = []
+		for key, values in self.args.items():
+			res[key.decode("latin-1")] = mapped = []
 			for value in values:
 				try:
 					if isinstance(value, str):
@@ -645,7 +522,7 @@ class Request(server.Request):
 		"""
 		if isinstance(perhapsFailure, failure.Failure):
 			base.ui.notifyFailure(
-				perhapsFailure, 
+				perhapsFailure,
 				"A failure escaped to finishCallback.  This probably means no"
 				" error was reported to the requester.  ")
 		self.finish()
@@ -653,47 +530,19 @@ class Request(server.Request):
 	def processingFailed(self, failure):
 		return produceErrorDocument(failure, self)
 
-	def process(self):
-		# overridden because of the FieldStorage hack (see below);
-		# the _FSHACK_MAGIC has been bytified by twisted's requestReceived
-		# Also, only do this if fields hasn't been set by the chunked
-		# upload hack (which needs to do that early, before twisted
-		# butchers the value).
-		if self.fields is None:
-			self.fields = self.args.pop(_FSHACK_MAGIC.encode("ascii"), None)
-		server.Request.process(self)
-
-	@staticmethod
-	def _parseHeader(line):
-		# cgi.parse_header requires a str
-		key, pdict = cgi.parse_header(line.decode('charmap'))
-
-		# We want the key as bytes, and cgi.parse_multipart (which consumes
-		# pdict) expects a dict of str keys but bytes values
-		key = key.encode('charmap')
-		pdict = {x:y.encode('charmap') for x, y in pdict.items()}
-		return (key, pdict)
-
 	def requestReceived(self, command, path, version):
-		# overridden because several upstream versions broke chunked
-		# uploads, and then to restore file uploads, which it broke
-		# a bit later.
-		# This is essentially requestReceived as of twisted 19.7 or so,
-		# with quite a few version switches removed because DaCHS only
-		# runs with python 3.7 or later anyway.
+		# overridden because I want to use multipart to parse
+		# POST payloads.
 		#
-		# changes vs. twisted:
+		# This is essentially requestReceived as of twisted 22.1 with
+		# the entire POST processing replaced.  Oh, and there's a
+		# TAP path hack to avoid breaking links to pre-TAP-corified
+		# DaCHS versions.
 		#
-		# * removed a strategic "and clength"
-		# * I'm pulling the _FSHACK_MAGIC from cgiArgs before twisted code
-		#   butchers it.
-		# * I'm reading in the whole request body into a BytesIO because
-    #   python cgi really can't live without clength.
-		#
-		# * There's an URL mapping hack in there for legacy TAP for now.
-		#
-		self.content.seek(0,0)
-		self.args = {}
+		# TODO: parsing this is still sync, which for large request bodies
+		# can take quite a while, during which we are otherwise inaccessible.
+		# I'd have to research hard how I can yield to the reactor from
+		# in here...
 
 		# Brute-force TAP compatibility redirect, 2021
 		# As long as old registry records may still contain the legacy
@@ -703,13 +552,14 @@ class Request(server.Request):
 		# I suppose this ugly stuff can go ~2025; or we'll need an alternative
 		# way of doing this kind of thing.
 		path = path.replace(
-			b"__system__/tap/run/tap/", 
+			b"__system__/tap/run/tap/",
 			b"__system__/tap/run/")
 
+		self.content.seek(0, 0)
+
 		self.method, self.uri = command, path
 		self.clientproto = version
-
-		x = self.uri.split(b'?', 1)
+		x = self.uri.split(b"?", 1)
 
 		if len(x) == 1:
 			self.path = self.uri
@@ -718,78 +568,71 @@ class Request(server.Request):
 			self.args = parse_qs(argstring, 1)
 
 		# Argument processing
-		args = self.args
-		ctype = self.requestHeaders.getRawHeaders(b'content-type')
-		clength = self.requestHeaders.getRawHeaders(b'content-length')
+		ctype = self.requestHeaders.getRawHeaders(b"content-type")
 		if ctype is not None:
 			ctype = ctype[0]
 
-		if clength is not None:
-			clength = clength[0]
-
 		if self.method == b"POST" and ctype:
-			mfd = b'multipart/form-data'
-			key, pdict = self._parseHeader(ctype)
-			pdict["CONTENT-LENGTH"] = clength
-			if key == b'application/x-www-form-urlencoded':
-				args.update(parse_qs(self.content.read(), 1))
-			elif key == mfd:
-				try:
-					# HACK ALERT: if clength is none, in this situation we probably have
-					# a chunked upload.  cgi.FieldStorage can't deal with that in
-					# python 3.7.  We pull in the whole stuff synchronously to cope
-					# and hope for the best in terms of blocking behaviour...
-					# TODO: We really should go back to just passing content.
-					if clength is None:
-						uploaded = io.BytesIO(self.content.read())
-						self.content = uploaded
-						pdict["CONTENT-LENGTH"] = len(uploaded.getvalue())
+			chunk_size = 1048576
 
-					cgiArgs = cgi.parse_multipart(
-						self.content, pdict, encoding='utf8',
-						errors="surrogateescape")
+			def addArg(field):
+				# The string/byte situation on urlparse.unquote* is confusingly
+				# bizarre.  If I want to be compatible with baseline twisted
+				# args, I don't think I get around the odd quote/dequote thing
+				# I'm doing here; and yes, it's latin1 on the incoming string,
+				# as any non-ascii in there probably isn't utf-8.
+				if field.value is None:
+					# ignore empty parameters coming in (it's what cgi did, too)
+					return
+				self.args.setdefault(field.field_name, []).append(
+					urlparse.unquote_plus(
+						field.value.decode("latin-1")).encode("utf-8"))
 
-					# The parse_multipart function on Python 3.7+
-					# decodes the header bytes as iso-8859-1 and
-					# decodes the body bytes as utf8 with
-					# surrogateescape -- we want bytes
-					self.fields = cgiArgs.pop(_FSHACK_MAGIC, {})
-					self.args.update({
-						x.encode('iso-8859-1'): \
-						[z.encode('utf8', "surrogateescape")
-						 if isinstance(z, str) else z for z in y]
-						for x, y in cgiArgs.items()})
+			def addFile(file_obj):
+				# some versions of multipart pass in the files already
+				# consumed.  Until this is fixed, rewind them here.
+				file_obj.file_object.seek(0)
+				self.files.setdefault(file_obj.field_name.decode("latin-1"), []
+				    ).append(file_obj)
 
-				except Exception as e:
-					# It was a bad request, or we got a signal.
-					self.channel._respondToBadRequestAndDisconnect()
-					if isinstance(e, (TypeError, ValueError, KeyError)):
-						return
-					else:
-						# If it's not a userspace error from CGI, reraise
-						raise
-				self.content.seek(0,0)
+			# we could enfoce maxUploadSize using MAX_BODY_SIZE in
+			# multipart; however, that just skips the extra bytes.
+			# I'd like to return a 413, and hence I count myself.
+			total_read = 0
 
-		# take the upload limiter out of content; it's done its work
-		try:
-			if hasattr(self.content, "finish"):
-				self.content = self.content.finish()
-		except svcs.RequestEntityTooLarge as ex:
-			return dali.serveDALIError(self, ex, httpStatus=413)
+			try:
+				strippedCtype, ctypeParams = multipart.parse_options_header(ctype)
+				parser = multipart.FormParser(
+					strippedCtype.decode("latin-1"), addArg, addFile,
+					boundary=ctypeParams.get(b"boundary"))
+
+				while True:
+					stuff = self.content.read(chunk_size)
+					if stuff==b"":
+						break
+
+					total_read += len(stuff)
+					if total_read>self.maxUploadSize:
+						raise svcs.RequestEntityTooLarge(
+							f"Would only accept {self.maxUploadSize} bytes of upload here.")
+
+					parser.write(stuff)
+				parser.finalize()
+
+			except svcs.RequestEntityTooLarge as ex:
+				return dali.serveDALIError(self, ex, httpStatus=413)
+
+			except Exception as e:
+				# It was a bad request, or we got a signal.
+				base.ui.notifyError(f"Fatal error while parsing request: {e}")
+				self.channel._respondToBadRequestAndDisconnect()
+				if isinstance(e, (TypeError, ValueError, KeyError)):
+					return
+				else:
+					# If it's not a "userspace" error, reraise
+					raise
+
+			self.content.seek(0, 0)
 
 		self.process()
 
-
-######################## cgi.make_file workaround
-# The cgi module creates temporary disk files under the assumption
-# its process is short-lived.  Well, DaCHS processes aren't and so
-# we override make_file; again, we have plenty of RAM and not terribly
-# many concurrent users, so we keep things off the disk and avoid
-# the use of FDs (that aren't returned if they are used).
-
-def _monkey_make_file(self):
-	if self._binary_file:
-		return io.BytesIO()
-	else:
-		return io.StringIO()
-cgi.FieldStorage.make_file = _monkey_make_file
