@@ -5,7 +5,7 @@ There's StaticRender just delivering certain files from within a service,
 and there's FixedPageRenderer that just formats a defined template.
 """
 
-#c Copyright 2008-2022, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -40,23 +40,24 @@ class StaticRenderer(grend.ServiceBasedPage):
 	renderer if you find yourself needing this.
 	"""
 	name = "static"
+	defaultType = "text/html"
 
 	def __init__(self, request, service):
 		grend.ServiceBasedPage.__init__(self, request, service)
 		try:
-			self.indexFile = os.path.join(service.rd.resdir, 
+			self.indexFile = os.path.join(service.rd.resdir,
 				service.getProperty("indexFile"))
 		except base.NotFoundError:
 			self.indexFile = None
 		try:
-			self.staticPath = os.path.join(service.rd.resdir, 
+			self.staticPath = os.path.join(service.rd.resdir,
 				service.getProperty("staticData"))
 		except base.NotFoundError:
 			self.staticPath = None
 
 	@classmethod
 	def isBrowseable(self, service):
-		return service.getProperty("indexFile", None) 
+		return service.getProperty("indexFile", None)
 
 	@classmethod
 	def isCacheable(cls, segments, request):
@@ -68,10 +69,14 @@ class StaticRenderer(grend.ServiceBasedPage):
 			raise svcs.WebRedirect(request.uri+b"/")
 
 		if self.indexFile:
-			return ifpages.StaticFile(self.indexFile, self.rd).render(request)
+			try:
+				return ifpages.StaticFile(self.indexFile, self.rd).render(request)
+			except os.error as ex:
+				raise base.NotFoundError(self.indexFile, "File", "file system",
+					hint=str(ex))
 		else:
 			if self.staticPath is None:
-				raise svcs.ForbiddenURI("No static data on this service") 
+				raise svcs.ForbiddenURI("No static data on this service")
 
 			return ifpages.DirectoryLister(self.staticPath).render(request)
 
@@ -81,7 +86,7 @@ class StaticRenderer(grend.ServiceBasedPage):
 			return self
 
 		if self.staticPath is None:
-			raise svcs.ForbiddenURI("No static data on this service") 
+			raise svcs.ForbiddenURI("No static data on this service")
 
 		relPath = "/".join(segments)
 		destName = os.path.join(self.staticPath, relPath)
@@ -99,14 +104,38 @@ class StaticRenderer(grend.ServiceBasedPage):
 			return static.File(destName).directoryListing()
 
 		elif os.path.isfile(destName):
-			return ifpages.StaticFile(destName, self.rd)
+			return ifpages.StaticFile(destName, self.rd,
+				defaultType=self.defaultType)
 
 		else:
 			raise svcs.UnknownURI("No %s available here."%relPath)
-		
+
+
+class HiPSRenderer(StaticRenderer):
+	"""A static renderer with a few amenities for HiPS trees.
+
+	To make this work, set the service's staticData property
+	"""
+	name = "hips"
+	# for the benefit of properties
+	defaultType = "text/plain"
+	urlUse = "base"
+
+	def __init__(self, request, service):
+		super().__init__(request, service)
+		self.indexFile = os.path.join(self.staticPath, "index.html")
+
+	@classmethod
+	def isCacheable(cls, segments, request):
+		return False
+
+	@classmethod
+	def isBrowseable(self, service):
+		return True
+
 
 class FixedPageRenderer(
-		grend.CustomTemplateMixin, 
+		grend.CustomTemplateMixin,
 		grend.ServiceBasedPage,
 		grend.HTMLResultRenderMixin):
 	"""A renderer that renders a single template.
@@ -115,7 +144,7 @@ class FixedPageRenderer(
 	in the enclosing service to tell the fixed renderer where to get
 	this template from.
 
-	In the template, you can fetch parameters from the URL using 
+	In the template, you can fetch parameters from the URL using
 	something like ``<n:invisible n:data="parameter FOO" n:render="string"/>``;
 	you can also define new render and data functions on the
 	service using customRF and customDF.
@@ -130,7 +159,7 @@ class FixedPageRenderer(
 	During development, users must add ?nocache=True to a fixed page
 	URI to force DaCHS to reload the template.
 
-	Built-in services for such browser apps should go through the //run 
+	Built-in services for such browser apps should go through the //run
 	RD.
 	"""
 	name = "fixed"
