@@ -4,7 +4,7 @@ Support for IVOA DAL and registry protocols.
 DALI-specified protocols are in dalirender, UWS/async is in asyncrender.
 """
 
-#c Copyright 2008-2022, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2023, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -39,7 +39,7 @@ MS = base.makeStruct
 class DALRenderer(grend.ServiceBasedPage):
 	"""A base class for renderers for the usual IVOA DAL protocols.
 
-	This is for simple, GET-based DAL renderers (where we allow POST as 
+	This is for simple, GET-based DAL renderers (where we allow POST as
 	well).  They work using nevow forms, but with standard-compliant error
 	reporting (i.e., in VOTables).
 
@@ -76,11 +76,11 @@ class DALRenderer(grend.ServiceBasedPage):
 			runner = self.runAsync
 		
 		# overwrite previous query meta for new limit
-		self.queryMeta = svcs.QueryMeta.fromRequest(request, 
+		self.queryMeta = svcs.QueryMeta.fromRequest(request,
 			defaultLimit=self.defaultLimit)
 
 		def execRunner():
-			if (self.queryMeta["dbLimit"]==0 
+			if (self.queryMeta["dbLimit"]==0
 					or self.queryMeta.ctxArgs.get("FORMAT", "").lower()=="metadata"):
 				return self._produceMetadata(request)
 			else:
@@ -135,9 +135,9 @@ class DALRenderer(grend.ServiceBasedPage):
 		data = rsc.makeData(dataDesc)
 		data.tables["results"].votCasts = self._outputTableCasts
 		data.setMeta("_type", "meta")
-		data.addMeta("info", 
+		data.addMeta("info",
 			base.getMetaText(self.service, "title") or "Unnamed",
-			infoName="serviceInfo", 
+			infoName="serviceInfo",
 			infoValue=str(self.service.getURL(self.name)))
 
 		return data
@@ -156,10 +156,10 @@ class DALRenderer(grend.ServiceBasedPage):
 				b"/xsl/meta-votable-to-html.xsl' type='text/xsl'?>"
 			)+votLit[splitPos:])
 
-	def _writeErrorTable(self, 
-			request, 
-			errmsg, 
-			httpStatus=200, 
+	def _writeErrorTable(self,
+			request,
+			errmsg,
+			httpStatus=200,
 			queryStatus="ERROR"):
 		# Unfortunately, most legacy DAL specs say the error messages must
 		# be delivered with a 200 response code.  I hope this is going
@@ -172,7 +172,7 @@ class DALRenderer(grend.ServiceBasedPage):
 		return dali.serveDALIError(request, errmsg, httpStatus, queryStatus)
 
 	def _formatOutput(self, data, request, stream=True):
-		if isinstance(data, tuple):  
+		if isinstance(data, tuple):
 			# core returned a complete document (mime and string)
 			mime, payload = data
 			request.setHeader("content-type", mime)
@@ -184,12 +184,12 @@ class DALRenderer(grend.ServiceBasedPage):
 				request.finish()
 			return server.NOT_DONE_YET
 
-		data.addMeta("info", "", infoName="QUERY_STATUS", 
-			infoValue=base.getMetaText(data.getPrimaryTable(), 
+		data.addMeta("info", "", infoName="QUERY_STATUS",
+			infoValue=base.getMetaText(data.getPrimaryTable(),
 				"_queryStatus"))
 
 		if self.standardId:
-			data.addMeta("info", 
+			data.addMeta("info",
 				"Written by DaCHS %s %s"%(base.getVersion(), self.__class__.__name__),
 				infoName="standardID",
 				infoValue=self.standardId)
@@ -205,7 +205,7 @@ class DALRenderer(grend.ServiceBasedPage):
 			requestedType = self.resultType
 
 		request.setHeader("content-type", requestedType)
-		request.setHeader('content-disposition', 
+		request.setHeader('content-disposition',
 			'attachment; filename="result.%s"'%
 			formats.getExtensionFor(requestedType))
 
@@ -237,7 +237,7 @@ class DALRenderer(grend.ServiceBasedPage):
 			httpStatus, queryStatus = 400, "EMPTY"
 
 		return self._writeErrorTable(
-			request, 
+			request,
 			failure.value,
 			httpStatus=httpStatus,
 			queryStatus=queryStatus)
@@ -276,7 +276,7 @@ class SCSRenderer(DALRenderer):
 		request.setHeader("content-type", base.votableType)
 		votable.write(V.VOTABLE[
 			V.DESCRIPTION[base.getMetaText(self.service, "description")],
-			V.RESOURCE[
+			V.RESOURCE(type="results")[
 				V.INFO(ID="Error", name="Error",
 						value=str(msg).replace('"', '\\"')),]], request)
 		request.write("\n")
@@ -292,11 +292,11 @@ class SCSRenderer(DALRenderer):
 			return DALRenderer._formatOutput(self, data, request, stream)
 
 		ucdCasts = {
-			"meta.id;meta.main": {"ucd": "ID_MAIN", "datatype": "char", 
+			"meta.id;meta.main": {"ucd": "ID_MAIN", "datatype": "char",
 				"arraysize": "*"},
-			"pos.eq.ra;meta.main": {"ucd": "POS_EQ_RA_MAIN", 
+			"pos.eq.ra;meta.main": {"ucd": "POS_EQ_RA_MAIN",
 				"datatype": "double"},
-			"pos.eq.dec;meta.main": {"ucd": "POS_EQ_DEC_MAIN", 
+			"pos.eq.dec;meta.main": {"ucd": "POS_EQ_DEC_MAIN",
 				"datatype": "double"},
 		}
 		realCasts = {}
@@ -316,7 +316,7 @@ class SCSRenderer(DALRenderer):
 			realCasts[idCol.name]["castFunction"] = str
 		table.votCasts = realCasts
 
-		return DALRenderer._formatOutput(self, 
+		return DALRenderer._formatOutput(self,
 			data, request, stream=stream)
 
 
@@ -335,7 +335,7 @@ class SIAPRenderer(DALRenderer):
 
 		- testQuery.pos.ra, testQuery.pos.dec -- RA and Dec for a query that
 			yields at least one image
-		- testQuery.size.ra, testQuery.size.dec -- RoI extent for a query that 
+		- testQuery.size.ra, testQuery.size.dec -- RoI extent for a query that
 			yields at least one image.
 	
 	You can set the following metadata items (there are defaults on them
@@ -353,15 +353,15 @@ class SIAPRenderer(DALRenderer):
 
 	_outputTableCasts = {
 		"pixelScale": {
-			"datatype": "double", 
+			"datatype": "double",
 			"arraysize": "*",
 			"ucd": "VOX:Image_Scale"},
 		"wcs_cdmatrix": {
-			"datatype": "double", 
+			"datatype": "double",
 			"arraysize": "*",
 			"ucd": "VOX:WCS_CDMatrix"},
 		"wcs_refValues": {
-			"datatype": "double", 
+			"datatype": "double",
 			"arraysize": "*",
 			"ucd": "VOX:WCS_CoordRefValue"},
 		"bandpassHi": {
@@ -374,11 +374,11 @@ class SIAPRenderer(DALRenderer):
 			"datatype": "double",
 			"ucd": "VOX:BandPass_RefValue"},
 		"wcs_refPixel": {
-			"datatype": "double", 
+			"datatype": "double",
 			"arraysize": "*",
 			"ucd": "VOX:WCS_CoordRefPixel"},
 		"wcs_projection": {
-			"arraysize": "3", 
+			"arraysize": "3",
 			"castFunction": lambda s: s[:3],
 			"ucd": "VOX:WCS_CoordProjection"},
 		"mime": {"ucd": "VOX:Image_Format"},
@@ -404,12 +404,12 @@ class SIAPRenderer(DALRenderer):
 			data.setMeta("_type", "results")
 			data.getPrimaryTable().votCasts = self._outputTableCasts
 
-		return DALRenderer._formatOutput(self, 
-			data, request, 
+		return DALRenderer._formatOutput(self,
+			data, request,
 			stream=stream)
 	
 	def _makeErrorTable(self, request, msg, queryStatus="ERROR"):
-		return V.VOTABLE11[
+		return V.VOTABLE[
 			V.RESOURCE(type="results")[
 				V.INFO(name="QUERY_STATUS", value=queryStatus)[
 					str(msg)]]]
@@ -440,7 +440,7 @@ class UnifiedDALRenderer(DALRenderer):
 			data.setMeta("_type", "results")
 			data.getPrimaryTable().votCasts = self._outputTableCasts
 
-		return DALRenderer._formatOutput(self, 
+		return DALRenderer._formatOutput(self,
 			data, request, stream=stream)
 	
 	def _makeErrorTable(self, request, msg, queryStatus="ERROR"):
@@ -479,7 +479,7 @@ class SIAP2Renderer(UnifiedDALRenderer):
 		if isinstance(failure.value, base.EmptyData):
 			httpStatus, queryStatus = 400, "EMPTY"
 
-		return self._writeErrorTable(request, 
+		return self._writeErrorTable(request,
 			"UsageFault: "+failure.getErrorMessage(),
 			httpStatus=httpStatus,
 			queryStatus=queryStatus)
@@ -488,7 +488,7 @@ class SIAP2Renderer(UnifiedDALRenderer):
 class SSAPRenderer(UnifiedDALRenderer):
 	"""A renderer for the simple spectral access protocol.
 
-	For registration, you must set the following metadata 
+	For registration, you must set the following metadata
 	for the ssap.xml renderer:
 
 	 - ssap.dataSource -- survey, pointed, custom, theory, artificial
@@ -514,8 +514,8 @@ class SSAPRenderer(UnifiedDALRenderer):
 	 - datalink -- if present, this must be the id of a datalink service
 	   that can work with the pubDIDs in this table (don't use this any more,
 	   datalink is handled through table-level metadata now)
-	 - defaultRequest -- by default, requests without a REQUEST parameter 
-	   will be rejected.  If you set defaultRequest to querydata, such 
+	 - defaultRequest -- by default, requests without a REQUEST parameter
+	   will be rejected.  If you set defaultRequest to querydata, such
 	   requests will be processed as if REQUEST were given (which is of
 	   course sane but is a violation of the standard).
 	"""
@@ -541,7 +541,7 @@ class SSAPRenderer(UnifiedDALRenderer):
 			data.addMeta("_votableRootAttributes",
 				'xmlns:ssa="http://www.ivoa.net/xml/DalSsap/v1.0"')
 			data.addMeta("info", "SSAP",
-				infoName="SERVICE_PROTOCOL", 
+				infoName="SERVICE_PROTOCOL",
 				infoValue=self.version)
 			
 			# In SSA, we want a "direct" SODA block if we have a dlget-capable
@@ -561,34 +561,31 @@ class SSAPRenderer(UnifiedDALRenderer):
 						# descriptor class from the datalink service)
 						continue
 
-					# endpoint 0 hopefully is the sync service. TODO: make this a bit
-					# more robust.
 					core = ssap.getDatalinkCore(dlService, table)
 					sodaGenerators.append(
-						lambda ctx, core=core, svcMeta=svcMeta, table=table: 
+						lambda ctx, core=core, svcMeta=svcMeta, table=table:
 							core.datalinkEndpoints[0].asVOT(
 								ctx,
-								dlService.getURL(core.datalinkEndpoints[0].rendName),
 								linkIdTo=ctx.getOrMakeIdFor(
 									table.tableDef.getByName(
 										base.getMetaText(svcMeta, "idColumn")))))
 			if sodaGenerators:
 				data.sodaGenerators = sodaGenerators
 
-		return UnifiedDALRenderer._formatOutput(self, 
+		return UnifiedDALRenderer._formatOutput(self,
 			data, request, stream=stream)
 
 
 class SLAPRenderer(UnifiedDALRenderer):
 	"""A renderer for the simple line access protocol SLAP.
 
-	For registration, you must set the following metadata on services 
+	For registration, you must set the following metadata on services
 	using the slap.xml renderer:
 
 	There's two mandatory metadata items for these:
 
-	- slap.dataSource -- one of observational/astrophysical, 
-	  observational/laboratory, or theoretical 
+	- slap.dataSource -- one of observational/astrophysical,
+	  observational/laboratory, or theoretical
 	- slap.testQuery -- parameters that lead to a non-empty response.
 	  The way things are written in DaCHS, MAXREC=1 should in general
 	  work.
@@ -606,7 +603,7 @@ class SLAPRenderer(UnifiedDALRenderer):
 				'xmlns:ssldm="http://www.ivoa.net/xml/SimpleSpectrumLineDM'
 					'/SimpleSpectrumLineDM-v1.0.xsd"')
 
-		return UnifiedDALRenderer._formatOutput(self, 
+		return UnifiedDALRenderer._formatOutput(self,
 			data, request, stream=stream)
 
 
@@ -642,7 +639,7 @@ class RegistryRenderer(grend.ServiceBasedPage):
 
 		streamResponse = request.strargs.get("verb")==["ListRecords"]
 
-		self.runAsync(inData, 
+		self.runAsync(inData,
 			).addCallback(self._renderXML, request, streamResponse
 			).addErrback(self._renderError, request, inData["args"])
 		return server.NOT_DONE_YET
@@ -695,7 +692,7 @@ class RegistryRenderer(grend.ServiceBasedPage):
 			return
 
 		try:
-			if not isinstance(failure.value, 
+			if not isinstance(failure.value,
 					(registry.OAIError, base.ValidationError)):
 				base.ui.notifyFailure(failure)
 			return self._renderXML(self._getErrorTree(failure.value, pars),
@@ -709,7 +706,7 @@ class RegistryRenderer(grend.ServiceBasedPage):
 
 
 def addAttachmentHeaders(request, mime=None):
-	"""adds a content-disposition header to request with a filename guessed 
+	"""adds a content-disposition header to request with a filename guessed
 	based on datalink arguments.
 
 	This tries a number of heuristics to try and preserve a bit of the
@@ -738,17 +735,17 @@ def addAttachmentHeaders(request, mime=None):
 			stem = "result"+stem
 
 		# if there's any arguments we suspect have changed the contents
-		argkeys = [k for k in 
+		argkeys = [k for k in
 				set(request.strargs.keys()) - set(["ID", "RESPONSEFORMAT"])
 			if request.strargs[k]]
 		if argkeys:
 			stem = stem+"_proc"
 
-		request.setHeader('content-disposition', 
+		request.setHeader('content-disposition',
 			'attachment; filename=%s%s'%(stem, ext))
 	except:
 		# if all fails, use a safe, if dumb, fallback
-		request.setHeader('content-disposition', 
+		request.setHeader('content-disposition',
 			'attachment; filename=result.dat')
 
 
@@ -822,7 +819,7 @@ def _doDatalinkXSLT(data, _cache={}):
 	"""a temporary hack to do server-side XSLT while the browser implementations
 	apparently suck.
 
-	Remove this once we've worked out how to make the datalink-to-xml 
+	Remove this once we've worked out how to make the datalink-to-xml
 	stylesheet compatible with actual browsers.
 	"""
 	if "etree" not in _cache:
